/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package com.jla.desc2.descartes;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import com.jla.desc2.math.*;
import com.jla.desc2.util.*;
import com.jla.desc2.gui.*;
import com.jla.desc2.parser.*;


public class Control {

	public R2 cpos;
	public int size;
	public String controlName,s_decimals="2",s_size="4",s_constraint,text,s_initialPos="(0,0)",s_cond;
	public Font font=mjaGui.Courier;
	public Color ccolor=Color.blue,icolor=Color.red,tcolor=Color.gray;
	public boolean hasTrace;
	public static final String _x=".x",_y=".y";

	private Descartes D;
	private Vector[] pstr;
	private Node constraint, decimals, cond, funcOfX, funcOfY;
	private R2Newton newt;


	Control (int L,String id) {
		controlName=id;
	}

	Control (int L,String id,String values) {
		this(L,id);
		Attribute[] aa=Attribute.parse(values);
		s_initialPos=data.getValue(L,aa,data.expr);
		text=data.getValue(L,aa,data.t_text);
		text=mjaStr.replace(text,"\\n","\n");
		font=mjaStr.parseFont(data.getValue(L,aa,data.font),font);
		ccolor=data.parseColor(data.getValue(L,aa,data.color),ccolor,ccolor);
		icolor=data.parseColor(data.getValue(L,aa,data.icolor),icolor,icolor);
		String s_trace=data.getValue(L,aa,data.trace);
		hasTrace=mjaStr.hasContent(s_trace);
		if (hasTrace) {
			tcolor=data.parseColor(s_trace,tcolor,tcolor);
		}
		s_decimals=data.getValue(L,aa,data.decimals,s_decimals);
		s_size=data.getValue(L,aa,data.size,s_size);
		s_constraint=data.getValue(L,aa,data.constraint);
		s_cond=data.getValue(L,aa,data.cond);
	}

	String toString(String sep,int L) {
		String s=data.toString(sep,L,data.expr,"'"+s_initialPos+"'");
		s+=data.toString(sep,L,data.color,mjaStr.getColorName(data.colorName[L],ccolor),data.newName[L][data.blue]);
		s+=data.toString(sep,L,data.icolor,mjaStr.getColorName(data.colorName[L],icolor),data.newName[L][data.red]);

		s+=data.toString(sep,L,data.t_text,"'"+mjaStr.replace(text,"\n","\\n")+"'");
		s+=data.toString(sep,L,data.decimals,"'"+s_decimals+"'","2");
		s+=data.toString(sep,L,data.size,s_size,"4");
		s+=data.toString(sep,L,data.cond,"'"+s_cond+"'");
		if (hasTrace) {
			s+=data.toString(sep,L,data.trace,mjaStr.getColorName(data.colorName[L],tcolor));
		}
		if (mjaStr.hasContent(s_constraint)) {
			s+=data.toString(sep,L,data.constraint,"'"+s_constraint+"'");
		}
		if (!font.equals(mjaGui.Courier)) {
			s+=data.toString(sep,L,data.font,mjaStr.FontToString(font));
		}
		return s.trim();
	}

	void Initialize(Descartes D) {
		this.D=D;
		String[] sa=mjaStr.toStringArray(text);
		pstr=new Vector[sa.length];
		for (int i=0;i<pstr.length;i++) {
			pstr[i]=D.p.parseString(sa[i]);
		}
		size=mjaStr.parseInteger(s_size,2,2);
		if (mjaStr.hasContent(s_constraint)) {
			constraint=D.p.Analyse(s_constraint);
			if (constraint.getSymbol().equals("=")) {
				Node left=constraint.getLeft();
				Node right=constraint.getRight()[0];
				if (left.getSymbol().equals("y")&&!right.contains("y")) {
					funcOfX=right;
				} else if (left.getSymbol().equals("x")&&!right.contains("x")) {
					funcOfY=right;
				}
				constraint.equalToMinus();
				newt=new R2Newton(D.p,constraint);
			} else {
				constraint=null;
			}
		}
		decimals=D.p.Analyse(s_decimals,2);
		cond=D.p.Analyse(s_cond,"1");
		cpos=new R2(0,0);
		try {
			sa=mjaStr.getTokens(s_initialPos);
			if (sa.length>=2) {
				Node xN=D.p.Analyse(sa[0]);
				Node yN=D.p.Analyse(sa[1]);
				cpos=new R2(xN.Evaluate(0),yN.Evaluate(0));
			} else {
				System.out.println("wrong expression: "+s_initialPos);
			}
		} catch (Exception exc) {
			System.out.println(exc);
		}
		D.p.Var(controlName+_x).r=cpos.x;
		D.p.Var(controlName+_y).r=cpos.y;
	}

	/* ------------------------ Controls ------------------------ */

	void setControlPos(double x,double y) {
//		D.updateSpinnersFromTextFields();
		cpos=D.cs.injection(x,y);
		if (constraint!=null) {
			try {
//				cpos=R2.Newton(D.p,constraint,cpos);
				cpos=newt.findZero(cpos);
			} catch (Exception e) {
			}
		}
		D.p.Var(controlName+_x).r=cpos.x;
		D.p.Var(controlName+_y).r=cpos.y;
		D.updateSpinners();
	}

	R2 getControlPos() {
		return D.cs.projection(cpos);
	}

	boolean isActive;

	void adjust() {
//		D.updateSpinnersFromTextFields();
		Node X=D.p.Var("x"),Y=D.p.Var("y");
		synchronized(X){
		synchronized(Y){
			double vx=X.r,vy=Y.r;
			double xa=cpos.x;
			double ya=cpos.y;
			cpos.x=D.p.value(controlName+_x);
			cpos.y=D.p.value(controlName+_y);
			X.r=cpos.x;
			Y.r=cpos.y;
			if (constraint!=null) {
				try {
					if (funcOfX!=null&&cpos.x!=xa) {
						cpos.y=funcOfX.Evaluate();
					} else if (funcOfY!=null&&cpos.y!=ya) {
						cpos.x=funcOfY.Evaluate();
					} else {
						cpos=newt.findZero(cpos);
	//					cpos=R2.Newton(D.p,constraint,cpos);
					}
				} catch (Exception e) {
						cpos.x=xa; cpos.y=xa;
				}
			}
			D.p.Var(controlName+_x).r=cpos.x;
			D.p.Var(controlName+_y).r=cpos.y;
			X.r=vx; Y.r=vy;
		}
		}
		D.updateSpinners();
	}

	void drawControl(boolean trace,Graphics[] g)	{
		Node X=D.p.Var("x"),Y=D.p.Var("y");
		synchronized(X){
		synchronized(Y){
			double vx=X.r,vy=Y.r;
			X.r=cpos.x;
			Y.r=cpos.y;
			if (cond.Evaluate(1)>0) {
				R2 q=D.cs.projection(cpos);
				for (int i=0;i<g.length;i++) {
					if (trace) {
						g[i].setColor(tcolor);
	//					mjaGraph.circle(g[i],q.ix(),q.iy(),size-1);
						g[i].drawOval(q.ix()-size,q.iy()-size,2*size,2*size);
					} else {
						g[i].setColor(icolor);
						mjaGraph.circle(g[i],q.ix(),q.iy(),size);
						if (isActive) {
							g[i].setColor(mjaGui.bestContrast(icolor));
							g[i].drawOval(q.ix()-size+2,q.iy()-size+2,2*size-4,2*size-4);
						}
						g[i].setColor(ccolor);
						g[i].drawOval(q.ix()-size,q.iy()-size,2*size,2*size);
					}
				}
				if (!trace) {
					drawText(q,g);
				}
			}
			X.r=vx; Y.r=vy;
		}
		}
	}

	void drawText(R2 q,Graphics[] g) {
		if (D.cs.inside(q)) {
			for (int i=0;i<g.length;i++) {
				g[i].setFont(font);
				FontMetrics fm=g[i].getFontMetrics(font);
				for (int n=0;n<pstr.length;n++) {
					String texto="";
					int y0=0;
					y0+=n*fm.getHeight();
					int dec=(int)Math.round(decimals.Evaluate(2));
					for (int j=0;j<pstr[n].size();j++) {
						Node auxn=(Node) pstr[n].elementAt(j);
						texto+=auxn.toString(dec,D.language);
					}
					g[i].drawString(texto,q.ix()+size,y0+q.iy()-size);
				}
			}
		}
	}


}