/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package com.jla.desc2.gui;
/*------------------------------------------------------------*/
/*                         Spinner                            */
/*                         (c)  MJA                           */
/*                     Agosto de 2000                         */
/*------------------------------------------------------------*/
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.math.*;
import com.jla.desc2.util.*;
import com.jla.desc2.parser.*;

public class Spinner extends Panel
implements Runnable, MouseListener, MouseMotionListener, KeyListener, FocusListener {


	public Label lb;
	public TextField tf;
	public spinnerControl sp;

	private Parser parser;
	private Node dvN;
	private Node pvN;
	private Vector listeners=new Vector();
	private Thread thr;
	private BigDecimal V,iV,sV,dV0;
	private int p0;
	private Node var,notexp;
	private volatile boolean stopRequested;

	public Spinner(Parser parser,String ne,String nv,String s_p,String s_v,String s_vincr,String s_vmin,String s_vmax,boolean visible,String s_notexp) {
		this.parser=parser;
if (nv.equals("m")) {
	System.out.println("s_p="+s_p);
}
		pvN=parser.Analyse(s_p,2);
		notexp=parser.Analyse(s_notexp,0);
		if (pvN.isANumber()) {
			p0=Math.max(0,(int) Math.round(pvN.Evaluate(2)));
			pvN=null;
		}
		if (mjaStr.hasContent(s_vmin)) {
			iV=new BigDecimal(parser.Analyse(s_vmin).Evaluate(-(Double.MAX_VALUE-1)));
		} else {
			iV=new BigDecimal(-(Double.MAX_VALUE-1));
		}
		if (mjaStr.hasContent(s_vmax)) {
			sV=new BigDecimal(parser.Analyse(s_vmax).Evaluate(Double.MAX_VALUE));
		} else {
			sV=new BigDecimal(Double.MAX_VALUE);
		}
		setName(nv);
		if (!mjaStr.osIsWindows()) {
			setFont(mjaGui.Arial_10);
		} else {
			setFont(mjaGui.Courier);
		}
		lb=new Label(ne);
		lb.setBackground(Color.lightGray);
		lb.setForeground(Color.black);
		lb.setAlignment(Label.CENTER);
		tf=new TextField();
		tf.addKeyListener(this);
		tf.addFocusListener(this);
		tf.setBackground(Color.white);
		tf.setForeground(Color.black);
		if (!visible) {
			tf.setVisible(false);
		}
		sp=new spinnerControl();
		GridBagLayout gbl=new GridBagLayout();
		setLayout(gbl);
		GridBagConstraints c=new GridBagConstraints();
		c.insets=new Insets(0,0,0,0);
		c.fill=GridBagConstraints.BOTH;
		c.anchor=GridBagConstraints.NORTHWEST;
		c.gridwidth=1; c.gridheight=1;
		c.gridy=0; c.weighty=1.0;
		c.gridx=0; c.weightx=1.0; gbl.setConstraints(lb,c); add(lb);
		c.gridx=1; c.weightx=0.0; gbl.setConstraints(sp,c); add(sp);
		c.gridx=2; c.weightx=1.0; gbl.setConstraints(tf,c); add(tf);

		sp.addMouseListener(this);
		sp.addMouseMotionListener(this);
		dvN=parser.Analyse(s_vincr,0.1);
		if (dvN.isANumber()) {
			if (pvN!=null && pvN.isANumber()) { // condicin agregada el 29 de octubre de 2009 para ver. 4.06
				dV0=new BigDecimal(Math.max(mjaStr.minIncr(p0),dvN.Evaluate(0.01)));
				dvN=null;
			}
		} else {
		}
		var=parser.Var(getName());
		V=new BigDecimal(0);
		defineValue(s_v);
	}

	public String getLabel() {
		return lb.getText();
	}

	public void setLabel(String ne) {
		lb.setText(ne);
	}

	public void updateFromNode() {
		BigDecimal bd=new BigDecimal(var.r);
		if (V.compareTo(bd)!=0) {
			setValue(bd);
		}
	}

	public void addActionListener(ActionListener al) {
		listeners.addElement(al);
	}

	public void removeActionListener(ActionListener al) {
		listeners.removeElement(al);
	}

/* ------------- private operation ------------- */

	private void defineValue(String s) {
		BigDecimal bd=V;
		try {
			bd=new BigDecimal(s);
		} catch (Exception exc) {
			if (mjaStr.hasContent(s)) {
				try {
					Node N=parser.Analyse(s,false);
					bd=new BigDecimal(N.Evaluate());
				} catch (Exception ex2) {
				}
			}
		}
		setValue(bd);
	}

	private void setValue(BigDecimal bd) {
		if (V.compareTo(bd)!=0) {
			V=bd;
			if (V.compareTo(iV)<0) {
				V=iV;
			}
			if (V.compareTo(sV)>0) {
				V=sV;
			}
			var.r=V.doubleValue();
		}
		updateTextField();
	}

	private void updateTextField() {
		int p=p0;
		if (pvN!=null) {
			p=Math.max(0,(int) Math.round(pvN.Evaluate(2)));
		}
		if (notexp.Evaluate(0)>0) {
			tf.setText(mjaStr.notExp(var.r,p));
		} else {
			tf.setText(mjaStr.DoubleToString(var.r,p,true));
//			tf.setText(mjaStr.BigDecimalToString(V,p));
		}
		tf.paint(tf.getGraphics());
	}

	public void updateFromTextField() {
		defineValue(tf.getText());
	}

	private void spinStep() {
		updateFromTextField();
		BigDecimal dV=dV0;
		if (dvN!=null) {
			int p=p0;
			if (pvN!=null) {
				p=Math.max(0,(int) Math.round(pvN.Evaluate(2)));
			}
			dV=new BigDecimal(Math.max(mjaStr.minIncr(p),dvN.Evaluate(0.01)));
		}
		String arg="";
		BigDecimal auxV=V;
		if (sp.up) {
			arg="up";
			auxV=auxV.add(dV);
			if (auxV.compareTo(sV)>0) {
				Toolkit.getDefaultToolkit().beep();
				auxV=sV;
			}
		} else if (sp.down) {
			arg="down";
			auxV=auxV.subtract(dV);
			if (auxV.compareTo(iV)<0) {
				Toolkit.getDefaultToolkit().beep();
				auxV=iV;
			}
		}
		setValue(auxV);
		callListeners(arg);
	}

	/* -------------- Listeners ------------- */

	private void callListeners(String arg) {
		for (int i=0;i<listeners.size();i++) {
			ActionListener al=(ActionListener)listeners.elementAt(i);
			al.actionPerformed(new ActionEvent(this,ActionEvent.ACTION_PERFORMED,arg));
		}
	}

	/* -------------- Runnable ------------- */
	private boolean stop() {
		return stopRequested;
	}
	public void run() {
		long T=System.currentTimeMillis()+500;
		spinStep();
		while (!stop()) {
			while  (!stop() && System.currentTimeMillis()<T) {
				try {
					thr.sleep(20);
				} catch (InterruptedException ie) {
					stopRequested=true;
				}
			}
			if (stop()) {
				break;
			}
			spinStep();
			try {
				thr.sleep(20);
			} catch (InterruptedException ie) {
				stopRequested=true;
			}
			T=System.currentTimeMillis()+40;
		}
		sp.up=false;
		sp.down=false;
		sp.repaint();
		thr=null;
	}
	
	/* ------------- Key Listener ------------ */

	public void keyTyped(KeyEvent e) {}
	public void keyPressed(KeyEvent e) {
		if (e.getKeyCode()==e.VK_ENTER||e.getKeyCode()==e.VK_TAB) {
			defineValue(tf.getText());
			callListeners("updateSpinnersFromTextFields");
		}
	}
	public void keyReleased(KeyEvent e) {}

	/* -------------- Mouse Listener ------------- */
	public void mousePressed(MouseEvent e) {
		if (thr!=null) {
			thr.stop();
			thr=null;
		}
		if (e.getSource()==sp) {
			if (e.getY()<sp.getSize().height/2) {
				sp.up=true;
			} else {
				sp.down=true;
			}
			sp.repaint();
			stopRequested=false;
			thr=new Thread(this);
			thr.start();
		}
	}
	public void mouseReleased(MouseEvent e) {
		stopRequested=true;
	}
	public void mouseEntered (MouseEvent e) {
		stopRequested=true;
	}
	public void mouseExited (MouseEvent e) {
		stopRequested=true;
	}
	public void mouseClicked (MouseEvent e) {}

	/* -------------- MouseMotion Listener ------------- */
	public void mouseDragged(MouseEvent e) {}
	public void mouseMoved(MouseEvent e) {
		stopRequested=true;
	}

	/* -------------- Focus Listener ------------- */
	public void focusLost(FocusEvent fe) {
//		defineValue(tf.getText());
//		updateTextField();
//		callListeners("");
	}
	public void focusGained(FocusEvent fe) {}

}