/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package com.jla.desc2.parser;
/*------------------------------------------------------------*/
/*                        parser.Node                         */
/*                         (c)  MJA                           */
/*                       Mayo de 2001                         */
/*------------------------------------------------------------*/
import java.util.*;
import com.jla.desc2.util.*;

public class Node {

	static final double epsilon=0.000001;
	static final String derivation_symbol="";

	public double r;
	boolean evOnce;
	double[] array;
	Node[] Narray;
	Node[] var;
	Node left,range,func,size,arr;
	Node[] right;
	int type;
	int symb;

	public Node getLeft() {
		return left;
	}

	public Node[] getRight() {
		return right;
	}

	private static final double[] zero={0.0};
	private static final int noerror=0,undefined=1,divzero=2,outofbounds=3;
	private static String[][] errMsg= {
		{"","Function not defined; ","Division by zero; ","Index out of bounds" },
		{"","Funcin no definida; ","Divisin por cero; ","ndice invlido" }
	};
	Algorithm alg;
	private boolean error;
	private int errType;
	String symbstr;

	static final int // TYPE
		No=0,Sep=1,Lp=2,Coma=3,Rp=4,
		As=5,If=6,ThenElse=7,
		B0=8,B1=9,B2=10,B3=11,In=12,
		Us=13,Ua=14,Za=15,Vi=16,Str=17;

	static final int // SYMBOL
		END    = 0,SEP   = 1,LP    = 2,COMA   =3,RP    = 4,
		ASSIGN = 5, IF =6, THENELSE= 7,
		RND   = 8,IGUAL = 9,NOI    =10,
		OR     =11,AND   =12,MEN   =13,MAY   =14,
		MEOI   =15,MAOI  =16,MAS   =17,MENOS =18,
		POR    =19,DIV   =20,EXPON =21,MOD   =22,
		NO     =23,SQR   =24,SQRT  =25,EXP   =26,
		LOG    =27,LOG10 =28,ABS   =29,ENT   =30,
		SGN    =31,IND   =32,SIN   =33,COS   =34,
		TAN    =35,COT   =36,SEC   =37,CSC   =38,
		SINH   =39,COSH  =40,TANH  =41,SECH  =42,
		CSCH   =43,COTH  =44,ASIN  =45,ACOS  =46,
		ATAN   =47,
		/* ACOT  =48,ASEC  =49,ACSC  =50,
		ASINH  =51,ACOSH =52,ATANH =53,ACOTH =54,
		ASECH  =55,ACSCH =56,FAC   =57,SUMA  =58,
		SUMN   =59,*/
		MIN    =60, MAX  =61,
		FUNC   =70, DER  =71,
		ARRAY  =72,
		VAR    =73,
		NOOP   =74;

/* ---------------------- Token Analysis ---------------------- */

	static String[] getTokens(String expr) {
		String expstr=expr.trim();
		int pos=0;
		Vector v=new Vector();
		String auxs="";
		do {
			StringBuffer sb=new StringBuffer();
			char ch='\0';
			if (pos>=expstr.length()) {
				sb.append(ch);
			} else {
				while (isSpace(expstr.charAt(pos))) { pos++; }
				sb.append(ch=expstr.charAt(pos++));
				if (pos<expstr.length()) {
					if (isOperator(ch)) {
						while (isOperator(ch=expstr.charAt(pos))) {
							if (knownOperator(new String(sb)+String.valueOf(ch))) {
								sb.append(ch); pos++;
							} else {
								break;
							}
							if (pos>=expstr.length()) { break; }
						}
					} else if (isDecimal(ch)) {   // new 2006-Oct-31
						while(true) {
							ch=expstr.charAt(pos);
							boolean minusExp= (ch=='-' && sb.length()>1
											&& sb.charAt(sb.length()-1)=='E'
											&& isDigit(sb.charAt(0)));
							if (isDecimal(ch)||minusExp||ch=='E') {
								sb.append(ch); pos++;
								if (pos>=expstr.length()) { break; }
							} else if (isAlphaNum(ch)) {
								auxs=new String(sb);
								v.addElement(auxs);
								sb=new StringBuffer();
								sb.append("*");
								break;
							} else {
								break;
							}
						}
					} else if (isAlphaNum(ch)) {
						while (isAlphaNum(ch=expstr.charAt(pos))) {
							sb.append(ch); pos++;
							if (pos>=expstr.length()) { break; }
						}
					}
				}
			}
			auxs=new String(sb);
			v.addElement(auxs);
		} while (!isSeparator(auxs));
		String[] sa=new String[v.size()];
		for (int i=0;i<sa.length;i++) {
			sa[i]=(String) v.elementAt(i);
		}
		return sa;
	}

	private static boolean isSpace(char ch) {
		return ch==' ' || ch=='\10' || ch=='\13';
	}

	private static boolean isAlphaNum(char ch) {
		return isDecimal(ch)||isAlpha(ch);
	}
	private static boolean isDecimal(char ch)  {
		return isDigit(ch)||ch=='.';
	}
	private static boolean isDigit(char ch) {
		return ch>='0'&&ch<='9';
	}
	private static boolean isAlpha(char ch) {
	    switch (ch) {
		case '' : case '' : case '' : case '' :
		case '' : case '' : case '' : case '' :
		case '' : case '' : case '' : case '' :
		case '' : case '' : case '' : case '' :
		case '' : case '' : case '' : case '' :
		case '' : case '' : case '' : case '' :
		case '' : case '' : case '' : case '' :
		case '' : case '' : case '' : case '' :
		case '' : case '' : case '' : case '' :
		case '' : case '' : case '' : case '' :
		case '_' : case '\'' : case '' :
				 return true;
		default  : return (ch>='A' && ch<='Z')||(ch>='a' && ch<='z');
		}
	}

	private static boolean isOperator(char ch) {
		switch (ch) {
		case '|' : case '&' :
		case '=' : case '#' : case '<' : case '>' :
		case '+' : case '-' : case '*' : case '/' :
		case '%' : case '^' : case ':' : case '~' :
		case '!' : case '?' :
				   return true;
		default  :  return false;
		}
	}

	private static boolean isSeparator(String s)   {
		return s.equals(";") || s.equals("\0") || s.equals("\26");
	}

/* ----------------------------------------------------------------- */

	public boolean isANumber() {
		try {
			double x=Double.valueOf(symbstr).doubleValue();
			return true;
		} catch(NumberFormatException nfe) {
			return false;
		}
	}
	public String getSymbol() {
		return symbstr;
	}

	public void equalToMinus() { // This is to convert "f(x,y)=g(x,y)" into "f(x,y)-g(x,y)=0" .
		symb=MENOS;
		type=B1;
	}

	Node (String name,double d) {
		symbstr=name; r=d; type=Vi;
	}
	Node (String name,String s) {
		symbstr=name;
		type=Str;
	}
	Node (String name,Node szN,Node[] Narr0,boolean evOnce) {
		symbstr=name;
		symb=ARRAY;
		type=nodeType(symb);
		Narray=Narr0;
		array=new double[Narray.length];
		this.evOnce=evOnce;
		size=szN;
	}
	Node (String name,Node arrN) {
		symbstr=name;
		symb=ARRAY;
		type=nodeType(symb);
		this.arr=arrN;
	}
	Node (String name,boolean der_exp,Node func,Algorithm a) {
		symbstr=name;
		this.func=func;
		if (der_exp) {
			symb=DER;
		} else {
			symb=FUNC;
		}
		this.alg=a;
		type=nodeType(symb);
	}
	Node (String s,boolean isvar) {
		symbstr=s;
		if (isvar) {
			symb=VAR;
			type=Vi;
		} else {
			symb=symbol(s);
			type=nodeType(symb);
		}
	}

	private int nodeType(int symbol) {
		switch (symbol) {
		case END    : return Sep;
		case SEP    : return Sep;
		case LP     : return Lp;
		case COMA   : return Coma;
		case RP     : return Rp;
		case ASSIGN : return As;
		case IF     : return If;
		case THENELSE: return ThenElse;
		case RND    : return Za;
		case MAS    :
		case MENOS  : return In;
		case IGUAL  :
		case NOI    :
		case MEN    :
		case MAY    :
		case MEOI   :
		case MAOI   : return B0;
		case OR     : return B1;
		case AND    :
		case POR    :
		case DIV    : return B2;
		case EXPON  :
		case MOD    : return B3;
		case NO     : return Us;
		case SQR    :
		case SQRT   :
		case EXP    :
		case LOG    :
		case LOG10  :
		case ABS    :
		case ENT    :
		case SGN    :
		case IND    :
		case SIN    :
		case COS    :
		case TAN    :
		case COT    :
		case SEC    :
		case CSC    :
		case SINH   :
		case COSH   :
		case TANH   :
		case SECH   :
		case CSCH   :
		case COTH   :
		case ASIN   :
		case ACOS   :
		case ATAN   :
		/*    case ACOT   :
		case ASEC   :
		case ACSC   :
		case ASINH  :
		case ACOSH  :
		case ATANH  :
		case ACOTH  :
		case ASECH  :
		case ACSCH  :
		case FAC    :
		case SUMA   :
		case SUMN   :  */
		case MIN    :
		case MAX    :
		case FUNC   :
		case DER    :
		case ARRAY  : return Ua;
		case VAR    : return Vi;
		case NOOP   :
		default     : return No;
		}
	}

	public static boolean knownOperator(String s) {
		int sy=symbol(s);
		return (ASSIGN<=sy && sy<=NO);
	}

	private static int symbol(String s) {
		if (s.equals("\0"))      { return END; }
		if (s.equals(";"))       { return SEP; }
		if (s.equals("("))       { return LP; }
		if (s.equals(")"))       { return RP; }
		if (s.equals("["))       { return LP; }
		if (s.equals("]"))       { return RP; }
		if (s.equals(","))       { return COMA; }
		if (s.equals(":="))      { return ASSIGN; }
		if (s.equals("?"))       { return IF; }
		if (s.equals(":"))       { return THENELSE; }
		if (s.equals("="))       { return IGUAL; }
		if (s.equals("#"))       { return NOI; }
		if (s.equals("!="))      { return NOI; }
		if (s.equals("|"))       { return OR; }
		if (s.equals("||"))      { return OR; }
		if (s.equals("&"))       { return AND; }
		if (s.equals("&&"))      { return AND; }
		if (s.equals("<"))       { return MEN; }
		if (s.equals(">"))       { return MAY; }
		if (s.equals("<="))      { return MEOI; }
		if (s.equals(">="))      { return MAOI; }
		if (s.equals("+"))       { return MAS; }
		if (s.equals("-"))       { return MENOS; }
		if (s.equals("*"))       { return POR; }
		if (s.equals("/"))       { return DIV; }
		if (s.equals("^"))       { return EXPON; }
		if (s.equals("%"))       { return MOD; }
		if (s.equals("~"))       { return NO; }
		if (s.equals("!"))       { return NO; }
		if (s.equals("sqr"))     { return SQR; }
		if (s.equals("sqrt"))    { return SQRT; }
		if (s.equals("raz"))    { return SQRT; }
		if (s.equals("exp"))     { return EXP ; }
		if (s.equals("log"))     { return LOG ; }
		if (s.equals("log10"))   { return LOG10;}
		if (s.equals("abs"))     { return ABS ; }
		if (s.equals("ent"))     { return ENT ; }
		if (s.equals("sgn"))     { return SGN ; }
		if (s.equals("ind"))     { return IND ; }
		if (s.equals("sin"))     { return SIN ; }
		if (s.equals("sen"))     { return SIN ; }
		if (s.equals("cos"))     { return COS ; }
		if (s.equals("tan"))     { return TAN ; }
		if (s.equals("cot"))     { return COT ; }
		if (s.equals("sec"))     { return SEC ; }
		if (s.equals("csc"))     { return CSC ; }
		if (s.equals("sinh"))    { return SINH; }
		if (s.equals("senh"))    { return SINH; }
		if (s.equals("cosh"))    { return COSH; }
		if (s.equals("tanh"))    { return TANH; }
		if (s.equals("coth"))    { return SECH; }
		if (s.equals("sech"))    { return CSCH; }
		if (s.equals("csch"))    { return COTH; }
		if (s.equals("asin"))    { return ASIN; }
		if (s.equals("asen"))    { return ASIN; }
		if (s.equals("acos"))    { return ACOS; }
		if (s.equals("atan"))    { return ATAN; }
		/*
		if (s.equals("acot"))    { return ACOT; }
		if (s.equals("asec"))    { return ASEC; }
		if (s.equals("acsc"))    { return ACSC; }
		if (s.equals("asinh"))   { return ASINH;}
		if (s.equals("asenh"))   { return ASINH;}
		if (s.equals("acosh"))   { return ACOSH;}
		if (s.equals("atanh"))   { return ATANH;}
		if (s.equals("acoth"))   { return ACOTH;}
		if (s.equals("asech"))   { return ASECH;}
		if (s.equals("acsch"))   { return ACSCH;}
		if (s.equals("fac"))     { return FAC;  }
		if (s.equals("sum"))     { return SUMA; }
		if (s.equals("suma"))    { return SUMN; }
		*/
		if (s.equals("min"))     { return MIN;  }
		if (s.equals("max"))     { return MAX;  }
		if (s.equals("rnd"))     { return RND;  }
		if (s.equals(""))        { return NOOP; }
		return VAR;
	}

	public int NumVars() {
		int nvars=0;
		if (type==Ua) {
			if (func!=null) {
				nvars=func.var.length;
			} else if (symb==ARRAY) {
				nvars=1;
			} else {
				switch (symb) {
				case SQR    :
				case SQRT   :
				case EXP    :
				case LOG    :
				case LOG10  :
				case ABS    :
				case ENT    :
				case SGN    :
				case IND    :
				case SIN    :
				case COS    :
				case TAN    :
				case COT    :
				case SEC    :
				case CSC    :
				case SINH   :
				case COSH   :
				case TANH   :
				case SECH   :
				case CSCH   :
				case COTH   :
				case ASIN   :
				case ACOS   :
				case ATAN   :
				/*    case ACOT   :
				case ASEC   :
				case ACSC   :
				case ASINH  :
				case ACOSH  :
				case ATANH  :
				case ACOTH  :
				case ASECH  :
				case ACSCH  :
				case FAC    :
				case SUMA   :
				case SUMN   :  */
					nvars=1;
					break;
				case MIN    :
				case MAX    :
					nvars=2;
					break;
				default     :
					nvars=0;
					break;
				}
			}
		}
		return nvars;
	}

/* ------------------------------------------------------------------ */

	private double divide(double num,double den) throws Exception {
		boolean denIsZero=Math.abs(den)<epsilon;
		if (num==0.0&&!denIsZero) {
			return 0.0;
		}
		if (denIsZero) {
			error=true;
			errType=divzero;
			throw new Exception(errMsg[1][errType]);
		} else {
			return num/den;
		}
	}

	private double log(double x) throws Exception {
		if (x<epsilon) {
			error=true;
			errType=undefined;
			throw new Exception(errMsg[1][errType]);
		} else {
			return Math.log(x);
		}
	}

	private double posexpon(double x, double y) throws Exception {
		if (y==0) {
			return 1;
		}
		if (y<0) {
			error=true;
			errType=undefined;
			throw new Exception(errMsg[1][errType]);
		}
		if ((x>=0)||((int) Math.floor(y)==y)) {
			return Math.pow(x,y);
		}
		if (x<0) {
			double yinv=1/y;
			int q=(int) Math.floor(yinv);
			if (q==yinv) {
				if (q%2==1) {
					return -Math.pow(-x,y);
				}
			}
		}
		error=true;
		errType=undefined;
		throw new Exception(errMsg[1][errType]);
	}

	private double expon(double x, double y) throws Exception {
		if (y>=0) {
			return posexpon(x,y);
		} else {
			return 1/posexpon(x,-y);
		}
	}

	private double sqrt(double x) throws Exception {
		if (x<0) {
			error=true;
			errType=undefined;
			throw new Exception(errMsg[1][errType]);
		} else {
			return Math.sqrt(x);
		}
	}

	public boolean rangeOK() {
		try {
			Evaluate(false);
			return true;
		} catch (Exception e) {
			return false;
		}
	}

	private double UnOp(boolean ignoreRange, double[] v) throws Exception {
		double x=0;
		if (v!=null&&v.length>0) {
			x=v[0];
		}
		switch (symb) {
		case NO       : if (x==1.0) return 0.0; else return 1.0;
		case MAS      : return x;
		case MENOS    : return -x;
		case ABS      : if (x<0) return -x; else return x;
		case SQR      : return x*x;
		case SQRT     : return sqrt(x);
		case EXP      : return Math.exp(x);
		case LOG      : return log(x);
		case LOG10    : return log(x)/Math.log(10.0);
		case COS      : return Math.cos(x);
		case SIN      : return Math.sin(x);
		case TAN      : return divide(Math.sin(x),Math.cos(x));
		case SEC      : return divide(1.0,Math.cos(x));
		case CSC      : return divide(1.0,Math.sin(x));
		case COT      : return divide(1.0,Math.tan(x));
		case SINH     : return (Math.exp(x)-Math.exp(-x))/2.0;
		case COSH     : return (Math.exp(x)+Math.exp(-x))/2.0;
		case TANH     : return divide(Math.exp(x)-Math.exp(-x),Math.exp(x)+Math.exp(-x));
		case COTH     : return divide(Math.exp(x)+Math.exp(-x),Math.exp(x)-Math.exp(-x));
		case SECH     : return 2.0/(Math.exp(x)+Math.exp(-x));
		case CSCH     : return divide(2.0,Math.exp(x)-Math.exp(-x));
		case ENT      : return Math.floor(x);
		case SGN      : if (x>0.0) return 1.0; else if (x<0) return -1.0;
		case IND      : if (x!=0.0) return 1.0; else return 0.0;
		case ASIN     : return Math.asin(x);
		case ACOS     : return Math.acos(x);
		case ATAN     : return Math.atan(x);
		case RND      : return Math.random();
		case MIN      : return Math.min(v[0],v[1]);
		case MAX      : return Math.max(v[0],v[1]);
		case FUNC     :
			double result=0.0;
			if (func.var!=null && func.var.length==v.length) {
				double[] va=new double[v.length];
				for (int i=0;i<v.length;i++) {
					va[i]=func.var[i].r;
					func.var[i].r=v[i];
				}
				boolean ok=true;
				if (ignoreRange || func.range==null || func.range.Evaluate(1.0)>0) {
					if (alg!=null) {
						alg.perform();
					}
					try {
						result=func.Evaluate(ignoreRange);
					} catch (Exception e) {
						ok=false;
					}
				} else {
					ok=false;
				}
				for (int i=0;i<v.length;i++) {
					func.var[i].r=va[i];
				}
				if (!ok) {
					error=true;
					errType=undefined;
					throw new Exception(errMsg[1][undefined]);
				}
			}
			return result;
		case DER      :
			double y0=0,y1=0;
			if (func.var!=null && func.var.length==v.length) {
				double[] va=new double[v.length];
				for (int i=0;i<v.length;i++) {
					va[i]=func.var[i].r;
					func.var[i].r=v[i];
				}
				boolean ok=true;
				try {
					if (alg!=null) {
						alg.perform();
					}
					y0=func.Evaluate(ignoreRange);
					func.var[0].r=x+epsilon;
					if (alg!=null) {
						alg.perform();
					}
					y1=func.Evaluate(ignoreRange);
				}  catch (Exception e) {
					ok=false;
				}
				for (int i=0;i<v.length;i++) {
					func.var[i].r=va[i];
				}
				if (!ok) {
					error=true;
					errType=undefined;
					throw new Exception(errMsg[1][undefined]);
				}
			}
			return (y1-y0)/epsilon;
		case ARRAY:
			double[] a=arr.array;
			int n=(int) Math.round(x);
			if (a!=null && n>=0 && n<a.length) {
				return a[n];
			} else {
				error=true;
				errType=outofbounds;
				throw new Exception(errMsg[1][outofbounds]);
			}
		default       : return 0.0;
		}
	}

	private double BnOp(double x, double y) throws Exception {
		switch (symb) {
		case IGUAL : if (x==y) return 1.0; else return 0.0;
		case MEN   : if (x<y)  return 1.0; else return 0.0;
		case MEOI  : if (x<=y) return 1.0; else return 0.0;
		case MAY   : if (x>y)  return 1.0; else return 0.0;
		case MAOI  : if (x>=y) return 1.0; else return 0.0;
		case NOI   : if (x!=y) return 1.0; else return 0.0;
		case AND   : if ((x==1.0)&&(y==1.0)) { return 1.0; } else { return 0.0; }
		case OR    : if ((x==1.0)||(y==1.0)) { return 1.0; } else { return 0.0; }
		case MAS   : return (x+y);
		case MENOS : return (x-y);
		case POR   : if ((x==0.0)||(y==0.0)) { return 0.0; } else { return (x*y); }
		case DIV   : return divide(x,y);
		case EXPON : return expon(x,y);
		case MOD   : return x - y*Math.floor(x/y); // no va --> (Math.IEEEremainder(x,y));
		default    : return 0.0;
		}
	}

	public double Evaluate() throws Exception {
		return Evaluate(false);
	}

	public double Evaluate(boolean ignoreRange) throws Exception {
		errType=0;
		error=false;
		switch(type) {
		case Vi: return r;
		case Za: return UnOp(ignoreRange, zero);
		case Us:
		case Ua:
			double[] v=new double[right.length];
			for (int i=0;i<v.length;i++) {
				v[i]=right[i].Evaluate(ignoreRange);
			}
			return UnOp(ignoreRange,v);
		case B0:
		case B1:
		case B2:
		case B3: return BnOp(left.Evaluate(ignoreRange),right[0].Evaluate(ignoreRange));
		case As: return left.r=right[0].Evaluate(ignoreRange);
		case If :
			if (left.Evaluate(ignoreRange)>0) {
				return right[0].left.Evaluate(ignoreRange);
			} else {
				return right[0].right[0].Evaluate(ignoreRange);
			}
		default: return 0;
		}
	}

	public double Evaluate(double defaultValue) {
		try {
			return Evaluate(false);
		} catch (Exception e) {
			return defaultValue;
		}
	}

	public boolean error() {
		if (error) {
			return true;
		}
		switch(type) {
		case Us:
		case Ua:
			for (int i=0;i<right.length;i++) {
				if (right[i].error()) {
					return true;
				}
			}
			return false;
		case B0:
		case B1:
		case B2:
		case B3:
		case If:
		case ThenElse:
		case As: return left.error()||right[0].error();
		default: return false;
		}
	}


	private String translate(int Lang,int em) {
		return errMsg[Lang][em];
	}

	public String errorMsg(int Lang) {
		String msg=translate(Lang,errType);
		switch(type) {
		case Us:
		case Ua:
			for (int i=0;i<right.length;i++) {
				msg+=right[i].errorMsg(Lang);
			}
			break;
		case B0:
		case B1:
		case B2:
		case B3:
		case If:
		case ThenElse:
		case As: msg+=left.errorMsg(Lang)+" "+right[0].errorMsg(Lang); break;
		}
		return msg;
	}

	public boolean contains(String s) {
		if (symbstr.equals(s)) { return true; }
		switch(type) {
		case Us:
		case Ua:
			for (int i=0;i<right.length;i++) {
				if (right[i].contains(s)) {
					return true;
				}
			}
			return false;
		case B0:
		case B1:
		case B2:
		case B3:
		case If:
		case ThenElse:
		case As: return left.contains(s)||right[0].contains(s);
		default: return false;
		}
	}

		/* ------------------------------------------------------ */

	public String toString(int precision,int Lang) {
		StringBuffer textb=new StringBuffer("");
		if (type==Node.Str) {
			textb.append(symbstr);
		} else {
			try {
				double x=Evaluate();
				textb.append(mjaStr.DoubleToString(x,precision));
			} catch (Exception e) {
				textb.append(errorMsg(Lang));
			}
		}
		return new String(textb);
	}

/*
	private static String symbolStr(int symbol) {
		switch (symbol) {
		case END      : return "\0";
		case SEP      : return ";";
		case LP       : return "(";
		case RP       : return ")";
		case ASSIGN: return ":=";
		case IF : return "?";
		case THENELSE : return ":";
		case IGUAL: return "=";
		case NOI: return "!=";
		case OR: return "|";
		case AND: return "&";
		case MEN: return "<";
		case MAY: return ">";
		case MEOI: return "<=";
		case MAOI: return ">=";
		case MAS: return "+";
		case MENOS: return "-";
		case POR: return "*";
		case DIV: return "/";
		case EXPON: return "^";
		case MOD: return "%";
		case NO: return "!";
		case SQR: return "sqr";
		case SQRT: return "sqrt";
		case EXP: return "exp";
		case LOG: return "log";
		case LOG10: return "log10";
		case ABS: return "abs";
		case ENT: return "ent";
		case SGN: return "sgn";
		case IND: return "ind";
		case SIN: return "sin";
		case COS: return "cos";
		case TAN: return "tan";
		case COT: return "cot";
		case SEC: return "sec";
		case CSC: return "csc";
		case SINH: return "sinh";
		case COSH: return "cosh";
		case TANH: return "tanh";
		case SECH: return "coth";
		case CSCH: return "sech";
		case COTH: return "csch";
		case ASIN: return "asin";
		case ACOS: return "acos";
		case ATAN: return "atan";
//		case ACOT: return "acot";
//		case ASEC: return "asec";
//		case ACSC: return "acsc";
//		case ASINH: return "asinh";
//		case ACOSH: return "acosh";
//		case ATANH: return "atanh";
//		case ACOTH: return "acoth";
//		case ASECH: return "asech";
//		case ACSCH: return "acsch";
//		case FAC: return "fac";
//		case SUMA: return "sum";
//		case SUMN: return "suma";
		case MIN: return "min";
		case MAX: return "max";
		case RND: return "rnd";
		case NOOP: return "";
		default :return "";
		}
	}

	private String parenthesis(Node[] N) {
		String expr="(";
		if (N!=null) {
			for (int i=0;i<N.length;i++) {
				expr+=N[i].toExpression();
				if (i+1<N.length) {
					expr+=",";
				}
			}
		}
		return expr+")";
	}

	private String parenthesis(Node N) {
		Node[] na=new Node[1];
		na[0]=N;
		return parenthesis(na);
	}

	public String toExpression() {
		String expr="";
		switch(type) {
		case Vi:
			expr+=getSymbol();
			break;
		case Za:
			expr+=symbstr;
			break;
		case Us:
		case Ua:
			expr+=symbstr+parenthesis(right);
			break;
		case B0:
		case B1:
		case B2:
		case B3:
		case As:
		case If:
			expr+=parenthesis(left)+symbstr+parenthesis(right);
			break;
		case ThenElse :
			expr+="("+parenthesis(left)+symbstr+parenthesis(right)+")";
			break;
		default:
			expr+=getSymbol();
			break;
		}
		return expr;
	}
*/

}