/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package com.jla.desc2.parser;
/*------------------------------------------------------------*/
/*                       parser.Parser                        */
/*                         (c)  MJA                           */
/*                       Mayo de 2001                         */
/*------------------------------------------------------------*/
import java.util.*;
import com.jla.desc2.util.*;

public class Parser {

	private Hashtable FS=new Hashtable(); // FUNCTIONS
	private Hashtable VS=new Hashtable(); // VARIABLES
	private Hashtable XS=new Hashtable(); // AUXILIARIES
	private Hashtable AS=new Hashtable(); // ARRAYS
	private Hashtable FAS=new Hashtable();// FUNCTION_ALGORITHMS
	private Vector ALS=new Vector();      // ALGORITHMS

	private void defineFunc(String name,String[] varname,String expr) {
		FS.put(name,Analyse(expr,0));
		Func(name).var=new Node[varname.length];
		for (int i=0;i<varname.length;i++) {
			Func(name).var[i]=Var(varname[i]);
		}
	}
	public void defineFunc(String name,String[] varname,String expr,String s_range,
		String s_do,String s_ini,String s_while) {
		defineFunc(name,varname,expr,s_range);
		FAS.put(name,new Algorithm(this,false,s_do,s_ini,s_while));
	}
	public void defineFunc(String name,String[] varname,String expr,String s_range) {
		defineFunc(name,varname,expr);
		if (mjaStr.hasContent(s_range)) {
			Func(name).range=Analyse(s_range);
		}
	}
	public void defineExpr(String name,String expr) {
		XS.put(name,Analyse(expr,0));
	}

	public Node Var(String name) {
		if (!isVar(name)) {
			VS.put(name,new Node(name,0));
		}
		return (Node)VS.get(name);
	}
	public double value(String name) {
		return value(name,0);
	}
	public double value(String name,double dflt) {
		if (isAux(name)) {
			return ((Node)XS.get(name)).Evaluate(dflt);
		} else if (isVar(name)) {
			return Var(name).r;
		} else {
			return dflt;
		}
	}

/* --------- some private auxiliary methods --------- */

	private Algorithm funcAlg(String name) {
		if (FAS.containsKey(name)) {
			return (Algorithm) FAS.get(name);
		} else {
			return null;
		}

	}
	private Node Func(String name) {
		if (isFun(name)) {
			return (Node)FS.get(name);
		} else {
			return null;
		}
	}
	private boolean isAux(String name) {
		return XS.containsKey(name);
	}
	public boolean isVar(String name) {
		return VS.containsKey(name);
	}
	private boolean isFun(String name) {
		return FS.containsKey(name);
	}
	private boolean isFunDer(String name) {
		if ( (name.length()>1) && name.endsWith(Node.derivation_symbol)) {
			String s0=name.substring(0,name.length()-1);
			return isFun(s0);
		}
		return false;
	}
	private void getInfo(String expr) {
		String[] sa=mjaStr.getElements(expr);
		for (int i=0;i<sa.length;i++) {
			int ixeq=sa[i].indexOf("=");
			if (ixeq>0) {
				String left=(sa[i].substring(0,ixeq)).trim();
				String right=(sa[i].substring(ixeq+1)).trim();
				int ixlb=left.indexOf("[");
				if (ixlb>0) {
					StringTokenizer st=new StringTokenizer(left,"[]");
					if (st.countTokens()==2) {
						String name=st.nextToken();
						if (isArray(name)) {
							Node N=Array(name);
							String ixs=st.nextToken().trim();
							try {
								int n=(int) Math.round(Analyse(ixs).Evaluate());
								if (N.Narray==null) {
									if (n>=0 && n<N.array.length) {
										N.array[n]=Analyse(right).Evaluate();
									}
								} else {
									if (n>=0 && n<N.Narray.length) {
										N.Narray[n]=Analyse(right);
										N.array[n]=N.Narray[n].Evaluate();
									}
								}
							} catch (Exception e) {
							}
						}
					}
				} else {
					try {
						Var(left).r=Analyse(right).Evaluate();
					} catch(Exception e) {
					}
				}
			}
		}
	}

/*----------------------- Analysis-Tree construction ----------------------*/

	public Node Analyse(String expression,double defaultValue) {
		return Analyse(expression,Double.toString(defaultValue));
	}
	public Node Analyse(String expression,String defaultExpression) {
		Node N=Analyse(expression);
		if (N==null) {
			N=Analyse(defaultExpression);
		}
		return N;
	}
	public Node Analyse(String expression) {
		Node N=null;
		try {
			N=Analyse(expression,true);
		} catch (Exception e) {
		}
		return N;
	}
	public Node Analyse(String EXPR,boolean defineVars) throws Exception {
		if (!mjaStr.hasContent(EXPR)) {
			return null;
		}
		String expression=new String(EXPR);  
		// added in version 2.8910, 2006-12-13
		expression=mjaStr.replace(expression,"\u00b2","^2");
		expression=mjaStr.replace(expression,"\u00b3","^3");
		
		String[] tok=Node.getTokens(expression);
		Stack NS=new Stack();
		Stack TNS=new Stack();
		int synstate=0;
		int countpar=0;
		int opstate=0;
		int ix=0;
		while (ix<tok.length) {
			String s=tok[ix++];
			Node node=null;
			if (!isVar(s) ||
				( ix<tok.length && (tok[ix].startsWith("(")||tok[ix].startsWith("[")) )
				) {
				if (ix<tok.length && tok[ix].startsWith("[")) {
					node=new Node(s,Array(s));
				} else {
					if (isFun(s)) {
						node=new Node(s,false,Func(s),funcAlg(s));
					} else if (isFunDer(s)) {
						String fn=s.substring(0,s.length()-1);
						node=new Node(s,true,Func(fn),funcAlg(fn));
					}
				}
			}
			if (node==null) {
				node=new Node (s,isVar(s));
			}
			switch (node.type) {
			case Node.Rp :
			case Node.Vi :
			case Node.Coma :
			case Node.Za : opstate=1; break;
			case Node.In : 	if (opstate==0) { node.type=Node.Us; }
							else { node.type=Node.B1; }
			case Node.Ua :
			default      : opstate=0; break;
			}
			if (node.type!=Node.Sep) {
				if (node.type==Node.Rp) {
					synstate=1;
					if (countpar>0) {
						countpar--;
						while (true) {
							Node N=(Node)TNS.pop();
							if (N.type==Node.Lp) {
								if (!TNS.empty()) {
									Node tN=(Node)TNS.peek();
									if (tN.type==Node.Ua) {
										tN.right=new Node[tN.NumVars()];
									}
								}
								break;
							}
							addNode(NS,N,defineVars);
						}
					} else {
						Error("too many ) in "+expression);
					}
				} else if (synstate==0) {
					switch (node.type) {
					case Node.Lp : countpar++;
					case Node.Us :
					case Node.Ua : TNS.push(node); break;
					case Node.Za : case Node.Vi : addNode(NS,node,defineVars); synstate=1; break;
					default      :
System.out.println(ix+"!!!! "+node.getSymbol());
						Error("expression expected in "+expression);
					}
				} else {
					switch (node.type) {
					case Node.Coma :
						if (countpar>0) {
							while (true) {
								Node N=(Node)TNS.pop();
								if (N.type==Node.Lp) {
									TNS.push(N);
									break;
								}
								addNode(NS,N,defineVars);
							}
						} else {
							Error(", not expected in "+expression);
						}
						synstate=0;
						opstate=0;
						break;
					case Node.As : case Node.B0: case Node.B1 : case Node.B2:
					case Node.If : case Node.ThenElse:
						while (!TNS.isEmpty()) {
							Node N=(Node)TNS.peek();
							if (N.type<node.type) {
								break;
							}
							addNode(NS,(Node)TNS.pop(),defineVars);
						}
						TNS.push(node); synstate=0;
						break;
					case Node.B3 :
						while (!TNS.isEmpty()) {
							Node N=(Node)TNS.peek();
							if (N.type<=node.type) {
								break;
							}
							if (N.type==Node.Us) {
								break;
							}
							addNode(NS,(Node)TNS.pop(),defineVars);
						}
						TNS.push(node); synstate=0;
						break;
					default      :
System.out.println(ix+"!!!! "+node.getSymbol());
						Error("operator expected in "+expression);
					}
				}
			} else {
				if (synstate==0) {
					Error("incomplete expression "+expression);
				}
				if (countpar>0) {
					Error(") expected in "+expression);
				}
				while (!TNS.isEmpty()) {
					addNode(NS,(Node)TNS.pop(),defineVars);
				}
				Node resultN=(Node)NS.peek();
//System.out.println(resultN.toExpression());
				return resultN;
			}
		}
		Error("Parser internal error.");
		return null;
	}

	private void addNode(Stack NS,Node node,boolean defineVars) throws Exception {
		switch (node.type) {
		case Node.Us : node.right=new Node[1];
		case Node.Ua :
			for (int i=node.right.length-1;i>=0;i--) {
				node.right[i]=(Node)NS.pop();
			}
			break;
		case Node.Za : break;
		case Node.B0 :
		case Node.B1 :
		case Node.B2 :
		case Node.B3 :
		case Node.If:
		case Node.ThenElse:
		case Node.As :
			node.right=new Node[1];
			node.right[0]=(Node)NS.pop();
			node.left=(Node)NS.pop();
			break;
		case Node.Vi :
			if (isAux(node.getSymbol())) {
				node=(Node) XS.get(node.getSymbol());
			} else if (isVar(node.getSymbol())) {
				node=(Node) VS.get(node.getSymbol());
			} else {
				try {
					node.r=Double.valueOf(node.getSymbol()).doubleValue();
				} catch(NumberFormatException nfe) {
					if (defineVars) {
						node=Var(node.getSymbol());
					} else {
						String errs="\""+node.getSymbol()+"\""+" no declarado";
						Error(errs);
						throw new Exception(errs);
					}
				}
			}
			break;
		default      : node.type=Node.In; break;
		}
		NS.push(node);
	}

/* ---------------------- String handling -------------------- */

	public Vector parseString(String str) {
		Vector v=new Vector();
		if (mjaStr.hasContent(str)) {
			v.addElement(new Node("",""));
			StringBuffer sb=new StringBuffer("");
			int state=0,level=0;
			boolean escape=false;
			for (int i=0;i<str.length();i++) {
				char ch=str.charAt(i);
				if (escape) {
					sb.append(ch);
					escape=false;
				} else {
					if (ch=='\\') {
						escape=true;
					} else {
						if (ch=='[') {
							level++;
						} else if (ch==']') {
							level--;
						}
						if ((level==1&&ch=='[')||(level==0&&ch==']')) {
							if (sb.length()>0) {
								String auxs=new String(sb);
								if (state==0) {
									v.addElement(new Node(auxs,""));
								} else {
									v.addElement(Analyse(auxs));
								}
								sb=new StringBuffer("");
							}
							state=1-state;
						} else {
							sb.append(ch);
						}
						if (level<0) {
							level=0;
						}
					}
				}
			}
			String auxs=new String(sb);
			if (state==0) {
				v.addElement(new Node(auxs,""));
			} else {
				v.addElement(Analyse(auxs));
			}
		}
		return v;
	}

/* ------------------ Algorithms ----------------------- */

	public void defineAlgorithm(boolean evOnce,String expr,String s_ini,String s_while) {
		ALS.addElement(new Algorithm(this,evOnce,expr,s_ini,s_while));
	}

	public void doAlgorithms(boolean firstTime) throws Exception {
		Enumeration an=ALS.elements();
		while (an.hasMoreElements()) {
			Algorithm a=(Algorithm) an.nextElement();
			if (firstTime||!a.evOnce) {
				a.perform();
			}
		}
	}

/* ------------------ Arrays ----------------------- */

	boolean isArray(String name) {
		return AS.containsKey(name);
	}
	Node Array(String name) {
		if (!isArray(name)) {
			defineArray("3",name,"",true);
		}
		return (Node) AS.get(name);
	}
	private double[] getArray(String name) {
		if (isArray(name)) {
			return ((Node) AS.get(name)).array;
		}
		return null;
	}
	public void defineArray(String s_size,String name,String expr,boolean evOnce) {
		Node arrSz=Analyse(s_size);
		int sz=(int) Math.round(Analyse(s_size,"3").Evaluate(3));
		AS.put(name,new Node(name,arrSz,new Node[sz],evOnce));
		getInfo(expr);
	}
	public void prepareArrays(boolean firstTime) {
		Enumeration aN=AS.elements();
		while (aN.hasMoreElements()) {
			Node N=(Node) aN.nextElement();
			int sz=(int) Math.round(N.size.Evaluate(3));
			double[] ar=N.array;
			if (ar.length!=sz) {
				double [] newar=new double[sz];
				if (!firstTime) {
					for (int i=0;i<Math.min(ar.length,newar.length);i++) {
						newar[i]=ar[i];
					}
				}
				N.array=newar;
			}
			if (!N.evOnce) {
				for (int i=0;i<N.array.length;i++) {
					if (N.Narray[i]!=null) {
						N.array[i]=N.Narray[i].Evaluate(0);
					}
				}
			}
		}
	}

/*--------------------- Error Handling ------------------------*/

	private void Error(String errmsg) {
		System.out.println(errmsg);
	}

}