/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package com.mja.algebra;

import java.util.*;
import java.io.*;
import java.net.*;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;

import com.mja.util.*;
import com.mja.gui.*;
import com.mja.descgui.spinnerControl;
import com.mja.file.*;
import com.mja.lang.*;

public class DescartesAConfig extends Frame implements WindowListener, MouseListener, ActionListener, ItemListener {

	static final Color scColor=new Color(0x88aaee);
	static final Color scColorLight=new Color(0xaabbff);
	static final Color seColor=new Color(0xaae866);
	static final Color seColorLight=new Color(0xbbff88);
	static final Color apColor=new Color(0xffaa66);
	static final Color apColorLight=new Color(0xffddbb);

	CardLayout cL;
	Panel CP;

	DescartesA M;
	ExerciseEditor EE;
	SeriesConfig[] SCA=new SeriesConfig[0];
	TextArea TA;
	TextField tf_nombre;
	TextField tf_codebase;
	TextField tf_ancho;
	TextField tf_alto;

	Button b_apply;
	Button b_close;
	Button b_accept;
	Button b_appletApply;
	
	Button b_saveAs;
//	Button b_undo,b_redo;

	Label lb_scene,lb_series,lb_applet;
	Panel P_scene,P_series,P_applet;
	java.awt.List LS=new java.awt.List(10);
	java.awt.List LE=new java.awt.List(10);

	private static Font lbFont;
	private static Font pnFont;
	private static Font txFont;
	
	DescartesAConfig (DescartesA M) {
		this.M=M;
		lbFont=mjaFont.makeFont("SansSerif",Font.PLAIN,17);
		pnFont=mjaFont.AWTSansSerif;
		txFont=mjaFont.AWTMono;
		setTitle("Editor de configuraciones del Manipulador de Expresiones");
		setFont(pnFont);
		setLayout(new BorderLayout());
		LS.setSize(120,120);
		LS.addItemListener(this);
		LE.setSize(120,120);
		LE.addItemListener(this);

		Panel NP=new Panel();
		NP.setBackground(Color.gray);
		NP.setFont(lbFont);
		NP.setLayout(new FlowLayout(FlowLayout.LEFT,1,0));
		lb_scene=new Label("Escena");
		lb_scene.setAlignment(Label.CENTER);
		lb_scene.setForeground(Color.black);
		lb_scene.setBackground(scColor);
		lb_scene.addMouseListener(this);
		NP.add(lb_scene);
		lb_series=new Label("Ejercicios");
		lb_series.setAlignment(Label.CENTER);
		lb_series.setForeground(Color.black);
		lb_series.setBackground(seColor);
		lb_series.addMouseListener(this);
		NP.add(lb_series);
		lb_applet=new Label("Applet");
		lb_applet.setAlignment(Label.CENTER);
		lb_applet.setForeground(Color.black);
		lb_applet.setBackground(apColor);
		lb_applet.addMouseListener(this);
		NP.add(lb_applet);
		add("North",NP);

		CP=new Panel();
		CP.setLayout(cL=new CardLayout());
		build_P_scene();
		CP.add(P_scene,"scene");
		build_P_series();
		CP.add(P_series,"series");
		build_P_applet();
		CP.add(P_applet,"applet");
		add("Center",CP);

		Panel SP=new Panel();
		SP.setBackground(Color.gray);
		SP.setLayout(new FlowLayout(FlowLayout.CENTER,64,8));
		SP.add(b_accept=new Button("  aceptar  "));
		SP.add(b_close=new Button("  cerrar  "));
		SP.add(b_apply=new Button("  aplicar  "));
		b_apply.addActionListener(this);
		b_close.addActionListener(this);
		b_accept.addActionListener(this);
		add("South",SP);

		pack();
		center();
		addWindowListener(this);
	}
	
	void destroy() {
		if (EE!=null) {
			EE.setVisible(false);
			EE.dispose();
		}
		super.dispose();
	}

	void center() {
		Dimension SD=Toolkit.getDefaultToolkit().getScreenSize();
		Dimension sz=getSize();
		int x=(SD.width-sz.width)/2;
		int y=(SD.height-sz.height)/2;
		setLocation(x,y);
	}

	static final int	
		ix_controles=0,
		ix_arbol=1,
		ix_arbol_sensible=2,
		ix_altura_paso=3,
		ix_fuente_tipo=4,
		ix_fuente_puntos=5,
		ix_por=6,
		ix_parntesisSiempre=7,
		ix_decimales=8;

	public static final String[] paramName = {
		"controles",			//(0) arriba; abajo,arriba
		"rbol",				//(1) no,izquierda,derecha,solo
		"rbol_sensible",		//(2)  ,no,s
		"altura_paso",			//(3) 1,2,3,...
		"fuente_tipo",			//(4) Serif; Serif, SansSerif, Monospaced
		"fuente_puntos",		//(5) 25; 8,9,...,80
		"por",					//(6) (punto,siempre); (punto|asterisco|aspas;siempre|no_siempre)
		"parntesisSiempre",	//(7) s; no,s
		"decimales"				//(8) 2; 0,1,2,...
	};

	private String[] paramContent;
	private Choice[] ch_scene=new Choice[paramName.length];
	private Choice[] ch_anode=new Choice[ANodeConfig.paramName.length];
	private Choice[] ch_series=new Choice[SeriesConfig.attrName.length];
	private TextField tf_fileName;
	
	private static final String[][] paramOptions = {
		{"arriba","abajo","no"},
		{"no","izquierda","derecha","solo"},
		{"","no","s"},
		{"1","2","3","4","5","6","7","8"},
		{"Monospaced","Serif","SansSerif"},
		{"10","11","12","13","14","15","16","17","18","19","20","21","22","23","24","25","26","27","28","29","30","32","34","36","38","40","45","50","60","80"},
		{"punto,siempre","punto,no siempre","aspas,siempre","aspas,no siempre"},//(4) por
		{"no","s"},			//(5) parntesisSiempre
		{"0","1","2","3","4","5","6","7","8"}	//(6) decimales
	};

	private static final String[] paramDefault = {
		"arriba",		//(0) controles
		"no",			//(1) rbol
		"",				//(2) rbol_sensible
		"1",			//(3)
		"SansSerif",	//(4) fuente_tipo
		"25",			//(5) fuente_puntos
		"punto,siempre",//(6) por
		"s",			//(7) parntesisSiempre
		"2"				//(8) decimales
	};
	
	private static final int sep_L=6;
	private static final int sep_R=6;
	

	private void build_P_scene() {
		P_scene=new Panel();
		P_scene.setBackground(scColor);
		P_scene.setLayout(new GridBagLayout());
		GridBagConstraints gbc=new GridBagConstraints();
		gbc.insets=new Insets(30,5,30,10);
		gbc.anchor=GridBagConstraints.CENTER;
		Panel auxPLN=new Panel();
		Panel auxPLS=new Panel();
		Panel auxPRN=new Panel();
		Panel auxPRS=new Panel();
		gbc.gridx=0;gbc.gridy=0;
		P_scene.add(auxPLN,gbc);
		gbc.gridx=1;gbc.gridy=0;
		P_scene.add(auxPRN,gbc);
		gbc.gridx=0;gbc.gridy=1;
		P_scene.add(auxPLS,gbc);
		gbc.gridx=1;gbc.gridy=1;
		P_scene.add(auxPRS,gbc);
		auxPLN.setLayout(new GridLayout(sep_L,2,5,5));
		auxPLS.setLayout(new GridLayout(paramName.length-sep_L,2,5,5));
		for (int i=0;i<paramName.length;i++) {
			Label lb=new Label(paramName[i]+"  ");
			lb.setAlignment(Label.RIGHT);
			if (i<sep_L) {
				auxPLN.add(lb);
			} else {
				auxPLS.add(lb);
			}
			ch_scene[i]=new Choice();
			ch_scene[i].setFont(txFont);
			for (int j=0;j<paramOptions[i].length;j++) {
				ch_scene[i].add(paramOptions[i][j]);
			}
			if (i<sep_L) {
				auxPLN.add(ch_scene[i]);
			} else {
				auxPLS.add(ch_scene[i]);
			}
		}

		auxPRN.setLayout(new GridLayout(sep_R,2,5,5));
		auxPRS.setLayout(new GridLayout(ANodeConfig.paramName.length-sep_R,2,5,5));
		for (int i=0;i<ANodeConfig.paramName.length;i++) {
			Label lb=new Label(ANodeConfig.paramName[i]+"  ");
			lb.setAlignment(Label.RIGHT);
			if (i<sep_R) {
				auxPRN.add(lb);
			} else {
				auxPRS.add(lb);
			}
			ch_anode[i]=new Choice();
			ch_anode[i].setFont(txFont);
			for (int j=0;j<ANodeConfig.paramOptions[i].length;j++) {
				ch_anode[i].add(ANodeConfig.paramOptions[i][j]);
			}
			if (i<sep_R) {
				auxPRN.add(ch_anode[i]);
			} else {
				auxPRS.add(ch_anode[i]);
			}
		}
	}

	Button bLS_new,bLS_cpy,bLS_del;
	spinnerControl Ssc=new spinnerControl(false);
	Button bLE_new,bLE_cpy,bLE_del;
	spinnerControl Esc=new spinnerControl(false);
	TextField TF_instructions;

	Choice ch_var=new Choice();
	Checkbox chb_active=new Checkbox(" ");
	TextField TF_min=new TextField(3);
	TextField TF_max=new TextField(3);
	
	ExercisePanel exerP;

	private Panel RP;

	private void build_P_series() {
		Panel auxP=new Panel();
		auxP.setLayout(new BorderLayout(8,8));
		Panel LP=new Panel(new GridLayout(2,1));
		Panel NLP=new Panel();
		NLP.setLayout(new BorderLayout());
		Panel LBP=new Panel();
		LBP.setLayout(new FlowLayout(FlowLayout.CENTER,0,0));
		bLS_new=new Button("+");
		bLS_cpy=new Button("*");
		bLS_del=new Button("-");
		bLS_new.setFont(txFont);
		bLS_cpy.setFont(txFont);
		bLS_del.setFont(txFont);
		bLS_new.addActionListener(this);
		bLS_cpy.addActionListener(this);
		bLS_del.addActionListener(this);
		LBP.add(bLS_new);
		LBP.add(bLS_cpy);
		LBP.add(Ssc);
		Ssc.addMouseListener(this);
		LBP.add(bLS_del);
		Label lb=new Label("    Series    ");
		lb.setFont(lbFont);
		lb.setAlignment(Label.CENTER);
		lb.setBackground(new Color(0x222288));
		lb.setForeground(Color.white);
		LBP.add(lb);
		NLP.add("North",LBP);
		NLP.add("Center",LS);

		Panel SLP=new Panel();
		SLP.setLayout(new BorderLayout());
		LBP=new Panel();
		LBP.setLayout(new FlowLayout(FlowLayout.CENTER,0,0));
		bLE_new=new Button("+");
		bLE_cpy=new Button("*");
		bLE_del=new Button("-");
		bLE_new.setFont(txFont);
		bLE_cpy.setFont(txFont);
		bLE_del.setFont(txFont);
		bLE_new.addActionListener(this);
		bLE_cpy.addActionListener(this);
		bLE_del.addActionListener(this);
		LBP.add(bLE_new);
		LBP.add(bLE_cpy);
		LBP.add(Esc);
		Esc.addMouseListener(this);
		LBP.add(bLE_del);
		lb=new Label("  Ejercicios  ");
		lb.setFont(lbFont);
		lb.setAlignment(Label.CENTER);
		lb.setBackground(new Color(0x222288));
		lb.setForeground(Color.white);
		LBP.add(lb);
		SLP.add("North",LBP);
		SLP.add("Center",LE);
		LP.add(NLP);
		LP.add(SLP);

		auxP.add("West",LP);

		Panel auxRP=new Panel(new BorderLayout(4,4));
		RP=new Panel();
		RP.setLayout(new GridLayout(SeriesConfig.attrName.length,2,2,2));
		for (int i=0;i<SeriesConfig.attrName.length;i++) {
			ch_series[i]=new Choice();
			ch_series[i].setFont(txFont);
			for (int j=0;j<SeriesConfig.attrOptions[i].length;j++) {
				ch_series[i].add(SeriesConfig.attrOptions[i][j]);
			}
			ch_series[i].addItemListener(this);
			lb=new Label(SeriesConfig.attrName[i]+"  ");
			lb.setAlignment(Label.RIGHT);
			RP.add(lb);
			if (i!=SeriesConfig.ix_filename) {
				RP.add(ch_series[i]);
			}
		}
		tf_fileName=new TextField(16);
		tf_fileName.addActionListener(this);
		auxRP.add("North",RP);
		Panel middleP=new Panel(new GridLayout(2,1,0,0));
		Panel msP=new Panel(new FlowLayout(FlowLayout.CENTER,2,2));
		msP.add(new Label("Descripcin"));
		TF_instructions=new TextField(36);
		TF_instructions.setFont(txFont);
		msP.add(TF_instructions);
		b_saveAs=new Button("guardar como");
		b_saveAs.addActionListener(this);
		msP.add(b_saveAs);
		Panel mnP=new Panel(new FlowLayout(FlowLayout.CENTER,2,2));
		for (char ch='A';ch<='Z';ch++) {
			ch_var.add(""+ch);
		}
		ch_var.addItemListener(this);
		chb_active.addItemListener(this);
		TF_min.addActionListener(this);
		TF_max.addActionListener(this);
		mnP.add(new Label("Variables aleatorias"));
		mnP.add(ch_var);
		mnP.add(new Label("activa"));
		mnP.add(chb_active);
		mnP.add(new Label("min"));
		mnP.add(TF_min);
		mnP.add(new Label("max"));
		mnP.add(TF_max);
		middleP.add(msP);
		middleP.add(mnP);
		auxRP.add("Center",middleP);
		exerP=new ExercisePanel(M);
		auxRP.add("South",exerP);
		auxP.add("Center",auxRP);
		
		
		Panel auxPS=new Panel();
		auxPS.setLayout(new FlowLayout(FlowLayout.CENTER,25,4));

//		b_undo=new Button(" deshacer ");
//		b_redo=new Button(" re-hacer ");
//		b_undo.setEnabled(false);
//		b_redo.setEnabled(false);
//		b_undo.addActionListener(this);
//		b_redo.addActionListener(this);
//		auxPS.add(b_undo);
//		auxPS.add(b_redo);

//		b_save=new Button("guardar");
//		b_save.addActionListener(this);
//		auxPS.add(b_save);
//		auxP.add("South",auxPS);

		P_series=new Panel();
		P_series.setBackground(seColor);
		P_series.setLayout(new FlowLayout(FlowLayout.CENTER,16,16));
		P_series.add(auxP);
		LS.select(0);
	}

	private String codebase="./";
	private String nombre="";
	private String course="";
	private String s_width=null,s_height=null;

	private void build_P_applet() {
		P_applet=new Panel();
		P_applet.setBackground(apColor);
		P_applet.setLayout(new BorderLayout());
		Panel NNP=new Panel();
		NNP.setLayout(new FlowLayout(FlowLayout.LEFT,5,3));
		Label lb=new Label("nombre");
		lb.setAlignment(Label.CENTER);
		NNP.add(lb);
		tf_nombre=new TextField(12);
		tf_nombre.setBackground(apColorLight);
		tf_nombre.setFont(txFont);
		NNP.add(tf_nombre);
		lb=new Label("codebase");
		lb.setAlignment(Label.CENTER);
		NNP.add(lb);
		tf_codebase=new TextField(8);
		tf_codebase.setBackground(apColorLight);
		tf_codebase.setFont(txFont);
		NNP.add(tf_codebase);
		lb=new Label("ancho");
		lb.setAlignment(Label.CENTER);
		NNP.add(lb);
		tf_ancho=new TextField(4);
		tf_ancho.setBackground(apColorLight);
		tf_ancho.setFont(txFont);
		NNP.add(tf_ancho);
		lb=new Label("alto");
		lb.setAlignment(Label.CENTER);
		NNP.add(lb);
		tf_alto=new TextField(4);
		tf_alto.setBackground(apColorLight);
		tf_alto.setFont(txFont);
		NNP.add(tf_alto);
		P_applet.add("North",NNP);
		TA=new TextArea(16,80);
		TA.setEditable(false);
		TA.setBackground(apColorLight);
		TA.setFont(txFont);
		b_appletApply=new Button("aplicar");
		b_appletApply.addActionListener(this);
		NNP.add(b_appletApply);
		P_applet.add("Center",TA);
		Label lb_copy=new Label("Puede copiar este texto y pegarlo en una pgina web");
		lb_copy.setAlignment(Label.CENTER);
		P_applet.add("South",lb_copy);
	}

/* ----------------------------------------------------------- */

	static void select(Choice ch,String s0) {
		if (ch.getItemCount()==2 && ch.getItem(0).equals("no")) {
			String s=s0.toLowerCase();
			if (s.startsWith("n")||s.startsWith("f")) {
				ch.select(0);
			} else {
				ch.select(1);
			}
		} else {
			ch.select(s0);
		}
	}

/* ----------------------------------------------------------- */

	private boolean firstTime=true;

	void updateSize() {
		if (firstTime) {
			firstTime=false;
			s_width=Integer.toString(M.getSize().width);
			s_height=Integer.toString(M.getSize().height);
			tf_ancho.setText(s_width);
			tf_alto.setText(s_height);
			updateTA();
		}
	}

	void readDescartesACfg() {
		codebase=getReducedCodeBase(M);
		tf_codebase.setText(codebase);
		nombre=M.getParameter("nombre");
		tf_nombre.setText(nombre);
		paramContent=new String[paramName.length];
		for (int i=0;i<paramName.length;i++) {
			paramContent[i]=M.getParameter(paramName[i]);
			if (!BasicStr.hasContent(paramContent[i])) {
				paramContent[i]=paramDefault[i];
			}
			select(ch_scene[i],paramContent[i]);
		}

		for (int i=0;i<ANodeConfig.paramName.length;i++) {
			M.an_cfg.paramContent[i]=M.getParameter(ANodeConfig.paramName[i]);
			if (!BasicStr.hasContent(M.an_cfg.paramContent[i])) {
				M.an_cfg.paramContent[i]=ANodeConfig.paramDefault[i];
			}
			select(ch_anode[i],M.an_cfg.paramContent[i]);
		}
		SCA=new SeriesConfig[M.serie.length];
		for (int i=0;i<SCA.length;i++) {
			SCA[i]=M.serie[i].cloneSeriesConfig(M,M.parser,M.decimales);
		}
		refreshListS(0);
		applyDescartesACfg();
	}

	void refreshListS(int n) {
		LS.removeAll();
		for (int i=0;i<SCA.length;i++) {
			String sn=Integer.toString(i+1);
			while (sn.length()<2) {
				sn="0"+sn;
			}
			LS.add(sn+" "+SCA[i].getIndicator());
		}
		if (SCA.length>0) {
			sel_ix=Math.min(SCA.length-1,n);
			LS.select(sel_ix);
			SCA[sel_ix].applyConfig(true);
			updateSeriesTFs();
		} else {
			refreshListE(0);
		}
	}
	
	private int exer_num;

	void refreshListE(int n) {
		LE.removeAll();
		if (SCA.length>0) {
			for (int i=0;i<SCA[sel_ix].countExercises();i++) {
				String sn=Integer.toString(i+1);
				while (sn.length()<2) {
					sn="0"+sn;
				}
				LE.add(sn+" "+SCA[sel_ix].getExercise(i).getIndicator());
			}
			if (SCA[sel_ix].countExercises()>0) {
				exer_num=Math.min(SCA[sel_ix].countExercises()-1,n);
				LE.select(exer_num);
				exerP.edit(SCA[sel_ix].getExercise(exer_num));
			} else {
				exerP.apply(null);
				exer_num=-1;
			}
		} else {
			exerP.apply(null);
			exer_num=-1;
		}
	}

	void applyChanges() {
		for (int i=0;i<paramName.length;i++) {
			paramContent[i]=ch_scene[i].getSelectedItem();
			if (!BasicStr.hasContent(paramContent[i])) {
				paramContent[i]=paramDefault[i];
			}
		}
		for (int i=0;i<ANodeConfig.paramName.length;i++) {
			M.an_cfg.paramContent[i]=ch_anode[i].getSelectedItem();
			if (!BasicStr.hasContent(M.an_cfg.paramContent[i])) {
				M.an_cfg.paramContent[i]=ANodeConfig.paramDefault[i];
			}
		}
		M.an_cfg.applyANodeCfg();
		M.serie=SCA;
		applyDescartesACfg();
	}

	void applyDescartesACfg() {
		M.setConfig(paramContent);
		updateSeriesTFs();
		updateTA();
	}

	void updateTA() {
		TA.setText(toApplet(null));
		TA.setSelectionStart(0);
		TA.setSelectionEnd(TA.getText().length());
		TA.requestFocus();
	}

	int sel_ix=-1;
	
	void updateSeriesFromPanel() {
		if (SCA.length==0) {
			exerP.apply(null);
		} else if (sel_ix>=0) {
			String exercisesFileName="";
			if (M.inWeb()) {
				exercisesFileName=tf_fileName.getText();
			} else {
//				exercisesFileName=ch_series[SeriesConfig.ix_filename].getSelectedItem();
				exercisesFileName=SCA[sel_ix].getFileName();
			}
			if (!SCA[sel_ix].getFileName().equals(exercisesFileName)) {
				String ser_cfg="";
				for (int i=0;i<ch_series.length-1;i++) {
					ser_cfg+=SeriesConfig.attrName[i]+"='"+ch_series[i].getSelectedItem()+"' ";
				}
				ser_cfg+=" ejercicios='"+exercisesFileName+"'";
				SCA[sel_ix]=new SeriesConfig(M,M.Tr,M.parser,ser_cfg,M.decimales,M.inWeb());
			} else {
				for (int i=0;i<ch_series.length-1;i++) {
					SCA[sel_ix].updateAttribute(i,ch_series[i].getSelectedItem());
				}
				SCA[sel_ix].applyConfig(false);
			}
			SCA[sel_ix].setInstructions(TF_instructions.getText());
			if (chb_active.getState()) {
				double min=BasicStr.parseDouble(TF_min.getText(),0,0);
				double max=BasicStr.parseDouble(TF_max.getText(),0,0);
				SCA[sel_ix].resetRandomVar(Vname,min,max,0,false);
			} else {
				SCA[sel_ix].deleteRandomVar(Vname);
			}

			exerP.apply(SCA[sel_ix].getExercise(exer_num));
			SCA[sel_ix].resetContent();
		}
	}
	
	private String Vname="A";
	
	boolean updateSeriesTFs() {
		boolean adelante=false;
		if (M.inWeb()) {
			if ( (0<=sel_ix && sel_ix<M.serie.length) &&
				!M.serie[sel_ix].getContents().equals(SCA[sel_ix].getContents())) {
				if (mjaText.ask("Cuidado!",6,50,"Perder los cambios a la serie si contina.\nPara no perder los cambios responda no,\npulse <ver contenido> y copie el texto\nen el archivo "+SCA[sel_ix].getFileName()+".\nAn as desea continuar?","S","No")) {
					adelante=true;
				}
			} else {
				adelante=true;
			}
		} else {
			adelante=true;
		}
		if (adelante) {
			sel_ix=LS.getSelectedIndex();
			if (sel_ix>=0) {
				for (int i=0;i<SeriesConfig.attrName.length;i++) {
					select(ch_series[i],SCA[sel_ix].attrValue[i]);
				}
				tf_fileName.setText(SCA[sel_ix].getFileName());
				// seleccionar el modo correcto ---
				String modos=SCA[sel_ix].getModos();
				if (modos!=null) {
					if (modos.length()>1) {
						modos=modos.substring(0,1);
					}
				} else {
					modos="2";
				}
				for (int j=0;j<SeriesConfig.attrOptions[SeriesConfig.ix_mode].length;j++) {
					if (modos.equals(Integer.toString(j+1))) {
						ch_series[SeriesConfig.ix_mode].select(j);
					}
				}
				// ---------------------------------
				updateRandomVarFields();
				TF_instructions.setText(SCA[sel_ix].getInstructions());
				refreshListE(0);
			}
		} else {
			LS.removeItemListener(this);
			LS.select(sel_ix);
			LS.addItemListener(this);
		}
		return adelante;
	}
	
	void updateRandomVarFields() {
		Vname=ch_var.getSelectedItem();
		boolean isVar=SCA[sel_ix].isRandomVar(Vname);
		chb_active.setState(isVar);
		if (isVar) {
			RandomNumber R=SCA[sel_ix].getRandomVar(Vname);
			TF_min.setText(BasicStr.DoubleToString(R.getMin(),R.getDecimals(),false));
			TF_max.setText(BasicStr.DoubleToString(R.getMax(),R.getDecimals(),false));
		} else {
			TF_min.setText("");
			TF_max.setText("");
		}
	}

	public Dimension proposedDimension() {
		Dimension d=M.getSize();
		d.width=BasicStr.parseInteger(s_width,d.width,d.width);
		d.height=BasicStr.parseInteger(s_height,d.height,d.height);
		return d;
	}
	
	
	public String toApplet(Dimension d0) {
		String s="<applet ";
		if (BasicStr.hasContent(nombre)) {
			s+="name=\""+nombre+"\"\n        ";
		}
		s+="code=\"DescartesA.class\"\n"+
		"        codebase=\""+codebase+"\"\n"+
		"        archive=\"DescartesA.jar";
		if (BasicStr.hasContent(course)) {
			s+=",classes/LMS_Learner.jar";
		}
		s+="\"\n"+
		"        MAYSCRIPT\n";
		if (d0!=null && (s_width==null||s_height==null)) {
			s_width=Integer.toString(d0.width);
			s_height=Integer.toString(d0.height);
		}
		s+="        width="+s_width+" height="+s_height+">\n";
		s+="   <param name=\"Version\" value=\""+DescartesA.Version+"\">\n";
		if (BasicStr.hasContent(nombre)) {
			s+="   <param name=\"nombre\" value=\""+nombre+"\">\n";
		}
		if (".".equals(BasicStr.decimal_symbol)) {
			s+="   <param name=\"decimal_symbol\" value=\".\">\n";
		}
		for (int i=0;i<paramName.length;i++) {
			if (!paramDefault[i].equals(paramContent[i])) {
				s+="   <param name=\""+paramName[i]+"\" value=\""+paramContent[i]+"\">\n";
			}
		}
		s+=M.an_cfg.toAppletParams();
		for (int i=0;i<M.serie.length;i++) {
			String auxs=Integer.toString(i+1);
			while (auxs.length()<2) {
				auxs="0"+auxs;
			}
			s+="   <param name=\"SERIE_"+auxs+"\" value=\""+M.serie[i].getHeader()+"\">\n";
		}
		s=s+"</applet>\n";
		return s;
//		return mjaHtml.getHTMLEncoding(s);
	}

	void updateFiles() {
		if (!M.inWeb()) {
			File docFolder=new File(BasicStr.DocPath(M)+File.separator+"ejercicios"+File.separator);
			if (!docFolder.exists()) {
				docFolder.mkdirs();
			}
			File[] files=docFolder.listFiles();
			ch_series[SeriesConfig.ix_filename].removeAll();
			ch_series[SeriesConfig.ix_filename].add(SeriesConfig.attrDefault[SeriesConfig.ix_filename]);
			for (int j=0;j<files.length;j++) {
				if (files[j].getName().toLowerCase().endsWith(".txt") && 
					!files[j].getName().equals(SeriesConfig.attrDefault[SeriesConfig.ix_filename])) {
					ch_series[SeriesConfig.ix_filename].add(files[j].getName());
				}
			}
			if (sel_ix>=0 && sel_ix<SCA.length) {
				ch_series[SeriesConfig.ix_filename].select(SCA[sel_ix].getFileName());
			}
		}
	}

	boolean firsttime=true;

	void editSeries() {
		updateFiles();
		if (firsttime) {
			if (M.inWeb()) {
				b_saveAs.setLabel("ver contenido");
				RP.add(tf_fileName);
			} else {
				b_saveAs.setLabel("guardar como");
				RP.add(ch_series[SeriesConfig.ix_filename]);
			}
			firsttime=false;
		}
		updateSeriesTFs();
		setVisible(true);
		toFront();
		requestFocus();
	}

	public static String getReducedCodeBase(Applet A) {
		return "./";
//		String docbase=A.getDocumentBase().toExternalForm();
//		docbase=docbase.substring(0,docbase.lastIndexOf('/')+1);
//		String codebase=A.getCodeBase().toExternalForm();
//		if (docbase.equals(codebase)) {
//			codebase="./";
//		} else if (codebase.startsWith(docbase)) {
//			codebase=codebase.substring(docbase.length());
//		} else if (docbase.startsWith(codebase)) {
//			docbase=docbase.substring(0,docbase.lastIndexOf('/'));
//			codebase=codebase.substring(0,codebase.lastIndexOf('/'));
//			String cb="";
//			while (docbase.length()>codebase.length()) {
//				docbase=docbase.substring(0,docbase.lastIndexOf('/'));
//				cb+="../";
//			}
//			codebase=cb;
//		}
//		return codebase;
	}

/* ----------------- Listener Implementation ------------------ */

//	Vector Vsca=new Vector();
//	int Vsca_ix=0;

	public void actionPerformed(ActionEvent e) {
		updateSeriesFromPanel();
		int exnum=LE.getSelectedIndex();
		if (e.getSource()==b_appletApply) {
			nombre=tf_nombre.getText();
			codebase=tf_codebase.getText();
			s_width=tf_ancho.getText();
			s_height=tf_alto.getText();
			applyChanges();
		} else if (e.getSource()==b_accept) {
			setVisible(false);
			applyChanges();
			M.configApplied();
			M.restart();
		} else if (e.getSource()==b_close) {
			setVisible(false);
		} else if (e.getSource()==b_apply) {
			applyChanges();
			M.restart();
		} else if (e.getSource()==bLS_new) {
			SeriesConfig[] SCA0=SCA;
			SCA=new SeriesConfig[SCA0.length+1];
			for (int i=0;i<SCA0.length;i++) {
				SCA[i]=SCA0[i].cloneSeriesConfig(M,M.parser,M.decimales);
			}
			SCA[SCA.length-1]=new SeriesConfig(M,M.parser,M.decimales,M.inWeb());
//			push(SCA);
			refreshListS(SCA.length-1);
		} else if (e.getSource()==bLS_cpy) {
			int n=LS.getSelectedIndex();
			if (n>=0) {
				SeriesConfig[] SCA0=SCA;
				SCA=new SeriesConfig[SCA0.length+1];
				for (int i=0;i<SCA0.length;i++) {
					SCA[i]=SCA0[i].cloneSeriesConfig(M,M.parser,M.decimales);
				}
				SCA[SCA.length-1]=SCA0[n].cloneSeriesConfig(M,M.parser,M.decimales);
//				push(SCA);
				refreshListS(SCA.length-1);
			}
		} else if (e.getSource()==bLS_del) {
			int n=LS.getSelectedIndex();
			if (n>=0) {
				SeriesConfig[] SCA0=SCA;
				SCA=new SeriesConfig[SCA0.length-1];
				for (int i=0;i<SCA.length;i++) {
					if (i<n) {
						SCA[i]=SCA0[i].cloneSeriesConfig(M,M.parser,M.decimales);
					} else {
						SCA[i]=SCA0[i+1].cloneSeriesConfig(M,M.parser,M.decimales);
					}
				}
//				push(SCA);
				refreshListS(n);
			}
//		} else if (e.getSource()==b_undo) {
//			if (Vsca_ix>0) {
//				Vsca_ix--;
//				SCA=(SeriesConfig[]) peek();
//			}
//			refreshListS(0);
//		} else if (e.getSource()==b_redo) {
//			if (Vsca_ix+1<Vsca.size()) {
//				Vsca_ix++;
//				SCA=(SeriesConfig[]) peek();
//			}
//			refreshListS(0);
		} else if (e.getSource()==bLE_del) {
			ExerciseConfig[] exercise0=SCA[sel_ix].exercise;
			SCA[sel_ix].exercise=new ExerciseConfig[exercise0.length-1];
			int j=0;
			for (int i=0;i<SCA[sel_ix].exercise.length;i++) {
				if (j==exnum) {
					j++;
				}
				SCA[sel_ix].exercise[i]=exercise0[j++];
			}
			SCA[sel_ix].resetContent();
			refreshListE(Math.min(exnum,SCA[sel_ix].countExercises()-1));
		} else if (e.getSource()==bLE_new) {
			ExerciseConfig[] exercise0=SCA[sel_ix].exercise;
			SCA[sel_ix].exercise=new ExerciseConfig[exercise0.length+1];
			for (int i=0;i<exercise0.length;i++) {
				SCA[sel_ix].exercise[i]=exercise0[i];
			}
			SCA[sel_ix].exercise[exercise0.length]=new ExerciseConfig(SCA[sel_ix].getRandomVector(),SCA[sel_ix].parser,M.decimales);
			SCA[sel_ix].exercise[exercise0.length].setValues("Instrucciones","x=0","");
			SCA[sel_ix].resetContent();
			refreshListE(exercise0.length);
		} else if (e.getSource()==bLE_cpy) {
			ExerciseConfig[] exercise0=SCA[sel_ix].exercise;
			SCA[sel_ix].exercise=new ExerciseConfig[exercise0.length+1];
			for (int i=0;i<exercise0.length;i++) {
				SCA[sel_ix].exercise[i]=exercise0[i];
			}
			SCA[sel_ix].exercise[exercise0.length]=SCA[sel_ix].getExercise(LE.getSelectedIndex()).cloneExercise();
			SCA[sel_ix].resetContent();
			refreshListE(exercise0.length);
		} else if (e.getSource()==b_saveAs) {
			if (M.inWeb()) {
				mjaText.showPlainTextFrame("ejercicios/"+SCA[sel_ix].getFileName(),"aceptar",SCA[sel_ix].getContents(),25,100,false);
			} else {
				String newFileName=mjaText.getInfo("Guardar como...","Guardar:","aceptar","cancelar",false,SCA[sel_ix].getFileName());
				if (BasicStr.hasContent(newFileName)) {
					SCA[sel_ix].setFileName(newFileName);
					if (exer_num>=0 && exer_num<SCA[sel_ix].countExercises()) {
						exerP.apply(SCA[sel_ix].getExercise(exer_num));
					}
					if (SCA[sel_ix].saveContents()) {
						updateFiles();
						SCA[sel_ix].setFileName(newFileName);
						refreshListS(sel_ix);
					}
				}
			}
		} else if (e.getSource()==tf_fileName) {
			String ser_cfg="";
			for (int i=0;i<ch_series.length-1;i++) {
				ser_cfg+=SeriesConfig.attrName[i]+"='"+ch_series[i].getSelectedItem()+"' ";
			}
			ser_cfg+=SeriesConfig.attrName[SeriesConfig.ix_filename]+"='"+tf_fileName.getText();
			SCA[sel_ix]=new SeriesConfig(M,M.Tr,M.parser,ser_cfg,M.decimales,M.inWeb());
			refreshListS(sel_ix);
		} else if  (e.getSource()==TF_min||e.getSource()==TF_max) {
			updateSeriesFromPanel();
		}
	}

	boolean editExercise(int six,int exnum) {
		if (EE==null) {
			EE=new ExerciseEditor(M);
		}
		EE.edit(SCA[six].getExercise(exnum));
		if (EE.accepted()) {
			SCA[six].resetContent();
			refreshListE(exnum);
			applyChanges();
			return true;
		}
		return false;
	}

//	void push(Object O) {
//		Vsca.addElement(O);
//		Vsca_ix=Vsca.size()-1;
//	}
//
//	Object pop() {
//		return Vsca.elementAt(--Vsca_ix);
//	}
//
//	Object peek() {
//		return Vsca.elementAt(Vsca_ix);
//	}

	
/* ---------------------- Item Listener  --------------------- */

	public void itemStateChanged(ItemEvent e) {
		updateSeriesFromPanel();
		if (e.getStateChange()==ItemEvent.SELECTED) {
			if (e.getSource()==LS) {
				if (sel_ix!=LS.getSelectedIndex() && updateSeriesTFs()) {
					refreshListS(sel_ix);
				}
			} else if (e.getSource()==LE) {
				exerP.apply(SCA[sel_ix].getExercise(exer_num));
				SCA[sel_ix].resetContent();
				refreshListE(LE.getSelectedIndex());
			} else if (e.getSource()==ch_series[SeriesConfig.ix_filename]) {
				boolean fileChanged=!SCA[sel_ix].getFileName().equals(ch_series[SeriesConfig.ix_filename].getSelectedItem());
				if (fileChanged) {
					SCA[sel_ix].setFileName(ch_series[SeriesConfig.ix_filename].getSelectedItem());
					SCA[sel_ix].applyConfig(true);
				} else {
					SCA[sel_ix].applyConfig(false);
				}
				refreshListS(sel_ix);
			} else if (e.getSource()==ch_var) {
				updateRandomVarFields();
			}
		}
	}

/* ---------------------- Mouse Listener  --------------------- */

	public void mouseClicked(MouseEvent e) {
	}
	public void mousePressed(MouseEvent e) {
		updateSeriesFromPanel();
		if (e.getSource()==Ssc) {
			if (e.getY()<Ssc.getSize().height/2) {
				if (sel_ix>0 && sel_ix<SCA.length) {
					SeriesConfig serCfg=SCA[sel_ix-1];
					SCA[sel_ix-1]=SCA[sel_ix];
					SCA[sel_ix]=serCfg;
					sel_ix--;
					refreshListS(sel_ix);
				}
			} else {
				if (sel_ix>=0 && sel_ix+1<SCA.length) {
					SeriesConfig serCfg=SCA[sel_ix+1];
					SCA[sel_ix+1]=SCA[sel_ix];
					SCA[sel_ix]=serCfg;
					sel_ix++;
					refreshListS(sel_ix);
				}
			}
		} else if (e.getSource()==Esc) {
			int exnum=LE.getSelectedIndex();
			if (e.getY()<Esc.getSize().height/2) {
				if (exnum>0) {
					ExerciseConfig auxex=SCA[sel_ix].exercise[exnum-1];
					SCA[sel_ix].exercise[exnum-1]=SCA[sel_ix].exercise[exnum];
					SCA[sel_ix].exercise[exnum]=auxex;
					SCA[sel_ix].resetContent();
					refreshListE(exnum-1);
				}
			} else {
				if (exnum+1<SCA[sel_ix].countExercises()) {
					ExerciseConfig auxex=SCA[sel_ix].exercise[exnum+1];
					SCA[sel_ix].exercise[exnum+1]=SCA[sel_ix].exercise[exnum];
					SCA[sel_ix].exercise[exnum]=auxex;
					SCA[sel_ix].resetContent();
					refreshListE(exnum+1);
				}
			}
		}
	}
	public void mouseReleased(MouseEvent e) {
	}
	public void mouseEntered(MouseEvent e) {
		updateSeriesFromPanel();
		if (e.getSource()==lb_scene) {
			cL.show(CP,"scene");
		} else if (e.getSource()==lb_series) {
			cL.show(CP,"series");
		} else if (e.getSource()==lb_applet) {
			cL.show(CP,"applet");
		}
	}
	public void mouseExited(MouseEvent e) {
	}

/* --------------------- Window Listener  --------------------- */

	public void windowOpened(WindowEvent e) {
	}
	public void windowClosed(WindowEvent e) {
	}
	public void windowClosing(WindowEvent e) {
		setVisible(false);
	}
	public void windowActivated(WindowEvent e) {
	}
	public void windowDeactivated(WindowEvent e) {
	}
	public void windowIconified(WindowEvent e) {
	}
	public void windowDeiconified(WindowEvent e) {
	}

}