/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package com.mja.algebra;

import java.util.*;
import java.applet.*;
import com.mja.file.*;
import com.mja.util.*;
import com.mja.parser.*;

public class ExerciseConfig {

	private Vector Vstep,rnv;
	private String instruction="";
	private Parser parser;
	private int decimales;
	private int errors=0;
	private int steps=0;
	private boolean done=false;
	
	public int getErrors() {
		return errors;
	}
	
	public void addError() {
		errors++;
	}
	
	public void addStep() {
		steps++;
	}
	
	public boolean isDone() {
		return done;
	}
	
	public void setDone() {
		done=true;
	}
	
	public void reset() {
		errors=0;
		steps=0;
		done=false;
	}
	
	public double getAverage() {
		if (done) {
			return ((double) Math.max(0,countSteps()-errors))*100.0/Math.max(1,countSteps());
		} else {
			return 0;
		}
	}
	
	public String getIndicator() {
		String s=instruction;
//		if (countSteps()>0) {
//			s+=" "+getStep(0).getOExpression(false);
//		}
		return s;
	}
	
	public String toString() {
		String s=instruction+"\n\n";
		for (int i=0;i<countSteps();i++) {
			s+=getStep(i).toString()+"\n";
		}
		return s;
	}
	
	public String getOExpressions() {
		String s="";
		for (int i=0;i<countSteps();i++) {
			s+=getStep(i).getOExpression(true)+"\n";
		}
		return s;
	}

	public String getAllExplanations() {
		String s="";
		for (int i=0;i<countSteps();i++) {
			s+=getStep(i).getExplanation()+"\n";
		}
		return s;
	}

	public String getInstruction() {
		return instruction;
	}

	public ExerciseConfig(Vector rnv,Parser p,int decimales) {
		Vstep=new Vector();
		this.parser=p;
		this.rnv=rnv;
		this.decimales=decimales;
	}
	
	public ExerciseConfig cloneExercise() {
		ExerciseConfig ecfg=new ExerciseConfig(rnv,parser,decimales);
		ecfg.setValues(instruction,getOExpressions(),getAllExplanations());
		return ecfg;
	}

	public void setValues(String instruction,String exprs,String expls) {
		this.instruction=instruction;
		String[] expra=BasicStr.toStrArr(exprs);
		String[] expla=BasicStr.toStrArr(expls);
		int N=expra.length;
		Vstep=new Vector();
		for (int i=0;i<N;i++) {
			if (expra[i].length()>0) {
				String expl="";
				if (i<expla.length) {
					expl=expla[i];
				}
				Vstep.add(new StepConfig(rnv,parser,expra[i],expl,decimales));
			}
		}
		
	}

	public int countSteps() {
		return Vstep.size();
	}

	public String[] getSExpressions() {
		String[] expr=new String[countSteps()];
		for (int i=0;i<countSteps();i++) {
			expr[i]=getStep(i).getSExpression();
		}
		return expr;
	}

	public String[] getExplanations() {
		String[] expl=new String[countSteps()];
		int N=countSteps();
		for (int i=0;i<N;i++) {
			expl[i]=((StepConfig) Vstep.elementAt(i)).getExplanation();
		}
		return expl;
	}

	public StepConfig getStep(int n) {
		if (n>=0 && n<Vstep.size()) {
			return (StepConfig) Vstep.elementAt(n);
		}
		return null;
	}

	private void addStep(Parser p,String expr,String expl,int decimales) {
		Vstep.addElement(new StepConfig(rnv,p,expr,expl,decimales));
	}
	
	public static ExerciseConfig[] readExercises(Vector rnv,Parser p,String s,int decimales) {
		String[] sa=BasicStr.toStrArr(s);
		Vector Vex=new Vector();
		int i=1;
		boolean in=false;
		ExerciseConfig ec=null;
		String instr_candidate="";
		while (i<sa.length) {
			int pos=0;
			if ((pos=sa[i].indexOf("|"))>0) {
				if (!in || ec==null) {
					ec=new ExerciseConfig(rnv,p,decimales);
					in=true;
				}
				String expr=sa[i].substring(0,pos);
				expr=BasicStr.trim(expr,'	');
				expr=BasicStr.trim(expr,' ');
				String expl=sa[i].substring(pos+1);
				expl=BasicStr.trim(expl,' ');
				ec.addStep(p,expr,expl,decimales);
			} else {
				if (BasicStr.hasContent(sa[i])&&!sa[i].toLowerCase().startsWith("variable")) {
					instr_candidate=sa[i];
				}
				in=false;
				if (ec!=null) {
					for (int k=0;k<SeriesConfig.attrName.length;k++) {
						if (instr_candidate.startsWith(SeriesConfig.attrName[k])) {
							instr_candidate="";
						}
					}
					ec.instruction=instr_candidate;
					Vex.addElement(ec);
					instr_candidate="";
				}
				ec=null;
			}
			i++;
		}
		if (ec!=null) {
			Vex.addElement(ec);
		}
		ExerciseConfig[] eca=new ExerciseConfig[Vex.size()];
		for (i=0;i<eca.length;i++) {
			eca[i]=(ExerciseConfig) Vex.elementAt(i);
		}

		return eca;
	}

}