/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package com.mja.algebra;

import java.awt.*;
import java.util.*;

import com.mja.gui.*;
import org.unam.matem.Attribute;
import com.mja.util.BasicStr;

public class RandomNumber {

	private static final String[] dec_options={"0","1","2","3","4","5"};

	private String name;
	private double value;
	private int decimals;
	private double min,max;
	private boolean allowNegatives;
	private Panel editPanel;
	private TextField tf_name,tf_min,tf_max;
	private Choice ch_dec;
	private Checkbox cb_allowNegatives;

	public RandomNumber(String name,double min,double max,int decimals,boolean allowNegatives) {
		this.name=name;
		this.min=min;
		this.max=max;
		this.decimals=decimals;
		this.allowNegatives=allowNegatives;
		value=random(this.min,this.max,this.decimals,this.allowNegatives);
	}

	public RandomNumber(String name,String s_min,String s_max,String s_decimals,String s_allowNegatives) {
		this.name=name;
		min=BasicStr.parseDouble(s_min,0,0);
		max=BasicStr.parseDouble(s_max,9,9);
		decimals=BasicStr.parseInteger(s_decimals,0,0);
		allowNegatives=false;
		if (s_allowNegatives!=null) {
			allowNegatives|=s_allowNegatives.startsWith("t");
			allowNegatives|=s_allowNegatives.startsWith("y");
			allowNegatives|=s_allowNegatives.startsWith("s");
		}
		recalculate();
	}

	public void recalculate() {
		value=random(min,max,decimals,allowNegatives);
	}
	
	public double getMin() {
		return min;
	}

	public double getMax() {
		return max;
	}
	
	public int getDecimals() {
		return decimals;
	}
	
	public boolean allowNegatives() {
		return allowNegatives;
	}

	public double getValue() {
		return value;
	}

	public String getStrValue(boolean fixed) {
		return BasicStr.DoubleToString(value,decimals,fixed);
	}

	public String getStrValue() {
		return getStrValue(false);
	}

	public String getName() {
		return name;
	}

	public static double random(double min, double max,int decimals,boolean allowNegatives) {
		int d=(int)Math.round(Math.pow(10,Math.max(0,Math.min(8,decimals))));
		while (true) {
			double x=(max-min)*Math.random()+min;
			x=((double)((int)Math.floor(x*d)))/(double)d;
			if (allowNegatives) {
				if (Math.random()<0.5) {
					x=-x;
				}
			}
			return x;
		}
	}

	public Component getVisualComponent() {
		if (editPanel==null) {
			editPanel=new Panel(new FlowLayout(FlowLayout.LEFT,0,0));
			tf_name=new TextField(1);
			tf_min=new TextField(1);
			tf_max=new TextField(1);
			ch_dec=new Choice();
			for (int i=0;i<dec_options.length;i++) {
				ch_dec.addItem(dec_options[i]);
			}
			cb_allowNegatives=new Checkbox("");
			tf_name.setText(name);
			tf_min.setText(BasicStr.DoubleToString(min,decimals,false));
			tf_max.setText(BasicStr.DoubleToString(max,decimals,false));
			ch_dec.select(String.valueOf(decimals));
			cb_allowNegatives.setState(allowNegatives);
			editPanel.add(mjaLayout.pair(0,1,new mjaLabel("nombre",mjaLabel.RIGHT,2),tf_name));
			editPanel.add(mjaLayout.pair(0,1,new mjaLabel("mnimo",mjaLabel.RIGHT,2),tf_min));
			editPanel.add(mjaLayout.pair(0,1,new mjaLabel("mximo",mjaLabel.RIGHT,2),tf_max));
			editPanel.add(mjaLayout.pair(0,1,new mjaLabel("decimales",mjaLabel.RIGHT,2),ch_dec));
			editPanel.add(mjaLayout.pair(1,0,new mjaLabel("permitir negativos",mjaLabel.RIGHT,2),cb_allowNegatives));
		}
		return editPanel;
	}

	public void updateFromControls() {
		name=tf_name.getText();
		min=BasicStr.parseDouble(tf_min.getText(),0,0);
		max=BasicStr.parseDouble(tf_max.getText(),9,9);
		decimals=BasicStr.parseInteger(ch_dec.getSelectedItem(),0,0);
		allowNegatives=cb_allowNegatives.getState();
		value=random(min,max,decimals,allowNegatives);
	}

	public String toString() {
		String sneg="no";
		if (allowNegatives) {
			sneg="si";
		}
		String smin=BasicStr.DoubleToString(min,decimals,false);
		String smax=BasicStr.DoubleToString(max,decimals,false);
		String s="("+name+","+smin+","+smax+","+decimals+","+sneg+")";
		return s;
	}

	public String toString(boolean forDescartesA) {
		if (forDescartesA) {
			String sneg="no";
			if (allowNegatives) {
				sneg="si";
			}
			String smin=BasicStr.DoubleToString(min,decimals,false);
			String smax=BasicStr.DoubleToString(max,decimals,false);
			String s="variable nombre="+name+" rango=["+smin+","+smax+"]";
			if (decimals>0) {
				s+=" decimales="+decimals;
			}
			if (allowNegatives) {
				s+=" negativos="+sneg;
			}
			return s;
		} else {
			return toString();
		}
	}

	public static RandomNumber[] parseRandomVariables(String content) {
		String[] sa=BasicStr.toStrArr(content);
		Vector V=new Vector();
		int n=0;
		while (n<sa.length) {
			if (sa[n].startsWith("variable ")) {
				String s=sa[n].substring(9);
				Attribute[] aa=Attribute.parse(s);
				String name=Attribute.getValue(aa,"nombre");
				String range=Attribute.getValue(aa,"rango");
				String smin="1";
				String smax="9";
				if (BasicStr.hasContent(range)) {
					StringTokenizer st=new StringTokenizer(range,"[,]");
					if (st.countTokens()==2) {
						smin=st.nextToken();
						smax=st.nextToken();
					}
				}
				String decimals=Attribute.getValue(aa,"decimales");
				String sneg=Attribute.getValue(aa,"negativos");
				V.addElement(new RandomNumber(name,smin,smax,decimals,sneg));
			}
			n++;
		}
		RandomNumber[] rna=new RandomNumber[V.size()];
		for (int i=0;i<rna.length;i++) {
			rna[i]=(RandomNumber) V.elementAt(i);
		}
		return rna;
	}

}