/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package com.mja.algebra;

import java.util.*;
import java.applet.*;
import java.io.*;
import com.mja.file.*;
import com.mja.lang.*;
import com.mja.parser.*;
import com.mja.util.*;
import com.mja.gui.*;
import org.unam.matem.Attribute;

public class SeriesConfig {

	public static final int auto=1,mouse=2,mousenkbd=3,kbd=4;

	private static translator Tr;
	private Applet A;
	Parser parser;
	int decimales=2;
	private boolean inWeb=true;
	private boolean guided=false;
	private int[] mode={mouse};
	private int bienConsecutivosParaAvance=3;
	private int notaDeAvance=75;
	private int notaDeRetroceso=50;
	private String instructions;
	ExerciseConfig[] exercise;
	private RandomNumber[] randomvar;
	private Vector randomVector;

	public static final int ix_guided=0,
							ix_mode=1,
							ix_equation=2,
							ix_okconsec=3,
							ix_perNext=4,
							ix_perPrev=5,
							ix_filename=6;

	public static final String[] attrName= {
		"guiado",
		"modos",
		"ecuacin",
		"bien_consecutivos",
		"nota_para_avanzar",
		"nota_para_retroceder",
		"ejercicios"
	};

	public static final String[][] attrOptions= {
		{"no","s"},
		{"automtico","clic o arrastre","clic y escribir","escribir"},
		{""},
		{"","1","2","3","4","5","6","7","8","9","10"},
		{"","100","95","90","85","80","75","70","65","60","55","50"},
		{"","60","50","40","30","20","10"},
		{""}
	};
	
	public static final int[] modeix={data.automatic,data.clickanddrag,data.clickandwrite,data.write};

	public static final String[] attrDefault= {
		"n",
		"clic o arrastre",
		"",
		"3",
		"75",
		"50",
		"_nuevo.txt"
	};
	public String[] attrValue;
	
	void updateAttribute(int i,String value) {
		attrValue[i]=value;
	}
	
	public int notaDeAvance() {
		return notaDeAvance;
	}

	public int notaDeRetroceso() {
		return notaDeRetroceso;
	}

	public int bienConsecutivosParaAvance() {
		return bienConsecutivosParaAvance;
	}

	public int bienParaAvance() {
		return (int) Math.round(notaDeAvance*countExercises()/100);
	}

	public int countExercisesOk() {
		int n=0;
		for (int i=0;i<countExercises();i++) {
			if (exercise[i].isDone() && exercise[i].getErrors()==0) {
				n++;
			}
		}
		return n;
	}

	public int countConsecutiveOks(int exer_num) {
		if (exercise[exer_num].getErrors()!=0) {
			return 0;
		}
		int n=0;
		for (int i=exer_num;i>=0;i--) {
			if (exercise[i].isDone()) {
				if (exercise[i].getErrors()!=0) {
					break;
				}
				n++;
			}
		}
		return n;
	}

	public double getAverage() {
		int n=0;
		double N=0;
		for (int i=0;i<exercise.length;i++) {
			if (exercise[i].isDone()) {
				n++;
				N+=exercise[i].getAverage();
			}
		}
		if (n==0) {
			n=1;
		}
		return N/n;
	}

	public void reset() {
		for (int i=0;i<exercise.length;i++) {
			exercise[i].reset();
		}
	}

	public String getIndicator() {
		return getFileName()+" "+getModos()+" "+getGuiado();
	}

	public String getFileName() {
		return attrValue[ix_filename];
	}

	public void setFileName(String newFileName) {
		attrValue[ix_filename]=newFileName;
	}

	public void setInstructions(String instr) {
		instructions=new String(instr);
	}
	
	public String getInstructions() {
		return instructions;
	}
	
	void resetContent() {
		applyContents(getContents());
		saveContents();
	}

	boolean saveContents() {
		if (!inWeb) {
			String path=BasicStr.DocPath(A);
			path=path+"ejercicios/"+attrValue[ix_filename];
			path=BasicStr.replace(path,"/",File.separator);
			boolean doit=false;
			File f=new File(path);
			if (f.exists()) {
	//			if (mjaText.ask("Cuidado!",5,40,"Al modificar este archivo\npueden verse afectadas otras series.\nDeseas continuar?","S","No")) {
					doit=true;
	//			}
			} else {
				if (mjaText.ask("Crear"+getFileName(),4,40,"Desea crear el archivo\n"+attrValue[ix_filename]+"?","S","No")) {
					doit=true;
				}
			}
			if (doit) {
//				mjaFile.save(path,getContents());
				mjaHtml.writeInUTF8File(getContents(),path);
				return true;
			} 
		}
		return false;
	}
	
	public SeriesConfig(DescartesA DA,Parser p,int decimales,boolean inWeb) {
		this.A=DA;
		this.Tr=DA.Tr;
		this.parser=p;
		this.decimales=decimales;
		this.inWeb=inWeb;
		attrValue=new String[attrName.length];
		for (int i=0;i<attrName.length;i++) {
			attrValue[i]=attrDefault[i];
		}
		applyConfig(true);
	}

	public SeriesConfig cloneSeriesConfig(DescartesA DA,Parser p,int decimales) {
		Tr=DA.Tr;
		SeriesConfig sc=new SeriesConfig(DA,p,decimales,inWeb);
		sc.parser=parser;
		sc.decimales=decimales;

		sc.attrValue[ix_filename]=new String(attrValue[ix_filename]);

		sc.mode=new int[mode.length];
		for (int i=0;i<mode.length;i++) {
			sc.mode[i]=mode[i];
		}
		sc.guided=guided;
		sc.bienConsecutivosParaAvance=bienConsecutivosParaAvance;
		sc.notaDeAvance=notaDeAvance;
		sc.notaDeRetroceso=notaDeRetroceso;

		sc.attrValue=new String[attrName.length];
		for (int i=0;i<attrName.length;i++) {
			sc.attrValue[i]=new String(attrValue[i]);
		}
		sc.applyContents(getContents());
		return sc;
	}

	public static String getModeName(int m) {
		if (1<=m && m<=attrOptions[ix_mode].length) {
			return attrOptions[ix_mode][m-1];
		}
		return "";
	}

	public SeriesConfig(Applet A,translator Tr,String info,String ser_cfg,Parser p,int decimales,boolean inWeb) {
		this.A=A;
		this.Tr=Tr;
		this.parser=p;
		this.decimales=decimales;
		this.inWeb=inWeb;
		attrValue=new String[attrName.length];
		for (int i=0;i<attrName.length;i++) {
			attrValue[i]=attrDefault[i];
		}
		Attribute[] aa=Attribute.parse(ser_cfg);
		attrValue=new String[attrName.length];
		for (int i=0;i<attrName.length;i++) {
			attrValue[i]=Attribute.getValue(aa,attrName[i]);
		}
		applyConfig(false);
		applyContents(info);
	}

	public SeriesConfig(Applet A,translator Tr,Parser p,String ser_cfg,int decimales,boolean inw) {
		this.A=A;
		this.Tr=Tr;
		this.decimales=decimales;
		this.parser=p;
		this.inWeb=inw;
		Attribute[] aa=Attribute.parse(ser_cfg);
		attrValue=new String[attrName.length];
		for (int i=0;i<attrName.length;i++) {
			attrValue[i]=Attribute.getValue(aa,attrName[i]);
		}
		applyConfig(true);
	}

	void applyConfig(boolean read) {
		if (read) {
			String contents="";		
			if (BasicStr.hasContent(attrValue[ix_filename])) {
				contents=mjaFile.readFile(A,inWeb,"ejercicios"+File.separator+getFileName(),true);
				contents=mjaHtml.decodeHTMLEncoding(contents);
			} else if (BasicStr.hasContent(attrValue[ix_equation])) {
				contents="Ejercicio\n\n"+attrValue[ix_equation]+"  |";
			}
			applyContents(contents);
		}
		parseModesString(attrValue[ix_mode]);
		guided=attrValue[ix_guided].toLowerCase().startsWith("s")
			||attrValue[ix_guided].toLowerCase().startsWith("y")
			||attrValue[ix_guided].toLowerCase().startsWith("t");
		bienConsecutivosParaAvance=BasicStr.parseInteger(attrValue[ix_okconsec],3,3);
		attrValue[ix_okconsec]=Integer.toString(bienConsecutivosParaAvance);
		notaDeAvance=BasicStr.parseInteger(attrValue[ix_perNext],75,75);
		attrValue[ix_perNext]=Integer.toString(notaDeAvance);
		notaDeRetroceso=BasicStr.parseInteger(attrValue[ix_perPrev],50,50);
		attrValue[ix_perPrev]=Integer.toString(notaDeRetroceso);
	}

	private void applyContents(String contents) {
		readInstructionsAndAttr(contents);
		setRandomVariables(contents,contents);
	}
	
	private void setRandomVariables(String rvs,String contents) {
		randomvar=RandomNumber.parseRandomVariables(rvs);
		randomVector=new Vector();
		for (int i=0;i<randomvar.length;i++) {
			randomVector.addElement(randomvar[i]);
		}
		randomizeExercises(contents);
	}

	public String getRandomVariablesStr() {
		String rvs="";
		if (randomvar.length>0) {
			for (int i=0;i<randomvar.length;i++) {
				rvs+=randomvar[i].toString(true)+"\n";
			}
			rvs+="\n";
		}
		return rvs;
	}
	
	void resetRandomVar(String name,double min,double max,int decimals,boolean negatives) {
		RandomNumber RN=new RandomNumber(name,min,max,decimals,negatives);
		if (isRandomVar(name)) {
			for (int i=0;i<randomvar.length;i++) {
				if (randomvar[i].getName().equals(name)) {
					randomvar[i]=RN;
					break;
				}
			}
		} else {
			setRandomVariables(getRandomVariablesStr()+RN.toString(true),getContents());
		}
		saveContents();
	}
	
	public void deleteRandomVar(String name) {
		if (isRandomVar(name)) {
			RandomNumber[] rna=randomvar;
			randomvar=new RandomNumber[rna.length-1];
			int n=0;
			for (int i=0;i<rna.length;i++) {
				if (!rna[i].getName().equals(name)) {
					randomvar[n++]=rna[i];
				}
			}
		}
		saveContents();
	}
	
	public boolean isRandomVar(String name) {
		RandomNumber R=getRandomVar(name);
		return R!=null;
	}
	
	public RandomNumber getRandomVar(String name) {
		for (int i=0;i<randomvar.length;i++) {
			if (randomvar[i].getName().equals(name)) {
				return randomvar[i];
			}
		}
		return null;
	}
	
	public RandomNumber[] getRandomVariables() {
		return randomvar;
	}
	
	public void randomizeExercises(String contents) {
		for (int i=0;i<randomvar.length;i++) {
			randomvar[i].recalculate();
		}
		exercise=ExerciseConfig.readExercises(randomVector,parser,contents,decimales);
	}

	public Vector getRandomVector() {
		return randomVector;
	}
	
	public void randomize() {
		randomizeExercises(getContents());
	}

	public static SeriesConfig[] readAllSeries(DescartesA DA,Parser p,int decimales,boolean inw) {
		Vector VS=new Vector();
		for (int i=0;i<100;i++) {
			String ser_cfg=DA.getParameter("SERIE_"+BasicStr.IntegerToString(i,2));
			if (BasicStr.hasContent(ser_cfg)) {
				try {
					VS.addElement(new SeriesConfig(DA,DA.Tr,p,ser_cfg,decimales,inw));
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
//		if (VS.size()==0) {
//			VS.addElement(new SeriesConfig(A,p,"guiado='s'",decimales,inw));
//			VS.addElement(new SeriesConfig(A,p,"guiado='n'",decimales,inw));
//		}
		SeriesConfig[] serie=new SeriesConfig[VS.size()];
		for (int i=0;i<serie.length;i++) {
			serie[i]=(SeriesConfig) VS.elementAt(i);
		}
		return serie;
	}

	public static SeriesConfig[] readAllSeries(Applet A,translator Tr,String ser_cfg,String info,Parser p,int decimales,boolean inw) {
		Vector VS=new Vector();
		try {
			VS.addElement(new SeriesConfig(A,Tr,info,ser_cfg,p,decimales,inw));
		} catch (Exception e) {
			e.printStackTrace();
		}
		SeriesConfig[] serie=new SeriesConfig[VS.size()];
		for (int i=0;i<serie.length;i++) {
			serie[i]=(SeriesConfig) VS.elementAt(i);
		}
		return serie;
	}

	public int countModes() {
		return mode.length;
	}

	public int getMode(int i) {
		if (i>=0 && i<mode.length) {
			return mode[i];
		}
		return auto;
	}

	public int countExercises() {
		return exercise.length;
	}

	public ExerciseConfig getExercise(int n) {
		if (0<=n && n<countExercises()) {
			return exercise[n];
		}
		return null;
	}

	public String[] getSExpressions(int n) {
		if (0<=n && n<countExercises()) {
			return exercise[n].getSExpressions();
		} else {
			return new String[0];
		}
	}

	public String getSExpression(int n,int m) {
		return getSExpressions(n)[m];
	}

	public String[] getExplanations(int n) {
		if (0<=n && n<countExercises()) {
			return exercise[n].getExplanations();
		} else {
			return new String[0];
		}
	}

	private void parseModesString(String modes) {
		Vector V=new Vector();
		if (BasicStr.hasContent(modes)) {
			for (int j=0;j<attrOptions[ix_mode].length;j++) {
				if (attrOptions[ix_mode][j].equals(modes) || Tr.equals(modes,modeix[j])) {
					V.addElement(new Integer(j+1));
					break;
				}
			}
			if (V.size()==0) {
				StringTokenizer st=new StringTokenizer(modes,",");
				while (st.hasMoreTokens()) {
					try {
						int n=Integer.parseInt(st.nextToken());
						if (n>=auto && n<=kbd) {
							V.addElement(new Integer(n));
						}
					} catch (NumberFormatException nfe) {
					}
				}
			}
		}
		if (V.size()==0) {
			V.addElement(new Integer(2));
		}
		mode=new int[V.size()];
		for (int i=0;i<V.size();i++) {
			mode[i]=((Integer) V.elementAt(i)).intValue();
		}
	}

	private void readInstructionsAndAttr(String content) {
		instructions="";
		String[] sa=BasicStr.toStrArr(content);
		int n=0;
		while (n<sa.length) {
			if (sa[n].length()==0||sa[n].startsWith("variable ")) {
				break;
			}
			instructions+=sa[n++];
		}
		while (n<sa.length && sa[n].length()==0) {
			n++;
		}
		if (n<sa.length && !sa[n].startsWith("variable ")) {
			while (n<sa.length) {
				boolean isAttr=false;
				for (int i=0;i<attrName.length;i++) {
					String value=Attribute.getValue(sa[n],attrName[i]);
					if (BasicStr.hasContent(value)) {
						attrValue[i]=value;
						isAttr=true;
					}
				}
				if (!isAttr) {
					break;
				}
				n++;
			}
		}
	}

	public boolean isGuided() {
		return guided;
	}

	public String getGuidedStr() {
		if (guided) {
			return "guiado";
		} else {
			return "libre";
		}
	}

	public String getModeAndGuidedStr() {
		return getModeName(mode[0])+","+getGuidedStr();
	}


	public String getModos() {
		String s="";
		for (int i=0;i<mode.length;i++) {
			if (i>0) {
				s+=",";
			}
			s+=mode[i];
		}
		return s;
	}

	public String getGuiado() {
		if (guided) {
			return "s";
		} else {
			return "n";
		}
	}

	public String getHeader() {
		String s="";
		for (int i=0;i<attrName.length;i++) {
			s+=attrName[i]+"='"+attrValue[i]+"' ";
		}
		return s;
	}

	public String getContents() {
		String s=instructions+"\n\n";
		s+=getRandomVariablesStr();
		for (int i=0;i<exercise.length;i++) {
			s+=exercise[i].toString()+"\n";
		}
		return s;
	}
	
}