/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package com.mja.algebra;


import java.util.*;
import com.mja.parser.*;
import com.mja.text.*;

public class StepConfig {

	private Parser p;
	private String oexpr,sexpr,expl;
	private Node ON,SN;
	private Vector rv;
	private int decimals;

	public StepConfig (Vector rv,Parser p,Node N,String expl,int decimales) {
		this.rv=rv;
		this.p=p;
		this.ON=N;
		this.oexpr=this.ON.toExpression(true,true);
		this.SN=Node_R.replaceRandomVariables(p,N,rv,decimales,true);
		this.sexpr=this.SN.toExpression(true);
		this.expl=expl;
		this.decimals=decimals;
	}

	public StepConfig (Vector rv,Parser p,String expr,String expl,int decimales) {
		this(rv,p,p.Analyse(parseSquareBrackets(expr,decimales)),expl,decimales);
	}

	public StepConfig cloneStep() {
		StepConfig sc=new StepConfig(rv,p,ON,expl,decimals);
		return sc;
	}

	private static String parseSquareBrackets(String txt,int decimales) {
		StringBuffer sb=new StringBuffer();
		int state=0;
		int i=0;
		while (i<txt.length()) {
			char ch=txt.charAt(i++);
			if (ch=='[') {
				sb.append("expr(");
				state++;
			} else if (ch==']') {
				if (state<=0) {
					(new Exception(txt.substring(0,i+1)+"\ntoo many ] ")).printStackTrace();
				}
				if (state>0) {
					sb.append(","+decimales+",0)");
				}
				state--;
			} else {
				sb.append(ch);
			}
		}
		return new String(sb);
	}

	public String toString() {
		String s=oexpr;
		while (s.length()<30) {
			s+=" ";
		}
		s+=" | "+expl;
		return s;
	}

	public String getOExpression(boolean allPar) {
		return ON.toExpression(allPar,true);
	}

	public String getSExpression(boolean allPar) {
		return SN.toExpression(allPar);
	}

	public String getOExpression() {
		return oexpr;
	}

	public String getSExpression() {
		return sexpr;
	}

	public String getExplanation() {
		return expl;
	}

	public Node getSNode() {
		return SN;
	}

	public void setExplanation(String expl) {
		this.expl=expl;
	}
}