/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package com.mja.algebra;


import java.awt.*;
import java.awt.event.*;
import com.mja.parser.*;
import com.mja.text.*;

class TreeCanvas extends Canvas implements MouseListener, Runnable {

	Thread thr;
	Node Ns;
	TreeNode DN;
	DescartesA DA;
	Image ima;
	int W,H;
	Graphics G;
	double t=1.0;

	TreeCanvas(DescartesA DA,boolean sensitive) {
		setBackground(new Color(0x2244ff));
		this.DA=DA;
		if (sensitive) {
			addMouseListener(this);
		}
	}

	void start() {
		W=getSize().width;
		H=getSize().height;
		ima=createImage(W,H);
		G=ima.getGraphics();
	}

	public void mousePressed(MouseEvent e) {
		if (e.getSource()==this) {
			Ns=DN.getNodeAt(e.getX(),e.getY());
			DA.FEC.unmarkSegment();
			if (Ns!=null) {
				Formula sF=Ns.F;
				if (sF!=null) {
					sF.markSegment(sF.begin(),sF.end());
					DA.FEC.updateScreen();
				}
				DA.setEditableFormula();
			}
			paint(getGraphics());
		}
	}
	public void mouseReleased(MouseEvent e) {
	}

	public void mouseClicked(MouseEvent e) {
		if (e.getSource()==this && Ns!=null) {
			if (e.getClickCount()==2) {
//				if (Ns.isBasicBinaryOperation() || DA.an_cfg.associateOnDoubleClick) {
//					Condicin eliminada el 9 de abril de 2009
					DA.performOperation(false,false,Ns);
//				}
			} else if (e.getClickCount()==1) {
				if (DA.an_cfg.associateOnClick) {
					ANode AN=new ANode(Ns,DA.an_cfg);
					if (!AN.isVar()) {
						DA.performOperation(true,false,Ns);
						DA.FEC.unmarkSegment();
						DA.FEC.updateScreen();
					}
				}
			}
		}
	}

	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}

	public void update(Graphics g) {
//		paint(g);
	}

	public void paint(Graphics g) {
//new Exception().printStackTrace();
		try {
			if (G==null) {
				start();
			}
			G.setColor(getBackground());
			G.fillRect(0,0,W,H);
			Dimension D=getSize();
			if (D!=null && D.width!=0) {
				DN=new TreeNode(G.getFontMetrics(),DA.tn_cfg,DA.FEC.getMainFormula().getNode(),D.width/2,20,(D.width-15)/4,35);
				DN.draw(G,Ns,t);
			}
			g.drawImage(ima,0,0,this);
		} catch (Exception e) {
		}
	}

	void animate() {
		thr=new Thread(this);
		thr.start();
	}
	
	private boolean paint_slowly=false;

	void paintSlowly() {
		paint_slowly=true;
		animate();
	}

	public void run () {
		try {
			if (paint_slowly) {
				if (G==null) {
					start();
				}
				Graphics g=ima.getGraphics();
				g.setColor(getBackground());
				g.fillRect(0,0,W,H);
				paint(g);
				int h=0;
				Graphics G=getGraphics();
				while (0<=h && h<=Math.max(DN.max_y,H/2)) {
					G.setColor(Color.white);
					G.drawLine(0,h,W,h);
					Thread.sleep(5);
					G.clipRect(0,h,W,1);
					G.drawImage(ima,0,0,this);
					h++;
					G=getGraphics();
				}
				G.setColor(getBackground());
				G.fillRect(0,h,W,H-h);
				paint_slowly=false;
			} else {
				long dT=100;
				t=1.0;
				Thread.sleep(2*dT);
				while (t>0) {
					Thread.sleep(dT);
					t=t-0.125;
					paint(getGraphics());
				}
				t=1.0;
				Thread.sleep(dT);
			}
		} catch (InterruptedException e) {
		}
		thr=null;
	}

}

class TreeNode {

	static Color nodeColor=new Color(0xff,0xff,0x22,0xc0);
	static Color lineColor=new Color(0xc0,0xc0,0xc0,0x80);

	Node N;
	String symbol;
	TreeNodeConfig tn_cfg;
	TreeNode L,R,parent;
	int x0,y0,lx,ly,sw,A,D,a,b;
	int sign;
	boolean isArtificialEndNode=false;
	int max_y;

	TreeNode (FontMetrics fm,TreeNodeConfig tn_cfg,Node N,int x0,int y0, int lx, int ly) {
		max_y=0;
		this.tn_cfg=tn_cfg;
		this.N=N;
		symbol=ANode.getPrettySymbol(N.getSymbol());
		this.x0=x0;
		this.y0=y0;
		this.lx=lx;
		this.ly=ly;
		tn_cfg.negativeNumbersAsNumbers=true;
		tn_cfg.fractionsAsNumbers=false;
		if (tn_cfg.fractionsAsNumbers && isIntegerFraction(N)) {
			isArtificialEndNode=true;
			symbol=N.getLeft().getSymbol()+"/"+N.getRight()[0].getSymbol();
		}
		if (tn_cfg.negativeNumbersAsNumbers && isNegativeNumber(N)) {
			isArtificialEndNode=true;
			symbol="-"+N.getRight()[0].getSymbol();
		}
		sw=fm.stringWidth(symbol);
		D=fm.getDescent();
		A=fm.getAscent();
		b=(A+D)/2+sw/16;
		a=Math.max(sw/2+sw/16+8,b);
		max_y=Math.max(max_y,y0+b+1);
		if (isArtificialEndNode) {
			return;
		}
		if (N.getLeft()!=null) {
			L=new TreeNode(fm,tn_cfg,(Node) N.getLeft(),x0-lx,y0+ly,lx/2,ly);
			max_y=Math.max(max_y,L.max_y);
			L.sign=-1;
			L.parent=this;
		}
		if (N.getRight()!=null && N.getRight().length>0 && N.getRight()[0]!=null) {
			R=new TreeNode(fm,tn_cfg,(Node) N.getRight()[0],x0+lx,y0+ly,lx/2,ly);
			max_y=Math.max(max_y,R.max_y);
			R.sign=1;
			R.parent=this;
		}
	}

	boolean isIntegerFraction(Node N) {
		return N.getSymb()==Node.DIV
			&& N.getLeft().isInteger()
			&& N.getLeft().isInteger();
	}

	boolean isNegativeNumber(Node N) {
		return N.getSymb()==Node.MENOS
			&& N.getType()==Node.Us
			&& N.getRight()[0].isPositiveNumber();
	}

	Node getNodeAt(int x,int y) {
		if ((x-x0)*(x-x0)/(a*a)+(y-y0)*(y-y0)/(b*b)<=1) {
			return N;
		} else {
			if (L!=null) {
				Node tn=L.getNodeAt(x,y);
				if (tn!=null) {
					return tn;
				}
			}
			if (R!=null) {
				Node tn=R.getNodeAt(x,y);
				if (tn!=null) {					
					return tn;
				}
			}
			return null;
		}
	}

	void draw(Graphics g,Node Ns,double t) {
		Color nC=nodeColor;
		Color tC=Color.black;
		double T=1;
		int X0=x0,Y0=y0;
		if (N==Ns||(Ns!=null && Ns.contains(N))) {
			nC=new Color(0xff,0x66,0x33,0xc0);
			tC=Color.black;
			T=t;
		}
		if (parent!=null&&N!=Ns) {
			X0=parent.x0+(int)Math.round(sign*T*parent.lx);
			Y0=parent.y0+(int)Math.round(T*parent.ly);
		}
		if (!isArtificialEndNode) {
			if (R!=null) {
				g.setColor(lineColor);
				g.drawLine(x0,y0,x0+(int)(lx*T),y0+(int)(ly*T));
			}
			if (L!=null) {
				g.setColor(lineColor);
				g.drawLine(x0,y0,x0-(int)(lx*T),y0+(int)(ly*T));
			}
			if (R!=null) {
				g.setColor(nC);
				R.draw(g,Ns,t*1);
			}
			if (L!=null) {
				g.setColor(nC);
				L.draw(g,Ns,t*1);
			}
		}
		g.setColor(nC);
		g.fillOval(X0-a,Y0-b,2*a+1,2*b+1);
		g.setColor(tC);
		g.drawString(symbol,X0-sw/2+1,Y0+(A-D)/2);
	}

}

class TreeNodeConfig {

	boolean fractionsAsNumbers=false;
	boolean negativeNumbersAsNumbers=false;

}