/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package com.mja.algebra;
/*
 * @(#)mjaLabel.java    1.0   2003/04/25
 *
 * This software is property of MJA
 *
 */
import java.awt.*;
import com.mja.cmp.mjaComponent;

public class mjaLabel extends mjaComponent {

	public static final int LEFT=0, CENTER=1, RIGHT=2;

	private Image img;
	private String text;
	private FontMetrics fm;
	private int align=LEFT;
	int sw;
	int extraSpace=8;

	public mjaLabel(Image img) {
		this("");
		this.img=img;
		if (img!=null) {
			setSize(img.getWidth(this),img.getHeight(this));
		}
	}

	public mjaLabel(String s) {
		this(s,CENTER);
	}

	public mjaLabel(String s,int align) {
		this(s,align,8);
	}

	public mjaLabel(String s,int align,int xs) {
		text=s;
		setAlignment(align);
		extraSpace=xs;
		setFont(getFont());
	}

	public void setAlignment(int align) {
		if (align>=LEFT && align<=RIGHT) {
			this.align=align;
		}
	}

	private void updateSize() {
		if (img==null) {
			setSize(2*extraSpace+fm.stringWidth(text),4+fm.getMaxAscent()+fm.getMaxDescent());
		}
	}

	public Dimension getPreferedSize() {
		if (img!=null) {
			return new Dimension(img.getWidth(this),img.getHeight(this));
		} else {
			return new Dimension(2*extraSpace+fm.stringWidth(text),4+fm.getMaxAscent()+fm.getMaxDescent());
		}
	}

	public void setFont (Font f) {
		if (img==null) {
			super.setFont(f);
			fm=getFontMetrics(getFont());
			sw=fm.stringWidth(text);
			updateSize();
		}
	}

	public void setText(String text) {
		this.text=text;
		sw=fm.stringWidth(text);
		paint(getGraphics());
	}

	public String getText() {
		return text;
	}

	public void setImage(Image img) {
		this.img=img;
		paint(getGraphics());
	}

	public Image getImage() {
		return img;
	}

	public void paint(Graphics g) {
		super.paint(g);
		if (g!=null) {
			if (img!=null) {
				int W=getSize().width;
				int H=getSize().height;
				int w=img.getWidth(this);
				int h=img.getHeight(this);
				int wn=W;
				int hn=(W*h)/w;
				if (h>w) {
					hn=H;
					wn=(H*w)/h;
				}
				int x0=(W-wn)/2;
				int y0=(H-hn)/2;
				g.drawImage(img,x0,y0,wn,hn,this);
			} else {
				g.setFont(getFont());
				int y=(getSize().height-fm.getMaxAscent()-fm.getMaxDescent())/2+fm.getMaxAscent();
				int x=extraSpace;
				if (align==RIGHT) {
					x=getSize().width-sw-extraSpace;
				} else if (align==CENTER) {
					x=(getSize().width-sw)/2;
				}
				g.setColor(getForeground());
				g.drawString(text,x,y);
			}
		}
	}

}