/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package com.mja.cmpedit;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import com.mja.util.BasicStr;
import com.mja.lang.*;
import com.jla.calc.*;
import com.mja.file.mjaFont;

public class mjaLineEditor extends Canvas implements Runnable, KeyListener,
								MouseListener, MouseMotionListener, FocusListener {

	private int align=data.a_center;
	private int adjust=data.wrap;

	private static String clipboardStr="";
	private Thread thr;
	private volatile boolean isRunning, pleaseStop, hasTheFocus, mustupdate, mustmove, editable;
	private int width, height, points;
	private Font font;
	private FontMetrics fm;
	private int X0=0,Y0=0;
	private int xl=4,yl=2,a,d,caret_pos=0,decimales=-1;
	private String string="";
	private int dx=0;
	private int selection_0, selection_1;
	private volatile boolean caretOn=false;
	private boolean fijo=false;
	private int calcalfanum=1;

	public mjaLineEditor(int width,int height,int points,boolean editable,int decimales,boolean fijo,int calcalfanum) {
		this.width=width;
		this.height=height;
		setFont("Serif",Font.PLAIN,points);
		setEditable(editable);
		this.decimales=decimales;
		this.fijo=fijo;
		this.calcalfanum=calcalfanum;
	}
	
	public void setDecimals(int decimales,boolean fijo) {
		this.decimales=decimales;
		this.fijo=fijo;
	}

	public void setInitialPoint(int x0,int y0) {
		this.X0=x0;
		this.Y0=y0;
	}

	private Vector listeners=new Vector();

	public void addActionListener(ActionListener c) {
		listeners.addElement(c);
	}

	public void removeActionListener(ActionListener c) {
		listeners.removeElement(c);
	}

	private void callListeners(String arg) {
		Enumeration e=listeners.elements();
		while (e.hasMoreElements()) {
			ActionListener al=(ActionListener) e.nextElement();
			al.actionPerformed(new ActionEvent(this,ActionEvent.ACTION_PERFORMED,arg));
		}
	}

	public String getText() {
		return string;
	}

	public void setText(String s) {
		string=s;
		if (decimales>=0) {
			try {
				double d=Double.parseDouble(s);
				string=BasicStr.DoubleToString(d,decimales, fijo,".");
			} catch (Exception e) {
			}
		}
		caret_pos=0;
		dx=0;
		selection_0=0;
		selection_1=0;
		paintAll(getGraphics());
	}

	public void setEditable(boolean toEdit) {
		if (editable) {
			removeKeyListener(this);
			removeFocusListener(this);
			removeMouseListener(this);
			removeMouseMotionListener(this);
		}
		editable=toEdit;
		if (editable) {
			addKeyListener(this);
			addFocusListener(this);
			addMouseListener(this);
			addMouseMotionListener(this);
		} else {
			hasTheFocus=false;
		}
	}
	
	public boolean isEditable() {
		return editable;
	}

	public void setFont(String fname,int fstyle, int points) {
		this.points=points;
		font=mjaFont.makeFont(fname,fstyle,points);
		fm=getFontMetrics(font);
		a=fm.getMaxAscent();
		d=fm.getMaxDescent();
	}

	public void setFont(Font font) {
		setFont(font.getName(),font.getStyle(),font.getSize());
	}

	public void setAlignment(int align) {
		this.align=align;
	}

	public void setAdjust(int adjust) {
		this.adjust=adjust;
	}

	private void start() {
		stop();
		thr=new Thread(this);
		thr.start();
	}

	private boolean pleaseStop() {
		return pleaseStop;
	}

	private boolean isRunning() {
		return isRunning;
	}

	private void stop() {
		pleaseStop=true;
		if (thr!=null) {
			try {
				while (isRunning()) {
					Thread.sleep(10);
				}
			} catch (InterruptedException e) {
			}
		}
	}

	private boolean inZone(int x,int y) {
		return (0<=x && 0<=y && x<width && y<height);
	}

	private boolean mustUpdate() {
		return mustupdate;
	}

	private boolean mustMove() {
		return mustmove;
	}

	private void setTheFocus(boolean b) {
		mustupdate=true;
		hasTheFocus=b;
		if (hasTheFocus) {
			start();
		} else {
			stop();
		}
	}

	boolean hasTheFocus() {
		return hasTheFocus;
	}

	void deleteSelection() {
		if (selection_0!=selection_1) {
			string=string.substring(0,selection_m())+string.substring(selection_M());
			caret_pos=selection_m();
			selection_0=caret_pos;
			selection_1=caret_pos;
			mustupdate=true;
		}
	}

	void moveCaret(int n,boolean shiftOn) {
		if (n!=0) {
			mustmove=true;
		}
		caret_pos+=n;
		if (caret_pos<0) {
			caret_pos=0;
		}
		if (caret_pos>string.length()) {
			caret_pos=string.length();
		}

		while (2*xl+dx+fm.stringWidth(string.substring(firstChar(),caret_pos))>=width) {
			dx--;
			mustupdate=true;
		}

		while (dx+caret_x()<0) {
			dx++;
			mustupdate=true;
		}
		if (shiftOn) {
			selection_1=caret_pos;
			mustupdate=true;
		} else {
			mustupdate|=selection_0!=selection_1;
			selection_0=caret_pos;
			selection_1=caret_pos;
		}
	}

	int selection_m() {
		return Math.min(selection_0,selection_1);
	}

	int selection_M() {
		return Math.max(selection_0,selection_1);
	}

/* --------------------- Line and Cursor Location ---------------- */

	int getCaretPos(int px,int py,boolean ShiftOn) {
		mustmove=true;
		int l=0;
		int L=string.length();
		caret_pos=0;
		String [] sa=wrap();
		while (l+1<sa.length && y(l)+d<py) {
			caret_pos+=sa[l++].length();
		}
		L=caret_pos+sa[l].length();
		while (caret_pos+1<L || (caret_pos<L)&&(L==string.length()) ) {
			char ch=string.charAt(caret_pos);
			if (x(l)+caret_x()+fm.charWidth(ch)/2<px) {
				caret_pos++;
			} else {
				break;
			}
		}
		selection_1=caret_pos;
		if (ShiftOn) {
			mustupdate=true;
		} else {
			selection_0=caret_pos;
		}
		return caret_pos;
	}

	private int firstChar() {
		String[] sa=wrap();
		int pos=0;
		int l=0;
		while (l+1<sa.length && pos+sa[l].length()<=caret_pos) {
			pos+=sa[l++].length();
		}
		return pos;
	}

	private int lastChar() {
		String[] sa=wrap();
		int pos=0;
		int l=0;
		while (l+1<sa.length && pos+sa[l].length()<=caret_pos) {
			pos+=sa[l++].length();
		}
		pos+=sa[l].length();
		if (l+1<sa.length) {
			pos--;
		}
		return pos;
	}

	private int lastCharOf(int line) {
		String[] sa=wrap();
		int pos=0;
		int l=0;
		while (l+1<sa.length && l+1<line) {
			pos+=sa[l++].length();
		}
		pos+=sa[l].length();
		if (l+1<sa.length) {
			pos--;
		}
		return pos;
	}

	private int firstCharOf(int line) {
		String[] sa=wrap();
		int pos=0;
		int l=0;
		while (l+1<sa.length && l+1<line) {
			pos+=sa[l++].length();
		}
		return pos;
	}

	private int lineWidth(int l) {
		String[] sa=wrap();
		return fm.stringWidth(sa[l]);
	}

	private int x(int l) {
		String[] sa=wrap();
		switch (align) {
			case data.a_left 	: return xl+dx;
			case data.a_center	: return (width-fm.stringWidth(sa[l]))/2;
			case data.a_right 	: return width-xl-fm.stringWidth(sa[l]);
			default		: return xl+dx;
		}
	}

	private int xAt(int pos) {
		int l=lnAt(pos);
		return x(l)+caret_x(pos);
	}

	private int y(int l) {
		switch (align) {
			case data.a_left 	:
			case data.a_center	:
			case data.a_right 	: return yl+a+(a+d)*l;
			default		: return a+yl;
		}
	}

	int ln() {
		return lnAt(caret_pos);
	}

	int NL() {
		return wrap().length;
	}

	int lnAt(int pos) {
		int l=0;
		String[] sa=wrap();
		int sz=0;
		while(l+1<sa.length && pos>=sz+sa[l].length()) {
			sz+=sa[l++].length();
		}
		return l;
	}

	int caret_x() {
		return caret_x(caret_pos);
	}

	int caret_x(int pos) {
		int sz=0;
		String[] sa=wrap();
		int L=lnAt(pos);
		int l=0;
		while (l<L) {
			sz+=sa[l++].length();
		}
		return fm.stringWidth(string.substring(sz,pos));
	}

/* --------------------- Wrap ------------------ */

	String[] wrap() {
		Vector V=new Vector();
		int N=0;
		int n=0;
		int nl=0;
		while (n<string.length()) {
			int lastSpace=0;
			if (adjust==data.wrap) {
				while (n<string.length() && 2*xl+fm.stringWidth(string.substring(N,n))<width )  {
					if (string.charAt(n)==' '||string.charAt(n)=='\n') {
						lastSpace=n;
					}
					if (string.charAt(n)=='\n') {
						n++;
						break;
					}
					n++;
				}
			} else {
				while (n<string.length())  {
					if (string.charAt(n)=='\n') {
						lastSpace=n;
						n++;
						break;
					}
					n++;
				}
			}
			int M=n;
			if (adjust==data.wrap && n<string.length() && lastSpace>0) {
				M=lastSpace+1;
			}
			V.addElement(string.substring(N,M));
			nl++;
			N=M;
			n=N;
		}
		if (nl==0 || (string.length()>0&&string.charAt(string.length()-1)=='\n')) {
			nl++;
			V.addElement("");
		}
		String[] sa=new String[nl];
		for (int i=0;i<sa.length;i++) {
			sa[i]=(String) V.elementAt(i);
		}
		return sa;
	}

/* ----------------------- Run and paint -------------------- */

	public void run() {
		mustupdate=false;
		pleaseStop=false;
		isRunning=true;
		Graphics g=getGraphics();
//		((Graphics2D)g).setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		try {
			while (!pleaseStop()) {
				if (hasTheFocus()&&(selection_0==selection_1)&&isEditable()) {
					Color bg=getBackground();
					Color fg=getForeground();
					int l=ln();
					int xC=x(l)+caret_x()-1,yC=y(l),aC=a,dC=d;
					g.setXORMode(bg);
					g.setColor(fg);
					caretOn=true;
					g.fillRect(xC,yC-aC,2,aC+dC);
					Thread.sleep(10);
					int n=0;
					while (!pleaseStop()&&n++<30&& hasTheFocus()&&!mustUpdate()&&!mustMove()) {
						Thread.sleep(10);
					}
					if (caretOn()) {
						g.setXORMode(bg);
						g.setColor(fg);
						g.fillRect(xC,yC-aC,2,aC+dC);
					}
					caretOn=false;
				}
				if (mustUpdate()) {
					paint(g);
					mustupdate=false;
				}
				int n=0;
				while (!pleaseStop()&&n++<20&&!mustMove()&&!mustUpdate()) {
					Thread.sleep(10);
				}
				mustmove=false;
			}
		} catch (InterruptedException e) {
		}
		paint(g);
		thr=null;
		isRunning=false;
	}

	private boolean caretOn() {
		return caretOn;
	}

	public synchronized void paint(Graphics g) {
		super.paint(g);
//		((Graphics2D)g).setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		caretOn=false;
		if (g!=null) {
			g.setColor(getForeground());
			g.setPaintMode();
			g.setFont(font);
			String[] sa=wrap();
			for (int i=0;i<sa.length;i++) {
				g.drawString(sa[i],X0+x(i),Y0+y(i));
			}
			if (selection_0!=selection_1) {
				int xm=xAt(selection_m());
				int lm=lnAt(selection_m());
				int xM=xAt(selection_M());
				int lM=lnAt(selection_M());
				g.setXORMode(getBackground());
				if (lm==lM) {
					g.fillRect(xm,y(lm)-a,xM-xm,a+d);
				} else {
					g.fillRect(xm,y(lm)-a,xAt(lastCharOf(lm))-xm,a+d);
					lm++;
					while (lm<lM) {
						xm=x(lm);
						g.fillRect(xm,y(lm)-a,lineWidth(lm),a+d);
						lm++;
					}
					xm=x(lM);
					g.fillRect(xm,y(lM)-a,xM-xm,a+d);
				}
				g.setPaintMode();
			}
		}
	}

/* -----------------Key Listener ------------------ */

	public void keyPressed(KeyEvent e){
		boolean ShiftOn=e.isShiftDown();
		boolean CtrlOn =e.isControlDown();
		boolean AltOn  =e.isAltDown();
		switch (e.getKeyCode()) {
			case KeyEvent.VK_LEFT:
					if (CtrlOn||caret_pos==0) {
						callListeners("prev");
					} else {
						moveCaret(-1,ShiftOn);
						mustmove=true;
					}
					break;
			case KeyEvent.VK_RIGHT:
					if (CtrlOn||caret_pos==string.length()) {
						callListeners("next");
					} else {
						moveCaret(1,ShiftOn);
						mustmove=true;
					}
					break;
			case KeyEvent.VK_UP:
					if (CtrlOn||ln()==0) {
						callListeners("prevLine");
					} else {
						getCaretPos(x(ln())+caret_x(),y(ln())-a-d,ShiftOn);
						mustmove=true;
					}
					break;
			case KeyEvent.VK_DOWN:
					if (CtrlOn||ln()==NL()-1) {
						callListeners("nextLine");
					} else {
						getCaretPos(x(ln())+caret_x(),y(ln())+a+d,ShiftOn);
						mustmove=true;
					}
					break;
			case KeyEvent.VK_HOME:
					if (CtrlOn) {
						moveCaret(-string.length(),ShiftOn);
					} else {
						moveCaret(firstChar()-caret_pos,ShiftOn);
					}
					mustmove=true;
					break;
			case KeyEvent.VK_END:
					if (CtrlOn) {
						moveCaret(string.length(),ShiftOn);
					} else {
						moveCaret(lastChar()-caret_pos,ShiftOn);
					}
					mustmove=true;
					break;
			case KeyEvent.VK_C:
					if (CtrlOn) {
						clipboardStr=string.substring(selection_m(),selection_M());
					}
					break;
			case KeyEvent.VK_X:
					if (CtrlOn) {
						clipboardStr=string.substring(selection_m(),selection_M());
						deleteSelection();
					}
					break;
			case KeyEvent.VK_V:
					if (CtrlOn) {
						string=string.substring(0,caret_pos)
								+clipboardStr
								+string.substring(caret_pos);
						moveCaret(clipboardStr.length(),false);
						mustupdate=true;
					}
					break;
//			case KeyEvent.VK_TAB: // nunca pasa por aqu.
//					callListeners("next");
		}
		paintAll(getGraphics());
	}

	public void keyReleased(KeyEvent e){
	}

	public void keyTyped(KeyEvent e){
		boolean ShiftOn=e.isShiftDown();
		int code=e.getKeyCode();
		if (code==KeyEvent.VK_TAB) {
			if (ShiftOn) {
				callListeners("prev");
			} else {
				callListeners("next");
			}
			return;
		}
		boolean CtrlOn =e.isControlDown();
		boolean AltOn  =e.isAltDown();
		if (!CtrlOn) {
			char ch=e.getKeyChar();
			switch (code) {
				case KeyEvent.VK_LEFT:
				case KeyEvent.VK_RIGHT:
				case KeyEvent.VK_HOME:
				case KeyEvent.VK_END:
				case KeyEvent.VK_UP:
				case KeyEvent.VK_DOWN:
									return;
			}
			boolean selectionExists=(selection_0!=selection_1);
			deleteSelection();
			if (ch==KeyEvent.VK_DELETE) {
				if (caret_pos<string.length() && !selectionExists) {
					string=string.substring(0,caret_pos)+string.substring(caret_pos+1);
					moveCaret(0,false);
					mustupdate=true;
				}
			} else if (ch==KeyEvent.VK_BACK_SPACE) {
				if (caret_pos>0 && !selectionExists) {
					string=string.substring(0,caret_pos-1)+string.substring(caret_pos);
					moveCaret(-1,false);
					mustupdate=true;
				}
			} else {
				if (ch==KeyEvent.VK_ENTER) {
					ch='\n';
				}
				if (caret_pos==string.length()) {
					string+=ch;
				} else {
					string=string.substring(0,caret_pos)+ch+string.substring(caret_pos);
				}
				moveCaret(1,false);
				mustupdate=true;
			}
		}
		paintAll(getGraphics());
	}

	/* -------------- Mouse Listener ------------- */

	private int x0,x1,l0,l1,mouse_x,mouse_y;
	private boolean moving;

	public void mousePressed(MouseEvent e) {
		mouse_x=e.getX();
		mouse_y=e.getY();
		if (selection_0!=selection_1) {
			//start moving
			moving=true;
		} else {
			x0=xAt(getCaretPos(e.getX(),e.getY(),false));
			l0=ln();
			x1=x0;
			l1=l0;
			selection_0=caret_pos;
			selection_1=caret_pos;
			mustupdate=true;
			paintAll(getGraphics());
			moving=false;
		}
	}

	public void mouseReleased(MouseEvent e) {
		if (moving) {
			if (mouse_x!=e.getX()||mouse_y!=e.getY()) {
				if (inZone(e.getX(),e.getY())) {
					int s_m=selection_m();
					int s_M=selection_M();
					clipboardStr=string.substring(s_m,s_M);
					int c_p=getCaretPos(e.getX(),e.getY(),false);
					selection_0=s_m;
					selection_1=s_M;
					deleteSelection();
					if (c_p<=s_m) {
						caret_pos=c_p;
					} else if (s_m<c_p && c_p<s_M) {
						caret_pos=s_m;
					} else if (s_M<=c_p) {
						caret_pos=s_m+(c_p-s_M);
					}
					string=string.substring(0,caret_pos)
							+clipboardStr
							+string.substring(caret_pos);
					moveCaret(clipboardStr.length(),false);
				}
			}
			selection_0=caret_pos;
			selection_1=caret_pos;
			mustupdate=true;
		}
		moving=false;
	}
	public void mouseClicked(MouseEvent e) {
		if ((e.getSource()==this) &&(e.getClickCount()==2)) {
			if (calcalfanum>0) {
				calcWin cw=new calcWin(true,calcalfanum==2);
				Point P=getLocationOnScreen();
				cw.setLocation(P.x-(cw.getSize().width-getSize().width)/2,Math.max(0,P.y-cw.getSize().height));
				cw.show();
				if (cw.getValue()!=null) {
					setText(cw.getValue());
				}
				cw.dispose();
				paint(getGraphics());
			}
		}
	}
	public void mouseEntered(MouseEvent e) {
	}
	public void mouseExited(MouseEvent e) {
	}
	public void mouseMoved(MouseEvent e) {
	}
	public void mouseDragged(MouseEvent e) {
		if (!moving) {
			int x1a=x1;
			int l1a=l1;
			x1=xAt(getCaretPos(e.getX(),e.getY(),true));
			l1=ln();
			if (x1a!=x1 || l1a!=l1) {
				selection_1=caret_pos;
				mustupdate=true;
			}
		}
	}

	/* -------------- Focus Listener ------------- */

	public void focusGained(FocusEvent fe) {
		setTheFocus(true);
	}

	public void focusLost(FocusEvent fe) {
		setTheFocus(false);
	}
}