package com.mja.descartes;

/**
 * Espacio para insertar un espacio con un applet
 */
 
import java.applet.Applet;
import java.applet.AppletContext;
import java.applet.AppletStub;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Point;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FilePermission;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.net.URLConnection;
import java.security.Permission;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipInputStream;

import org.unam.matem.Communicator;

import com.mja.file.mjaFile;
import com.mja.file.mjaHtml;
import com.mja.math.R2;
import com.mja.util.BasicStr;
import com.mja.parser.*;

public class SpaceAP extends Space {
	
	private Node fileNode;
	private String presentFile="";
	private Applet A;
	private Communicator comm;
	private Hashtable<?, ?> ap;

	public SpaceAP(Descartes D, spaceConfig sc) {
		super(D, sc);
		setLayout(null);
		String fileName=sc.getFileName();
		if (fileName.startsWith("[") && fileName.endsWith("]")) {
			String fileNameExpr=fileName.substring(1,fileName.length()-1);
			fileNode=D.p.Analyse(fileNameExpr);
			try {
				fileNode.Evaluate();
				fileName=fileNode.getStr();
			} catch (Exception e) {
				e.printStackTrace();
			}
		} else {
			fileNode=new Node(D.p,fileName);
		}
		initSpaceAP(fileName);
	}
	
	public String toTeX(boolean grayscale) {
		return "";
	} 

	private void initSpaceAP(String fileName) {
		presentFile=fileName;
		if (BasicStr.hasContent(fileName)) {
			A=null;
			try {
				String str=mjaFile.readFile(D,D.inWeb(),fileName,true);
				Point[] P=mjaHtml.getAppletCodeLimits(str);
				if (P.length>0) {
					String appletCode=str.substring(P[0].x,P[0].y);
					appletCode=mjaHtml.decodeHTMLEncoding(appletCode);
					ap=mjaHtml.getAppletParams(appletCode);
					String str_codebase=mjaHtml.getAppletAttribute(appletCode,"codebase");
					String str_archive=mjaHtml.getAppletAttribute(appletCode,"archive");
					String str_code=mjaHtml.getAppletAttribute(appletCode,"code",true);
					if (str_code.endsWith(".class")) {
						str_code=str_code.substring(0,str_code.length()-".class".length());
					}
					A=loadApplet(null,str_code);
					started=false;
					if (A==null) {
						StringTokenizer st=new StringTokenizer(str_archive,",");
						while (st.hasMoreTokens()) {
							String jarfilename=st.nextToken();
							A=loadApplet(jarfilename,str_code);
							if (A!=null) {
								break;
							}
						}
					}
					if (A!=null) {
						A.setStub(new DStub(D,A,str_codebase,fileName,ap));
						A.init();
						if (A instanceof Communicator) {
							comm=(Communicator) A;
							comm.addCommunicator(D);
							D.addCommunicator(comm);
							Communicator[] CA=D.getRegistered();
							for (int i=0;i<CA.length;i++) {
								CA[i].addCommunicator(comm);
								comm.addCommunicator(CA[i]);
							}
						} else {
							comm=null;
						}
					} else {
//						System.out.println("class: "+str_code+", not found");
					}
				} else {
//					System.out.println("No applet found in "+fileName);
				}
			} catch(MalformedURLException e1) {
				e1.printStackTrace();
			} catch(Exception e2) {
				e2.printStackTrace();
			}
		} else {
			A=null;
		}
	}
	
	private Applet loadApplet(String str_archive,String str_code) {
		try {
			ClassLoader classLoader;
			if(BasicStr.hasContent(str_archive)){
				String docBase = D.getDocumentBase().toExternalForm();
				String codeBase = D.getCodeBase().toExternalForm();
				
				String[] bases=new String[]{codeBase,docBase,""};
				URL u = null;
				for (int i = 0; i < bases.length; i++) {
					String uri = bases[i];
					int ix=uri.lastIndexOf("/");
					if (ix>=0) {
						uri=uri.substring(0,ix);
					}
					uri=uri+"/"+str_archive;
					if (uri.indexOf("file://")>=0) {
						uri=uri.replaceFirst("file://", "file:/");
					}
					
					u = new URL(uri);
					try{
						u.getContent();
						break;
					}catch (IOException e) {
						u = null;
					}
				}
				if(u == null){
//					System.out.println("No se pudo cargar el jar "+str_archive);
					return null;
				}
					
//				System.out.println("Leyendo desde:"+u.toExternalForm()+"\nla clase :"+str_code);
				SecurityManager ant = System.getSecurityManager();
				if(ant != null && !MySecurityManager.class.isInstance(ant)){
					System.setSecurityManager(new MySecurityManager(ant));
				}
				String uri = u.toExternalForm();
				int ix=uri.lastIndexOf("/");
				if (ix>=0) {
					uri=uri.substring(0,ix+1);
				}
				URL u2 = new URL(uri);
				URL[] urls = new URL[]{u2,u,D.getDocumentBase(),D.getCodeBase()};
				URLClassLoader urlClassLoader = new URLClassLoader(urls, ClassLoader.getSystemClassLoader());
				urlClassLoader.setDefaultAssertionStatus(true);
		
				if(System.getSecurityManager() instanceof MySecurityManager && System.getSecurityManager()!= null){
					MySecurityManager actP = (MySecurityManager)System.getSecurityManager();
					Vector<Permission> perm = new Vector<Permission>();
					
					if(D.inWeb()){
						String uriCodeBase = u2.toExternalForm();
						uriCodeBase = uriCodeBase.replace("file:/", File.separator);
						uriCodeBase = uriCodeBase.replace("/", File.separator);
						perm.add(new FilePermission(u2.toExternalForm()+"-","read"));
						perm.add(new FilePermission(uriCodeBase+"-","read"));
					}
					actP.addPermissions(perm.toArray(new Permission[perm.size()]));
				}
				classLoader = urlClassLoader;
			} else {
				URL[] urls = new URL[]{D.getDocumentBase(),D.getCodeBase()};
				URLClassLoader urlClassLoader = new URLClassLoader(urls, ClassLoader.getSystemClassLoader());
				classLoader = urlClassLoader;
			}
			Class<?> classLoaded = classLoader.loadClass(str_code);
			return (Applet) classLoaded.newInstance();
		}catch (Exception e) {
//			if(BasicStr.hasContent(str_archive)) {
//				e.printStackTrace();
//			}
		}
		return null;
		
		
		/* CAMBIO DE OSCAR
		try {
			Class cl;
			if (BasicStr.hasContent(str_archive)) {
				try {
					ZipClassLoader ZCL=new ZipClassLoader(null,str_archive);
					cl=ZCL.findClass(str_code);
				} catch (Exception e1) {
					ZipClassLoader ZCL=new ZipClassLoader(D,str_archive);
					cl=ZCL.findClass(str_code);
				}
			} else {
				ClassLoader SCL=ClassLoader.getSystemClassLoader();
				cl=Class.forName(str_code,false,SCL);
			}
			Constructor[] ca=cl.getConstructors();
			if (ca.length>0) {
				return (Applet) ca[0].newInstance();
			}
		} catch (Exception ex) {
		}
		return null;
		*/
	}
	
	boolean started=false;
	
	private void checkApplet() {
		try {
			fileNode.Evaluate();
			if ((A==null) || (!presentFile.equals(fileNode.getStr()))) {
				started=false;
				if (A!=null) {
					A.stop();
					remove(A);
//					A.destroy();
					A=null;
				}
				initSpaceAP(fileNode.getStr());
			}
		} catch(Exception e) {
		}
	}
		
	private void startApplet() {
		if (A!=null) {
			A.setBounds(0,0,getSize().width,getSize().height);
			add(A);
			A.paintAll(A.getGraphics());
			A.start();
			if (A instanceof Descartes) {
				((Descartes) A).restart(true);
			}
			started=true;
		}
	}

/* ----------------- Space Implementation --------------------- */

	@Override
	public void start() {
		super.start();
		if (A!=null && isVisible()) {
			startApplet();
		}
	}

	@Override
	void doBack(boolean clear) {
		checkApplet();
		setVisible(visibilityN.Evaluate(1)>0);
		if (A!=null && isVisible()) {
			if (!started) {
				super.start();
				startApplet();
			}
		}
		super.doBack(clear);
	}
	
	@Override
	protected void doFront(boolean calc) {
		checkApplet();
		if (A!=null && isVisible()) {
			if (!started) {
				super.start();
				startApplet();
			}
			if (back!=null && gmain!=null) {
				gmain.drawImage(back,0,0,this);
			}
		}
		super.doFront(calc);
		if (gmain!=null) {
			gmain.setColor(Color.cyan);
			gmain.fillRect(0,0,2048,2048);
			gmain.setColor(Color.blue);
			gmain.setFont(new Font("SansSerif",Font.BOLD,16));
			gmain.drawString("Descartes ",10,20);
			gmain.setColor(Color.darkGray);
			gmain.setFont(new Font("SansSerif",Font.PLAIN,14));
			gmain.drawString("SpaceAP source: "+presentFile,10,40);
		}
	}

	@Override
	protected void stopMoving() {
		// TODO Auto-generated method stub

	}

	@Override
	protected void flushImages() {
		// TODO Auto-generated method stub

	}

	@Override
	protected boolean isMoving() {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	protected void doCalc() {
		// TODO Auto-generated method stub
	}
	

	@Override
	protected boolean onDrag(boolean right, int dx, int dy) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	protected void writeCoordinates(Graphics g, int x, int y) {
		// TODO Auto-generated method stub

	}

	@Override
	protected R2 injection(double x, double y) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	protected R2 projection(R2 p) {
		// TODO Auto-generated method stub
		return null;
	}
	
	
}

class DStub implements AppletStub {
	
	Applet D;
	Applet A;
	Hashtable<?, ?> ap;
	URL codebase,docbase;
	
	DStub(Applet D,Applet A,String str_cb,String filename,Hashtable<?, ?> ap) {
		this.D=D;
		this.A=A;
		this.ap=ap;

		String str_docbase=D.getDocumentBase().toExternalForm();
		if (filename.startsWith("http:") || filename.startsWith("file:")) {
			str_docbase=filename;
		} else if (str_docbase.startsWith("http:") || filename.startsWith("file:")) {
			str_docbase=str_docbase+filename;
		} else {
			str_docbase=D.getDocumentBase()+filename;
		}
		int ix=str_docbase.lastIndexOf("/");
		if (ix>=0) {
			str_docbase=str_docbase.substring(0,ix+1);
		} else {
			ix=str_docbase.lastIndexOf(":");
			if (ix>=0) {
				str_docbase=str_docbase.substring(0,ix+1);
			}
		}
		try {
			docbase = new URL(str_docbase);
		} catch (Exception e) {
			docbase=D.getDocumentBase();
		}

		String str_codebase="";
		if (str_cb.startsWith("http:") || str_cb.startsWith("file:")) {
			str_codebase=str_cb;
		} else if (docbase.toExternalForm().startsWith("http:") || docbase.toExternalForm().startsWith("file:")) {
			str_codebase=docbase.toExternalForm()+str_cb;
		} else {
			str_codebase=D.getCodeBase().toExternalForm()+str_cb;
		}
		while (str_codebase.endsWith("./")) {
			str_codebase=str_codebase.substring(0,str_codebase.length()-2);
		}
		try {
			codebase = new URL(str_codebase);
		} catch (Exception e) {
			codebase=D.getCodeBase();
		}
//System.out.println("codebase="+codebase.toExternalForm());
//System.out.println("doc base="+docbase.toExternalForm());
	}
	
  /* ---------------------  Applet Stub Implementation ------------------ */

	public void appletResize(int w,int h) {
	}

	public AppletContext getAppletContext() {
		return D.getAppletContext();
	}
	
	public URL getCodeBase() {
		return codebase;
	}
	
	public URL getDocumentBase() {
		return docbase;
	}
	
	public String getParameter(String name) {
		String value=(String) ap.get(name);
		return value;
	}
	public boolean isActive() {
		return A.isActive();
	}
	
}

final class ZipClassLoader extends ClassLoader {
	private ZipFile file;
    private ZipInputStream zin;

    ZipClassLoader(Applet A,String filename) throws IOException {
    	if (A!=null) {
	    	String uri=A.getDocumentBase().toExternalForm();
	    	int ix=uri.lastIndexOf("/");
	    	if (ix>=0) {
	    		uri=uri.substring(0,ix);
	    	}
			uri=uri+"/"+filename;
			try {
				URL url=new URL(uri);
				URLConnection urlc=url.openConnection();
				zin=new ZipInputStream(urlc.getInputStream());
			} catch (MalformedURLException e) {
				e.printStackTrace();
				throw e;
			}
    	} else {
	        file = new ZipFile(filename);
    	}
    }

    @Override
    protected Class<?> findClass(String name) throws ClassNotFoundException {
    	ZipEntry entry=null;
    	if (zin!=null) {
	    	do {
				try {
		    		entry=zin.getNextEntry();
				} catch (Exception ex) {
					ex.printStackTrace();
				}
				String s1=name+".class";
				String s2=entry.getName().replace('/', '.');
	    		if (s1.equals(s2)) {
	    			break;	
	    		}
	    	} while (entry!=null);
    	} else {
	        entry = this.file.getEntry(name.replace('.', '/') + ".class");
    	}
        if (entry == null) {
            throw new ClassNotFoundException(name);
        }
        try {
            byte[] array = new byte[1024];
            InputStream in = null;
            if (file!=null) {
            	in=file.getInputStream(entry);
            } else {
            	in=zin;
            }
            ByteArrayOutputStream out = new ByteArrayOutputStream(array.length);
            int length = in.read(array);
            while (length > 0) {
                out.write(array, 0, length);
                length = in.read(array);
            }
            return defineClass(name, out.toByteArray(), 0, out.size());
        } catch (IOException exception) {
            throw new ClassNotFoundException(name, exception);
        }
    }

    public class MySecurityManager extends SecurityManager {
    	
    }
}

class MySecurityManager extends SecurityManager{
	Vector<Permission> permisosExtra = new Vector<Permission>();
	public MySecurityManager(SecurityManager orig){
		super();
	}
	
	@Override
	public void checkPermission(Permission perm, Object context) {
		try{
			super.checkPermission(perm, context);
		}catch (SecurityException e) {
			System.out.println("Se esta pidiendo checar \n\t"+perm+"\n\t"+context);
			throw e;
		}
	}

	@Override
	public void checkPermission(Permission perm) {
		try{
			super.checkPermission(perm);
		}catch (SecurityException e) {
			System.out.println("Tratando de ver con nuestros permisos");
			for (int i = 0; i < permisosExtra.size(); i++) {
				Permission tmpPerm = permisosExtra.get(i);
				boolean esta = tmpPerm.implies(perm);
				System.out.println("Encontramos uno :D \n"+tmpPerm+"\n"+perm+"\n=>"+esta);
				if(esta){
					return;
				}
			}
			System.out.println("Se esta pidiendo checar \n\t"+perm);
			throw e;
		}
	}


	public void addPermissions(Permission[] array) {
		for (int i = 0; i < array.length; i++) {
			Permission tmpPerm1 = array[i];
			boolean agregar = true;
			for (int j = 0; j < permisosExtra.size(); j++) {
				Permission tmpP2 = permisosExtra.get(j);
				if(tmpP2.implies(tmpPerm1)){
					agregar = false;
					break;
				}
			}
			if(agregar)
				permisosExtra.add(tmpPerm1);
		}
		
	}
}