/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package com.mja.descartes;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import com.mja.util.*;
import com.mja.math.*;
import com.mja.gui.*;
import com.mja.descgui.*;
import com.mja.parser.*;
import com.mja.lang.*;
import com.mja.file.mjaFont;

public class SpaceR2 extends Space {

	private Graphics2D[] tga,fga;
	private boolean fillInFront;
	private R2 v=new R2();

	public SpaceR2(Descartes D,spaceConfig sc) {
		super(D,sc);
		D.p.Var(spcfg.prefix()+"_refresh_back_").setDouble(1);
	}
	
	Vector GV;

	public String toTeX(boolean grayscale) {
//		obtener todos los grficos de este espacio
		GV=new Vector();
		for (int i=0;i<D.scene.graph.length;i++) {
			if (D.scene.graph[i]!=null && D.scene.graph[i].cs==this && D.scene.graph[i].gc.isInBack) {
				GV.addElement(D.scene.graph[i]);
//				drawGraph(D.p,D.scene.graph[i]);
			}
		}
		for (int i=0;i<D.scene.graph.length;i++) {
			if (D.scene.graph[i]!=null && D.scene.graph[i].cs==this) {
				if (!D.scene.graph[i].gc.isInBack) {
					GV.addElement(D.scene.graph[i]);
//					drawGraph(D.p,D.scene.graph[i]);
				}
			}
		}
//		convertirlas a curves
		Vector curva=new Vector();
		for (int n=0;n<GV.size();n++) {
			mjaGraph mjag=((mjaGraph) GV.elementAt(n));
			for (int j=0;j<mjag.lasCurvas.length;j++) {
				curva.addElement(mjag.lasCurvas[j]);
			}
		}
//		vaciarlas a TeX con \definecolor, \psline y \pspolygon
		double SCL=Nesc.Evaluate(48);
		int W=getWidth()/2;
		int H=getHeight()/2;
		double DX=(W+NOx.Evaluate(0))/SCL;
		double DY=(H-NOy.Evaluate(0))/SCL;		
		String str=BasicStr.getTeX_Header(W,H);
		str=str+BasicStr.getTeX_BeginDoc(SCL,W,H,DX,DY);
		for (int n=0;n<curva.size();n++) {
			str=str+((Curve) curva.elementAt(n)).toTeX(this,getHeight(),grayscale);
		}
		str=str+BasicStr.TeX_Tail;
		return str;
	} 
	
/* -----------------------   Overwrites  -------------------------*/

//	public void start() {
//		super.start();
//	}

/* ---------------- implementation of Space -------------------- */

	protected R2 injection(double x,double y) {
		return injection(x,y,false);
	}

	protected R2 injection(double x,double y,boolean absCoor) {
		if (absCoor) {
			return new R2(x,y);
		} else {
			return new R2((double)(x-Ox)/Nesc.getDouble(),(double)(Oy-y)/Nesc.getDouble());
		}
	}

	protected R2 projection(R2 p,boolean absCoor) {
		double x=p.x;
		double y=p.y;
		if (!absCoor) {
			x=Ox+Nesc.getDouble()*p.x;
			y=Oy-Nesc.getDouble()*p.y;
		}
		return new R2(x,y);
	}

	protected R2 projection(R2 p) {
		return projection(p,false);
	}

	protected void stopMoving() {
		v.x=0; v.y=0;
	}

	protected boolean isMoving() {
		return ( v.x!=0 || v.y!=0 );
	}

	void doBack(boolean clear) {
		setVisible(visibilityN.Evaluate(1)>0);
		if (isVisible()) {
			if (!started) {
				start();
			}
			boolean forceBackUpdate=(backColor.variable() || (backColor.getAdaptedColor().getAlpha()<0xff));
			boolean inGrSt=initializeGraphicStuff();
			if (inGrSt||clear||(main==null)||forceBackUpdate) {
				super.doBack(inGrSt||clear);
				double rsc=Nesc.getDouble();
				int dec=0;
				while (rsc>=10 && rsc>(w+h)) { 
					rsc/=10; 
					dec++; 
				}
				while (rsc<(w+h)/10) { 
					rsc*=10;	
				}
				if (spcfg.isOn[net]) {
					gback.setColor(spcfg.DC[net].getAdaptedColor());
					drawMarks(rsc/10,-1);
				}
				if (spcfg.isOn[net10]) {
					gback.setColor(spcfg.DC[net10].getAdaptedColor());
					drawMarks(rsc,-1);
				}
				if (spcfg.isOn[axe]) {
					gback.setFont(mjaFont.AWTSansSerif);
					gback.setColor(spcfg.DC[axe].getAdaptedColor());
					if (spcfg.x_On) {
						gback.drawLine(0,Oy,w,Oy);
					}
					if (spcfg.y_On) {
						gback.drawLine(Ox,0,Ox,h);
					}
					if (spcfg.numbers) {
						if (rsc>(w+h)/2) {
							drawNumbers(rsc/5,(rsc<=Nesc.getDouble())?dec+1:dec);
						} else if (rsc>(w+h)/4) {
							drawNumbers(rsc/2,(rsc<=Nesc.getDouble())?dec+1:dec);
						} else {
							drawNumbers(rsc,dec);
						}
					}
					drawMarks(rsc,3);
					rsc/=2; drawMarks(rsc,2);
					rsc/=5; drawMarks(rsc,1);
					gback.setColor(spcfg.DC[axe].getAdaptedColor());
					int xL=Ox-2-gback.getFontMetrics(gback.getFont()).stringWidth(spcfg.y_axis);
					int yL=gback.getFontMetrics(gback.getFont()).getAscent();
					if (spcfg.y_On) {
						gback.drawString(spcfg.y_axis,xL,yL);
					}
					xL=w-3-gback.getFontMetrics(gback.getFont()).stringWidth(spcfg.x_axis);
					yL=Oy+yL;
					if (spcfg.x_On) {
						gback.drawString(spcfg.x_axis,xL,yL);
					}
				}
				drawGraphsInBack();
			}
			if (fillInFront) {
				gmain.setColor(backColor.getAdaptedColor());
				gmain.fillRect(0,0,w,h);
			}
		}
	}

	protected void doCalc() {
	}

	protected void doFront(boolean calc) {
		if (isVisible()) {
			prepareMainImage();
			fga[0]=gmain;
			adjustControls();
			drawGraphsInFront();
			drawBackControls();
			super.doFront(calc);
			gmain.setColor(new Color(0xccbbaa));
		} else {
//			System.out.println("not visible");
		}
	}

	protected void writeCoordinates(Graphics g,int x, int y) {
		if (isVisible() && spcfg.isOn[tex]) {
			g.setFont(mjaFont.AWTMono);
			g.setColor(spcfg.DC[tex].getAdaptedColor());
			g.drawOval(x-2,y-2,4,4);
			FontMetrics fm=g.getFontMetrics();
			TFont tf=new TFont(mjaFont.AWTMono);
			int pp=0;
			while (Nesc.getDouble()>Math.pow(10,pp)) { pp++; }
			if (pp>0) {
				pp++;
			}
			double X=(double)(x-Ox)/Nesc.getDouble();
			double Y=(double)(Oy-y)/Nesc.getDouble();
			String str_x=BasicStr.DoubleToString(X,pp,true,".");
			String str_y=BasicStr.DoubleToString(Y,pp,true,".");
			String str="("+str_x+","+str_y+")";
			int sw=fm.stringWidth(str);
			int posx=x-sw/2;
			int posy=y-12;
			if (posx<0) { posx=0; }
			int ld=w-sw;
			if (posx>ld) { posx=ld; }
			int li=tf.getAscent();
			if (posy<li) { posy=li; }
			int ls=h-tf.getDescent();
			if (posy>ls) { posy=ls; }
			g.drawString(str,posx,posy);
		}
	}

	protected boolean onDrag(boolean right,int dx,int dy) {
		D.updateNodesFromControls();
		if (right) {
			double esc0=Nesc.getDouble();
			double mult=((double)h-(double)dy)/(double)h;
			Nesc.setDouble(Nesc.getDouble()*mult);
			correctScale();
			NOx.setDouble(NOx.getDouble()*(Nesc.getDouble()/esc0));
			NOy.setDouble(NOy.getDouble()*(Nesc.getDouble()/esc0));
			Ox=(int) Math.round(NOx.getDouble())+w/2;
			Oy=(int) Math.round(NOy.getDouble())+h/2;
		} else {
			v.x=dx;
			v.y=dy;
			Ox+=v.x;
			Oy+=v.y;
			NOx.setDouble(Ox-(double)w/2.0);
			NOy.setDouble(Oy-(double)h/2.0);
			v.x=0; v.y=0;
		}
		D.updateTextFieldsFromNodes();
		return backColor.variable();
	}

/* ----------------- utility methods ------------------ */

	private double _X_(int size,double v,boolean absCoor) {
//System.out.println(size+" "+v+" "+absCoor);
		double x=v;
		if (!absCoor) {
			x=Ox+Nesc.getDouble()*v;
		}
		if (x<-size) { x=-size; }
		if (x>w+size) { x=w+size; }
		return x;
	}

	int XX(int size,double v,boolean absCoor) {
		return (int) Math.round(_X_(size,v,absCoor));
	}

	private double _Y_(int size, double v,boolean absCoor) {
		double y=v;
		if (!absCoor) {
			y=Oy-Nesc.getDouble()*v;
		}
		if (y<-size) { y=-size; }
		if (y>h+size) { y=h+size; }
		return y;
	}

	int YY(int size, double v,boolean absCoor) {
		return (int) Math.round(_Y_(size,v,absCoor));
	}

/* ---------------------- Private Methods --------------------- */
/* ---------------------- Graphic Stuff --------------------- */

	private boolean initializeGraphicStuff() {
		boolean bgch=false;
		fillInFront=false;
		if (D.scene.graph!=null) {
			for (int i=0;i<D.scene.graph.length;i++) {
				if (D.scene.graph[i]!=null && !D.scene.graph[i].gc.isInBack) {
					if (D.scene.graph[i].gc.type==data.fill) {
						fillInFront=true;
						break;
					}
				}
			}
		}
		fga=new Graphics2D[1];
		fga[0]=gmain;
		tga=new Graphics2D[2];
		tga[0]=gback;
		tga[1]=gmain;
		bgch=spaceChanged()|bgch;
		return bgch;
	}


	private void drawGraphsInBack() {
		for (int i=0;i<D.scene.graph.length;i++) {
			if (D.scene.graph[i]!=null && D.scene.graph[i].cs==this && D.scene.graph[i].gc.isInBack) {
				drawGraph(D.p,D.scene.graph[i]);
			}
		}
	}

	private void drawGraphsInFront() {
		for (int i=0;i<D.scene.graph.length;i++) {
			if (D.scene.graph[i]!=null && D.scene.graph[i].cs==this) {
				if (!D.scene.graph[i].gc.isInBack) {
					drawGraph(D.p,D.scene.graph[i]);
				}
			}
		}
	}

	protected void flushImages() {
		for (int i=0;i<D.scene.graph.length;i++) {
			if (D.scene.graph[i].ith!=null) {
				D.scene.graph[i].ith.pleaseStop();
			}
		}
		for (int i=0;i<D.scene.graph.length;i++) {
			if (D.scene.graph[i].image!=null) {
				D.scene.graph[i].flush();
			}
		}
	}

	private void drawMarks(double rsc,int sz) {
		int x1=0,x2=w,y1=0,y2=h;
		if (sz>=0) {
			x1=Ox-sz; x2=Ox+sz; y1=Oy-sz; y2=Oy+sz;
		}
		int x,y;
		if ((spcfg.x_On&&sz>0) || sz<0) {
			for (int i=-(int)Math.round(Ox/rsc);(x=Ox+(int)Math.round(i*rsc))<w;i++) {
				gback.drawLine(x,y1,x,y2);
			}
		}
		if ((spcfg.y_On&&sz>0) || sz<0) {
			for (int i=-(int)Math.round(Oy/rsc);(y=Oy+(int)Math.round(i*rsc))<h;i++) {
				gback.drawLine(x1,y,x2,y);
			}
		}
	}

	private void drawNumbers(double rsc,int dec) {
		// Se hicieron cambios el 21 de mayo de 2009, se usa una fuente SansSerif y se ajustaron las posiciones de los nmeros
		int x,y;
		if (spcfg.x_On) {
			for (int i=-(int)Math.round(Ox/rsc);(x=Ox+(int)Math.round(i*rsc))<w;i++) {
				double X=(double)i*rsc/Nesc.getDouble();
//				String s_x=BasicStr.notExp(X,0.0015,9999.5,4,false);
				String s_x=BasicStr.DoubleToString(X,4,false);
				gback.drawString(s_x,x-((X<0)?4:0),Oy-5);
			}
		}
		if (spcfg.y_On) {
			for (int i=-(int)Math.round(Oy/rsc);(y=Oy+(int)Math.round(i*rsc))<h;i++) {
				double Y=-(double)i*rsc/Nesc.getDouble();
//				String s_y=BasicStr.notExp(Y,0.0015,9999.5,4,false);
				String s_y=BasicStr.DoubleToString(Y,4,false);
				if (!"0".equals(s_y)) {
					gback.drawString(s_y,Ox+5,y+2);
				}
			}
		}
	}

/* --------------------- Graphs drawing -------------------- */

	private void drawGraph(Parser p, mjaGraph G) {
		if (G.cs==this) {
			Node X=p.Var("x");
			Node Y=p.Var("y");
			synchronized (X) {
				synchronized (Y) {
					double vx=X.getDouble(), vy=Y.getDouble();
					if (G.sparz!=null) {
						int sN=(int)Math.round(G.sparz.NN.Evaluate(32));
						double si=G.sparz.Ni.Evaluate(0);
						double sf=G.sparz.Nf.Evaluate(1);
						Node S=p.Var(G.sparz.name);
						synchronized (S) {
							double s=S.getDouble();
							if (sN==0) {
								S.setDouble(si);
								G.createLasCurvas(1);
								drawGraph(G,0);
							} else if (sN>0) {
								G.createLasCurvas(sN+1);
								for (int n=0;n<=sN;n++) {
									S.setDouble(si+(sf-si)*(double)n/(double)sN);
									drawGraph(G,n);
								}
							}
							S.setDouble(s);
						}
					} else {
						G.createLasCurvas(1);
						drawGraph(G,0);
					}
					X.setDouble(vx); Y.setDouble(vy);
				}
			}
		}
	}
	
	private void drawGraph(mjaGraph G,int n) {
		if (G.cs==this) {
			if (G.gc.isInBack) {
				mjaColor[] mjac=new mjaColor[1];
				for (int i=0;i<1;i++) {
					mjac[i]=G.gc.gcolor;
				}
				if (back!=null) {
					Graphics[] bgar={back.getGraphics()};
					G.draw(back,bgar,mjac,n);
				}
			} else if (G.gc.hasTrace) {
				mjaColor[] mjac=new mjaColor[tga.length];
				for (int i=0;i<tga.length;i++) {
					if (i==0) {
						mjac[i]=G.gc.traceColor;
					} else {
						mjac[i]=G.gc.gcolor;
					}
				}
				G.draw(main,tga,mjac,n);
			} else {
				mjaColor[] mjac=new mjaColor[fga.length];
				for (int i=0;i<fga.length;i++) {
					mjac[i]=G.gc.gcolor;
				}
				G.draw(main,fga,mjac,n);
			}
		}
	}

}