/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package com.mja.descgui;
/*------------------------------------------------------------*/
/*                     TextAreaControl                        */
/*                      (c)  JLAbreu                          */
/*                      Marzo de 2010                         */
/*------------------------------------------------------------*/
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import java.net.*;
import javax.swing.JTextField;
import javax.swing.JLabel;
//import com.jla.calc.*;
import com.mja.util.*;
import com.mja.parser.*;
import com.mja.descartes.*;
import com.mja.gui.*;
import com.mja.cmp.*;
import com.mja.file.*;
import com.mja.textedit.*;

public class TextAreaControl extends Panel implements ActionListener {

	private EditorQPanel EQP;
	private Descartes D;
	private String name;
	private controlConfig txtctrl;
	private mjaButton b_switch;
	private static final Color pbgc=new Color(0xc0d0d8);
	private boolean hasSol=false;
	private String solution;

	public TextAreaControl(Descartes D,controlConfig txtctrl) {
		this.D=D;
		this.txtctrl=txtctrl;
		name=txtctrl.getName();
		setBackground(pbgc);
		setBounds(txtctrl.getTextX(),txtctrl.getTextY(),txtctrl.getTextW(),txtctrl.getTextH());
		setLayout(null);
		EQP=new EditorQPanel(D,new Frame(),D.Tr,D.p,true,false,false,true);
		EQP.init();
		EQP.setText(mjaHtml.decodeHTMLEncoding(txtctrl.s_value[controlConfig.ixtext]));
		D.p.defineVar(name,txtctrl.s_value[controlConfig.ixtext]);
		EQP.getEB().setAttributes(EditorButtons.ItemsAttributes);
		if (!D.Tr.isTrue(txtctrl.s_value[controlConfig.ixbuttons])) {
			EQP.hideEditorButtons();
		}
		hasSol=BasicStr.hasContent(txtctrl.s_value[controlConfig.ixsolution]);
		int hsz=txtctrl.getTextW()-8;
		int vsz=txtctrl.getTextH()-((hasSol)?28:8);
		EQP.setCanvasSize(hsz,vsz);
		EQP.setBounds(4,4,hsz,vsz);
		add(EQP);
		EQP.setEditing(true);
		if (hasSol) {
			solution=mjaHtml.decodeHTMLEncoding(txtctrl.s_value[controlConfig.ixsolution]);
System.out.println(solution);
			b_switch=new mjaButton("S");
			b_switch.addActionListener(this);
			b_switch.setBounds(txtctrl.getTextW()-24,txtctrl.getTextH()-20,20,16);
			add(b_switch);
			if (D.isArquimedesItem() && !D.inNippeEditor()) {
				b_switch.setVisible(false);
			}
		}
	}
	
	public void registerTextInParser() {
		String s=EQP.getText(false);
		D.p.defineVar(name,s);
	}
	
	private boolean showingSol=false;
	
	public void updateVisualComponent() {
		EQP.setText(D.p.Var(name).getStringValue());
		EQP.getEC().updateScreen();
		showingSol=false;
		EQP.repaint();
	}
	
	public void showSolution() {
		registerTextInParser();
		EQP.setText(solution);
		EQP.getEC().updateScreen();
		showingSol=true;
		EQP.repaint();
	}
	
	public void showUserText() {
		updateVisualComponent();
		EQP.getEC().updateScreen();
		showingSol=false;
		EQP.repaint();
	}
	
	protected void defineTextValue(String s0) {
		D.p.defineVar(name,s0);
		updateVisualComponent();
	}

	public void start() {
		if (D.disableControls) {
			EQP.setEditing(false);
		}
		EQP.start();
		updateVisualComponent();
		if (D.disableControls) {
			EQP.setEnabled(false);
		}
	}

	public void locate() {
		setVisible(txtctrl.condVis.Evaluate(1)>0);
		setEnabled(txtctrl.condActiv.Evaluate(1)>0);
		if (txtctrl.xN!=null && txtctrl.yN!=null&& txtctrl.wN!=null&& txtctrl.hN!=null) {
			Dimension d=getSize();
			double w=txtctrl.wN.Evaluate(100);
			double h=txtctrl.hN.Evaluate(23);
			int w1=(int)Math.floor(w+0.5);
			int h1=(int)Math.floor(h+0.5);
			int x1=(int)Math.floor(txtctrl.xN.Evaluate(0)+0.5);
			int y1=(int)Math.floor(txtctrl.yN.Evaluate(0)+0.5);
			Point loc=getLocation();
			if (loc.x!=x1||loc.y!=y1||d.width!=w1||d.height!=h1) {
				setBounds(x1,y1,w1,h1);
				int hsz=txtctrl.getTextW()-8;
				int vsz=txtctrl.getTextH()-((hasSol)?28:8);
				EQP.setCanvasSize(hsz,vsz);
				EQP.setBounds(4,4,hsz,vsz);
				b_switch.setBounds(txtctrl.getTextW()-24,txtctrl.getTextH()-20,20,16);
			}
		}
	}
	
	public void stop() {
		EQP.stop();
	}
	
	public void actionPerformed (ActionEvent e) {
		if (e.getSource()==b_switch) {
			if (showingSol) {
				showUserText();
				EQP.setEnabled(true);
				EQP.setEditing(true);
				b_switch.setLabel("S");
				b_switch.setBounds(txtctrl.getTextW()-24,txtctrl.getTextH()-20,20,16);
				b_switch.repaint();
			} else {
				showSolution();
				EQP.setEditing(false);
				EQP.setEnabled(false);
				b_switch.setLabel("T");
				b_switch.setBounds(txtctrl.getTextW()-24,txtctrl.getTextH()-20,20,16);
				b_switch.repaint();
			}
		}
	}
	
	public void paint(Graphics g) {
		super.paint(g);
		EQP.repaint();
	}

}