/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package com.mja.descgui.edit;

import java.util.*;
import com.mja.lang.*;
import com.mja.util.*;
import com.mja.krypto.*;
import com.mja.gui.mjaColor;
import org.unam.matem.Attribute;


/*------------------------------------------------------------*/
/*                         editObject                         */
/*                          (c) MJA                           */
/*                       Febrero de 2001                      */
/*------------------------------------------------------------*/

public abstract class editObject {

	public static final int select=0,string=1,color_button=2,color_select=3,text=4,choice=5,textArea=6,macro=7;

	public editField[] field;
	public editObjectType[] eot;
	public String[] s_value;
	public String[] h_value;
	public String[][] expl=new String[0][0];
	protected String ID;
	protected String[] nullv;

	protected abstract void defineFields();
	public abstract void updateInfo(translator Tr);
	public abstract void setEditName(String name);
	public abstract String getEditName();

	public abstract int getNumTypes();
	public abstract String getType(translator Tr,int i);
	public abstract String getDefaultExpresion(translator Tr,int i);
	public abstract String getListTitle(translator Tr);
	public abstract String getTypesInfo(translator Tr);

	public abstract int getTypeIndex(translator Tr);
	
	protected void initialize(translator Tr,editField[] field,editObjectType[] eot,Attribute[] aa) {
		initialize(Tr,field,eot,aa,null);
	}

	protected void initialize(translator Tr,editField[] field,editObjectType[] eot,Attribute[] aa,String[] null_values) {
		this.field=field;
		this.eot=eot;
		s_value=new String[field.length];
		h_value=new String[field.length];
		nullv=null_values;
		if (nullv==null) {
			nullv=new String[field.length];
			for (int i=0;i<field.length;i++) {
				nullv[i]=field[i].null_val;
			}
		}
		for (int i=0;i<field.length;i++) {
			if (field[i].prefix>=0) {
				String prefix=s_value[field[i].prefix]+".";
				s_value[i]=Attribute.decodeQuotes(Tr.getValue(aa,prefix,field[i].attrix,nullv[i]));
			} else {
				s_value[i]=Attribute.decodeQuotes(Tr.getValue(aa,field[i].attrix,nullv[i]));
			}
			if (field[i].encoded&&s_value[i].startsWith("krypto_")) {
				s_value[i]=s_value[i].substring("krypto_".length());
				s_value[i]=Encript.decode(s_value[i]);
				s_value[i]=Attribute.decodeQuotes(s_value[i]);
			}
			if (field[i].hattrix>=0) {
				String hprefix="";
				if (field[i].hprefix>=0) {
					hprefix=Tr.getTr(field[i].hprefix)+".";
				}
				h_value[i]=Tr.getValue(aa,hprefix,field[i].hattrix,"");
			} else {
				h_value[i]="";
			}
		}
		updateInfo(Tr);
	}

	public void translateValues(translator Tr) {
		if (field!=null) {
			for (int i=0;i<field.length;i++) {
				if (BasicStr.hasContent(s_value[i])) {
					if (field[i].eType==editObject.select) {
						if (Tr.isTrue(s_value[i])) {
							s_value[i]=Tr.getTr(data.Yes);
						} else if (Tr.isFalse(s_value[i])) {
							s_value[i]=Tr.getTr(data.No);
						}
					} else if (field[i].eType==editObject.color_button) {
						s_value[i]=mjaColor.getColorName(Tr,Tr.parseColor(s_value[i],null,null));
					} else if (field[i].eType==editObject.color_select) {
						if (Tr.isTrue(s_value[i])) {
							s_value[i]=Tr.getTr(data.Yes);
						} else if (Tr.isFalse(s_value[i])) {
							s_value[i]=Tr.getTr(data.No);
						} else {
							s_value[i]=mjaColor.getColorName(Tr,Tr.parseColor(s_value[i],null,null));
						}
					} else if (field[i].eType==editObject.choice) {
						if (field[i].chvalues!=null) {
							int ix=Tr.getIndex(field[i].chvalues,s_value[i],-1);
							if (ix>=0) {
								s_value[i]=Tr.getTr(ix);
							}
						}
					}
				}
			}
		}
	}

	public String getId() {
		return ID;
	}

	public void setId(String id) {
		this.ID=id;
	}

	public String toString(String sep,translator Tr) {
		String s="";
		int objType = getTypeIndex(Tr);
		editObjectType eotAct = eot[objType];
		for (int i=0;i<field.length;i++) {
			editField fieldAct = field[i];
			boolean write=(eotAct.attributes.get(i)||fieldAct.isFixed)&&fieldAct.write;
			if (fieldAct.writeifix>=0) {
				write&=Tr.isNotFalse(s_value[fieldAct.writeifix]);
			}
			if (write) {
				String nowriteval="";
				if (nullv!=null) {
					nowriteval=nullv[i];
				}
				if (fieldAct.writeifix>=0) {
					nowriteval="";
				}
				String sval=Attribute.encodeQuotes(s_value[i]);
				if (fieldAct.encoded&&BasicStr.hasContent(sval)) {
					sval="krypto_"+Encript.encode(sval);
				}
				if (fieldAct.prefix>=0) {
					String prefix=s_value[fieldAct.prefix];
					s+=Tr.toString(sep,prefix+"."+Tr.getTr(fieldAct.attrix),"'"+sval+"'",nowriteval);
				} else {
					s+=Tr.toString(sep,fieldAct.attrix,"'"+sval+"'",nowriteval);
				}
				if (BasicStr.hasContent(h_value[i]) && fieldAct.hattrix>=0) {
					String name=Tr.getTr(fieldAct.hattrix);
					if (field[i].hprefix>=0) {
						name=Tr.getTr(fieldAct.hprefix)+"."+name;
					}
					s+=name+"='"+h_value[i]+"'"+sep;
				}
			}
		}
		return s.trim();
	}

	public String getTypesTitle(translator Tr) {
		return Tr.getTr(data.add);
	}
	public String getAcceptLabel(translator Tr) {
		return Tr.getTr(data.accept);
	}
	public String getCancelLabel(translator Tr) {
		return Tr.getTr(data.cancel);
	}

}