/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package com.mja.krypto;

import java.io.*;
import java.util.*;

public class Encript {

	public static void main(String[] args) {
		if (args[0].equals("encript")) {
			System.out.println(encode(args[1]));
		} else {
			System.out.println(decode(args[1]));
		}
	}

	public static String encode(String s) {
		int n=(int) (31*Math.random());
		Encript e = new Encript(getKey(n));
		byte[] b=new byte[1];
		b[0]=Encript.Alfanum(n);
		return (new String(b))+e.Encripta(s);
	}

	public static String decode(String s) {
		int n=Numalfa((byte) s.charAt(0));
		Encript e = new Encript(getKey(n));
		return e.Desencripta(s.substring(1,s.length()));
	}

	/* ---------------------- private methods ---------------------- */

	private String llave;

	private static String getKey(int n) { 
		double a1=1.0, a2=1.4, a3= 0.6, a4= 2.2;
		byte[] ll = new byte[256];
		for (int i=0;i<256;i++) {
		ll[i]=Encript.Alfanum(
			(int)(Math.abs(7.5*(Math.sin(a1*(double)i-(double)n)
							   +Math.sin(a2*(double)i+(double)n)
							   +Math.sin(a3*(double)i-(double)n)
							   +Math.sin(a4*(double)i+(double)n)))));
		}
		return new String(ll);
	} 

	public Encript(String ll) { 
		llave = new String(ll); 
	}

	public String Encripta(String OrigMeu) {
		return BytesToString(Encripta(StringToBytes(OrigMeu)));   
	} 

	public String Desencripta(String OrigMeu) {
		return BytesToString(Desencripta(StringToBytes(OrigMeu)));   
	} 

	private byte[] Encripta(byte[] OrigMeu)	{
		if (OrigMeu == null) {
			return null;
		}
		if (llave == null) {
			return null;
		}
		byte[] EncripMeu = new byte[3*OrigMeu.length];
		for (int i=0;i<OrigMeu.length;i++) {
			int x = (((int)OrigMeu[i]+128))*256
				  + (int)Math.round(Math.random()*255)
				  + (int)Math.round(Math.random()*255)*256*256;
			int y = ((x<<shift(i))/256);
			EncripMeu[3*i] =   Alfanum (y%32); 
			EncripMeu[3*i+1] = Alfanum ((y/32)%32);
			EncripMeu[3*i+2] = Alfanum ((y/1024)%32);
		}
		return EncripMeu;
	}

	private byte[] Desencripta(byte[] OrigMeu) {
		if (OrigMeu == null) {
			return null;
		}
		if (llave == null) {
			return null;
		}
		byte[] DesencripMeu = new byte[OrigMeu.length/3];
		for (int i=0;i<DesencripMeu.length;i++) {
			int y = Numalfa(OrigMeu[3*i]) 
				+ Numalfa(OrigMeu[3*i+1])*32 
				+ Numalfa(OrigMeu[3*i+2])*1024;  
			int x = (y*256)>>shift(i);
			DesencripMeu[i] = (byte) ((((x/256)%256))-128);
		}
		return DesencripMeu;
	}

	private static byte[] StringToBytes(String OrigMeu) {
		byte[] b = new byte[OrigMeu.length()];
		for (int i=0;i<OrigMeu.length();i++) {
			b[i] = (byte) OrigMeu.charAt(i); 
		}
		return b;
	}

	private static String BytesToString(byte[] b) {
		return new String(b);
	}

	private static byte Alfanum (int k) {
		if (k<10){
			return (byte)(48 + k);
		} else {
			return (byte)(87 + k);
		}
	}

	private static int Numalfa (byte b) {
		if (b<58){
			return (int) (b-48);
		} else {
			return (int) (b-87);
		}
	}

	private int shift(int i) {
		byte a = (byte) llave.charAt(i%llave.length());
		int b = Numalfa(a);
		int c=(b/2)%8;
		if (c==0) c=4;
		return c;
	}

}