/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package com.mja.math;
/*------------------------------------------------------------*/
/*                            R2                              */
/*                         (c) MJA                            */
/*                     Septiembre de 1998                     */
/*------------------------------------------------------------*/
import com.mja.parser.*;
import com.mja.util.BasicStr;
import java.util.StringTokenizer;

public class R2Newton {

	static double delta=0.000001;

	private Parser P;
	private Node F,X,Y;
	private double f0;
	private R2 normal;
	static final int igual=0,menor=-1,mayor=1;
	private int signo=igual;

	public R2Newton(Parser P, Node F) throws Exception {
		this.P=P;
		this.F=F;
		if (F.getStr()!=null) {
			init("_Eval_("+F.getStr()+")");
		} else if (F.getSymbol().equals("=")
			||F.getSymbol().equals("<")
			||F.getSymbol().equals("<=")
			||F.getSymbol().equals(">=")
			||F.getSymbol().equals(">"))
			{
			if (F.getSymbol().equals("<")||F.getSymbol().equals("<=")) {
				signo=menor;
			} else if (F.getSymbol().equals(">")||F.getSymbol().equals(">=")) {
				signo=mayor;
			} else {
				signo=igual;
			}
			F.equalToMinus();
			X=P.Var("x");
			Y=P.Var("y");
		}
	}
	
	private void init(String expr) throws Exception {
		boolean hasException=false;
		if (BasicStr.hasContent(expr)) {
			try {
				F=P.Analyse(expr);
				if (F.getSymbol().equals("=")) {
					F.equalToMinus();
					X=P.Var("x");
					Y=P.Var("y");
				} else {
					hasException=true;
				}
			} catch (Exception e) {
				hasException=true;
			}
		} else {
			hasException=true;
		}
		if (hasException) {
			System.out.println("R2Newton("+expr+")");
			Exception ex=new Exception("\""+expr+"\""+" is not a formula");
			throw ex;
		}
	}

	public R2Newton(Parser P, String expr) throws Exception {
		this.P=P;
		init(expr);
	}

	public R2 getUnitNormal() {
		normal.normalize();
		return normal.copy();
	}

	public boolean rangeOK() {
		return F.rangeOK();
	}

	public R2 gradient(R2 q0) throws Exception {
		R2 q=new R2(0,0);
		synchronized (X) {
			synchronized (Y) {
				double savex=X.getDouble();
				double savey=Y.getDouble();
				X.setDouble(q0.x); 
				Y.setDouble(q0.y); 
				f0=F.Evaluate(F,true);
				X.setDouble(X.getDouble()+delta);
				double FV=F.Evaluate(F,true);
				q.x=(FV-f0)/delta;
				X.setDouble(X.getDouble()-delta);
				Y.setDouble(Y.getDouble()+delta);
				FV=F.Evaluate(F,true);
				q.y=(FV-f0)/delta;
				X.setDouble(savex);
				Y.setDouble(savey);
			}
		}
		return q;
	}

	public R2 findZero(R2 q0,double dist) throws Exception {
		double dist2=dist*dist;
		R2 q=q0.copy();
		double savex=X.getDouble();
		double savey=Y.getDouble();
		X.setDouble(q0.x); 
		Y.setDouble(q0.y); 
		f0=F.Evaluate(F,true);
		if (signo==menor && f0<=0) {
			return q;
		} else if (signo==mayor && f0>=0) {
			return q;
		}
		X.setDouble(savex);Y.setDouble(savey);
		for (int i=0;i<16;i++) { // eran 256
			double xa=q.x, ya=q.y;
			normal=gradient(q);
			normal.mul(-f0/normal.norm2());
			q.x=xa+normal.x; 
			q.y=ya+normal.y;
			if (normal.norm2()<dist2) {
				if ((Math.abs(normal.x)<delta)&&(Math.abs(normal.y)<delta)) {
					normal.x=q.x-q0.x;
					normal.y=q.y-q0.y;
				}
				return q;
			}
		}
		throw new Exception("Zero not found.");
	}

}