/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package com.mja.math3D;

public class LinearTransformation { 

	private double[][] M={ {1.0,0.0,0.0} , {0.0,1.0,0.0} , {0.0,0.0,1.0} };
	private R3 T = new R3( 0.0, 0.0, 0.0 );

	public LinearTransformation() {
	}
	
	static public LinearTransformation Scale(double esc) {
		LinearTransformation LT=new LinearTransformation();
		for (int i=0;i<3;i++) {
			LT.M[i][i]*=esc;
		}
		return LT;
	}

	static public LinearTransformation Rotation(int axis,double a) {
		LinearTransformation LT=new LinearTransformation();
		double cos=Math.cos(a);
		double sin=Math.sin(a);
		if (axis==0) {
			LT.M[1][1]=cos; LT.M[1][2]=-sin   ;
			LT.M[2][1]=sin; LT.M[2][2]=cos   ;
		} else if (axis==1) {
			LT.M[2][2]=cos; LT.M[2][0]=-sin   ;
			LT.M[0][2]=sin; LT.M[0][0]=cos   ;
		} else if (axis==2) {
			LT.M[0][0]=cos; LT.M[0][1]=-sin   ;
			LT.M[1][0]=sin; LT.M[1][1]=cos   ;
		}
		return LT;
	}
	
	static public LinearTransformation Transposed(LinearTransformation R) {
		LinearTransformation Rt=new LinearTransformation();
		for (int i=0;i<3;i++) {
			for (int j=0;j<3;j++) {
				Rt.M[i][j]=R.M[j][i];
			}
		}
		return Rt;
	}

	static public LinearTransformation Rotation(R3 rot) {
		LinearTransformation Rx=Rotation(0,rot.x);
		LinearTransformation Ry=Rotation(1,rot.y);
		LinearTransformation Rz=Rotation(2,rot.z);
		return Multiply(Rz,Multiply(Ry,Rx));
	}

	static public LinearTransformation EulerRotation(R3 rot) {
		LinearTransformation R1=Rotation(2,rot.z);
		LinearTransformation R2=Rotation(0,rot.y);
		LinearTransformation R3=Rotation(2,rot.x);
		return Multiply(R3,Multiply(R2,R1));
	}

	static public LinearTransformation Rotation(LinearTransformation LT,R3 rot) {
		return Multiply(LT,Rotation(rot));
	}

	static public LinearTransformation RigidMotion(boolean EulerAngles,R3 r,R3 t) {
		LinearTransformation LT=Rotation(r);
		if (EulerAngles) {
			LT=EulerRotation(r);
		}
		LT.T=t;
		return LT;
	}

	public void Apply(R3 p) {
		double x=p.x, y=p.y, z=p.z;
		p.x=T.x+M[0][0]*x+M[0][1]*y+M[0][2]*z;
		p.y=T.y+M[1][0]*x+M[1][1]*y+M[1][2]*z;
		p.z=T.z+M[2][0]*x+M[2][1]*y+M[2][2]*z;
	}

	static public LinearTransformation Multiply(LinearTransformation L1,LinearTransformation L2) {
		LinearTransformation LT=new LinearTransformation();
		for (int i=0;i<3;i++) {
			for (int j=0;j<3;j++) {
				LT.M[i][j]=0;
				for (int k=0;k<3;k++) {
					LT.M[i][j]+=L1.M[i][k]*L2.M[k][j];
				}
			}
		}
		return LT;
	}
	
	static public double[][] Multiply(double[][] M1,double[][] M2) {

		int cols=M2.length;
		int rows=M1[0].length;
		double[][] result=new double[cols][rows];
		int N=M1.length; // debe ser igual a M2[0].length;
		for (int j=0;j<rows;j++) {
			for (int i=0;i<cols;i++) {
				double sum=0;
				for (int k=0;k<N;k++) {
					sum=sum+M1[k][j]*M2[i][k];
				}
				result[i][j]=sum;
			}
		}
		return result;
	}

/* --------------------------------------------------------------- */	

	public static double[][] Minor(int I,int J,double[][] T) {
		double[][] M=new double[T.length-1][T.length-1];
		for (int i=0;i<M.length;i++) {
			for (int j=0;j<M.length;j++) {
				if (i<I) {
					if (j<J) {
						M[i][j]=T[i][j];
					} else {
						M[i][j]=T[i][j+1];
					}
				} else {
					if (j<J) {
						M[i][j]=T[i+1][j];
					} else {
						M[i][j]=T[i+1][j+1];
					}
				}
			}
		}
		return M;
	}

	public static double Determinant(double[][] T) {
		if (T.length>1) {
			double D=0;
			double s=1;
			for (int j=0;j<T.length;j++) {
				D+=s*T[0][j]*Determinant(Minor(0,j,T));
				s=-s;
			}
			return D;
		} else {
			return T[0][0];
		}
	}
	
//	public LinearTransformation inverse() throws Exception {
//		LinearTransformation LT=new LinearTransformation();
//		LT.M=inverse(M);
//		return LT;
//	}

	public static double[][] inverse(double[][] T) throws Exception {
		double[][] S = new double[T.length][T.length];
		double det=Determinant(T);
		if (det==0) {
			throw new Exception("Matrix not invertible");
		}
		double s=1/det;
		if (T.length>1) {
			for (int i=0;i<T.length;i++) {
				double t=s;
				for (int j=0;j<T.length;j++) {
					S[j][i]=t*Determinant(Minor(i,j,T));
					t=-t;
				}
				s=-s;
			}
		} else {
			S[0][0]=s;
		}
		return S;
	}
	
//	public static R3[] inverse(R3[] p) throws Exception {
//		double[][] T=new double[3][3];
//		for (int i=0;i<3;i++) {
//			T[i][0]=p[i].x;
//			T[i][1]=p[i].y;
//			T[i][2]=p[i].z;
//		}
//		double[][] I=inverse(T);
//		R3[] r=new R3[3];
//		for (int i=0;i<3;i++) {
//			r[i]=new R3(I[0][i],I[1][i],I[2][i]);
//		}
//		return r;
//	}

	public static R3 apply(R3[] p,R3 q) {
		return new R3(p[0].x*q.x+p[1].x*q.y+p[2].x*q.z,
					  p[0].y*q.x+p[1].y*q.y+p[2].y*q.z,
					  p[0].z*q.x+p[1].z*q.y+p[2].z*q.z);
	}
	
//	public static R3 solve(R3[] p,R3 q) throws Exception {
//		return apply(inverse(p),q);
//	}
	
/*	
	public static void main(String[] args) {
		R3[] p=new R3[3];
		p[0]=new R3(1,0,0.5);
		p[1]=new R3(-1,2,0);
		p[2]=new R3(0,0.25,1);
		R3 q=new R3(1,1,2);
		R3 r=solve(p,q);
		R3 Q=apply(p,r);
		System.out.println("q.x="+q.x+" q.y="+q.y+" q.z="+q.z);
		System.out.println("Q.x="+Q.x+" Q.y="+Q.y+" Q.z="+Q.z);
	}
*/
	
}