/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package com.mja.parser;
/*------------------------------------------------------------*/
/*                       parser.Parser                        */
/*                         (c)  MJA                           */
/*                       Mayo de 2001                         */
/*------------------------------------------------------------*/
import java.text.BreakIterator;
import java.util.*;
import java.util.regex.Pattern;
import java.applet.*;

import com.mja.file.mjaFile;
import com.mja.util.BasicStr;
import com.jla.nippe.LMSApplet;
import com.jla.nippe.NotSoAbstractNippe;

import org.unam.matem.Attribute;

public class Parser {

	public LMSApplet lms_applet;
	public NotSoAbstractNippe nsa_nippe;
	public static final String pi="\u03c0";
	private Vector dataFiles=new Vector();
	
	private Hashtable FS; // FUNCTIONS
	private Hashtable VS; // VARIABLES
	private Hashtable XS; // AUXILIARIES
	private Hashtable FAS;// FUNCTION_ALGORITHMS
	private Hashtable AS; // ARRAYS
	private Hashtable MS; // MATRICES
	private Vector ALS;   // ALGORITHMS
	
	public static final int DESCARTES=1, ATUTOR=2, PECEI=3;

	public Parser(NotSoAbstractNippe nsa_nippe) {
		this(DESCARTES);
		this.nsa_nippe=nsa_nippe;
		Node.C=nsa_nippe;
		if (nsa_nippe instanceof LMSApplet) {
			lms_applet=(LMSApplet) nsa_nippe;
		}
	}
	
	private int option=DESCARTES;

	public Parser(int option) {
		this.option=option;
		renew();
	}
	
	public void renew() {
		dataFiles=new Vector();
		FS=new Hashtable(); // FUNCTIONS
		VS=new Hashtable(); // VARIABLES
		XS=new Hashtable(); // AUXILIARIES
		FAS=new Hashtable();// FUNCTION_ALGORITHMS
		AS=new Hashtable(); // ARRAYS
		MS=new Hashtable(); // MATRICES
		ALS=new Vector();      // ALGORITHMS
		switch (option) {
			case DESCARTES:
				defineVar("e",2.7182818284590452353602874);
			case ATUTOR:
			case PECEI:
				defineVar(pi,Math.PI);
				defineVar("pi",Math.PI);
				break;
			default :
				break;
		}
	}
	
	public boolean inNippeEditor() {
		if (nsa_nippe!=null) {
			return nsa_nippe.inNippeEditor();
		} else {
			return false;
		}
	}
	
	private static boolean seemsFileName(String str) {
		return (!str.toLowerCase().startsWith("http://"))
			&& (str.toLowerCase().endsWith(".mp3")
				|| str.toLowerCase().endsWith(".txt")
				|| str.toLowerCase().endsWith(".png")
				|| str.toLowerCase().endsWith(".jpg")
				|| str.toLowerCase().endsWith(".jpeg"));
	}
	
	public String[] guessFilenames() {
		Vector Vfn=new Vector();
		String ds=BasicStr.decimal_symbol;
		BasicStr.decimal_symbol=".";
		Enumeration en=VS.elements();
		while (en.hasMoreElements()) {
			Node n=(Node) en.nextElement();
			String s=n.getString();
			if (seemsFileName(s)) {
				Vfn.addElement(s);
			}
		}
		en=AS.elements();
		while (en.hasMoreElements()) {
			Node n=(Node) en.nextElement();
			String[] sa=n.sarray;
			for (int i=0;i<sa.length;i++) {
				if (sa[i]!=null && seemsFileName(sa[i])) {
					Vfn.addElement(sa[i]);
				}
			}
		}
		en=MS.elements();
		while (en.hasMoreElements()) {
			Node n=(Node) en.nextElement();
			String[][] sm=n.smatrix;
			for (int i=0;i<sm.length;i++) {
				for (int j=0;j<sm[i].length;j++) {
					if (sm[i][j]!=null && seemsFileName(sm[i][j])) {
						Vfn.addElement(sm[i][j]);
					}
				}
			}
		}
		BasicStr.decimal_symbol=ds;
		String[] fn=new String[Vfn.size()];
		for (int i=0;i<fn.length;i++) {
			fn[i]=(String) Vfn.elementAt(i);
		}
		return fn;
	}

	public String dump() {
		String ds=BasicStr.decimal_symbol;
		BasicStr.decimal_symbol=".";
		String s="";
		Enumeration en=VS.elements();
		while (en.hasMoreElements()) {
			Node n=(Node) en.nextElement();
			if (isVar(n.toExpression())) { 
				s+=n.toExpression()+" = "+n.getString()+"\n";
			} else {
//System.out.println("VS OJO: "+n.toExpression()+" = "+n.getString()); 
			}
		}
		en=AS.elements();
		while (en.hasMoreElements()) {
			Node n=(Node) en.nextElement();
			double[] ra=getRealArray(n.toExpression());
			String[] sa=getStringArray(n.toExpression());
			for (int i=0;i<ra.length;i++) {
				if (sa[i]!=null) {
					s+=n.toExpression()+"["+i+"] = "+sa[i]+"\n";
				} else {
					s+=n.toExpression()+"["+i+"] = "+ra[i]+"\n";
				}
			}
		}
		en=MS.elements();
		while (en.hasMoreElements()) {
			Node n=(Node) en.nextElement();
			double[][] rm=getRealMatrix(n.toExpression());
			String[][] sm=getStringMatrix(n.toExpression());
			for (int i=0;i<rm.length;i++) {
				for (int j=0;j<rm[i].length;j++) {
					if (sm[i][j]!=null) {
						s+=n.toExpression()+"["+i+"]["+j+"] = "+sm[i][j]+"\n";
					} else {
						s+=n.toExpression()+"["+i+"]["+j+"] = "+rm[i][j]+"\n";
					}
				}
			}
		}
		BasicStr.decimal_symbol=ds;
//System.out.println("DUMP");
//System.out.println(s);
//System.out.println("");
		return s;
	}

	public void undump(String dump) {
//System.out.println("UNDUMP");
		String[] sa=BasicStr.toStringArray(dump);
		for (int i=0;i<sa.length;i++) {
			int pos=sa[i].indexOf("=");
			String name=sa[i].substring(0,pos).trim();
			if (BasicStr.hasContent(name) && !name.equals("-")) {
//System.out.println("sa["+i+"]  "+sa[i]);
				String value=sa[i].substring(pos+1).trim();
				if (value.startsWith("'") && value.endsWith("'")) {
					Var(name).setOnlyStr(value.substring(1,value.length()-1));
				} else if (value.startsWith("{\\rtf") && value.endsWith("}")) {
					Var(name).setOnlyStr(value);
				} else {
					if (isVar(name)) {
						double v=Double.parseDouble(value);
						Var(name).setDouble(v);
					} else {
//System.out.println("UNDUMP: Variable not found: "+name+" using algorithm");
						Algorithm A=new Algorithm("DUMP"+i,this,true,sa[i],"","");
//System.out.println(A.toExpression());
						try {
							A.perform();
						} catch (Exception e) {
							e.printStackTrace();
						}
					}
				}
			}
		}
//System.out.println("");
	}


	private String dumpy() {
		String ds=BasicStr.decimal_symbol;
		BasicStr.decimal_symbol=".";
		String s="";
		Enumeration en=VS.elements();
		while (en.hasMoreElements()) {
			Node n=(Node) en.nextElement();
			if (isVar(n.toExpression())) { 
				s+=n.toExpression()+" = "+n.getString()+"\n";
			} else {
//System.out.println("VS OJO: "+n.toExpression()+" = "+n.getString()); 
			}
		}
		BasicStr.decimal_symbol=ds;
		return s;
	}

	public Attribute[] getState() {
		String str=this.dumpy();
		String[] stra=BasicStr.toStringArray(str);
		Attribute[] aa= new Attribute[stra.length];
		for (int i=0;i<aa.length;i++) {
			int ix=stra[i].indexOf("=");
			if (ix>0) {
				String name=stra[i].substring(0,ix).trim();
				String value=stra[i].substring(ix+1).trim();
				if (name.startsWith("public.")) {
					aa[i]=new Attribute(name,value);
				} else {
					aa[i]=new Attribute("","");
				}
			} 
		}
		return aa;
	}
	
	public boolean adoptValues(Attribute[] aa) {
		boolean changed=false;
		for (int i=0;i<aa.length;i++) {
			if (aa[i]!=null && aa[i].name.length()>0 && isVar(aa[i].name)) {
				String sv0=Var(aa[i].name).getStr();
				double dv0=Double.NaN;
				if (sv0==null) {
					dv0=Var(aa[i].name).getDouble();
				}
				String sv1=BasicStr.trim(aa[i].value,'\'');
				double dv1=Double.NaN;
				try {
					dv1=Double.parseDouble(sv1);
				} catch (NumberFormatException e) {
					dv1=Double.NaN;
				}
				if (sv0!=null) {
					if (!sv0.equals(sv1)) {
						changed=true;
						Var(aa[i].name).setStrAndAnalyse(sv1);
					}
				} else if (dv0!=Double.NaN) {
					if (dv0!=dv1) {
						changed=true;
						Var(aa[i].name).setDouble(dv1);
					}
				}
			}
		}
		return changed;
	}
	

	public void randomizeVariables(double min,double max,int decimals) {
		Enumeration E=VS.keys();
		while (E.hasMoreElements()) {
			String name=(String)E.nextElement();
			if (!pi.equals(name)) {
				Node n=Var(name);
				int d=(int)Math.round(Math.pow(10,Math.max(0,Math.min(8,decimals))));
				double x=(max-min)*Math.random()+min;
    			n.setDouble(((double)((int)Math.round(x*d)))/(double)d);
//System.out.println(name+"="+n.r);
			}
		}
	}

	private boolean numericallyDifferent(Node n1,Node n2) {
		for (int i=0;i<20;i++) {
			randomizeVariables(-100,100,8);
			double x1=n1.Evaluate(Math.random());
			double x2=n2.Evaluate(Math.random());
			if (Math.abs(x1-x2)>Node.epsilon) {
				return true;
			}
		}
		return false;
	}

	public boolean areNumericallyEqual(Node N1,Node N2) {
		if (N1.symb==Node.IGUAL && N2.symb==Node.IGUAL) {
			Node n1=(Node) Node.Subtract(N1.left,N1.right[0]);
			Node n2=(Node) Node.Subtract(N2.left,N2.right[0]);
//System.out.println(n1.toExpression(true)+" =? "+n2.toExpression(true));
			boolean distinct1=numericallyDifferent(n1,n2);
			n1=(Node) Node.Multiply(N1.left,N2.right[0]);
			n2=(Node) Node.Multiply(N2.left,N1.right[0]);
//System.out.println(n1.toExpression(true)+" =? "+n2.toExpression(true));
			boolean distinct2=numericallyDifferent(n1,n2);
			n1=(Node) Node.Multiply(N1.left,N2.left);
			n2=(Node) Node.Multiply(N1.right[0],N2.right[0]);
//System.out.println(n1.toExpression(true)+" =? "+n2.toExpression(true));
			boolean distinct3=numericallyDifferent(n1,n2);
			return !(distinct1 && distinct2 && distinct3);
		} else {
			return !numericallyDifferent(N1,N2);
		}
	}

	public void defineVar(String name,String s) {
		Node n=new Node(this,name,0);
		n.setOnlyStr(s);
		VS.put(name,n);
	}

	void defineVar(String name,double d) {
		VS.put(name,Analyse(Double.toString(d),d));
	}
	
	public Node Var(String name) {
		if (!isVar(name)) {
			Node n=null;
			if (name.startsWith("'")&&name.endsWith("'")) {
				n=new Node(this,BasicStr.trim(name,'\''));
			} else {
				n=new Node(this,name,0);
			}
			if (Node.isVarName(name)) {
				VS.put(name,n);
			} else {
				return n;
			}
		} 
		return (Node)VS.get(name);
	}

	public String[][] getVariables() {
		String[][] varNameAndValue= new String[VS.size()][2];
		Enumeration e=VS.elements();
		Enumeration k=VS.keys();
		for (int i=0;i<VS.size();i++) {
			Node N=(Node) e.nextElement();
			varNameAndValue[i][0]=(String) k.nextElement();	
			varNameAndValue[i][1]=N.getStringValue();	
		}
		return varNameAndValue;
	}

	public void defineFunc(String name,String[] varname,String expr,boolean sequence) {
		Node n1=(Node)FS.get(name);
		Algorithm a1=(Algorithm)FAS.get(name+".func-alg");
		if (n1==null) {
			n1=new Node(this,name,0);
			FS.put(name,n1);
			a1=new Algorithm(name);
			FAS.put(name+".func-alg",a1);
		}

		n1.var=new Node[varname.length];
		for (int i=0;i<varname.length;i++) {
			n1.var[i]=Var(varname[i]);
		}
		n1.sequence=sequence;
		Node n2;
		try {
			n2=Analyse("Function "+name,expr,true,true);
		} catch (Exception e) {
			n2=new Node(this,0);
		}
		n1.assign(n2);

	}
	
	public void defineFunc(String name,String[] varname,String expr,String s_range,boolean sequence) {
		defineFunc(name,varname,expr,sequence);
		if (BasicStr.hasContent(s_range)) {
			try {
				Func(name).range=Analyse("Function "+name+".domain",s_range,true,true);
			} catch (Exception e) {
			}
		}
	}

	/**
	 * 
	 * @param name
	 * @param varname
	 * @param expr
	 * @param s_range
	 * @param localVarIds 
	 * 	The ids list for the local variables
	 * @param s_do
	 * @param s_ini
	 * @param s_while
	 * @param sequence
	 */
	public void defineFunc(String name,String[] varname,String expr,String s_range,
		String localVarIds, String s_do,String s_ini,String s_while,boolean sequence) {
		defineFunc(name,varname,expr,s_range,sequence);
		Node tmpF = Func(name);
		Algorithm a1=(Algorithm)FAS.get(name+".func-alg");
		Algorithm a2=new Algorithm(name,this,false,"Function",s_do,s_ini,s_while);
		a1.assign(a2);
		
		
		if(localVarIds == null){
			localVarIds = "";
		}
		String[] listIdLocals = localVarIds.split("\\s*,\\s*");
		// Guardamos los nodos de las variables locales 
		tmpF.localVarsList = new Node[listIdLocals.length];
		for (int i=0;i<listIdLocals.length;i++) {
			tmpF.localVarsList[i]=Var(listIdLocals[i]);
		}
	}

	

	public void defineExpr(String name,String expr) {
		Node n1=(Node) XS.get(name);
		Node n2;
		try {
			n2=Analyse("Variable "+name,expr,true,true);
		} catch (Exception e) {
			n2=new Node(this,0);
		}
		if (n1!=null) {
			n1.assign(n2);
		} else {
			Node n3=new Node(this,name,0);
			n3.assign(n2);
			XS.put(name,n3);
		}
	}

	public double rvalue(String name) {
		return rvalue(name,0);
	}

	private double rvalue(String name,double dflt) {
		if (isAux(name)) {
			return ((Node)XS.get(name)).Evaluate(dflt);
		} else if (isVar(name)) {
			return Var(name).getDouble();
		}
		return dflt;
	}

	public String svalue(String name) {
		if (isAux(name)) {
			return ((Node)XS.get(name)).getStr();
		} else if (isVar(name)) {
			return Var(name).getStr();
		} else {
			return null;
		}
	}

/* --------- some private auxiliary methods --------- */

	public Algorithm funcAlg(String name) {
		if (FAS.containsKey(name+".func-alg")) {
			return (Algorithm) FAS.get(name+".func-alg");
		} else {
			return null;
		}

	}

	private Node defFunc(String name,String[] varName) {
		defineFunc(name,varName,"0",false);
		return (Node)FS.get(name);
	}
	
	private Node Func(String name) {
		if (isFun(name)) {
			return (Node)FS.get(name);
		} else {
			return null;
		}
	}
	
	public boolean isAux(String name) {
		return XS.containsKey(name);
	}
	public boolean isVar(String name) {
		return VS.containsKey(name);
	}
	public boolean isArray(String name) {
		return AS.containsKey(name);
	}
	public Node getArray(String name) {
		if (AS.containsKey(name)) {
			return (Node) AS.get(name);
		}
		return null;
	}
	public boolean isMatrix(String name) {
		return MS.containsKey(name);
	}
	public Node getMatrix(String name) {
		if (MS.containsKey(name)) {
			return (Node) MS.get(name);
		}
		return null;
	}
	private boolean isFun(String name) {
		return FS.containsKey(name);
	}
	private boolean isFunDer(String name) {
		if ( (name.length()>1) && name.endsWith(Node.derivation_symbol)) {
			String s0=name.substring(0,name.length()-1);
			return isFun(s0);
		}
		return false;
	}

/*----------------------- Check Style ----------------------*/

    public static String checkStyle(String expr) {
    	String s="";
    	if (
    		expr.indexOf("--")>=0 ||
    		expr.indexOf("++")>=0 ||
    		expr.indexOf("+-")>=0 ||
    		expr.indexOf("-+")>=0
    		) {
    		s+="\n signos consecutivos";
    	}
    	String ps=new String(expr);
    	while (ps.length()>0) {
	    	int i1=ps.indexOf("((");
    		if (i1>=0) {
		    	int i2=i1+ps.substring(i1).indexOf("))");
    			if (i2>i1) {
	    			int count=0;
	    			for (int i=i1+2;i<i2;i++) {
	    				if (ps.charAt(i)=='(') {
	    					count++;
	    				} else if (ps.charAt(i)==')') {
	    					count--;
	    				}
	    			}
	    			if (count==0) {
	    				s+="\n parntesis innecesarios";
	    				break;
	    			}
	    		}
    			ps=ps.substring(i1+1);
    		} else {
    			break;
    		}
    	}
    	if (s.length()>0) {
    		s="Hay errores de estilo:"+s;
    	}
    	return s;
    }
    
    public String evaluateString(String str,int precision,boolean fixed) {
    	Vector V=parseString(str);
    	String s="";
    	for (int i=0;i<V.size();i++) {
    		if (V.elementAt(i) instanceof Node) {
    			Node N=(Node) V.elementAt(i);
				double d=N.Evaluate(0);
				s+=N.getStringValue(precision,fixed);
    		} else {
    			s+=(String) V.elementAt(i);
    		}
    	}
    	return s;
    }

	private Vector parseString(String str) {
		Vector v=new Vector();
		if (BasicStr.hasContent(str)) {
			v.addElement("");
			StringBuffer sb=new StringBuffer("");
			int state=0,level=0;
			boolean escape=false;
			for (int i=0;i<str.length();i++) {
				char ch=str.charAt(i);
				if (escape) {
					sb.append(ch);
					escape=false;
				} else {
					if (ch=='\\') {
						escape=true;
					} else {
						if (ch=='[') {
							level++;
						} else if (ch==']') {
							level--;
						}
						if ((level==1&&ch=='[')||(level==0&&ch==']')) {
							if (sb.length()>0) {
								String auxs=new String(sb);
								if (state==0) {
									v.addElement(auxs);
								} else {
									v.addElement(Analyse(auxs));
								}
								sb=new StringBuffer("");
							}
							state=1-state;
						} else {
							sb.append(ch);
						}
						if (level<0) {
							level=0;
						}
					}
				}
			}
			String auxs=new String(sb);
			if (state==0) {
				v.addElement(auxs);
			} else {
				v.addElement(Analyse(auxs));
			}
		}
		return v;
	}

/*----------------------- Analysis-Tree construction ----------------------*/

	String whereIsIt="";

	public Node Analyse(String expression,double defaultValue) {
		return Analyse(expression,Double.toString(defaultValue));
	}
	public Node Analyse(String expression,String defaultExpression) {
//		whereIsIt=wit;
		Node N=Analyse(expression);
		if (N==null) {
			N=Analyse(defaultExpression);
		}
		return N;
	}
	public Node Analyse(String expression) {
		Node N=null;
		try {
			N=Analyse("",expression,true);
		} catch (Exception e) {
//			e.printStackTrace();
		}
		return N;
	}

	Node Analyse(String wit,String expression,boolean defineVars) throws Exception {
		return Analyse(wit,expression,defineVars,true); // OJO
	}
	
	public Node Analyse(String wit,String EXPR,boolean defineVars,boolean reportErrors) throws Exception {
		whereIsIt=wit;
		String expression=new String(EXPR);
		if (!BasicStr.hasContent(expression)) {
			return null;
		} 
		if (expression.startsWith("|") && expression.endsWith("|")) {	// modificado el 15 de junio de 2009
			expression=BasicStr.replace(expression,"|","'");
		}
		if (!expression.startsWith("'")) {
			expression=BasicStr.replace(expression,"\u00b2","^2");
			expression=BasicStr.replace(expression,"\u00b3","^3");
		}
		if (expression.startsWith("'") && expression.indexOf(1,'\'')==expression.length()-1) {	// 12 de agosto de 2010, ver 4.105
			Node N=new Node(this,0.0);
			N.setOnlyStr(BasicStr.trim(expression,'\''));
			return N;
		}
		String[] tok=Node.getTokens(expression);
		Stack NS=new Stack();
		Stack TNS=new Stack();
		int synstate=0;
		int countpar=0,countkeys=0;
		int opstate=0;
		int node_level=0;
		int ix=0;
		while (ix<tok.length) {
			String s=tok[ix++];
			Node node=null;
			// el !isVar(s) || en la lnea que sigue se coment para la ver. 3.915, 28 de abril de 2008
			// para volver a permitir que una misma letra representa a una variable y a una funcin. 
			// Queda pendiente saber porqu se hab aconsiderado necesario incluirlo.
			if (/*!isVar(s) ||*/ ( ix<tok.length && (tok[ix].startsWith("(")||tok[ix].startsWith("[")))) {
				if (ix<tok.length && tok[ix].startsWith("[")) {
					if (isMatrix(s)) {
						node=new Node(this,s,Matrix(s),true);
					} else {
						node=new Node(this,s,Array(s),false);
					}
				} else {
					if (isFun(s)) {
						node=new Node(this,s,false,Func(s),funcAlg(s));
					} else if (isFunDer(s)) {
						String fn=s.substring(0,s.length()-1);
						node=new Node(this,s,true,Func(fn),funcAlg(fn));
					} else if (ix<tok.length && tok[ix].startsWith("(") && Node.symbol(s)==Node.VAR ) {
						node=new Node(this,s,false,defFunc(s,new String[0]),funcAlg(s));
						node.predefined=true;
						node.level=NS.size();
					}
				}
			}
			if (node==null) {
				node=new Node (this,s,isVar(s));
			}
			switch (node.type) {
			case Node.Rp :
			case Node.Vi :
			case Node.Coma :
			case Node.Za : opstate=1; break;
			case Node.In : 	if (opstate==0) { node.type=Node.Us; }
							else { node.type=Node.B1; }
			case Node.Ua :
			default      : opstate=0; break;
			}
			if (node.type!=Node.Sep) {
				if (node.type==Node.Rp) {
					synstate=1;
					if (countpar>0) {
						countpar--;
						while (true) {
							Node N=(Node)TNS.pop();
							if (N.type==Node.Lp) {
								if (!TNS.empty()) {
									Node tN=(Node)TNS.peek();
									if (tN.type==Node.Ua) {
										tN.right=new Node[tN.NumVars()];
									}
								}
								break;
							}
							addNode(NS,N,defineVars,reportErrors);
						}
					} else {
						reportError("too many ) in "+expression,reportErrors);
					}
				} else if (synstate==0) {
					switch (node.type) {
					case Node.Lp : countpar++;
					case Node.Us :
					case Node.Ua : 
									TNS.push(node); break;
					case Node.Za : 
					case Node.Vi : addNode(NS,node,defineVars,reportErrors); synstate=1; break;
					case Node.Str : addNode(NS,node,defineVars,reportErrors); synstate=1; break;
					
					default      :
//						if (isMatrixExpression) {
//							System.out.println("using a matrix as a variable "+node.toExpression());
//							addNode(NS,node,defineVars,reportErrors); synstate=1; break;
// 						} else {
							reportError("expression expected in "+expression+"\nAt token "+ix+" \""+node.getSymbol()+"\"",reportErrors);
//						}
					}
				} else {
					switch (node.type) {
					case Node.Coma :
						if (countpar>0) {
							while (true) {
								Node N=(Node)TNS.pop();
								if (N.type==Node.Lp) {
									TNS.push(N);
									break;
								}
								addNode(NS,N,defineVars,reportErrors);
							}
						} else {
							reportError(", not expected in "+expression,reportErrors);
						}
						synstate=0;
						opstate=0;
						break;
					case Node.As : case Node.B0: case Node.B1 : case Node.B2:
					case Node.If : case Node.ThenElse:
						while (!TNS.isEmpty()) {
							Node N=(Node)TNS.peek();
							if (N.type<node.type) {
								break;
							}
							addNode(NS,(Node)TNS.pop(),defineVars,reportErrors);
						}
						TNS.push(node); synstate=0;
						break;
					case Node.B3 :
						while (!TNS.isEmpty()) {
							Node N=(Node)TNS.peek();
							if (N.type<=node.type) {
								break;
							}
							if (N.type==Node.Us) {
								break;
							}
							addNode(NS,(Node)TNS.pop(),defineVars,reportErrors);
						}
						TNS.push(node); synstate=0;
						break;
					default      :
//						if (isMatrixExpression) {
//							System.out.println("Pushing matrix node "+node.toExpression());
//							TNS.push(node); synstate=0;
//						} else {
							reportError("operator expected in "+expression+"\nAt token "+ix+" \""+node.getSymbol()+"\"",reportErrors);
//						}
					}
				}
			} else {
				if (synstate==0) {
					reportError("incomplete expression "+expression,reportErrors);
				}
				if (countpar>0) {
					reportError(") expected in "+expression,reportErrors);
				}
				while (!TNS.isEmpty()) {
					addNode(NS,(Node)TNS.pop(),defineVars,reportErrors);
				}
				Node resultN=(Node)NS.peek();
				try {
					Node.check(resultN);
				} catch (Exception e) {
					System.out.println("NOT OK expr="+expression+" //"+defineVars);
					e.printStackTrace();
				}
				return resultN;
			}
		}
		reportError("Parser internal error.",reportErrors);
		return null;
	}

	private void addNode(Stack NS,Node node,boolean defineVars,boolean reportErrors) throws Exception {
		switch (node.type) {
		case Node.Us : node.right=new Node[1];
		case Node.Ua :
			Node funcN=Func(node.getSymbol());
			if (funcN!=null && node.predefined) {
				int NV=NS.size()-node.level;
				node.right=new Node[NV];
				funcN.var=new Node[NV];
				for (int i=0;i<NV;i++) {
					funcN.var[i]=Var("x"+i);
				}
			}
			for (int i=node.right.length-1;i>=0;i--) {
				if (!NS.isEmpty()) {
					node.right[i]=(Node)NS.pop();
				}
			}
			if (funcN!=null && node.predefined) {
//				System.out.println("function pre-defined "+node.toExpression());
				node.predefined=false;
			}
			break;
		case Node.Za : break;
		case Node.B0 :
		case Node.B1 :
		case Node.B2 :
		case Node.B3 :
		case Node.If:
		case Node.ThenElse:
		case Node.As :
			node.right=new Node[1];
			if (!NS.isEmpty()) {
				node.right[0]=(Node)NS.pop();
			}
			if (!NS.isEmpty()) {
				node.left=(Node)NS.pop();
			}
			break;
		case Node.Vi :
			String symbol=node.getSymbol();
			if (isAux(symbol)) {
				node=(Node) XS.get(symbol);
			} else if (isVar(symbol)) {
				node=(Node) VS.get(symbol);
			} else if (isMatrix(symbol)) {
				node=(Node) MS.get(symbol);
			} else {
				if (symbol.startsWith(".")) {
					symbol="0"+symbol;
				}
				try {
					node.setDouble(Double.valueOf(symbol).doubleValue());
				} catch(NumberFormatException nfe) {
					if (defineVars) {
						node=Var(node.getSymbol());
					} else {
						String errs="\""+node.getSymbol()+"\""+" no declarado";
						reportError(errs,reportErrors);
						throw new Exception(errs);
					}
				}
			}
			break;
		default      : node.type=Node.In; break;
		}
		NS.push(node); 
	}

/* ---------------------- String handling -------------------- */


	public String getSolution(String pattern) {
		StringTokenizer st=new StringTokenizer(pattern,"|");
		if (st.hasMoreTokens()) {
			String proposal=st.nextToken();
			if (proposal.startsWith("'") && proposal.endsWith("'") ) {
				proposal=BasicStr.toLowerCase(proposal.substring(1,proposal.length()-1));
			}
			if (proposal.startsWith("`") && proposal.endsWith("") ) {
				proposal=BasicStr.eliminateAccents(proposal.substring(1,proposal.length()-1));
			}
			if (proposal.startsWith("'") && proposal.endsWith("'") ) {
				proposal=BasicStr.toLowerCase(proposal.substring(1,proposal.length()-1));
			}
			if ( (proposal.startsWith("(")||proposal.startsWith("[")) && (proposal.endsWith(")")||proposal.endsWith("]"))) {
				String solstr=proposal.substring(1,proposal.length()-1);
				StringTokenizer st2;
				if (solstr.indexOf(":")>0) {
					st2=new StringTokenizer(solstr,":");
				} else {
					st2=new StringTokenizer(solstr,",");
				}
				if (st2.countTokens()==2) {
					String sa=st2.nextToken();
					String sb=st2.nextToken();
					try {
						Node Na=Analyse(BasicStr.replace(sa,",","."));
						Node Nb=Analyse(BasicStr.replace(sb,",","."));
						double a=Na.Evaluate(0);
						double b=Nb.Evaluate(0);
						if (sa.equals(sb) && Na.getStr()!=null) {
							return Na.getStr();
						} else if (a<=b) {
							return BasicStr.DoubleToString((a+b)/2,6,false,".");
						} else {
							return "No hay solucin";
						}
					} catch (Exception ex) {
						return "NaN";
					}
				}
			}
			return proposal;
		}
		return "";
	}
	
	boolean isOk(String pattern,String answer) {
		return isCorrect(0,pattern,answer);
	}

	boolean isok(String pattern,String answer) {
		return isCorrect(3,pattern,answer);
	}

	static final String RADIAL="--radial--"; 

	public boolean isCorrect(String pattern,String answer) {
		return isCorrect("",pattern,answer);
	}

	public boolean isCorrect(String varname,String pattern0,String answer) {
		boolean isOk=false;
		int rix=pattern0.toLowerCase().indexOf(RADIAL);
		if (rix==0) {
			String pattern2=pattern0.substring(rix+RADIAL.length()); 
			isOk=radialOk(varname,pattern2,answer);
		} else if (rix>0) {
			String pattern1=pattern0.substring(0,rix);
			String pattern2=pattern0.substring(rix+RADIAL.length()); 
			isOk=isCorrect(3,pattern1,answer) || radialOk(varname,pattern2,answer);
		} else {
			isOk=isCorrect(3,pattern0,answer);
		}
		if (BasicStr.hasContent(varname)) {
			Var(varname+"."+"ok").setDouble((isOk)?1:0);
		}
		return isOk;
	}


	private boolean radialOk(String varname,String pattern,String answer) {
		edu.lite.radial.RadialServices rs=new edu.lite.radial.RadialServices();
		String resp=rs.getAnswer(pattern,answer);
		boolean b=rs.isCorrect();
		String correction=rs.getCorrection();
		if (BasicStr.hasContent(varname)) {
			Var(varname+"."+"correction").setOnlyStr(correction);
		}
		if (b) {
			if (BasicStr.hasContent(correction)) { 
				System.out.println("correction="+correction);
			}
			System.out.print("RADIAL: ");
		}
		return b;
	}


	private boolean isCorrect(int style,String pattern,String answer0) {
		boolean isOk=false;
		int rix=pattern.toLowerCase().indexOf(RADIAL);
		if (rix==0) {
			String pattern2=pattern.substring(rix+RADIAL.length()); 
			isOk=radialOk("",pattern2,answer0);
			return isOk;
		} else if (rix>0) {
			String pattern1=pattern.substring(0,rix);
			String pattern2=pattern.substring(rix+RADIAL.length()); 
			isOk=isInternallyCorrect(style,pattern1,answer0) || radialOk("",pattern2,answer0);
		} else {
			isOk=isInternallyCorrect(style,pattern,answer0);
		}
//System.out.println(isOk+" "+answer0+" =?= "+pattern+" ");
		return isOk;
	}
		
	private boolean isInternallyCorrect(int style,String pattern,String answer0) {
		StringTokenizer st=new StringTokenizer(pattern,"|");
		while (st.hasMoreTokens()) {
			String pattern1=st.nextToken();
			StringTokenizer st1=new StringTokenizer(pattern1,"&");
			boolean[] correct=new boolean[st1.countTokens()];
			int n=0;
			while (st1.hasMoreTokens()) {
				String proposal=st1.nextToken();
				String answer=answer0;
				if (proposal.startsWith("'") && proposal.endsWith("'") ) {
					answer=BasicStr.toLowerCase(answer);
					proposal=BasicStr.toLowerCase(proposal.substring(1,proposal.length()-1));
				}
				if (proposal.startsWith("`") && proposal.endsWith("") ) {
					answer=BasicStr.eliminateAccents(answer);
					proposal=BasicStr.eliminateAccents(proposal.substring(1,proposal.length()-1));
				}
				if (proposal.startsWith("'") && proposal.endsWith("'") ) {
					answer=BasicStr.toLowerCase(answer);
					proposal=BasicStr.toLowerCase(proposal.substring(1,proposal.length()-1));
				}
				if (answer.equals(proposal)) {
					correct[n++]=true;
					continue;
				}
				switch (style) {
					case 1 :
						answer=BasicStr.toLowerCase(answer);
						proposal=BasicStr.toLowerCase(proposal);
						break;
					case 2 :
						answer=BasicStr.eliminateAccents(answer);
						proposal=BasicStr.eliminateAccents(proposal);
						break;
					case 3 :
						answer=BasicStr.toLowerCase(answer);
						proposal=BasicStr.toLowerCase(proposal);
						answer=BasicStr.eliminateAccents(answer);
						proposal=BasicStr.eliminateAccents(proposal);
//						proposal="*"+BasicStr.trim(proposal,'*')+"*";
						break;
					default:
						break;
				}
				if (proposal.endsWith("*") && answer.startsWith(proposal.substring(0,proposal.length()-1))) {
					correct[n++]=true;
					continue;
				}
				if (proposal.startsWith("*") && answer.endsWith(proposal.substring(1))) {
					correct[n++]=true;
					continue;
				}
				if (proposal.startsWith("*") && proposal.endsWith("*") 
					&& answer.indexOf(proposal.substring(1,proposal.length()-1))>=0) {
					correct[n++]=true;
					continue;
				}
				if ( (proposal.startsWith("(")||proposal.startsWith("[")) && (proposal.endsWith(")")||proposal.endsWith("]"))) {
					String solstr=proposal.substring(1,proposal.length()-1);
					StringTokenizer st2;
					if (solstr.indexOf(":")>0) {
						st2=new StringTokenizer(solstr,":");
					} else {
						st2=new StringTokenizer(solstr,",");
					}
					if (st2.countTokens()==2) {
						String sa=st2.nextToken();
						String sb=st2.nextToken();
						try {
							Node Na=Analyse(BasicStr.replace(sa,",","."));
							Node Nb=Analyse(BasicStr.replace(sb,",","."));
							Node Nr=Analyse(BasicStr.replace(answer,",","."));
							try {
								double a=Na.Evaluate(0);
								double b=Nb.Evaluate(0);
								double r=Nr.Evaluate(0);
								if (sa.equals(sb) && Na.getStr()!=null) {
									if (answer.equals(Na.getStr())) {
										correct[n++]=true;
										continue;
									}
								} else if ( 
									( (a<r)||(a<=r && proposal.startsWith("[")) ) 
								 && ( (r<b)||(r<=b && proposal.endsWith("]")) ) 
								 )  {
									correct[n++]=true;
									continue;
								}
							} catch (Exception ex) {
							}
						} catch (Exception ex) {
						}
					}
				}
				if (answer.length()==proposal.length()) {
					boolean differ=false;
					for (int j=0;j<answer.length();j++) {
						if (proposal.charAt(j)!='?' && proposal.charAt(j)!=answer.charAt(j)) {
							differ=true;
							break;
						}
					}
					if (!differ) {
						correct[n++]=true;
						continue;
					}
				}
				n++;
			}
			boolean isOk=true;
			for (int i=0;i<correct.length;i++) {
				isOk&=correct[i];
			}
			if (isOk) {
				System.out.println("INTERNAL  ");
			}
			if (isOk) return true;
		}
		return false;
	}

/* ------------------ Algorithms ----------------------- */

	public Algorithm defineConstant(String name,boolean evOnce,String expr,String s_ini,String s_while) {
		Algorithm alg=new Algorithm(name,this,evOnce,"Constant",expr,s_ini,s_while);
		ALS.addElement(alg);
		return alg;
	}

	public Algorithm defineAlgorithm(String name,boolean evOnce,String expr,String s_ini,String s_while) {
		Algorithm alg=new Algorithm(name,this,evOnce,"Algorithm",expr,s_ini,s_while);
		ALS.addElement(alg);
		return alg;
	}

	public void doAlgorithms(boolean firstTime) throws Exception {
		Enumeration an=ALS.elements();
		while (an.hasMoreElements()) {
			Object o=an.nextElement();
			if (o instanceof Algorithm) {
				Algorithm a=(Algorithm) o;
				if (firstTime||!a.evOnce) {
					a.perform();
				}
			} else if (o instanceof Node) { // means it is an array or Matrix
				Node N=(Node) o;
				if (MS.contains(N)) {
					evaluateMatrix(N,firstTime);
				} else if (AS.contains(N)) {
					evaluateArray(N,firstTime);
				}
			}
		}
		if (nsa_nippe!=null) {
			nsa_nippe.sendChanges();
		}
	}

/* ------------------ Arrays ----------------------- */

	public Node Array(String name) {
		if (!isArray(name)) {
			defineArray("3",name,"","",true);
		}
		return (Node) AS.get(name);
	}

	public Node Matrix(String name) {
		if (!isMatrix(name)) {
			defineMatrix("3","3",name,"","",true);
			new Exception("Creating nondefined matrix!").printStackTrace();
		}
		return (Node) MS.get(name);
	}

	public double[] getRealArray(String name) {
		if (isArray(name)) {
			return ((Node) AS.get(name)).rarray;
		}
		return null;
	}

	public double[][] getRealMatrix(String name) {
		if (isMatrix(name)) {
			return ((Node) MS.get(name)).rmatrix;
		}
		return null;
	}

	public String[] getStringArray(String name) {
		if (isArray(name)) {
			return ((Node) AS.get(name)).sarray;
		}
		return null;
	}

	public String[][] getStringMatrix(String name) {
		if (isMatrix(name)) {
			return ((Node) MS.get(name)).smatrix;
		}
		return null;
	}

	public static String getExprFromContent(String content,String id) {
		if (BasicStr.hasContent(content)) {
			String[] sa=BasicStr.toStringArray(content);
			String expr="";
			for (int i=0;i<sa.length;i++) {
				expr+=id+"["+i+"]="+sa[i]+";";
			}
			return expr;
		}
		return "";
	}
	
	public Vector getDataFilePaths() {
		return dataFiles;
	}
	
	public void defineArray(String s_size0,String name,String expr0,String filename,boolean evOnce) {
 		Node n2=null;
		Node n=null;
		String s_size=s_size0;
		String expr=expr0;
		boolean fromFile = false;
		if (BasicStr.hasContent(filename)) {
			try {
				n2=Analyse("Vector "+name,filename,false,false);
				n2.evOnce=evOnce;
				fromFile = true;
			} catch (Exception ex) {
				String str=mjaFile.readFile(nsa_nippe,nsa_nippe.inWeb(),filename,true);
				if (BasicStr.hasContent(str)) {
					dataFiles.add(filename);
					String[] sa=BasicStr.toStringArray(str);
					s_size=Integer.toString(sa.length);
					fromFile = true;
				}
				
				expr=Parser.getExprFromContent(str,name);
			}
		}
		
		if(!fromFile && //No esta el archivo. Revisamos si es asignacin explicita o una lista de valores
				BasicStr.hasContent(expr)) {
			String str = expr.replaceAll("/;\\s*;/", ";0;");//Cambiamos las lineas en blanco por 0 
			str = str.replaceAll(";", "\n");
			String[] sa = BasicStr.toStringArray(str);
			if (sa != null && sa.length > 0) {
				String pattern = "(.|\\s)*]\\s*=\\s*.*"; //		Buscamos asignaciones del tipo v[a]=valor
				boolean sinAsig = !sa[0].matches(pattern);//	en el primer renglon
				if (sinAsig) {
					expr = Parser.getExprFromContent(str, name);
					s_size = String.valueOf(sa.length);
				}
			} 
		}
		
		
		if (n2==null) {
	        int sz=(int) Math.round(Analyse(s_size,"3").Evaluate(3));
    	    Var(name+".long").setDouble(sz);
        	Node arrSz=Analyse(s_size);
	        n2 = new Node(this,name,arrSz,new Node[sz],evOnce);
    	    n = (Node) AS.get(name);
    	}
        if (n==null) {
        	n=n2;
            AS.put(name,n2);
        } else {
        	n.assign(n2);
        }
        int indexALS = ALS.indexOf(n.alg);
        Algorithm alg=new Algorithm(name,this,evOnce,"Vector",expr,"","");
        if (n.alg==null) {
            n.alg=alg;
        } else {
	        n.alg.assign(alg);    
        }
        if (indexALS<0) {
            ALS.addElement(n);
        }
	}

	public void evaluateArray(String name,boolean firstTime) {
		if (isArray(name)) {
			Node N=Array(name);
			try {
				evaluateArray(N,firstTime);
			} catch (Exception e) {
				System.out.println("Error evaluating array "+name);
			}
		}
	}

	private void evaluateArray(Node N,boolean firstTime) throws Exception {
		int sz=3;
		Algorithm alg=null;
		double[] rar;
		String[] sar;
		if (VS.contains(N)) {
			if ((!N.evOnce || firstTime) && (!N.filename.equals(N.getStr()))) {
				Enumeration en=AS.keys();
				String name="";
				while (en.hasMoreElements()) {
					String key=(String) en.nextElement();
					Object O=AS.get(key);
					if (O==N) {
						name=key;
						break;
					}
				}
				String str=mjaFile.readFile(nsa_nippe,nsa_nippe.inWeb(),N.getStr(),true);
				if (BasicStr.hasContent(str)) {
					String[] sa=BasicStr.toStringArray(str);
					sz=sa.length;
		    	    Var(name+".long").setDouble(sz);
					String expr=Parser.getExprFromContent(str,name);			
			        alg=new Algorithm(name,this,false,"Vector",expr,"","");
			        rar=new double[sz];
			        sar=new String[sz];
			        N.rarray=rar;
			        N.sarray=sar;
				}
			}
		} else {
			sz=(int) Math.round(N.size.Evaluate(3));
			alg=N.alg;
			rar=N.rarray;
			sar=N.sarray;
			if (rar.length!=sz) {
				double [] newrar=new double[sz];
				String[] newsar=new String[sz];
				if (!firstTime) {
					for (int i=0;i<Math.min(rar.length,newrar.length);i++) {
						newrar[i]=rar[i];
						newsar[i]=sar[i];
					}
				}
				N.rarray=newrar;
				N.sarray=newsar;
			}
		}
		if (alg!=null && (!N.evOnce || firstTime)) {
			alg.perform();
		}
	}

	public void defineMatrix(String s_columns,String s_rows,String name,String filename, String expr,boolean evOnce) {
//System.out.println("defineMatrix "+s_columns+" x "+s_rows);
        Node NCols=Analyse(s_columns,"3");
        Node NRows=Analyse(s_rows,"3");
        int cols=(int) Math.round(NCols.Evaluate(3));
        int rows=(int) Math.round(NRows.Evaluate(3));
        int sz=cols*rows;
        Var(name+".filas").setDouble(rows);
        Var(name+".columnas").setDouble(cols);
        Var(name+".rows").setDouble(rows);
        Var(name+".columns").setDouble(cols);
		Node n2=new Node(this,name,NCols,NRows,new Node[sz],evOnce);
        Node n = (Node) MS.get(name);
        if (n==null) {
        	n=n2;
            MS.put(name,n2);
        } else {
        	n.ncols=NCols;
        	n.nrows=NRows;
        	n.assign(n2);
        }
        int indexALS = ALS.indexOf(n.alg);
        Algorithm alg=new Algorithm(name,this,evOnce,"Matrix",expr,"","");
        if (n.alg==null) {
            n.alg=alg;
        } else {
	        n.alg.assign(alg);    
        }
        if (indexALS<0) {
            ALS.addElement(n);
        }
	}

	public void evaluateMatrix(String name,boolean firstTime) {
		if (isMatrix(name)) {
			Node N=Matrix(name);
			try {
				evaluateMatrix(N,firstTime);
			} catch (Exception e) {
				System.out.println("Error evaluating matrix "+name);
			}
		}
	}

	private void evaluateMatrix(Node N,boolean firstTime) throws Exception {
		try {
			int rows=(int) Math.round(N.nrows.Evaluate(3));
			int cols=(int) Math.round(N.ncols.Evaluate(3));
//System.out.println("MATRIX "+cols+" x "+rows);
//new Exception().printStackTrace();
			double[][] rmx=N.rmatrix;
			String[][] smx=N.smatrix;
			if (rmx==null || rmx[0]==null || rmx.length!=cols ||rmx[0].length!=rows) {
				double [][] newrmx=new double[cols][rows];
				String[][] newsmx=new String[cols][rows];
				if (!firstTime) {
					for (int j=0;j<Math.min(rmx.length,newrmx.length);j++) {
						for (int i=0;i<Math.min(rmx[j].length,newrmx[j].length);i++) {
							newrmx[i][j]=rmx[i][j];
							newsmx[i][j]=smx[i][j];
						}
					}
				}
				N.rmatrix=newrmx;
				N.smatrix=newsmx;
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		if (!N.evOnce || firstTime) {
			N.alg.perform();
		}
	}

	public static void analyseEquation(Parser parser,String equation) {
//		System.out.println(equation);
		StringTokenizer st=new StringTokenizer(equation,"=");
//		System.out.println("Analysis:");
		String s="Analysis:";
		int n=0;
		while (st.hasMoreTokens()) {
			if (n++>0) {
				s+="=";
				System.out.print("=");
			}
			String t=st.nextToken();
			s+=t;
//			System.out.println(t);
			Node N=null;
			try {
				N=parser.Analyse("Equation",t,true);
				if (N!=null) {
					s+=N.toExpression();
//					System.out.print(N.toExpression());
				}
			} catch (Exception e) {
				System.out.println(equation);
				System.out.println(s);
				e.printStackTrace();
			}
		}
//		System.out.println("");
	}

/*--------------------- Error Handling ------------------------*/
//	private boolean firstError=true;

	private String errmsg_a="_";

	private void reportError(String errmsg,boolean report) throws Exception {
		if (report && (!errmsg_a.equals(errmsg))) {
			errmsg_a=errmsg;
			System.out.println("\n"+errmsg+"\n"+whereIsIt);
			if ( nsa_nippe!=null ) {
				if (inNippeEditor()) {
					com.mja.gui.mjaText.msg(nsa_nippe,"Error",errmsg+"\n"+whereIsIt);
				} else {
					com.mja.gui.mjaText.msg(nsa_nippe,"Error",errmsg);
				}
			}
		}
		Exception e=new Exception(errmsg);
		throw e;
	}

}