/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package com.mja.poly;

import java.awt.*;
import java.util.*;
import java.awt.image.*;
import com.mja.math.*;
import com.mja.math3D.*;
import com.mja.text.*;
import com.mja.gui.*;
import com.mja.util.BasicStr;
import com.mja.parser.Node;

public class Face {

//	public static final viewModel colorModel=new viewModel(viewModel.COLOR),
//								  lightModel=new viewModel(viewModel.LIGHT),
//								  metalModel=new viewModel(viewModel.METAL),
//								  wireModel =new viewModel(viewModel.WIRE);
	static final R3 Ux=new R3(1,0,0); // Uy=new R3(0,1,0), Uz=new R3(0,0,1);
	static final double epsilon=R3.epsilon;

	// package variables
	R3[] P;

	// variables used for rendering
	boolean drawn;
	boolean pointsOut;				// for determining face color
	R3 unitToEye=new R3(0,0,0);
	R3 unitNormal=new R3(0,0,0);

	private Color FC=Color.white, BC=Color.pink, EC=Color.gray;
	private boolean drawEdges=false;
	private viewModel vM=new viewModel();
	private viewR3 view;
	private int widthness=1;
	private Polygon poly;
	private R2[] pr;
	private int minx,miny,maxx,maxy;
	private double normalToEye;
	private double minDistanceToEye, maxDistanceToEye;
	double aveDistanceToEye;
	R3 average;
	private MathText mt;

	/* ---------------------- Constructors ------------------ */

	public Face cloneFace() {
		R3[] nP=new R3[P.length];
		for (int i=0;i<P.length;i++) {
			nP[i]=P[i].cloneR3();
		}
		Face nF=new Face(nP);
		nF.init();
		nF.FC=FC;
		nF.BC=BC;
		nF.EC=EC;
		nF.drawEdges=drawEdges;
		nF.vM=vM;
		nF.mt=mt;
		return nF;
	}

	public Face(R3[] p) { // General (plane) Polygonal Face
		boolean ok=true;
		for (int i=0;i<p.length;i++) {
			if (p[i]==null) {
				ok=false;
				break;
			}
		}
		if (ok) {
			P=p;
		} else {
			P=new R3[0];
		}
		init();
	} ;

	public Face(R3 p0) { // Point;
		if (p0!=null) {
			P=new R3[1];
			P[0]=p0;		
		} else {
			P=new R3[0];
		}
		init();
	}

	public Face(R3 p0,R3 p1) { // Segment;
		if ((p0!=null)&&(p1!=null)) {
			P=new R3[2];
			P[0]=p0; P[1]=p1;
		} else {
			P=new R3[0];
		}
		init();
	}

	public Face(R3 p0,R3 p1,R3 p2) { // Triangle;
		if ((p0!=null)&&(p1!=null)&&(p2!=null)) {
			P=new R3[3];
			P[0]=p0; P[1]=p1; P[2]=p2;
		} else {
			P=new R3[0];
		}
		init();
	}

	public Face(R3 p0,R3 p1,R3 p2,R3 p3) {  // Rectangle;
		if ((p0!=null)&&(p1!=null)&&(p2!=null)&&(p3!=null)) {
			P=new R3[4];
			P[0]=p0; P[1]=p1; P[2]=p2; P[3]=p3;
		} else if ((p1!=null)&&(p2!=null)&&(p3!=null)) {
			P=new R3[3];
			P[0]=p1; P[1]=p2; P[2]=p3;
		} else if ((p0!=null)&&(p2!=null)&&(p3!=null)) {
			P=new R3[3];
			P[0]=p0; P[1]=p2; P[2]=p3;
		} else if ((p0!=null)&&(p1!=null)&&(p3!=null)) {
			P=new R3[3];
			P[0]=p0; P[1]=p1; P[2]=p3;
		} else if ((p0!=null)&&(p1!=null)&&(p2!=null)) {
			P=new R3[3];
			P[0]=p0; P[1]=p1; P[2]=p2;
		} else {
			P=new R3[0];
		}
		init();
	}

	public Face(R3 p0,R3 p1,R3 p2,R3 p3,R3 p4) {  // Pentagon;
		if ((p0!=null)&&(p1!=null)&&(p2!=null)&&(p3!=null)&&(p4!=null)) {
			P=new R3[5];
			P[0]=p0; P[1]=p1; P[2]=p2; P[3]=p3; P[4]=p4;
		} else {
			P=new R3[0];
		}
		init();
	}

	public Face(R2[] p) {  // Cara plana;
		boolean ok=true;
		for (int i=0;i<p.length;i++) {
			if (p[i]==null) {
				ok=false;
				break;
			}
		}
		if (ok) {
			P=new R3[p.length];
			for (int i=0;i<P.length;i++) {
				P[i]=new R3(p[i].x,p[i].y,0);
			}
		} else {
			P=new R3[0];
		}
		init();
	}

	public void setWidthness (int w) {
		this.widthness=w;
	}

	public void setViewModel (viewModel vM) {
		this.vM=vM;
	}
	
	viewModel getViewModel() {
		return vM;
	}

	public void setFrontColor(Color c) {
		FC=c;
	}

	public void setEdgeColor(Color c) {
		EC=c;
	}

	public void setBackColor(Color c) {
		BC=c;
	}

	public Color getEdgeColor() {
		return EC;
	}

	public Color getFrontColor() {
		return FC;
	}

	public Color getBackColor() {
		return BC;
	}

	public void setDrawEdges(boolean oe) {
		this.drawEdges=oe;
	}

	public boolean getDrawEdges() {
		return drawEdges;
	}

	public double getArea() {
		double A=0;
		for (int i=1;i<P.length-1;i++) {
			A+=(P[0].Subtract(P[i]).CrossProduct(P[0].Subtract(P[i+1]))).Norm()/2;
		}
		return A;
	}

	/* ---------------------- Face Rendering -------------------- */

	private void init() {
		Vector V=new Vector();
		for (int j=0;j<P.length;j++) {
			if (!V.contains(P[j])) {
				boolean add=true;
				Enumeration e=V.elements();
				while (e.hasMoreElements()) {
					R3 test=(R3) e.nextElement();
					if (test.Subtract(P[j]).Norm()<epsilon) {
						add=false;
						break;
					}
				}
				if (add) {
					V.addElement(P[j]);
				}
			}
		}
		P=new R3[V.size()];
		Enumeration e=V.elements();
		for (int i=0;i<P.length;i++) {
			P[i]=(R3) e.nextElement();
		}
		calculateUnitNormal();
	}

	void calculateUnitNormal() {
		if (P.length==0) {
			unitNormal=null;
		} else if (P.length==1) {
			unitNormal=Ux;
		} else if (P.length==2) {
			R3 p=P[1].Subtract(P[0]);
			R3 q=Ux.CrossProduct(p);
			unitNormal=p.CrossProduct(q);
		} else {
			unitNormal=new R3(0,0,0);
			for (int i=0;i<P.length-2;i++) {
				R3 p=P[i+1].Subtract(P[0]);
				R3 q=P[i+2].Subtract(P[0]);
				unitNormal=unitNormal.Add(p.CrossProduct(q));
			}
		}
		if (unitNormal!=null) {
			double N=unitNormal.Norm();
			if (N!=0.0) {
				unitNormal=unitNormal.ScalarProduct(1/N);
			} else {
				unitNormal=Ux;
			}
		}
	}

	void updateView (viewR3 v) {
		if (P.length>0) {
			view=v;
			minx=Integer.MAX_VALUE;
			miny=Integer.MAX_VALUE;
			maxx=-Integer.MAX_VALUE;
			maxy=-Integer.MAX_VALUE;
			pr=new R2[P.length+1];
			poly = new Polygon();
			for (int i=0; i<P.length; i++) {
				pr[i]=view.pr(P[i]);
				poly.addPoint((int) Math.round(pr[i].x),(int) Math.round(pr[i].y));
				if (minx>poly.xpoints[i]) { minx=poly.xpoints[i]; }
				if (maxx<poly.xpoints[i]) { maxx=poly.xpoints[i]; }
				if (miny>poly.ypoints[i]) { miny=poly.ypoints[i]; }
				if (maxy<poly.ypoints[i]) { maxy=poly.ypoints[i]; }
			}
			pr[P.length]=pr[0];
			poly.addPoint(poly.xpoints[0],poly.ypoints[0]);

			if (P.length>1) {
				minDistanceToEye=Double.MAX_VALUE;
				maxDistanceToEye=-Double.MAX_VALUE;
				for (int i=0;i<P.length;i++) {
					double d=P[i].Subtract(view.Eye).Norm();
					minDistanceToEye=Math.min(minDistanceToEye,d);
					maxDistanceToEye=Math.max(maxDistanceToEye,d);
				}
				average=new R3(P[0].x,P[0].y,P[0].z);
				for (int i=1;i<P.length;i++) {
					average.x+=P[i].x; average.y+=P[i].y; average.z+=P[i].z;
				}
				average.x/=P.length; average.y/=P.length; average.z/=P.length;
				R3 toEye=view.Eye.Subtract(average);
				aveDistanceToEye=toEye.Norm();
				unitToEye=toEye.ScalarProduct(1/aveDistanceToEye);
				pointsOut=unitToEye.DotProduct(unitNormal) > 0;
				normalToEye=average.Subtract(view.Eye).DotProduct(unitNormal);
			} else {
				double d=P[0].Subtract(view.Eye).Norm();
				minDistanceToEye=d;
				maxDistanceToEye=d;
				average=P[0];
				R3 toEye=view.Eye.Subtract(average);
				aveDistanceToEye=d;
				unitToEye=toEye.ScalarProduct(1/aveDistanceToEye);
				pointsOut=true;
				normalToEye=average.Subtract(view.Eye).DotProduct(unitNormal);
			}
		}
	}

	private double svalue=0;
	private Node sNode=null;
	
	public void setSValue(double sv) {
		this.svalue=sv;
	}
	public void setSNode(Node sN) {
		this.sNode=sN;
	}

	void setMathText(MathText mt) {
		this.mt=mt;
	}

	private Container C;
	
	public void draw(Container C,Graphics2D g2) {
		this.C=C;
		Stroke bs=new BasicStroke(widthness,BasicStroke.CAP_ROUND,BasicStroke.JOIN_ROUND);
		g2.setStroke(bs);
		if (P.length==0) {
		} else if (P.length==1) {
			if (widthness>0) {
				bs=new BasicStroke(1,BasicStroke.CAP_ROUND,BasicStroke.JOIN_ROUND);
				g2.setStroke(bs);
				g2.setColor(BC);
				int sz=1+widthness+widthness;
				g2.fillOval(poly.xpoints[0]-widthness,poly.ypoints[0]-widthness,sz,sz);
				g2.setColor(FC);
				g2.drawOval(poly.xpoints[0]-widthness,poly.ypoints[0]-widthness,sz,sz);
			}
		} else if (P.length==2) {
			g2.setColor(FC);
			g2.drawLine(poly.xpoints[0],poly.ypoints[0],poly.xpoints[1],poly.ypoints[1]);
		} else if (vM.getModel()==viewModel.WIRE) {
			if (widthness>0) {
				g2.setColor(FC);
				g2.drawPolygon(poly);
			}
		} else {
			g2.setColor(vM.getFaceColor(this));
			if (BasicStr.antialias) {
			    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
			}
			g2.fillPolygon(poly);
			if (BasicStr.antialias) {
			    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
			}
			if (drawEdges) {
				g2.setColor(Color.gray);
				g2.drawPolygon(poly);
			}
		}
		drawText(C,g2);
		bs=new BasicStroke(1);
		g2.setStroke(bs);
		drawn=true;
	}
	
	String toTeX(Component C,int n,int H,boolean grayscale) {
		Color BC=(P.length>2)?vM.getFaceColor(this):this.BC;
		String str="";
		String bcn="bcolor";
		String fcn="fcolor";
		if (P.length==0) {
		} else if (P.length==1) { // es un punto
			if (widthness>0) {
				str=str+BasicStr.DefineColor(grayscale,bcn,BC);
				str=str+"\\pscircle[linewidth=0.1pt,fillstyle=solid,fillcolor="+bcn+"]("+BasicStr.DblStr(poly.xpoints[0])+","+BasicStr.DblStr(H-poly.ypoints[0])+"){"+BasicStr.DblStr(widthness+1)+"}\r\n";
				str=str+mt.getTeXColors(grayscale);
				str=str+"\\uput{"+BasicStr.DblStr(widthness)+"pt}[30]("+BasicStr.DblStr(poly.xpoints[0])+","+BasicStr.DblStr(H-mt.getTextY(C))+"){\\shortstack[l]{"+mt.toTeX()+"}}\r\n";
			}
		} else if (P.length==2) {
			str=str+BasicStr.DefineColor(grayscale,fcn,FC);
			str=str+"\\psline[showpoints=false,linewidth="+BasicStr.DblStr(widthness/2)+"pt,linecolor="+fcn+"]";
			str=str+"("+BasicStr.DblStr(poly.xpoints[0])+","+BasicStr.DblStr(H-poly.ypoints[0])+")";
			str=str+"("+BasicStr.DblStr(poly.xpoints[1])+","+BasicStr.DblStr(H-poly.ypoints[1])+")\r\n";
		} else {
			if (vM.getModel()==viewModel.WIRE) {
				if (widthness>0) {
					str=str+BasicStr.DefineColor(grayscale,fcn,FC);
					str=str+"\\psline[linewidth=0.5pt,linecolor="+fcn+"]";
					for (int j=0;j<P.length;j++) {
						str=str+"("+BasicStr.DblStr(poly.xpoints[j])+","+BasicStr.DblStr(H-poly.ypoints[j])+")";
					}
					str=str+"\r\n";
				}
			} else {
				str=str+BasicStr.DefineColor(grayscale,bcn,BC);
				if (drawEdges) {
					str=str+BasicStr.DefineColor(grayscale,fcn,Color.gray);
					str=str+"\\pspolygon[linewidth=0.5pt,linecolor="+fcn+",fillcolor="+bcn+",fillstyle=solid]";
				} else {
					str=str+"\\pspolygon[linewidth=0.01pt,linecolor="+bcn+",fillcolor="+bcn+",fillstyle=solid]";
				}
				for (int j=0;j<P.length;j++) {
					str=str+"("+BasicStr.DblStr(poly.xpoints[j])+","+BasicStr.DblStr(H-poly.ypoints[j])+")";
				}
				str=str+"("+BasicStr.DblStr(poly.xpoints[0])+","+BasicStr.DblStr(H-poly.ypoints[0])+")";
				str=str+"\r\n";
			}
		}
		return str+"\r\n";
	}

	void drawText(Container C,Graphics2D g) {
		if (mt!=null) {
			Graphics2D[] ga=new Graphics2D[1];
			ga[0]=g;
			R2 q=new R2(poly.xpoints[0]+5,poly.ypoints[0]-40);
			double savesv=sNode.getDouble();
			sNode.setDouble(svalue);
			mt.draw(C,q.ix(),q.iy(),ga);
			sNode.setDouble(savesv);
		}
	}

/* ----------------------------- Split Face ----------------------- */

	Face[] split(Face F) {
		if ( P.length>=3 && F.P.length>=2 && intersects(F) ) {
			R3 i1=null,i2=null;
			int ix1=0,ix2=0;
			for (int i=0;i<F.P.length;i++) {
				R3 Int=intersection(F.P[i],F.P[(i+1)%F.P.length]);
				if (Int!=null) {
					if (i1==null) {
						i1=Int;
						ix1=i;
						if (F.P.length==2) {
							i2=i1;
							break;
						}
					} else if (!Int.equals(i1,epsilon)) {
						i2=Int;
						ix2=i;
						break;
					}
				}
			}
			if ( i1!=null && i2!=null ) {
				boolean oneIsInterior=isInterior(i1)||isInterior(i2);
				R3 j1=null,j2=null;
				if (!oneIsInterior && F.P.length>=3) {
					for (int j=0;j<P.length;j++) {
						R3 Int=F.intersection(P[j],P[(j+1)%P.length]);
						if (Int!=null) {
							if (j1==null) {
								j1=Int;
							} else if (!Int.equals(j1,epsilon)) {
								j2=Int;
								break;
							}
						}
					}
				}
				if (oneIsInterior || (j1!=null&&j2!=null &&F.isInterior(j1)&&F.isInterior(j2)) ) {
					boolean splitted=true;
					R3[] P0=F.P;
					R3[] V=null,v=null;
					if (P0.length==2) {
						if (i1.equals(P0[0],epsilon) || i1.equals(P0[1],epsilon)) {
							splitted=false;
						} else {
							V=new R3[2];
							V[0]=P0[0]; V[1]=i1;
							v=new R3[2];
							v[0]=i1; v[1]=P0[1];
						}
					} else {
						V=new R3[2+ix1+P0.length-ix2];
						v=new R3[ix2-ix1+2];
						int J=0,j=0,k=0;
						for (int i=0;i<P0.length;i++) {
							if (i<ix1) {
								V[J++]=P0[i];
							} else if (i==ix1) {
								V[J++]=P0[i];
								V[J++]=i1;
								v[j++]=i1;
							} else if (i<ix2) {
								v[j++]=P0[i];
							} else if (i==ix2) {
								v[j++]=P0[i];
								v[j++]=i2;
								V[J++]=i2;
							} else {
								V[J++]=P0[i];
							}
						}
					}
					if (splitted) {
						Face[] fa=new Face[2];
						fa[0]=new Face(V);
						fa[1]=new Face(v);
						for (int i=0;i<fa.length;i++) {
							fa[i].vM=F.vM;
							fa[i].FC=F.FC;
							fa[i].BC=F.BC;
							fa[i].EC=F.EC;
							fa[i].drawEdges=F.drawEdges;
						}
						return fa;
					}
				}
			}
		}
		Face[] fa=new Face[1];
		fa[0]=F;
		return fa;
	}

	private boolean intersects(Face F) {
		return intersectsPlane(F) && F.intersectsPlane(this);
	}

	private boolean intersectsPlane(Face F) {
		if (P.length>0) {
			double d=F.P[0].DotProduct(F.unitNormal);
			double d0=P[0].DotProduct(F.unitNormal);
			if (Math.abs(d-d0)<epsilon) {
				return true;
			}
			for (int i=1;i<P.length;i++) {
				double di=P[i].DotProduct(F.unitNormal);
				if ( (Math.abs(d-di)<epsilon) || (di>d && d0<d) || (di<d && d0>d) ) {
					return true;
				}
			}
		}
		return false;
	}

	private R3 intersection(R3 p1,R3 p2) {
		if (P.length>0) {
			R3 p12=p2.Subtract(p1);
			double den=p12.DotProduct(unitNormal);
			if (den!=0) {
				double t=(P[0].Subtract(p1).DotProduct(unitNormal))/den;
				if (-epsilon<t && t<1+epsilon) {
					return p1.Add(p12.ScalarProduct(t));
				}
			}
		}
		return null;
	}

	private boolean isInterior(R3 r) {
		if (P.length>0) {
			double D=0;
			R3 u=P[0].Subtract(r);
			for (int i=0;i<P.length;i++) {
				R3 v=P[(i+1)%P.length].Subtract(r);
				double D1=u.CrossProduct(v).DotProduct(unitNormal);
				if (Math.abs(D1)<epsilon) {
					if (u.DotProduct(v)<0) {
						return true;
					}
				} else {
					if ( (D<0 && D1>0) || (D>0 && D1<0) ) {
						return false;
					}
					u=v;
					D=D1;
				}
			}
		}
		return true;
	}

/* ----------------- For the Sort and Paint algorithm --------------- */

//	boolean isCloserThan(Face F) {
//		return aveDistanceToEye<=F.aveDistanceToEye;
//	}

/* ----------------- For the Painter's Algorithm ------------------- */

	boolean inFrontOf(Vector V,Face F,double epsilon) {
		if (maxx<=F.minx || minx>=F.maxx || maxy<=F.miny || miny>=F.maxy) {
			return false;
		}
		if (minDistanceToEye>=F.maxDistanceToEye) {
			return false;
		}
		for (int state=0;state<3;state++) {
			Vector v=null;
			switch (state) {
			case 0:	v=intersections(F);
					break;
			case 1:	v=F.verticesContainedIn(this);
					break;
			case 2:	v=verticesContainedIn(F);
					break;
//			case 3:	v=F.intersections(this);
//					break;
			}
			if (v!=null && !v.isEmpty()) {
				for (int k=0;k<v.size();k++) {
					V.addElement(v.elementAt(k));
				}
				Enumeration e=v.elements();
				int numel=0;
				while (e.hasMoreElements()) {
					R2 p=(R2) e.nextElement();
					R3 ray=view.rayFromEye(p.x,p.y);
					try {
						double t=distanceToEyeAlong(ray)-F.distanceToEyeAlong(ray);
						if ( t<=-epsilon ) {
							return true;
						} else if ( t>=epsilon ) {
							return false;
						}
					} catch (Exception ex) {
						System.out.println("Error: inFrontOf");
//						return false;
					}
				}
			}
		}
		return false;
	}
	
	private Vector intersections(Face F) {
		Vector V=new Vector();
		for (int i=0;i<P.length;i++) {
			R3 pi=P[i],qi=P[(i+1)%P.length];
			for (int j=0;j<F.P.length;j++) {
				R3 pj=F.P[j],qj=F.P[(j+1)%F.P.length];
				if ( (pi!=pj) && (pi!=qj)  && (qi!=pj) && (qi!=qj) ) {
//				if (   (!pi.equals(pj,0.00000001)) 
//					&& (!pi.equals(qj,0.00000001))  
//					&& (!qi.equals(pj,0.00000001))  
//					&& (!qi.equals(qj,0.00000001))
//					)  {
					R2 ip=R2.intersection(pr[i],pr[(i+1)%P.length],F.pr[j],F.pr[(j+1)%F.P.length]);
					if (ip!=null) {
						V.addElement(ip);
					}
				}
			}
		}
		return V;
	}

	private double distanceToEyeAlong(R3 ray) throws Exception {
		double den=unitNormal.DotProduct(ray);
		if (Math.abs(den)>0.000001) {
			return normalToEye/den;
		}
		throw new Exception("Face is invisible");
	}

	private Vector verticesContainedIn(Face F) {
		Vector V=new Vector();
		for (int i=0;i<P.length;i++) {
			if (!F.isVertex(P[i]) && F.appearsToContain(pr[i]) ) {
				V.addElement(pr[i]);
			}
		}
		return V;
	}

	private boolean appearsToContain(R2 p) {
		double D=0;
		for (int i=0;i<P.length;i++) {
			double D1=((pr[i].x-p.x)*(pr[i+1].y-p.y))-((pr[i+1].x-p.x)*(pr[i].y-p.y));
			if (D!=0) {
				if (Math.abs(D1)<epsilon) {
					if (Math.abs(pr[i].x-pr[i+1].x)>epsilon) {
						return (Math.min(pr[i].x,pr[i+1].x)<=p.x+epsilon && p.x<=Math.max(pr[i].x,pr[i+1].x)+epsilon);
					} else if (Math.abs(pr[i].y-pr[i+1].y)>epsilon) {
						return (Math.min(pr[i].y,pr[i+1].y)<=p.y+epsilon && p.y<=Math.max(pr[i].y,pr[i+1].y)+epsilon);
					}
				} else if ( (D>0 && D1<0) || (D<0 && D1>0) ) {
					return false;
				}
			}
			D=D1;
		}
		return true;
	}

	private boolean isVertex(R3 p) {
		for (int i=0;i<P.length;i++) {
			if (p.equals(P[i])) {
				return true;
			}
		}
		return false;
	}

/* -------------------------- For Ray Tracing ----------------------- */

	Color getColorAt(int m,int n,boolean checkEdge) {
		if (checkEdge && drawEdges && isOnEdge(m,n)) {
			return EC;
		} else {
			return vM.getFaceColor(this);
		}
	}

	double intersectionToEye(double x,double y) {
		try {
			return distanceToEyeAlong(view.rayFromEye(x,y));
		} catch (Exception e) {
			return minDistanceToEye;
		}
	}


	boolean contains(int x,int y,int sz) {
		if (sz<=1) {
			return contains(x,y);
		} else {
			if (vM.getModel()==viewModel.WIRE) {
				return false;
			}
			return contains(x,y)&&contains(x+sz-1,y)&&contains(x+sz-1,y+sz-1)&&contains(x,y+sz-1);
		}
	}


	private boolean contains(int x,int y) {
		if (x<minx||x>maxx||y<miny||y>maxy) {
			return false;
		}
		for (int i=0;i<P.length;i++) {
			if (x==poly.xpoints[i] && y==poly.ypoints[i]) {
				return true;
			}
		}
		if (P.length==1) {
			return false;
		} else if (P.length==2) {
			return pointInSegment(x,y,poly.xpoints[0],poly.ypoints[0],poly.xpoints[1],poly.ypoints[1]);
		} else {
			if ((drawEdges||vM.getModel()==viewModel.WIRE) && isOnEdge(x,y)) {
				return true;
			}
			if (vM.getModel()==viewModel.WIRE) {
				return false;
			}
			return poly.contains(x,y);
		}
	}

	boolean touches(int x0,int y0,int sz) {
		if (sz==1) {
			return poly.contains(x0,y0);
		} else {
			return poly.intersects(x0,y0,sz,sz);
		}
	}

	boolean isOnEdge(int x0,int y0) {
		for (int i=0;i<P.length;i++) {
			if (pointInSegment(x0,y0,poly.xpoints[i],poly.ypoints[i],poly.xpoints[i+1],poly.ypoints[i+1])) {
				return true;
			}
		}
		return false;
	}


	static boolean pointInSegment(int x0,int y0, int x1, int y1, int x2, int y2) {
		return rectangleIntersectsSegment(x0,y0,1,1,x1,y1,x2,y2);
	}

	static boolean rectangleIntersectsSegment(int x0,int y0, int dx, int dy,int x1, int y1, int x2, int y2) {
		int nn=2;
		int[] xx={x1,x2},yy={y1,y2};
		Polygon pp=new Polygon(xx,yy,nn);
		if (pp.intersects(x0,y0,dx,dy)) {
			return true;
		}
		return false;
	}


}