/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package com.mja.poly;

import java.awt.Color;
import java.util.*;
import com.mja.math.*;
import com.mja.util.*;
import com.mja.parser.*;
import com.mja.gui.mjaColor;

public class SurfaceFactory {

	public static final int point=0;
	public static final int segment=1;
	public static final int polygon=2;
	public static final int arrow=3;
	public static final int curve=4;
	public static final int triangle=5;
	public static final int face=6;
	public static final int regpoly=7;
	public static final int box=8;
	public static final int cone=9;
	public static final int cylinder=10;
	public static final int tetrahedron=11;
	public static final int cube=12;
	public static final int octahedron=13;
	public static final int dodecahedron=14;
	public static final int icosahedron=15;
	public static final int sphere=16;
	public static final int ellipsoid=17;
	public static final int parametric=18;
	public static final int text=19;

	public static final String[] surfaceName = {
		"punto",
		"segmento",
		"polgono",
		"flecha",
		"curva",
		"tringulo",
		"cara",
		"polireg",
		"paraleleppedo",
		"cono",
		"cilindro",
		"tetraedro",
		"cubo",
		"octaedro",
		"dodecaedro",
		"icosaedro",
		"esfera",
		"elipsoide",
		"superficie",
		"texto"
	};

	public static final int getType(String str_type) {
		for (int i=0;i<surfaceName.length;i++) {
			if (surfaceName[i].equalsIgnoreCase(str_type)) {
				return i;
			}
		}
		return -1;
	}

	public static final String getEquation(int type) {
		switch (type) {
		case point          : return "";
		case segment        : return "";
		case polygon        : return "";
		case arrow          : return "";
		case curve          : return "x=_AnChO_*cos(pi*u) y=_LaRgO_*sen(pi*u) z=_AlTo_*(u-0.5)";
		case triangle		: return "";
		case face           : return "Cara poligonal plana";
		case regpoly        : return "U=2*pi*u x=(_AnChO_/2)*v*cos(U) y=-(_LaRgO_/2)*v*sin(U) z=0";
		case box            : return "Paraleleppedo de lados _AnChO_, _LaRgO_ y _AlTo_";
		case cone           : return "U=2*pi*u V=v x=(_AnChO_/2)*V*cos(U) y=(_LaRgO_/2)*V*sin(U) z=-(_AlTo_/2)+_AlTo_*V";
		case cylinder       : return "U=2*pi*u V=v x=(_AnChO_/2)*cos(U) y=(_LaRgO_/2)*sin(U) z=-(_AlTo_/2)+_AlTo_*V";
		case tetrahedron    : return "Tetraedro inscrito en la esfera de radio=_AnChO_/2";
		case cube           : return "Cubo inscrito en la esfera de radio=_AnChO_/2";
		case octahedron     : return "Octaedro inscrito en la esfera de radio=_AnChO_/2";
		case dodecahedron   : return "Dodecaedro inscrito en la esfera de radio=_AnChO_/2";
		case icosahedron    : return "Icosaedro inscrito en la esfera de radio=_AnChO_/2";
		case sphere         : return "U=pi*u V=2*pi*v x=(_AnChO_/2)*sen(U)*cos(V) y=(_AnChO_/2)*sen(U)*sin(V) z=(_AnChO_/2)*cos(U)";
		case ellipsoid      : return "U=pi*u V=2*pi*v x=(_AnChO_/2)*sen(U)*cos(V) y=(_LaRgO_/2)*sen(U)*sin(V) z=(_AlTo_/2)*cos(U)";
		case parametric     : return "x=_AnChO_*u-_AnChO_/2 y=_LaRgO_*v-_LaRgO_/2 z=_AlTo_*exp(-2*(x^2+y^2))";
		case text           : return "";
		default : return "";
		}
	}

	public static mjaSurface getMjaSurface(Parser p,String s_type,String equ,
											String s_Nu,String s_Nv,
											String s_gr,String s_w,String s_l, String s_h,mjaColor fc,mjaColor bc,
											boolean edges,String s_ir,String s_ip,String s_fr,String s_fp,String s_r,String s_p,
											int model, String cond,boolean fixed) {
		int type=getType(s_type);
		mjaSurface pS=new mjaSurface(p,type,s_Nu,s_Nv,s_gr,s_w,s_l,s_h,fc,bc,edges,s_ir,s_ip,s_fr,s_fp,s_r,s_p,model,fixed);
		pS.cond=p.Analyse(cond,"1");
		switch(type) {
		case regpoly        : pS.setEquation(getEquation(regpoly)); pS.setNv("1"); break;
		case cone           : pS.setEquation(getEquation(cone));  break;
		case cylinder       : pS.setEquation(getEquation(cylinder));  break;
		case sphere         : pS.setEquation(getEquation(sphere));  break;
		case ellipsoid      : pS.setEquation(getEquation(ellipsoid));  break;
		
		case text           : 
		case point          : pS.setEquation(getPointEquation(equ)); pS.setNu("0"); pS.setNv("0"); break;
		case segment        : pS.setEquation(getSegmentEquation(equ)); pS.setNv("0"); break;
		case polygon        : pS.setEquation(getPolygonEquation(equ)); 
							  pS.setNv("0"); 
							  pS.setNu(Integer.toString(countPoints(equ)-1)+"*("+s_Nu+")"); 
							  break;
		case triangle       : pS.set3DVertices(equ); break;
		case face           : pS.setVertices(equ); break;
		case arrow          : pS.setEquation(getSegmentEquation(equ)); pS.setNv("0"); break;
		case curve          : pS.setNv("0");
		case parametric     : pS.setEquation(equ); break;
		}
		return pS;
	}

	private static final String getPointEquation(String equ) {
		String s="x=0 y=0 z=0";
		try {
			String[] st=BasicStr.getTokens(equ);
			if (st.length==3) {
				s="";
				s+="x="+st[0];
				s+=" y="+st[1];
				s+=" z="+st[2];
			}
		} catch (Exception e) {
		}
		return s;
	}

	private static final String getSegmentEquation(String equ) {
		String s="x=u y=u z=u";
		try {
			String[] st=BasicStr.getTokens(equ);
			if (st.length==6) {
				String sx1=st[0];
				String sy1=st[1];
				String sz1=st[2];
				String sx2=st[3];
				String sy2=st[4];
				String sz2=st[5];
				s="x="+sx1+"+u*(("+sx2+")-("+sx1+"))"
				+" y="+sy1+"+u*(("+sy2+")-("+sy1+"))"
				+" z="+sz1+"+u*(("+sz2+")-("+sz1+"))";
			}
		} catch (Exception e) {
		}
		return s;
	}
	
	private static int countPoints(String equ) {
		StringTokenizer st=new StringTokenizer(equ,"(,)");
		return st.countTokens()/3;
	}

	private static final String getPolygonEquation(String equ) {
		String[] st=new String[0];
		try {
			st=BasicStr.getTokens(equ);
		} catch (Exception e) {
		}
		int N=st.length/3;
		String sN=Integer.toString(N-1);
		String x_s="x=",y_s="y=",z_s="z=";
		String sxa="",sya="",sza="";
		String s1="0";
		for (int i=0;i<N;i++) {
			String sx="0",sy="0",sz="0";
			try {
				sx=st[i*3  ];
				sy=st[i*3+1];
				sz=st[i*3+2];
			} catch (Exception e) {
			}
			if (i>0) {
				if (i>1) {
					x_s+="+";
					y_s+="+";
					z_s+="+";
				}
				String si=Integer.toString(i-1);
				String s2=Double.toString(0.00000001+(double)i/(double)(N-1));
				String U="ind((u>="+s1+")&(u<"+s2+"))";
				String V= sN+ "*u-" + si;
				x_s+=U+ "*(" +sxa+ "+(" +V+ ")*((" +sx+ ")-(" +sxa+")))";
				y_s+=U+ "*(" +sya+ "+(" +V+ ")*((" +sy+ ")-(" +sya+")))";
				z_s+=U+ "*(" +sza+ "+(" +V+ ")*((" +sz+ ")-(" +sza+")))";
				s1=s2;
			}
			sxa=sx; sya=sy; sza=sz;
		}
		return x_s+" "+y_s+" "+z_s;
	}

}