/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package com.mja.poly;

import java.awt.*;
import java.util.*;
import com.mja.parser.*;
import com.mja.util.*;
import com.mja.math.*;
import com.mja.math3D.*;
import com.mja.text.*;
import com.mja.gui.mjaColor;
import org.unam.matem.Attribute;

public class mjaSurface { 

	public static final double oneDegree = Math.PI/180;
	public boolean split=false;
	public Node cond;
	private Node[] var;
	private Node[] node;
	private Node NNu,NNv,Ngr,Nw,Nl,Nh;
	private Node X,Y,Z,U,V;
	private mjaColor fc,bc;
	private boolean edges;
	private Node[] irN=new Node[3];
	private Node[] ipN=new Node[3];
	private Node[] frN=new Node[3];
	private Node[] fpN=new Node[3];
	private Node[] rN=new Node[3];
	private Node[] pN=new Node[3];
	private int model;
	private Parser p;
	private int special=0;
	private R3[] v;
	private Color[] Fc,Bc;
	private boolean fixed;
	private boolean EulerInitialAngles=false,EulerFinalAngles=false, EulerAngles=false;
	
	public mjaSurface(Parser p,int type,String s_Nu,String s_Nv,String s_gr,String s_w,String s_l, String s_h,mjaColor fc,mjaColor bc,boolean edges,String s_ir,String s_ip,String s_fr,String s_fp,String s_r,String s_p,int model,boolean fixed) {
		this.p=p;
		special=type;
		X=p.Var("x"); Y=p.Var("y"); Z=p.Var("z");
		U=p.Var("u"); V=p.Var("v");
		setNu(s_Nu);
		setNv(s_Nv);
		Ngr=p.Analyse(s_gr,"1");
		Nw=p.Analyse(s_w,"2");
		Nl=p.Analyse(s_l,"2");
		Nh=p.Analyse(s_h,"2");
		this.fc=fc;
		this.bc=bc;
		this.fc.analyse(p);
		this.bc.analyse(p);
		this.edges=edges;
		int EulerNum=0;
		try {
			String[] inirot=BasicStr.getTokens(s_ir);
			EulerNum=0;
			if (s_ir.startsWith("Euler")) {
				EulerInitialAngles=true;
				EulerNum=1;
			}
			if (inirot.length>=3+EulerNum) {
				for (int i=EulerNum;i<inirot.length;i++) {
					irN[i-EulerNum]=p.Analyse(inirot[i],0.0);
				}
			} else {
				irN[0]=p.Analyse("0");
				irN[1]=irN[0]; irN[2]=irN[0];
			}
		} catch (Exception e) {
		}
		try {
			String[] inipos=BasicStr.getTokens(s_ip);
			if (inipos.length>=3) {
				for (int i=0;i<ipN.length;i++) {
					ipN[i]=p.Analyse(inipos[i],0.0);
				}
			} else {
				ipN[0]=p.Analyse("0");
				ipN[1]=ipN[0]; ipN[2]=ipN[0];
			}
		} catch (Exception e) {
		}
		try {
			String[] finrot=BasicStr.getTokens(s_fr);
			EulerNum=0;
			if (s_fr.startsWith("Euler")) {
				EulerFinalAngles=true;
				EulerNum=1;
			}
			if (finrot.length>=3+EulerNum) {
				for (int i=EulerNum;i<finrot.length;i++) {
					frN[i-EulerNum]=p.Analyse(finrot[i],0.0);
				}
			} else {
				frN[0]=p.Analyse("0");
				frN[1]=frN[0]; frN[2]=frN[0];
			}
		} catch (Exception e) {
		}
		try {
			String[] finpos=BasicStr.getTokens(s_fp);
			if (finpos.length>=3) {
				for (int i=0;i<fpN.length;i++) {
					fpN[i]=p.Analyse(finpos[i],0.0);
				}
			} else {
				fpN[0]=p.Analyse("0");
				fpN[1]=fpN[0]; fpN[2]=fpN[0];
			}
		} catch (Exception e) {
		}
		try {
			String[] rot=BasicStr.getTokens(s_r);
			EulerNum=0;
			if (s_r.startsWith("Euler")) {
				EulerAngles=true;
				EulerNum=1;
			}
			if (rot.length>=3+EulerNum) {
				for (int i=EulerNum;i<rot.length;i++) {
					rN[i-EulerNum]=p.Analyse(rot[i],0.0);
				}
			} else {
				rN[0]=p.Analyse("0");
				rN[1]=rN[0]; rN[2]=rN[0];
			}
		} catch (Exception e) {
		}
		try {
			String[] pos=BasicStr.getTokens(s_p);
			if (pos.length>=3) {
				for (int i=0;i<pN.length;i++) {
					pN[i]=p.Analyse(pos[i],0.0);
				}
			} else {
				pN[0]=p.Analyse("0");
				pN[1]=pN[0]; pN[2]=pN[0];
			}
		} catch (Exception e) {
		}
		this.model=model;
		this.fixed=fixed;
	}
	
	private MathText mt;
	
	public void setMathText(MathText mt) {
		this.mt=mt;
	}
	
	public boolean isFixed() {
		return fixed;
	}
	
	void setEquation (String equ_text) {
		special=0;
		String equ=BasicStr.replace(equ_text,"\\n"," ");
		Attribute[] attr=Attribute.parse(equ);
		var=new Node[attr.length];
		node=new Node[attr.length];
		for (int i=0;i<attr.length;i++) {
			var[i]=p.Var(attr[i].name);
			node[i]=p.Analyse(attr[i].value);
		}
	}


	Node[] Vx,Vy,Vz;
	R3[] vertices3D;
		
	public void set3DVertices (String equ) {
		Vx=new Node[3];
		Vy=new Node[3];
		Vz=new Node[3];
		for (int i=0;i<3;i++) {
			Vx[i]=p.Analyse("0",0);
			Vy[i]=p.Analyse("0",0);
			Vz[i]=p.Analyse("0",0);
		}
		try {
			String[] sa=BasicStr.getTokens(equ);
			int N=sa.length;
			for (int i=0;(i<N/3)&&(i<3);i++) {
				int k=3*i;
				if (k<sa.length) {
					Vx[i]=p.Analyse(sa[k++],0);
				}
				if (k<sa.length) {
					Vy[i]=p.Analyse(sa[k++],0);
				}
				if (k<sa.length) {
					Vz[i]=p.Analyse(sa[k++],0);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		calculate3DVertices();
	}
	
	private void calculate3DVertices() {
		vertices3D=new R3[Vx.length];
		for (int i=0;i<Vx.length;i++) {
			try {
				vertices3D[i]=new R3(Vx[i].Evaluate(),Vy[i].Evaluate(),Vz[i].Evaluate());
			} catch (Exception e) {
				vertices3D[i]=null;
			}
		}
	}



	R2[] vertices;
	Node[] VX,VY;
	public void setVertices (String equ) {
		VX=new Node[0];
		VY=new Node[0];
		try {
			String[] sa=BasicStr.getTokens(equ);
			int N=sa.length;
			if (N%2!=0) {
				N--;
			} 
			if (N>0) {
				N=N/2;
				VX=new Node[N];
				VY=new Node[N];
				for (int i=0;i<N;i++) {
					String sx=sa[2*i];
					String sy=sa[2*i+1];
					VX[i]=p.Analyse(sx,0);
					VY[i]=p.Analyse(sy,0);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		calculateVertices();
	}
	
	private void calculateVertices() {
		vertices=new R2[VX.length];
		for (int i=0;i<VX.length;i++) {
			double vx=VX[i].Evaluate(0);
			double vy=VY[i].Evaluate(0);
			vertices[i]=new R2(vx,vy);
		}
	}

	public void setNu(String s_Nu) {
		NNu=p.Analyse(s_Nu,"7");
	}
	
	public void setNv(String s_Nv) {
		NNv=p.Analyse(s_Nv,"7");
	}
	
	public Surface create() {
		Surface S=null;
		Node An=p.Var("_AnChO_"), La=p.Var("_LaRgO_"), Al=p.Var("_AlTo_");
		synchronized (An) { 
			synchronized (La) { 
				synchronized (Al) { 
					double ancho=An.getDouble(), largo=La.getDouble(), alto=Al.getDouble();
					An.setDouble(Nw.Evaluate(2)); La.setDouble(Nl.Evaluate(2)); Al.setDouble(Nh.Evaluate(2));
					if (special>0) {
							double w=An.getDouble()/2, l=La.getDouble()/2, h=Al.getDouble()/2;
							switch (special) {
								case SurfaceFactory.triangle       : calculate3DVertices();
																		S=getFace(vertices3D); 
																		break;
								case SurfaceFactory.face           : calculateVertices();
																		S=getFace(vertices); 
																		break;
								case SurfaceFactory.box            : S=getBox(w,l,h); break;
								case SurfaceFactory.tetrahedron    : S=getRegularTetrahedron(w); break;
								case SurfaceFactory.cube           : double sz=w/(Math.sqrt(3));
																	 S=getBox(sz,sz,sz); break;
								case SurfaceFactory.octahedron     : S=getRegularOctahedron(w); break;
								case SurfaceFactory.dodecahedron   : S=getRegularDodecahedron(w); break;
								case SurfaceFactory.icosahedron    : S=getRegularIcosahedron(w); break;
							}
							S.setFrontColor(fc.getAdaptedColor());
							S.setBackColor(bc.getAdaptedColor());
					} else {
						int Nu=(int) Math.round(NNu.Evaluate(7));
						int Nv=(int) Math.round(NNv.Evaluate(7));
						calculateVerticesAndColors(Nu,Nv);
						S=makeSurface(Nu,Nv,(int) Math.round(Ngr.Evaluate(1)));
					}
					An.setDouble(ancho); La.setDouble(largo); Al.setDouble(alto);
				}
			}
		}
		if (S!=null) {
			S.setDrawEdges(edges);
			
			R3 Tir=new R3(oneDegree*irN[0].Evaluate(0.0),oneDegree*irN[1].Evaluate(0.0),oneDegree*irN[2].Evaluate(0.0));
			R3 Tip= new R3(ipN[0].Evaluate(0.0),ipN[1].Evaluate(0.0),ipN[2].Evaluate(0.0));
			S.apply(LinearTransformation.RigidMotion(EulerInitialAngles,Tir,Tip));

			R3 Tfr=new R3(oneDegree*frN[0].Evaluate(0.0),oneDegree*frN[1].Evaluate(0.0),oneDegree*frN[2].Evaluate(0.0));
			R3 Tfp= new R3(fpN[0].Evaluate(0.0),fpN[1].Evaluate(0.0),fpN[2].Evaluate(0.0));
			S.apply(LinearTransformation.RigidMotion(EulerFinalAngles,Tfr,Tfp));

			R3 Tr=new R3(oneDegree*rN[0].Evaluate(0.0),oneDegree*rN[1].Evaluate(0.0),oneDegree*rN[2].Evaluate(0.0));
			R3 Tp= new R3(pN[0].Evaluate(0.0),pN[1].Evaluate(0.0),pN[2].Evaluate(0.0));
			S.apply(LinearTransformation.RigidMotion(EulerAngles,Tr,Tp));

			S.fixAsInitialPosition();
			viewModel vM=new viewModel();
			S.setViewModel(model);
			S.setMathText(mt);
		}
		return S;
	}
	
	private Surface makeSurface(int Nu,int Nv,int w) {
		Face[] f=null;
		if (Nv==0) {
			if (Nu==0) {
				f=new Face[1];
				f[0]=new Face(v[0]);
				f[0].setFrontColor(Fc[0]);
				f[0].setBackColor(Bc[0]);
				f[0].setWidthness(w);
			} else {
				f=new Face[Nu];
				for (int i=0;i<Nu;i++) {
					f[i]=new Face(v[i],v[i+1]);
					f[i].setFrontColor(Fc[i]);
					f[i].setBackColor(Bc[i]);
					f[i].setWidthness(w);
				}
			}
		} else {
			f=new Face[Nu*Nv];
			for (int j=0;j<Nv;j++) {
				for (int i=0;i<Nu;i++) {
					f[i+Nu*j]=new Face(v[i+j*(Nu+1)],v[i+1+j*(Nu+1)],v[i+1+(j+1)*(Nu+1)],v[i+(j+1)*(Nu+1)]);
					f[i+Nu*j].setFrontColor(Fc[i+j*(Nu+1)]);
					f[i+Nu*j].setBackColor(Bc[i+j*(Nu+1)]);
				}
			}
		}
		return new Surface(f);
	}
				
	private void calculateVerticesAndColors(int Nu,int Nv) {
		v=new R3[(Nu+1)*(Nv+1)];
		Fc=new Color[(Nu+1)*(Nv+1)];
		Bc=new Color[(Nu+1)*(Nv+1)];
		synchronized (X) { 
		synchronized (Y) { 
		synchronized (Z) { 
		synchronized (U) { 
		synchronized (V) { 
			// protect X,Y,Z,U,V,var[]
			double savex=X.getDouble(), savey=Y.getDouble(), savez=Z.getDouble(),saveu=U.getDouble(), savev=V.getDouble();
			double[] savevar=new double[node.length];
			for (int k=0;k<node.length;k++) {
				savevar[k]=var[k].getDouble();
			}
			// make calculations
			double dU=1.0/(double)Nu;
			double dV=1.0/(double)Nv;
			int ix=0;
			V.setDouble(0);
			for (int j=0;j<=Nv;j++) {
				U.setDouble(0);
				for (int i=0;i<=Nu;i++) {
					boolean ok=true;
					for (int k=0;k<node.length;k++) {
						try {
							var[k].setDouble(node[k].Evaluate());
						} catch (Exception e) {
//System.out.println("NaN");
							var[k].setDouble(Double.NaN);
							ok=false;
						}
					}
					Fc[ix]=fc.getAdaptedColor();
					Bc[ix]=bc.getAdaptedColor();
					if (ok&&(X.getDouble()!=Double.NaN)&&(Y.getDouble()!=Double.NaN)&&(Z.getDouble()!=Double.NaN)) {
						v[ix++]=new R3(X.getDouble(),Y.getDouble(),Z.getDouble());
					} else {
						v[ix++]=null;
					}
					if (i+1==Nu) {
						U.setDouble(1.0);
					} else {
						U.setDouble(U.getDouble()+dU);
					}
//System.out.println(U.getDouble());
				}
				if (j+1==Nv) {
					V.setDouble(1.0);
				} else {
					V.setDouble(V.getDouble()+dV);
				}
//System.out.println(V.getDouble());
			}
			// return protected values
			for (int k=0;k<node.length;k++) {
				var[k].setDouble(savevar[k]);
			}
			X.setDouble(savex); Y.setDouble(savey); Z.setDouble(savez); U.setDouble(saveu); V.setDouble(savev);
		}
		}
		}	
		}
		}
	}

  
	private static final Surface getFace(R2[] vertices) {
		Face[] f=new Face[1];
		if (vertices!=null&&vertices.length>=3)  {
			f[0]=new Face(vertices);
		} else {
			f[0]=new Face(new R3(-1,-1,0),new R3(-1,1,0),new R3(1,1,0),new R3(1,-1,0));
		}
		return new Surface(f);
	}

	private static final Surface getFace(R3[] vertices) {
		Face[] f=new Face[1];
		if (vertices!=null&&vertices.length==3)  {
			f[0]=new Face(vertices);
		} else {
			f[0]=new Face(new R3(1,0,0),new R3(0,1,0),new R3(0,0,1));
		}
		return new Surface(f);
	}

	private static final Surface getBox(double a,double b,double c) {
		R3[] v=new R3[8];
		v[0]=new R3( a, b, c); v[1]=new R3( a,-b, c);
		v[2]=new R3(-a,-b, c); v[3]=new R3(-a, b, c);
		v[4]=new R3( a, b,-c); v[5]=new R3( a,-b,-c);
		v[6]=new R3(-a,-b,-c); v[7]=new R3(-a, b,-c);
		Face[] f=new Face[6];
		f[0]=new Face(v[0],v[3],v[2],v[1]); 
		f[1]=new Face(v[0],v[1],v[5],v[4]); 
		f[2]=new Face(v[1],v[2],v[6],v[5]); 
		f[3]=new Face(v[2],v[3],v[7],v[6]); 
		f[4]=new Face(v[3],v[0],v[4],v[7]); 
		f[5]=new Face(v[7],v[4],v[5],v[6]); 
		return new Surface(f);
	}

	private static final Surface getRegularTetrahedron(double a) {
		R3[] v=new R3[4];
		v[0]=new R3(0.0,0.0,a);
		v[1]=new R3(a*2.0*Math.sqrt(2.0)/3.0,0.0,-a*1.0/3.0);
		v[2]=v[1].RotZ(2.0*Math.PI/3.0);
		v[3]=v[2].RotZ(2.0*Math.PI/3.0);
		Face [] f=new Face[4];
		f[0]=new Face(v[0],v[1],v[2]);
		f[1]=new Face(v[1],v[3],v[2]);
		f[2]=new Face(v[0],v[2],v[3]);
		f[3]=new Face(v[0],v[3],v[1]);
		return new Surface(f);
	}

	private static final Surface getRegularOctahedron(double a) {
		R3[] v=new R3[6];
		v[0]=new R3( 0, 0, a);
		v[1]=new R3( a, 0, 0);
		v[2]=new R3( 0, a, 0);
		v[3]=new R3(-a, 0, 0);
		v[4]=new R3( 0,-a, 0);
		v[5]=new R3( 0, 0,-a);
		Face[] f=new Face[8];
		f[0]=new Face(v[0],v[1],v[2]);
		f[1]=new Face(v[0],v[2],v[3]);
		f[2]=new Face(v[0],v[3],v[4]);
		f[3]=new Face(v[0],v[4],v[1]);
		f[4]=new Face(v[5],v[2],v[1]);
		f[5]=new Face(v[5],v[3],v[2]);
		f[6]=new Face(v[5],v[4],v[3]);
		f[7]=new Face(v[5],v[1],v[4]);
		return new Surface(f);
	}

	private static final Surface getRegularDodecahedron(double a) {
		R3[] v=new R3[20];
		R3 p0=new R3( 0, 0, a);
		// de las frmulas para una arista dado el radio=1.0 
		double d=4.0/(Math.sqrt(3)*(Math.sqrt(5)+1));
		double beta=2.0*Math.PI/5.0;
		// resolviendo: d2=sin(alfa)2*((1-cos(beta))2+sen(beta)2) 
		double r=1.0-Math.cos(beta); 
		r=(r*r)+Math.sin(beta)*Math.sin(beta);
		double alfa=Math.asin(d/(Math.sqrt(r)));
		double gamma=2.0*Math.asin(Math.sin(Math.PI/5.0)*Math.sin(alfa));
		v[0]=p0.RotY(alfa);
		v[1]=v[0].RotZ(beta);
		v[2]=v[1].RotZ(beta);
		v[3]=v[2].RotZ(beta);
		v[4]=v[3].RotZ(beta);
		v[5]=v[0].RotY(gamma);
		v[6]=v[5].RotZ(beta);
		v[7]=v[6].RotZ(beta);
		v[8]=v[7].RotZ(beta);
		v[9]=v[8].RotZ(beta);
		v[10]=v[0].Negative();
		v[11]=v[1].Negative();
		v[12]=v[2].Negative();
		v[13]=v[3].Negative();
		v[14]=v[4].Negative();
		v[15]=v[5].Negative();
		v[16]=v[6].Negative();
		v[17]=v[7].Negative();
		v[18]=v[8].Negative();
		v[19]=v[9].Negative();
		Face[] f=new Face[12];
		f[0]=new Face(v[0],v[1],v[2],v[3],v[4]); 
		f[1]=new Face(v[0],v[5],v[18],v[6],v[1]);
		f[2]=new Face(v[1],v[6],v[19],v[7],v[2]);
		f[3]=new Face(v[2],v[7],v[15],v[8],v[3]);
		f[4]=new Face(v[3],v[8],v[16],v[9],v[4]);
		f[5]=new Face(v[4],v[9],v[17],v[5],v[0]);
		f[6]=new Face(v[14],v[13],v[12],v[11],v[10]);
		f[7]=new Face(v[11],v[16],v[8],v[15],v[10]);
		f[8]=new Face(v[12],v[17],v[9],v[16],v[11]);
		f[9]=new Face(v[13],v[18],v[5],v[17],v[12]);
		f[10]=new Face(v[14],v[19],v[6],v[18],v[13]);
		f[11]=new Face(v[10],v[15],v[7],v[19],v[14]);
		return new Surface(f);
	}

	private static final Surface getRegularIcosahedron(double a) {
		R3[] v=new R3[12];
		v[0]=new R3(0,0,a);
		double dd=(2.0*Math.sqrt(2.0))/(Math.sqrt(5.0+Math.sqrt(5)));
		double alf=2.0*Math.asin(dd/2.0);
		double bet=2.0*Math.PI/5.0;
		v[1]=v[0].RotY(alf);
		v[2]=v[1].RotZ(bet);
		v[3]=v[2].RotZ(bet);
		v[4]=v[3].RotZ(bet);
		v[5]=v[4].RotZ(bet);
		v[6]=v[1].Negative();
		v[7]=v[2].Negative();
		v[8]=v[3].Negative();
		v[9]=v[4].Negative();
		v[10]=v[5].Negative();
		v[11]=v[0].Negative();
		Face[] f=new Face[20];
		f[ 0]=new Face(v[0],v[1],v[2]);
		f[ 1]=new Face(v[0],v[2],v[3]);
		f[ 2]=new Face(v[0],v[3],v[4]);
		f[ 3]=new Face(v[0],v[4],v[5]);
		f[ 4]=new Face(v[0],v[5],v[1]);
		f[ 5]=new Face(v[ 1],v[ 9],v[ 2]);
		f[ 6]=new Face(v[ 2],v[10],v[ 3]);
		f[ 7]=new Face(v[ 3],v[ 6],v[ 4]);
		f[ 8]=new Face(v[ 4],v[ 7],v[ 5]);
		f[ 9]=new Face(v[ 5],v[ 8],v[ 1]);
		f[10]=new Face(v[ 6],v[ 7],v[ 4]);
		f[11]=new Face(v[ 7],v[ 8],v[ 5]);
		f[12]=new Face(v[ 8],v[ 9],v[ 1]);
		f[13]=new Face(v[ 9],v[10],v[ 2]);
		f[14]=new Face(v[10],v[ 6],v[ 3]);
		f[15]=new Face(v[11],v[10],v[9]);
		f[16]=new Face(v[11],v[ 9],v[8]);
		f[17]=new Face(v[11],v[ 8],v[7]);
		f[18]=new Face(v[11],v[ 7],v[6]);
		f[19]=new Face(v[11],v[ 6],v[10]);
		return new Surface(f);
	}

} // mjaSurface;