/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package com.mja.poly;

import java.awt.Color;
import com.mja.math3D.R3;
import com.mja.gui.mjaColor;
import com.mja.lang.data;

public class viewModel { 

	public static final int COLOR=0;
	public static final int LIGHT=1;
	public static final int METAL=2;
	public static final int WIRE=3;
	private int model=COLOR;
	private float dim= 1, uI2=0;
	private static final float maxI=(float)1/(float)256;

	private static final R3[] Light = { 
		new R3( 50, 50, 70),
		new R3( 50,-50, 30),
		new R3( 20,  0,-80), 
		new R3(  0,  0,  0) 
	};
	private static R3[] light;

	private static final float[] Int0={ 
		(float)0.4, 
		(float)0.5, 
		(float)0.3, 
		(float)0.0, 
	};
	private static float[] Int={Int0[0],Int0[1],Int0[2],Int0[3]};
	
	public viewModel() {
		if (light==null) {
			light=new R3[Int.length];
			for (int i=0;i<Int.length;i++) {
				light[i]=Light[i].ScalarProduct(1.0/Light[i].Norm());
			}
		}
		setModel(COLOR);
	}

	public void setModel(int type) {
		model=type;
	}

	public int getModel() { 
		return model; 
	}
	
	public void setUserIllumination(double dim, double uI) {
		this.dim=(float) dim;
		this.uI2=((float) uI*(float) uI);
		Int[0]=Int0[0]*this.dim;
		Int[1]=Int0[1]*this.dim;
		Int[2]=Int0[2]*this.dim;
		Int[3]=(float) uI;
	}
	
	public void setUserLightPos(int x, int y, int z) {
		Light[3].x=x;
		Light[3].y=y;
		Light[3].z=z;
	}

	Color getFaceColor(Face F) {
		Color C=F.getFrontColor();
		if (model==WIRE) {
			return C;
		}
		if (!F.pointsOut) {
			C=F.getBackColor();
		}
		if (model==COLOR) {
			return C;
		}
		R3 uN=F.unitNormal;
		if (!F.pointsOut) {
			uN=uN.ScalarProduct(-1); 
		}
		int NI=0,NF=2;
		if (dim==0) {
			NI=3;
		}
		if (uI2>0) {
			light[3]=Light[3].cloneR3();
			light[3].subtract(F.average);
			float dl3=(float) light[3].Norm();
			Int[3]=uI2/dl3;
			NF=3;
		}
		float I=(model==LIGHT)?(float) (dim/4):(float) (dim/2);
		float c=0;
		for (int i=NI;i<=NF;i++) {
			if (model == LIGHT) {
				c=Math.max(0,(float) light[i].DotProduct(uN));
			} else { // model == COLOR
				c=Math.max(0,(float) R3.reflectedRay(light[i],uN).DotProduct(F.unitToEye));
				c=c*c*c;
			}
			I+=Int[i]*c;
		}
		I=Math.min(I/256,maxI);
		return mjaColor.scaleColor(I,C);
	}

}