/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package com.mja.text;

import java.awt.*;
import java.util.*;

import com.mja.parser.*;
import com.mja.util.BasicStr;
import com.mja.file.mjaFont;
import com.mja.util.TFont;

public class TNode {
	
	public static boolean hideExprs=true;

	static final int CHAR=0,TFONT=1,COLOR=2,EXPR=3,FRACT=4,RADICAL=5,SUBIX=6,SUPIX=7,SUM=8,INTEGRAL=9,LIMIT=10,MATRIX=11,DEFPARTS=12,FORMULA=13,MULT=14,HYPERLINK=15,COMPONENT=16;
	private int type=CHAR;
	private TFont tfont;
	private TColors tc;
	private String s,expr,eval,dec_expr;
	private Node n_dec;
	private FormulaItem FI;
	private Component Cmp;
	private String ctype,cid="",url="";
	private Parser p;
	private int x,y,A,D,W,x1,dx2,dx1,he,extra;
	private boolean fixed=false;
	private int cols=1, rows=1, parts=2;
	

	private BasicText parent;

	public void setLocation(int x,int y) {this.x=x; this.y=y;}
	public int getX1() {return x1;}
	public int getdX2() {return dx2;}
	public int getdX1() {return dx1;}
	public int getX() {	return x;}
	public int getY() {	return y;}
	public int getA() {	return A;}
	public int getD() {	return D;}
	public int getW() {	return W;}
	public int gethe() {return he;}
	public int getdx2() {return extra/2;}
	public FormulaItem getFI() { return FI; }

	private TNode(BasicText pt) {
		parent=pt;
	}

	public TNode(BasicText pt,String s) {
		parent=pt;
		build(CHAR,s);
	}

	public TNode(BasicText pt,String s,int type) {
		parent=pt;
		if (type==SUBIX||type==SUPIX||type==FORMULA) {
			Formula f=new Formula(pt.getParser());
			f.setColors(pt.getColors());
			f.insertString(s);
			build(type,f);
		} else if (type==CHAR) {
			build(type,s);
		} else if (type==HYPERLINK) {
			build(type,s);
		} else {
			new Exception("BAD TNode TYPE").printStackTrace();
		}
	}

	public TNode(BasicText pt,Formula f,int type) {
		parent=pt;
		build(type,f);
	}

	public TNode(BasicText pt,Formula formula) {
		parent=pt;
		build(FORMULA,formula);
	}

	public TNode(BasicText pt,Component C,String ctype,String cid) {
		parent=pt;
		build(COMPONENT,C);
		this.ctype=ctype;
		this.cid=cid;
	}

	public TNode(BasicText pt,TFont f) {
		parent=pt;
		build(TFONT,f);
	}

	public TNode(BasicText pt,TColors c) {
		parent=pt;
		build(COLOR,c);
	}

	public TNode(BasicText pt,int cols,int rows,FormulaItem fi) {
		this.cols=cols;
		this.rows=rows;
		parent=pt;
		build(MATRIX,fi);
		fi.setColors(parent.getColors());
	}

	public TNode(BasicText pt,int parts,FormulaItem fi) {
		this.parts=parts;
		parent=pt;
		build(DEFPARTS,fi);
		fi.setColors(parent.getColors());
	}

	public TNode(BasicText pt,FormulaItem fi,int type) {
		parent=pt;
		build(type,fi);
//		fi.setColors(parent.getColors());
	}

	public TNode(BasicText pt,Parser p,String expr) {
		parent=pt;
		this.p=p;
		this.expr=expr;
//		Node nexpr=null;
//		try {
//			nexpr=p.Analyse(expr);
//		} catch (Exception e) {
//		}
		build(EXPR,expr);
	}

	public void setParser(Parser p) {
		this.p=p;
	}

	public BasicText getParent() {
		return parent;
	}

	public void setExpr(String expr) {
		this.expr=expr;
		build(EXPR,expr);
	}

	private void build(int t,Object o) {
		type=t;
		if (t==CHAR) {
			s=(String) o;
			if (s.equals("\u00b7")||	// uniformizar operadores de multiplicacin
				s.equals("\u00d7")||
				s.equals("*")||
				s.equals("\u2022")||
				s.equals("\u2219")  ) {
//				s="\u00d7";
//				s="";
			}
		} else if (t==TFONT) {
			tfont=(TFont) o;
		} else if (t==COLOR) {
			tc=(TColors) o;
		} else if (t==EXPR) {
			s=" "+expr+" ";
			eval=s;
		} else if (t==FRACT||t==RADICAL||t==INTEGRAL||t==LIMIT||t==SUM||t==MATRIX||t==DEFPARTS||t==MULT) {
			FI=(FormulaItem) o;
			FI.setParent(this);
		} else if (t==FORMULA||t==SUBIX||t==SUPIX) {
			FI=new FormulaItem((Formula) o);
			FI.setParent(this);
			s=FI.getFormula(0).toString();
		} else if (t==HYPERLINK) {
			String auxs=(String) o;
			StringTokenizer st=new StringTokenizer(auxs,"|");
			if (st.countTokens()<2) {
				s="hv";
			} else {
				s=st.nextToken();
			}
			url=st.nextToken();
		} else if (t==COMPONENT) {
			this.Cmp=(Component) o;
		} else {
			type=-1;
		}
	}

	public int getAscent(TFont f) {
		int a=f.getAscent();
		int d=f.getDescent();
		he=(a-d)/2;
		switch (type) {
			case CHAR 	:
			case HYPERLINK :
			case EXPR 	: A=a; break;
			case SUPIX 	: A=FI.getFormula(0).getAscent()+he+FI.getFormula(0).getDescent(); break;
			case SUBIX 	: A=he; break;
			case FORMULA: A=FI.getFormula(0).getAscent(); break;
			case FRACT 	: A=FI.getFormula(FormulaItem.NUM).getHeight()+he; break;
			case RADICAL: A=Math.max(FI.getFormula(FormulaItem.RADICAND).getAscent(),FI.getFormula(FormulaItem.INDEX).getHeight()+he)+3; break;
			case SUM	:
			case INTEGRAL:A=Math.max(a+FI.getFormula(FormulaItem.TO).getHeight()-1,FI.getFormula(FormulaItem.WHAT).getAscent()); break;
			case LIMIT	: A=Math.max(a,FI.getFormula(FormulaItem.WHAT).getAscent()); break;
			case MATRIX	: A=he+(getMatrixHeight()+1)/2; break;
			case DEFPARTS: A=he+(getDefPartsHeight()+1)/2; break;
			case MULT 	: A=Math.max(FI.getFormula(FormulaItem.NUM).getAscent(),FI.getFormula(FormulaItem.NUM).getAscent()); break;
			case COMPONENT : A=a+3; break;
			default 	: A=0;
		}
		return A;
	}

	public int getDescent(TFont f) {
		int a=f.getAscent();
		int d=f.getDescent();
		he=(a-d)/2;
		switch (type) {
			case CHAR 	:
			case HYPERLINK :
			case EXPR 	: D=d; break;
			case FRACT 	: D=FI.getFormula(FormulaItem.DEN).getHeight()-he; break;
			case SUBIX 	: D=FI.getFormula(0).getDescent()-he+FI.getFormula(0).getAscent(); break;
			case SUPIX 	: D=-he; break;
			case FORMULA: D=FI.getFormula(0).getDescent(); break;
			case RADICAL: D=FI.getFormula(FormulaItem.RADICAND).getDescent(); break;
			case SUM	:
			case INTEGRAL:D=Math.max(d+FI.getFormula(FormulaItem.FROM).getHeight()-1,FI.getFormula(FormulaItem.WHAT).getDescent()); break;
			case LIMIT	: D=d+Math.max(FI.getFormula(FormulaItem.FROM).getHeight()-1,FI.getFormula(FormulaItem.TO).getHeight()); 
						  D=Math.max(D,FI.getFormula(FormulaItem.WHAT).getDescent());	break;
			case MATRIX	: D=-he+(getMatrixHeight()+1)/2; break;
			case DEFPARTS : D=-he+(getDefPartsHeight()+1)/2; break;
			case MULT 	: D=Math.max(FI.getFormula(FormulaItem.NUM).getDescent(),FI.getFormula(FormulaItem.NUM).getDescent()); break;
			case COMPONENT : D=d+3; break;
			default 	: D=0;
		}
		return D;
	}

	public int getExtraDescent(TFont f) {
		int d=f.getDescent();
		int m=0;
		if ((type==COMPONENT) && (Cmp!=null)) {
			m=Cmp.getSize().height;
		}
		return Math.max(0,m-d);
	}

	int getRows() {
		return rows;
	}

	int getColumns() {
		return cols;
	}

	int getParts() {
		return parts;
	}

	int getMaxA() {
		int maxa=0;
		if (type==DEFPARTS) {
			for (int i=0;i<parts;i++) {
				maxa=Math.max(maxa,FI.getFormula(i).getAscent());
			}
		} else {
			for (int j=0;j<cols;j++) {
				int aj=0;
				for (int i=0;i<rows;i++) {
					aj=Math.max(aj,FI.getFormula(i+j*rows).getAscent());
				}
				maxa=Math.max(maxa,aj);
			}
		} 
		return maxa;
	}

	int getMatrixHeight() {
		int maxa=0,maxd=0;
		for (int j=0;j<cols;j++) {
			int aj=0,dj=0;
			for (int i=0;i<rows;i++) {
			    aj=Math.max(aj,FI.getFormula(i+j*rows).getAscent());
		    	dj=Math.max(dj,FI.getFormula(i+j*rows).getDescent());
			}
			maxa=Math.max(maxa,aj);
			maxd=Math.max(maxd,dj);
		}
		return rows*(maxa+maxd);
	}

	int getDefPartsHeight() {
		int maxa=0,maxd=0;
		for (int j=0;j<parts;j++) {
			int aj=0,dj=0;
		    aj=Math.max(aj,FI.getFormula(j).getAscent());
	    	dj=Math.max(dj,FI.getFormula(j).getDescent());
			maxa=Math.max(maxa,aj);
			maxd=Math.max(maxd,dj);
		}
		return parts*(maxa+maxd+6)+10;
	}

	public int getCharWidth(TFont tf,Component C,boolean useExtra) {
		mjaFont f=tf.getMjaFont();
		extra=f.stringWidth(C," ");
		switch (type) {
			case CHAR 	:	if (!isOpSymbol()) {
								extra=0;
							}
							W=f.stringWidth(C,getChar());
							if (useExtra && ("(".equals(getChar()) || ")".equals(getChar())) ) {
								W=f.stringWidth(C," (");
							}
						 	break;
			case HYPERLINK :
			case EXPR 	: 	W=f.stringWidth(C,getChar()); break;
			case SUBIX 	:
			case SUPIX 	: 	extra=0;
			case FORMULA: 	W=FI.getFormula(0).getFormulaWidth(C);
							if (FI.getFormula(0).getParent().isFormula()) {
								dx1=f.stringWidth(C," ");
								W+=dx1+dx1;
							}
							break;
			case COMPONENT: W=Cmp.getSize().width;
							break;
			case FRACT 	: 	W=Math.max(FI.getFormula(FormulaItem.NUM).getFormulaWidth(C),FI.getFormula(FormulaItem.DEN).getFormulaWidth(C))+8;
							break;
			case RADICAL: 	//extra=0;
							dx2=f.stringWidth(C,"\u221A");
							x1=dx2+FI.getFormula(FormulaItem.INDEX).getFormulaWidth(C);
							W=x1+FI.getFormula(FormulaItem.RADICAND).getFormulaWidth(C);
							break;
			case SUM: 		//extra=0;
							dx2=f.stringWidth(C,"\u2211");
							x1=Math.max(dx2,Math.max(FI.getFormula(FormulaItem.FROM).getFormulaWidth(C),FI.getFormula(FormulaItem.TO).getFormulaWidth(C)));
							W=x1+FI.getFormula(FormulaItem.WHAT).getFormulaWidth(C);
							break;
			case INTEGRAL:	//extra=0;
							dx2=f.stringWidth(C,"\u222b");
							x1=dx2+Math.max(FI.getFormula(FormulaItem.FROM).getFormulaWidth(C),FI.getFormula(FormulaItem.TO).getFormulaWidth(C));
							W=x1+3+FI.getFormula(FormulaItem.WHAT).getFormulaWidth(C);
							break;
			case LIMIT	:	dx2=FI.getFormula(FormulaItem.FROM).getFormulaWidth(C)+f.stringWidth(C," \u2192");
							int d1=FI.getFormula(FormulaItem.TO).getFormulaWidth(C);
							x1=Math.max(d1+dx2,f.stringWidth(C,"lm"));
							W=x1+3+FI.getFormula(FormulaItem.WHAT).getFormulaWidth(C);
							break;
			case MATRIX	: int wmax=0;
							for (int i=0;i<rows;i++) {
								int wi=extra;
								for (int j=0;j<cols;j++) {
									wi=Math.max(wi,FI.getFormula(i+j*rows).getFormulaWidth(C));
								}
								wmax=Math.max(wmax,wi+extra);
							}
							W=2*extra+cols*wmax;
							break;
			case DEFPARTS : extra=6;
							wmax=0;
							int wi=extra;
							for (int j=0;j<parts;j++) {
								wi=Math.max(wi,FI.getFormula(j).getFormulaWidth(C));
							}
							wmax=Math.max(wmax,wi+extra);
							W=2*extra+wmax;
							break;
			case MULT 	:
							dx1=FI.getFormula(FormulaItem.NUM).getFormulaWidth(C);
							dx2=f.stringWidth(C,"  ");
							W=dx1+dx2+FI.getFormula(FormulaItem.DEN).getFormulaWidth(C);
							break;
			default 	: 	W=0; extra=0;
		}
		if (useExtra) {
			W+=extra;
		}
		return W;
	}
	
	void removeFI() {
		if (FI!=null) {
			for (int i=0;i<FI.size();i++) {
				Formula F=FI.getFormula(i);
				F.remove(0,F.info.size());
			}
		}
	}

	void clearColors() {
		if (FI!=null) {
			for (int i=0;i<FI.size();i++) {
				FI.getFormula(i).clearColors();
			}
		}
	}

	void clearFonts() {
		if (FI!=null) {
			for (int i=0;i<FI.size();i++) {
				FI.getFormula(i).clearFonts();
			}
		}
	}

	void setColors(TColors colors) {
		if (FI!=null) {
			for (int i=0;i<FI.size();i++) {
				FI.getFormula(i).setColors(colors);
			}
		}
	}

	void clean() {
		if (FI!=null) {
			for (int i=0;i<FI.size();i++) {
				FI.getFormula(i).clean();
			}
		}
	}

	public String toString() {
		String sext="";
//		String str="";
//		if (s!=null) {
//			for (int i=0;i<s.length();i++) {
//				str+=Integer.toString(s.charAt(i))+",";
//			}
//		}
		switch (type) {
			case CHAR 	: sext+=" CHAR "+s; break;
			case TFONT	: sext+=" TFONT "+tfont.toString(); break;
			case COLOR 	: sext+=" COLOR "+tc.toString(); break;
			case HYPERLINK : sext+=" HYPERLINK "+expr; break;
			case EXPR 	: sext+=" EXPR "+expr; break;
			case RADICAL: sext+=" RADICAL "+FI.toString(); break;
			case SUBIX 	: sext+=" SUBIX "+s; break;
			case SUPIX 	: sext+=" SUPIX "+s; break;
			case FRACT 	: sext+=" FRACT "+FI.toString(); break;
			case SUM	: sext+=" SUM "+FI.toString(); break;
			case INTEGRAL:sext+=" INTEGRAL "+FI.toString(); break;
			case LIMIT	: sext+=" LIMIT "+FI.toString(); break;
			case MATRIX	: sext+=" MATRIX "+FI.toString(); break;
			case DEFPARTS : sext+=" DEFPARTS "+FI.toString(); break;
			case FORMULA: sext+=" FORMULA "+s; break;
			case COMPONENT: sext+=" COMPONENT "+s; break;
			case MULT 	: sext+=" MULT "+FI.toString(); break;
			default 	: sext+=" INVALID TNode";
		}
		return sext;
	}

	public static String getComparisonSymbol(String str) {
		if (str.equals("<=")) return "\u2264";
		else if (str.equals(">=")) return "\u2265";
		else if (str.equals("!=")) return "\u2260";
		else if (str.equals("#")) return "\u2260";
		else if (str.equals("==")) return "<=>";
//		else if (str.equals("==")) return "\u2261";
		else if (str.equals("=?")) return "\u225f";
//		else if (str.equals("*")) return "\u00d7";
//		else if (str.equals("-")) return "\u2012";
//		else if (str.equals("+/-")) return "\u00b1";
//		else if (str.equals("-/+")) return "\u2213";
		else return str;
	}

	public String toExpresion(boolean content) {
		StringBuffer sb=new StringBuffer("");
		String auxs=s;
		switch (type) {
			case CHAR 	: 	auxs=auxs.replace('\u00b7','*');
							auxs=auxs.replace('\u00d7','*');
							auxs=auxs.replace('\u2022','*');
							auxs=auxs.replace('\u2219','*');
//							auxs=auxs.replace('\u2219','*');
//							auxs=auxs.replace('\u2219','*');
//							auxs=auxs.replace('\u2219','*');
//							auxs=auxs.replace('\u2219','*');
//							auxs=auxs.replace('\u2219','*');
//							auxs=auxs.replace('\u2219','*');
//							auxs=auxs.replace('\u2219','*');
							auxs=BasicStr.replace(auxs,"\u225f","=?");
							auxs=BasicStr.replace(auxs,"\u2260","#");
							auxs=BasicStr.replace(auxs,"<=>","==");
//							auxs=BasicStr.replace(auxs,"\u2261","==");
							auxs=BasicStr.replace(auxs,"\u2264","<=");
							auxs=BasicStr.replace(auxs,"\u2265",">=");

							if (Node.inAlgebraTutor) {
								auxs=BasicStr.replace(auxs,"#","eqn");
								auxs=BasicStr.replace(auxs,"@","eqa");
								auxs=BasicStr.replace(auxs,"\u007e","eqx");
								auxs=BasicStr.replace(auxs,"[","num(");
								auxs=BasicStr.replace(auxs,"]",")");
							}
							sb.append(auxs);
							break;
			case TFONT	: sb.append("<FONT:"+tfont.getName()+tfont.getSize()+">");
							break;
			case COLOR 	: sb.append("<COLOR:"+tc.getTextColor()+">");
							break;
			case RADICAL:	String radical=FI.getFormula(FormulaItem.INDEX).toExpresion(content);
							if (BasicStr.hasContent(radical)) {
								sb.append("(");
								sb.append(FI.getFormula(FormulaItem.RADICAND).toExpresion(content));
								sb.append(")^(1/(");
								sb.append(FI.getFormula(FormulaItem.INDEX).toExpresion(content));
								sb.append("))");
							} else {
								sb.append("sqrt("+FI.getFormula(FormulaItem.RADICAND).toExpresion(content)+")");
							}

							break;
			case HYPERLINK	: if (content) {
								sb.append(url);
							} else {
								sb.append("hyperlink("+url+")");
							}
							break;
			case EXPR 	: if (content) {
								sb.append(expr);
							} else {
								sb.append("expr("+expr+","+dec_expr+","+fixed+")");
							}
							break;
			case SUBIX 	: sb.append("_"+FI.getFormula(0).toExpresion(content));
							break;
			case SUPIX 	: sb.append("^("+FI.getFormula(0).toExpresion(content)+")");
							break;
			case FORMULA: sb.append(FI.getFormula(0).toExpresion(content));
							break;
			case COMPONENT: sb.append(Cmp.toString());
							break;
			case FRACT 	: 	sb.append("((");
							sb.append(FI.getFormula(FormulaItem.NUM).toExpresion(content));
							sb.append(")/(");
							sb.append(FI.getFormula(FormulaItem.DEN).toExpresion(content));
							sb.append("))");
							break;
			case SUM 	:
							sb.append("Sum from ("+FI.getFormula(FormulaItem.FROM).toExpresion(content)+") "+
								"to ("+FI.getFormula(FormulaItem.TO).toExpresion(content)+") "+
								" ("+FI.getFormula(FormulaItem.WHAT).toExpresion(content)+") ");
							break;
			case INTEGRAL: 	sb.append("Integral from ("+FI.getFormula(FormulaItem.FROM).toExpresion(content)+") "+
								"to ("+FI.getFormula(FormulaItem.TO).toExpresion(content)+") "+
								" ("+FI.getFormula(FormulaItem.WHAT).toExpresion(content)+") ");
							break;
			case LIMIT	: 	sb.append("Limit from ("+FI.getFormula(FormulaItem.FROM).toExpresion(content)+") "+
								"to ("+FI.getFormula(FormulaItem.TO).toExpresion(content)+") ");
							break;
			case MATRIX	: sb.append(rows+"x"+cols+"Matrix ");
							for (int j=0;j<cols;j++) {
								for (int i=0;i<rows;i++) {
									sb.append(FI.getFormula(i+rows*j).toExpresion(content));
								}
							}
							break;
			case DEFPARTS : sb.append(parts+" parts Definition ");
							for (int j=0;j<parts;j++) {
								sb.append(FI.getFormula(j).toExpresion(content));
							}
							break;
			case MULT 	: 	sb.append("((");
							sb.append(FI.getFormula(FormulaItem.NUM).toExpresion(content));
							sb.append(")(");
							sb.append(FI.getFormula(FormulaItem.DEN).toExpresion(content));
							sb.append("))");
			default 	: ;
		}
		return sb.toString();
	}

	public boolean isOpSymbol() {
		return s!=null && (
			s.equals("+")||
			s.equals("-")||
			s.equals("*")||
			s.equals("\u00b7")|| // *
			s.equals("\u00d7")|| // *
			s.equals("\u2022")|| // *
			s.equals("\u2219")|| // *
			s.equals("/")||
			s.equals("\u00f7")|| // divisin
			s.equals("\u00b1")|| // +/-
			s.equals("=") ||
			s.equals("<") ||
			s.equals(">") ||
			s.equals("\u225f")|| // =?
			s.equals("\u2260")|| // != , #
			s.equals("\u2261")|| // == (three hor.lines)
			s.equals("\u2264")|| // <=
			s.equals("\u2265")|| // >=
			s.equals("?")
		);
 	}

 	public boolean isEquivalenceDirective() {
 		return s!=null && (
 			s.equals("#") ||
 			s.equals("@") ||
 			s.equals("\u007e") ||
 			s.equals("[")
 		);
 	}

 	public boolean isRightSqrBracket() {
 		return s!=null && (
 			s.equals("]")
 		);
 	}

	public boolean isNumSymbol() {
		return s!=null && (
			s.equals("0")||
			s.equals("1")||
			s.equals("2")||
			s.equals("3")||
			s.equals("4")||
			s.equals("5")||
			s.equals("6")||
			s.equals("7")||
			s.equals("8")||
			s.equals("9")||
			s.equals(".")
		);
 	}

 	public boolean isPureChar() {
 		return type==CHAR;
 	}

	public boolean isChar() {
		switch (type) {
			case CHAR :
			case FORMULA :
			case COMPONENT :
			case HYPERLINK :
			case EXPR :
			case FRACT :
			case SUPIX :
			case SUBIX :
			case RADICAL :
			case SUM :
			case INTEGRAL :
			case MATRIX :
			case DEFPARTS :
			case MULT :
			case LIMIT :
					return true;
			default: return false;
		}
	}

	public boolean isAlgebraicToken() {
		return (type==CHAR && !isOpSymbol() && !isEquivalenceDirective())
			||type==FRACT||type==RADICAL||type==FORMULA||type==SUBIX || type==MULT;
	}

	public boolean isTFont() {
		return type==TFONT;
	}

	public boolean isColors() {
		return type==COLOR;
	}

	public boolean isMath() {
		return isExpr() || isFraction() || isProduct() || isRadical() || isSum() || isIntegral() || isLimit() || isMatrix() || isDefParts() || isSubIndex() || isSuperIndex() || isFormula();
	}

	public boolean isLineSeparator() {
		return isChar() && (isEndOfFile()||"\n".equals(s));
	}

	public boolean isWordSeparator() {
		return isChar() && (isWhiteSpace()||".".equals(s)||",".equals(s)||";".equals(s)||":".equals(s));
	}

	public boolean isWhiteSpace() {
		return isChar() && (isEndOfFile()||"\n".equals(s)||" ".equals(s));
	}

	public boolean isExpr() {
		return type==EXPR;
	}

	public boolean isHyperlink() {
		return type==HYPERLINK;
	}

	public boolean isFraction() {
		return type==FRACT;
	}

	public boolean isProduct() {
		return type==MULT;
	}

	public boolean isRadical() {
		return type==RADICAL;
	}

	public boolean isSum() {
		return type==SUM;
	}

	public boolean isIntegral() {
		return type==INTEGRAL;
	}

	public boolean isLimit() {
		return type==LIMIT;
	}

	public boolean isMatrix() {
		return type==MATRIX;
	}

	public boolean isDefParts() {
		return type==DEFPARTS;
	}

	public boolean isSubIndex() {
		return type==SUBIX;
	}

	public boolean isSuperIndex() {
		return type==SUPIX;
	}

	public boolean isFormula() {
		return type==FORMULA;
	}

	public boolean isComponent() {
		return type==COMPONENT;
	}

	public boolean isEndOfLine() {
		return isChar() && ("\n".equals(s) || isEndOfFile());
	}

	public boolean isEndOfFile() {
		return isChar() && "\u001A".equals(s);
	}

	public int getType() {
		return type;
	}

	public void setChar(String s) {
		this.s=s;
	}

	public String getChar() {
		String str="";
		if (isExpr()) {
			str=eval;
//		} else if (type==FORMULA && toExpresion(false).startsWith("num(")) {
//			String expr=toExpresion(false);
//			try {
//				Node N=p.Analyse(expr);
//				double d=N.Evaluate();
//				str=BasicStr.DoubleToString(d,6,false);
//			} catch (Exception e) {
//				e.printStackTrace();
//				str=eval;
//			}
		} else {
			str=s;
		}
		return str;
	}

	public String getUri() {
		return url;
	}

	public TFont getTFont() {
		return tfont;
	}

	public TColors getColors() {
		return tc;
	}

	public String getExpr() {
		return expr;
	}
	
	public Component getComponent() {
		return Cmp;
	}
	
	public void setNewComponent(Component Cmp,String cid) {
		this.Cmp=Cmp;
		this.cid=cid;
	}
	
	public boolean updateComponent(Component Cmp,String cID) {
		if ( (type==COMPONENT) && (this.cid.equals(cID)) ) {
			this.Cmp=Cmp;
			((com.mja.util.LiveOrDie) this.Cmp).setIsUsed(true);
			return true;
		} else if (type==FORMULA) {
			if (getFI().updateNodes(Cmp,cID)) {
				return true;
			}
		}
		return false;
	}
	
	public String getComponentType() {
		return ctype;
	}

	public String getComponentId() {
		return cid;
	}

	public String getEval() {
		return eval;
	}

	public void setFixed(boolean f) {
		fixed=f;
	}

	public void setDecExpr(String de) {
		dec_expr=de;
		n_dec=null;
		if (de!=null) {
			try {
				n_dec=p.Analyse(dec_expr,2);
			} catch (Exception e) {
			}
		}
	}

	public boolean getFixed() {
		return fixed;
	}

	public String getDecExpr() {
		return dec_expr;
	}
	
	void calcValues(int dec,boolean fixed,int aL,boolean editing,boolean learnerMode) {
		switch (type) {
			case CHAR : break;
			case TFONT: break;
			case COLOR :break;
			case EXPR : boolean exprok=true;
						eval="[ERROR]";
						if (hideExprs && editing && !learnerMode) {
							eval="[expr]";
						} else {
							Node n=null;
							if (p.isVar(expr)) {
								n=p.Var(expr);
							} else {
								n=p.Analyse(expr);
							}
							if (n!=null) {
								try {
									double aux=n.Evaluate();
									if (n.getStr()!=null) {
										eval=n.getStr();
									} else {
										try {
											if (n_dec!=null) {
												int owndec=(int) Math.round(n_dec.Evaluate(2));
												eval=n.toString(owndec,this.fixed,aL);
											} else {
												eval=n.toString(dec,fixed,aL);
											}
										} catch (Exception e) {
											System.out.println("\n"+e.getMessage()+"\nText expression");
											StringTokenizer st=new StringTokenizer(e.getMessage(),";");
											eval=st.nextToken();
											exprok=false;
										}
									}
								} catch (Exception e) {
									System.out.println("\n"+e.getMessage()+"\nText expression");
									StringTokenizer st=new StringTokenizer(e.getMessage(),";");
									eval=st.nextToken();
									exprok=false;
								}
							} else {
								eval="[error]";
								exprok=false;
							}
							if (exprok) {
								while (eval.indexOf("\\u")>=0) {
									int i=eval.indexOf("\\u")+2;
									String auxs="";
									while (i<eval.length() && Node.isHexDigit(eval.charAt(i))) {
										auxs=auxs+eval.charAt(i++);
									}
									String auxs1="";
									if (auxs.length()>0) {
										auxs1+=(char) Integer.parseInt(auxs,16);
									}
									if (i<eval.length()) {
										auxs=auxs+eval.charAt(i++);
									}
									eval=BasicStr.replace(eval,"\\u"+auxs,auxs1);
								}
							}
						}
						break;
			case SUBIX  :
			case SUPIX  :
			case FORMULA: FI.getFormula(0).calcValues(dec,fixed,aL,editing,learnerMode); break;
			case FRACT  :
			case RADICAL:
			case SUM:
			case INTEGRAL:
			case LIMIT:
			case MATRIX:
			case DEFPARTS:
			case MULT  :
							for (int i=0;i<FI.size();i++) {
								FI.getFormula(i).calcValues(dec,fixed,aL,editing,learnerMode);
							}
						  break;
			case HYPERLINK: break;
			case COMPONENT: break;
		}
	}

//	public void replaceRandomVariables(Vector v_random,boolean isolated) {
//		switch (type) {
//			case CHAR :
//				s=Node.replaceRandomVariables(s,v_random,isolated);
//				break;
//			case TFONT: break;
//			case COLOR :break;
//			case EXPR : n=n0.replaceRandomVariables(v_random,isolated);
//						break;
//			case SUBIX  :
//			case SUPIX  :
//			case FORMULA: FI.getFormula(0).replaceRandomVariables(v_random,isolated); break;
//			case FRACT  :
//			case RADICAL:
//			case SUM:
//			case INTEGRAL:
//			case MATRIX:
//							for (int i=0;i<FI.size();i++) {
//								FI.getFormula(i).replaceRandomVariables(v_random,isolated);
//							}
//						  break;
//		}
//	}
//

	public TNode cloneTNode(BasicText pt) {
		TNode tn=new TNode(pt);
		switch (type) {
			case CHAR : tn.build(type,s); break;
			case TFONT: tn.build(type,getTFont()); break;
			case COLOR : tn.build(type,getColors()); break;
			case EXPR : tn=new TNode(pt,p,expr);
						tn.eval=eval;
						tn.setDecExpr(dec_expr);
						tn.setFixed(fixed);
						break;
			case SUBIX :
			case SUPIX :
			case FORMULA : tn.build(type,FI.getFormula(0).cloneFormula()); 
							break;
			case COMPONENT : 	tn.build(COMPONENT,Cmp); 
								tn.ctype=ctype;
								tn.cid=cid;
							break;
			case HYPERLINK : 	tn.build(HYPERLINK,url); 
								tn.s=s;
							break;
			case FRACT :
			case RADICAL :
			case SUM :
			case INTEGRAL :
			case LIMIT	:
			case MATRIX :
			case DEFPARTS :
			case MULT :
							tn.build(type,getFI().cloneItem()); break;
		}
		return tn;
	}
	
	public void editHyperlink() {
		if (isHyperlink()) {
			String[] label={"etiqueta","URL"};
			String[] content={s,url};
			boolean[] edit={true,true};
			boolean[] hide={false,false};
			content=com.mja.gui.mjaText.getInfo("Hipervnculo",label,"aceptar","cancelar",edit,hide,content);
			if (content!=null) {
				s=content[0];
				url=content[1];
			}
		}
	}

}