/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package com.mja.text;

import java.awt.*;
import java.util.*;

import com.mja.parser.*;
import com.mja.file.mjaFont;
import com.mja.util.TFont;
import com.mja.util.BasicStr;
import com.mja.util.LiveOrDie;
import com.mja.descartes.Space;

public class Text extends BasicText {

	private int leftMargin=16,topMargin=8,bottomMargin=8;

	private boolean mustClean;

	public Text(Parser p) {
		super(p);
	}

	public Text(Parser p,TFont f) {
		super(p,f);
	}

	public Text(Parser p,TFont f,Color bc,Color tc) {
		super(p,f,bc,tc);
	}

	public void setLeftMargin(int lm) {
		leftMargin=lm;
	}

	public void setTopAndBottomMargin(int tm) {
		topMargin=tm;
		bottomMargin=tm;
	}

	public int getLeftMargin() {
		return leftMargin;
	}

	public int getTopMargin() {
		return topMargin;
	}

	public int getBottomMargin() {
		return bottomMargin;
	}

    public Text cloneText() {
    	Text nT=new Text(getParser());
    	nT.info=cloneInfo();
    	return nT;
    }

    public boolean updateNodes(Component Cmp,String cID) {
    	for (int i=begin();i<end();i++) {
    		if (TNode(i).updateComponent(Cmp,cID)) {
    			return true;
    		}
    	}
    	return false;
    }

    public void unmarkSegment() {
    	super.unmarkSegment();
   		for (int i=begin();i<end();i++) {
   			if (TNode(i).getFI()!=null) {
   				FormulaItem fi=TNode(i).getFI();
   				for (int j=0;j<fi.size();j++) {
   					fi.getFormula(j).unmarkSegment();
   				}
   			}
   		}
    }

    public void markSegment(int b,int e) {
    	unmarkSegment();
    	super.markSegment(b,e);
    }

	/* -------------------- editing functions ------------------- */

    public void insertText(String s) {
		super.insertText(s);
	    if (mustClean) {
			clean();
		}
	}

	public void clearColors() {
		clearColors(begin(),end());
	}

    public void clearColors(int n1,int n2) {
    	int n=n1,N=n2;
    	while (n<N) {
    		TNode tn=TNode(n);
    		if (tn.isColors()) {
    			if (removeNode(n)) {
	    			N--;
    			}
    		} else {
    			tn.clearColors();
    		}
			if (tn==TNode(n)) {
    			n++;
			}
    	}
    }

	public void clearFonts() {
		clearFonts(begin(),end());
	}

    public void clearFonts(int n1,int n2) {
    	int n=n1,N=n2;
    	while (n<N) {
    		TNode tn=TNode(n);
    		if (tn.isTFont()) {
    			if (removeNode(n)) {
	    			N--;
	    		}
	    	} else {
	    		tn.clearFonts();
	    	}
    		if (tn==TNode(n)) {
    			n++;
    		}
    	}
    }

    public void insertColors(TColors tc,boolean mustClean) {
		if (segMarked()) {
	    	insert(new TNode(this,getColorsAtTNode(segMax())),segMax());
	    	clearColors(segMin(),segMax()-1);
	    	insert(new TNode(this,tc),segMin());
		    setColorsInFormulas(segMin(),segMax());
		} else {
			int n=getCursorPos();
			TColors tc0=getColorsAtTNode(n);
	    	insert(new TNode(this,tc),n);
//	    	insert(new TNode(this,tc0),n+1);
	    	setCursorPos(n+1);
		    setColorsInFormulas(n+1);
	    }
	    this.mustClean=mustClean;
    }

    public void insertTFont(TFont f,boolean mustClean) {
		if (segMarked()) {
	    	insert(new TNode(this,getTFontAtTNode(segMax())),segMax());
	    	clearFonts(segMin()+1,segMax()-1);
	    	insert(new TNode(this,f),segMin());
		    setFontsInFormulas(segMin(),segMax());
		} else {
			int n=getCursorPos();
			TFont tf0=getTFontAtTNode(n);
	    	insert(new TNode(this,f),n);
//	    	insert(new TNode(this,tf0),n+1);
	    	setCursorPos(n+1);
		    setFontsInFormulas(n+1);
	    }
	    this.mustClean=mustClean;
    }

    private void setFontsInFormulas(int n1) {
    	TFont tf=getTFontAtTNode(n1);
    	for (int n=n1;n<end();n++) {
    		if (TNode(n).isTFont()) {
    			break;
    		} else if (TNode(n).isFormula()) {
    			Formula F=TNode(n).getFI().getFormula(0);
    			F.setFonts(tf);
    		}
    	}
    }

    private void setFontsInFormulas(int n1,int n2) {
    	TFont tf=getTFontAtTNode(n1);
    	for (int n=n1;n<n2;n++) {
    		if (TNode(n).isTFont()) {
    			tf=TNode(n).getTFont();
    		} else if (TNode(n).isFormula()) {
    			Formula F=TNode(n).getFI().getFormula(0);
    			F.setFonts(tf);
    		}
    	}
    }

    private void setColorsInFormulas(int n1) {
    	TColors tc=getColorsAtTNode(n1);
    	for (int n=n1;n<end();n++) {
    		if (TNode(n).isColors()) {
    			break;
    		} else if (TNode(n).isMath()) {
    			TNode(n).setColors(tc);
    		}
    	}
    }

    private void setColorsInFormulas(int n1,int n2) {
    	TColors tc=getColorsAtTNode(n1);
    	for (int n=n1;n<n2;n++) {
    		if (TNode(n).isColors()) {
    			tc=TNode(n).getColors();
    		} else if (TNode(n).isMath()) {
    			TNode(n).setColors(tc);
    		}
    	}
    }

	public void setColors(Color bc,Color fc) {
		super.setColors(bc,fc);
		setColorsInFormulas(begin(),end());
	}

	public void insertComponent(Component C, String ctype, String cid) {
		TNode tn=new TNode(this,C,ctype,cid);
		insert(tn,getCursorPos());
	}

	public void insertHyperlink(String label,String url) {
		insert(new TNode(this,label+"|"+url,TNode.HYPERLINK),getCursorPos());
		// hay que resolver que todos los TNodes entre segBegin y segEnd tengan fondo de link y sean sensibles al clic
		
	}

 	public void insertFormula(String s) {
		insert(new TNode(this,s,TNode.FORMULA),getCursorPos());
	}
	public void insertFormula(Formula f) {
		insert(new TNode(this,f),getCursorPos());
	}

   public boolean delete() {
    	if (super.delete()) {
	  		clean();
	  		return true;
    	}
    	return false;
    }

    public boolean backSpace() {
    	if (super.backSpace()) {
	  		clean();
	  		return true;
	    }
    	return false;
    }

	public void removeMarkedSegment() {
		if (segMarked()) {
			TColors tc=getColorsAtTNode(segMax());
			TFont f=getTFontAtTNode(segMax());
			remove(segMin(),segMax());
			insertColors(tc,false);
			insertTFont(f,false);
		}
		unmarkSegment();
	}


    /* ------------------ Graphic Text Metrics ------------------ */

    public TNode getTNodeAtPixel(Component C,int x,int y) {
    	int i=getIndexOfLineAtPixel(C,y);
   		int o=getOffsetOfTNodeAtPixel(C,i,x,true);
   		int n=i+o;
    	TNode tn=TNode(n);
    	int y0=getY(C,n)-getLineAscentAtTNode(C,n);
    	if (y0<=y && y<y0+getLineHeightAtTNode(C,n)) {
    		return tn;
    	}
    	return null;
    }


    public int getIndexOfTNodeAtPixel(Component C,int x,int y) {
    	int n=getIndexOfLineAtPixel(C,y);
    	int o=getOffsetOfTNodeAtPixel(C,n,x,false);
		return firstChar(n+o);
    }

    public int getIndexOfLineAtPixel(Component C,int y) {
    	int i=firstLine();
    	int yy=topMargin+getLineHeightAtTNode(C,i);
    	while (i<end() && yy<y) {
    		int i0=i;
    		i=nextLine(i0);
    		if (i==i0) {
    			break;
    		}
    		yy+=getLineHeightAtTNode(C,i);
    	}
    	return i;
    }

    public int getOffsetOfTNodeAtPixel(Component C,int n,int x,boolean strict) {
    	int i0=startOfLine(n);
    	int i=i0;
    	int xx=leftMargin;
    	while (i<end() && !TNode(i).isEndOfLine() && xx+getCharVisualLimitAtTNode(C,i,false,strict)<x+3 ) {
   			xx+=getCharWidthAtTNode(C,i++,false);
    	}
    	return i-i0;
    }

    public int getX(Component C,int n) {
		int n1=startOfLine(n);
		TFont tf=getTFontAtTNode(n1);
		int x=0;
		for (int i=n1;i<n;i++) {
			TNode tn=TNode(i);
			if (tn.isTFont()) {
				tf=tn.getTFont();
			} else if (tn.isChar()) {
				if ("\n".equals(tn.getChar())) {
					x=0;
				} else {
					int chw=tn.getCharWidth(tf,C,false);
					x+=chw;
				}
			}
		}
		return leftMargin+x;
    }

    public int getY(Component C,int N) {
		int y=topMargin+getLineHeightAtTNode(C,N);
		int i0=startOfLine(N);
		int i=previousLine(i0);
		while (i<i0) {
    		y+=getLineHeightAtTNode(C,i);
    		i0=i;
    		i=previousLine(i0);
    	}
    	int h=y-getLineDescentAtTNode(C,N);
    	return h;
    }

    public int getTextY(Component C) {
    	return getY(C,end());
    }

    public int getHeight(Component C) {
    	return getY(C,end())+getLineDescentAtTNode(C,end())+bottomMargin+getExtraDescent(C);
    }

	private int getExtraDescent(Component C) {
    	int i=startOfLine(0);
		TFont tf=getTFontAtTNode(i);
		int xtrd=TNode(i).getExtraDescent(tf);
    	while (i<=end()) {
			TNode tn=TNode(i);
	    	if (tn.isChar() && tn.isComponent()) {
				xtrd=Math.max(xtrd,tn.getExtraDescent(tf));
	    	} else if (tn.isTFont()) {
				tf=tn.getTFont();
			}
			if (tn.isEndOfLine()) {
				xtrd=Math.max(0,xtrd-getLineHeightAtTNode(C,i));
			}
    		i++;
    	}
    	return xtrd;
    }

    public int getWidth(Component C) {
		TFont tf=getTFontAtTNode(firstLine());
		int w=0;
		int x=0;
		for (int i=0;i<end();i++) {
			TNode tn=TNode(i);
			if (tn.isTFont()) {
				tf=tn.getTFont();
			} else if (tn.isChar()) {
				if ("\n".equals(tn.getChar())) {
					w=Math.max(w,x);
					x=0;
				} else {
					int dx=tn.getCharWidth(tf,C,false);
					x+=dx;
				}
			}
		}
		w=Math.max(w,x);
		w+=2*leftMargin;
		return w;
    }

	public String toExpression(boolean content) {
		String s="";
		for (int i=0;i<info.size();i++) {
			TNode tn=(TNode) info.elementAt(i);
			s+=tn.toExpresion(content);
		}
		return s;
	}

    /* ----------------- drawing methods ---------------- */
    
	protected void updateComponentLocation(Container C,Graphics G,int n) {
		TNode tn=TNode(n);
		if (tn.isComponent()) {
			TFont tf=getTFontAtTNode(n);
			int a=tf.getAscent();
			int d=tf.getDescent();
			int x=getX(C,n),y=getY(C,n);
			int dx=tn.getCharWidth(tf,C,false);
			Component Cmp =tn.getComponent();
			int dx1=(dx-Cmp.getSize().width)/2;
	    	int X=x+dx1,Y=y-a-3,W=Cmp.getSize().width,H=Cmp.getSize().height;
    		if (Cmp instanceof com.mja.descgui.NumericControl) {
    			H=a+d+6;
			}
	    	if (X!=Cmp.getLocation().x||Y!=Cmp.getLocation().y||Cmp.getSize().width!=W||Cmp.getSize().height!=H) {
    			Cmp.setBounds(X,Y,W,H);
    		}
    		if (Cmp instanceof com.mja.descgui.NumericControl) {
	    		if (!Cmp.getFont().getName().equals(tf.getName())
	    			||Cmp.getFont().getStyle()!=(tf.getStyle()%4)
    				||Cmp.getFont().getSize()!=tf.getSize()
    			) {
    				Cmp.setFont(new Font(tf.getName(),tf.getStyle()%4,tf.getSize()));
    			}
    		}
			if (!(Cmp.getParent()==C)) {
    			C.add(Cmp);
    			Cmp.setVisible(true);
    			if (Cmp instanceof LiveOrDie) {
    				((LiveOrDie) Cmp).start();
    			}
    		}
    		Cmp.repaint();
		}
	}

    void updateComponentLocations(Container C,Graphics G) {
    	for (int i=0;i<info.size();i++) {
    		TNode tn=TNode(i);
    		if (tn.isComponent()) {
	   			updateComponentLocation(C,G,i);
    		} else if (tn.getFI()!=null) {
    			FormulaItem FI=tn.getFI();
    			for (int j=0;j<FI.size();j++) {
    				Formula F=FI.getFormula(j);
    				F.locate(C,getX(C,i),getY(C,i));
    				F.updateComponentLocations(C);
    			}
    		} else {
//System.out.println("NODE TYPE "+tn.toExpresion(true));
    		}
    	}
    }

    public void drawSegment(Graphics g,Container C,boolean editing,int ini,int fin,boolean showholes) {
    	drawSegment(g,C,editing,ini,fin,true,showholes);
    }
    public void drawSegment(Graphics g,Container C,boolean editing,int ini,int fin,boolean initColor,boolean showholes) {
    	g.setColor(getColors().getTextColor());
    	int dY=0;
	    int i=ini;
    	while (i<=fin) {
    		int dy=drawLineAtTNode(g,C,editing,i,initColor,showholes,false);
    		dY=Math.max(dY,dy);
    		int i0=i;
    		i=nextLine(i);
    		if (i==i0) {
    			break;
    		}
    	}
    }

    private void drawLineAtTNode(Graphics g,Container C,boolean editing,int n,boolean showholes) {
    	drawLineAtTNode(g,C,editing,n,true,showholes,false);
    }

    private void calcAndSetLineFonts(int n) {
		int n1=startOfLine(n);
    	int n2=Math.min(endOfLine(n),end());
		TFont tf=getTFontAtTNode(n1);
    	for (int i=n1;i<n2;i++) {
    		TNode tn=TNode(i);
			if (tn.isTFont()) {
    			tf=tn.getTFont();
    		} else if (tn.isFormula()) {
				Formula form=tn.getFI().getFormula(0);
				form.calcAndSetFonts(tf);
    		}
    	}
    }
    
//    public void drawOnlyExpressions(Graphics g,Container C,boolean editing,int ini,int fin,boolean initColor,boolean showholes) {
//    	if (g!=null) {
//	    	g.setColor(getColors().getTextColor());
//	    	int dY=0;
//		    int i=ini;
//	    	while (i<=fin) {
//	    		int dy=drawLineAtTNode(g,C,editing,i,initColor,showholes,true);
//	    		dY=Math.max(dY,dy);
//	    		int i0=i;
//	    		i=nextLine(i);
//	    		if (i==i0) {
//	    			break;
//	    		}
//			}
//		}
//	}

    private int drawLineAtTNode(Graphics g0,Container C,boolean editing,int n,boolean initColor,boolean showholes,boolean onlyExpr) {
//System.out.println("drawLine ");
//long T0=System.currentTimeMillis();
		int n1=startOfLine(n);
		calcAndSetLineFonts(n1);
		int dy=getLineHeightAtTNode(C,n1);
		int lA=getLineAscentAtTNode(C,n1);
		int y1=getY(C,n1);
		int y=y1;
		Graphics2D g=(Graphics2D) g0;
		Image tima=null;
		if (C instanceof TextCanvas) {
   			tima=C.createImage(C.getSize().width,dy);
    		g=(Graphics2D) tima.getGraphics();
			y=lA;
		}
		TColors currentColors=getColorsAtTNode(n1,initColor);

		if (editing) {
			g.setColor(Color.white);
			g.fillRect(0,0,C.getSize().width,dy);
		}
	
    	int n2=Math.min(endOfLine(n),end());
		g.setColor(Color.black);
		Color bc=null;
		if (currentColors!=null) {
			bc=currentColors.getBackColor();
			if (currentColors.getTextColor()!=null) {
    			g.setColor(currentColors.getTextColor());
    		}
    	} else {
    		currentColors=getColors();
    	}
		if (segMarked()) {
			if (segMax()>=n1 && segMin()<n2) {
				Color gc=g.getColor();
				int x1=getX(C,Math.max(n1,segMin()));
				int x2=getX(C,Math.min(segMax(),n2));
				if (x1==leftMargin) {
					x1-=2;
				}
//				if (bc!=null) {
//					g.setColor(bestBG(bc));
//				} else {
					g.setColor(bestBG(Color.white));
//				}
				g.fillRect(x1,0,x2-x1,dy);
				int x3=getX(C,n2);
				g.setColor(gc);
			}
		}
		TFont tf=getTFontAtTNode(n1);
		int a=tf.getAscent();
		int d=tf.getDescent();
		int he=(a-d)/2;
		int uy=tf.getDescent()/2;
		int x=leftMargin;
		g.setColor(currentColors.getTextColor());
    	for (int i=n1;i<n2;i++) {
		    TFont.setAntialias(g,tf);
    		TNode tn=TNode(i);
    		if (tn.isColors()) {
    			currentColors=tn.getColors();
				if (currentColors!=null) {
					bc=currentColors.getBackColor();
					if (currentColors.getTextColor()!=null) {
		    			g.setColor(currentColors.getTextColor());
		    		}
		    	} else {
		    		currentColors=getColors();
		    	}
    		} else if (tn.isTFont()) {
    			tf=tn.getTFont();
			    TFont.setAntialias(g,tf);
    			a=tf.getAscent();
    			d=tf.getDescent();
    			he=(a-d)/2;
				uy=2;
    		} else if (tn.isComponent()) {
				int dx=tn.getCharWidth(tf,C,false);
    			Component Cmp =tn.getComponent();
				int dx1=(dx-Cmp.getSize().width)/2;
	    		if (segMarked() && ( (segMin()<=i) && (i<segMax()) ) ) {
		    		Cmp.setForeground(bestFG(Color.white));
		    		Cmp.setBackground(Color.black);
		    	} else {
	    			Cmp.setForeground(currentColors.getTextColor());
		    		Cmp.setBackground(null);
		    	}
		    	updateComponentLocation(C,g,i);
	    		x+=dx;
    		} else if (tn.isFormula()) {
				int dx=tn.getCharWidth(tf,C,false);
				Formula form=tn.getFI().getFormula(0);
		   		boolean isMarked=segMarked() && (segMin()<=i) && (i<segMax());
		   		form.locate(C,x+tn.getdX1(),y1);
				form.draw(g,C,tf,x+tn.getdX1(),y,editing,isMarked,true,showholes);
				x+=dx;
    		} else if (tn.isChar()) {
				String s=tn.getChar();
				if (s!=null) {
		    		if (segMarked()) {
		    			if ( (segMin()<=i) && (i<segMax()) ) {
		    				g.setColor(bestFG(Color.white));
		    			} else {
		    				g.setColor(currentColors.getTextColor());
		    			}
		    		}
					int dx=tn.getCharWidth(tf,C,true);
					Color c=g.getColor();
					if (bc!=null) {
						g.setColor(bc);
						int A=getLineAscentAtTNode(C,i);
						int D=getLineDescentAtTNode(C,i);
						g.fillRect(x,y-A,dx,A+D);
						g.setColor(c);
					}
					if (!tn.isHyperlink()) {
						while (i+1<n2) {
							TNode tni=TNode(i+1);
							if (tni.isPureChar()) {
								s+=tni.getChar();
								i++;
							} else {
								break;
							}
						}
					}
					if (tn.isHyperlink()) {
						g.setColor(new Color(0x662288));
					}
					dx=tf.getMjaFont().drawString(C,g,s,x,y);
					if (tf.isUnderline() || tf.isOverline() || tn.isHyperlink()) {
						TFont.setAntialias(g,false);
						if (tf.isUnderline() || tn.isHyperlink()) {
							g.drawLine(x,y+uy,x+dx-1,y+uy);
							if (tf.isBold()) {
								g.drawLine(x,y+uy+1,x+dx-1,y+uy+1);
							}
						}
						if (tn.isHyperlink()) {
							g.setColor(c);
						}
						if (tf.isOverline()) {
							g.drawLine(x,y-a,x+dx-1,y-a);
							if (tf.isBold()) {
								g.drawLine(x,y-a+1,x+dx-1,y-a+1);
							}
						}
						TFont.setAntialias(g,tf);
					}
					x+=dx;
 				}
	   		}
    	}
		if (C instanceof TextCanvas) {
			g0.drawImage(tima,0,y1-lA,C);
			C.getGraphics().drawImage(tima,0,y1-lA,C);
//System.out.println(C.getSize().height+" , "+y1);
		}
//long dT=System.currentTimeMillis()-T0;
//System.out.println("time elapsed "+dT);
    	return dy;
	}

}