/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package com.mja.textedit;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.io.*;

import com.mja.gui.editFrame;
import com.mja.gui.mjaText;
import com.mja.gui.mjaColor;
import com.mja.guiedit.*;
import com.mja.gui.MouseCursorHandler;
import com.mja.util.BasicStr;
import com.mja.util.TFont;
import com.mja.lang.*;
import com.mja.parser.*;
import com.mja.img.*;
import com.jla.nippe.NotSoAbstractNippe;
import com.mja.text.Text;
import com.mja.text.RTF;
import com.mja.file.mjaFont;

public class EditorQPanel extends Panel {

	private Frame pf;
	private NotSoAbstractNippe A;
	private translator Tr;
	private Parser parser;
	private boolean editable, moveimg, variableSize, addButtons;
	private boolean moveimg2; // MOS 040813
	private Color defaultFGColor;
	private TFont defaultTFont;
	private EditorButtons EB;
	private Panel NP;
	private EditorCanvas EC;
	private boolean useKB=false;

	public EditorQPanel(NotSoAbstractNippe A,Frame parent,translator Tr,Parser p,
		boolean editable,boolean moveimg,boolean variableSize,boolean addButtons) {
		this(A,parent,Tr,p,editable,moveimg,false,variableSize,addButtons);
	}

	public EditorQPanel(NotSoAbstractNippe A,Frame parent,translator Tr,Parser p,
		boolean editable,
		boolean moveimg,boolean moveimg2,
		boolean variableSize,boolean addButtons) {
		this.A=A;
		pf=parent;
		this.Tr=Tr;
		this.parser=p;
		this.editable=editable;
		this.moveimg=moveimg;
		this.moveimg2=moveimg2;
		this.variableSize=variableSize;
		this.addButtons=addButtons;
	}
	
	public void hideEditorButtons() {
		if (NP!=null) {
			NP.setVisible(false);
		}
	}

	public void showEditorButtons() {
		if (NP!=null) {
			NP.setVisible(true);
		}
	}

	public void useKB(boolean useit) {
		useKB=useit;
	}

	public void init() {
		setBackground(Color.lightGray);
		setFont(mjaFont.AWTSerif);
		setLayout(new BorderLayout());
		EC=new EditorCanvas(A,pf,Tr,null,parser,new TFont(mjaFont.makeFont("SansSerif",Font.PLAIN,16)),
			Color.black,moveimg,moveimg2,variableSize,false);
		EC.setBackground(Color.white);
		add("Center",EC);
		if (editable&&addButtons) {
			NP=new Panel();
			NP.setLayout(new GridLayout(1,1));
			NP.add(EB=new EditorButtons(A,pf,Tr,true,true,true,useKB));
			EB.setEditorCanvas(EC,false);
			EC.addActionListener(EB);
			add("North",NP);
		}
	}

	public void start() {
		EC.start();
		if (EB!=null) {
	 		setEditing(editable);
	 	}
	}

	public void stop() {
		EC.stop();
	}

	public void setEditing(boolean b) {
		if (EB!=null) {
			EB.setEditing(b);
		}
	}

	public EditorCanvas getEC() {
		return EC;
	}

	public void setCanvasSize(int w,int h) {
		EC.setSize(w,h);
	}

	public void setBGColor(Color c) {
		EC.setBackground(c);
	}

	public void setFGColor(Color c) {
		defaultFGColor=c;
	}

	public void setDefaultFont(TFont tfont) {
		defaultTFont=tfont;
	}

	public TFont getFirstFont() {
		return EC.getFirstFont();
	}

	public EditorButtons getEB() {
		return EB;
	}

	public void setText(String rtf_str) {
		Text T=new Text(parser,defaultTFont,null,defaultFGColor);
		RTF.parseRTF(rtf_str,T);
		T.setTopAndBottomMargin(12);
		T.setLeftMargin(12);
		EC.setText(T);
	}

	public String getText(boolean plain) {
		return EC.toRTF(plain);
	}

	public void setExprDefaults(int dec,boolean fixed,int aL) {
		EC.setExprDefaults(dec,fixed,aL);
	}
	
	public void paint(Graphics g) {
		EB.paint(g);
		EC.paint(g);
	}

}