/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package com.mja.textedit;

import java.awt.*;
import java.util.*;

import com.mja.parser.*;
import com.mja.util.BasicStr;
import org.unam.matem.Attribute;
import com.mja.text.Text;
import com.mja.text.Formula;

public class TextParser {

//	public static Formula parseStringToFormula (Parser p, String s0) {
//		String s=BasicStr.replace(s0,"\\n","\n");
//		String [] line=getLines(s);
//		Formula FF=new Formula(p);
//		for (int j=0;j<line.length;j++) {
//			if (j>0) {
//				FF.insertString("\n");
//			}
//			String[] word=getWords(line[j]);
//			for (int i=0;i<word.length;i++) {
//				if (i>0) {
//					FF.insertString(" ");
//				}
//				Formula F=getFormula(p,word[i]);
//				if (F!=null) {
//					FF.insertFormula(F);
//				} else {
//				}
//			}
//		}
//		return FF;
//	}

	public static Text parseString (Parser p, String s0) {
		String s=BasicStr.replace(s0,"\\n","\n");
		String [] line=getLines(s);
		Text T=new Text(p);
		for (int j=0;j<line.length;j++) {
			if (j>0) {
				T.insertString("\n");
			}
			String[] word=getWords(line[j]);
			for (int i=0;i<word.length;i++) {
				if (i>0) {
					T.insertString(" ");
				}
				Formula F=getFormula(p,word[i]);
				if (F!=null) {
					T.insertFormula(F);
				} else {
					T.insertString(word[i]);
				}
			}
		}
		return T;
	}

	public static Formula getFormula(Parser p,String s) {
		if (hasOperator(s)) {
			try {
				Node n=p.Analyse("TextParser",s,true,false);
				return n.toFormula(p,false,"\u00d7",true,false);
			} catch (Exception e) {
			}
		}
		return null;
	}

	public static Node getNode(Parser p,String s) {
		if (hasOperator(s)) {
			try {
				Node n=p.Analyse("TextParser",s,true,false);
				return n;
			} catch (Exception e) {
			}
		}
		return null;
	}

	public static boolean hasOperator(String s) {
		for (int i=0;i<s.length();i++) {
			if (Node.isOperator(s.charAt(i))) {
				return true;
			}
		}
		return false;
	}

	public static String[] getLines(String s) {
		StringTokenizer st=new StringTokenizer(s,"\n\r");
		Vector V=new Vector();
		while (st.hasMoreTokens()) {
			V.addElement(st.nextToken());
		}
		String [] sa=new String[V.size()];
		for (int i=0;i<V.size();i++) {
			sa[i]=(String) V.elementAt(i);
		}
		return sa;
	}

	public static String[] getWords(String s) {
		StringTokenizer st=new StringTokenizer(s," ");
		Vector V=new Vector();
		while (st.hasMoreTokens()) {
			V.addElement(st.nextToken());
		}
		String [] sa=new String[V.size()];
		for (int i=0;i<V.size();i++) {
			sa[i]=(String) V.elementAt(i);
		}
		return sa;
	}

	public static String toString(Parser p,Text T,boolean allP) {
		String [] line=getLines(T.toExpression(allP));
		String s="";
		for (int j=0;j<line.length;j++) {
			if (j>0) {
				s+="\\n";
			}
			String[] word=getWords(line[j]);
			for (int i=0;i<word.length;i++) {
				if (i>0) {
					s+=" ";
				}
				Node n=getNode(p,word[i]);
				if (n!=null) {
					s+=n.toExpression(allP);
				} else {
					s+=word[i];
				}
			}
		}
		return s;
	}

	public static String canonize(Parser p,String s,boolean allP) {
		Text T=parseString(p,s);
		return toString(p,T,allP);
	}

//	public static String canonizeAsFormula(Parser p,String s,boolean allP) {
//		Formula F=parseStringToFormula(p,s);
//		return toString(p,F,allP);
//	}

}