/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package edu.mec.descartesserver.beans;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Vector;
import java.util.zip.CRC32;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

public class EsceneBean {
	private String[] files;
	private String htmlCode;
	private String localPath;
	private String name;

	/**
	 * @return the files
	 */
	public String[] getFiles() {
		if (files == null)
			files = new String[0];
		return files;
	}

	/**
	 * @param files
	 *            the files to set
	 */
	public void setFiles(String[] files) {
		this.files = files;
	}

	/**
	 * @return the appletCode
	 */
	public String getHtmlCode() {
		return htmlCode;
	}

	/**
	 * @param appletCode
	 *            the appletCode to set
	 */
	public void setHtmlCode(String appletCode) {
		this.htmlCode = appletCode;
	}

	public void setLocalPath(String localUrl) {
		this.localPath = localUrl;
	}

	public String getLocalPath() {
		return localPath;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getName() {
		return name;
	}

	public ZipOutputStream preparaZip(ByteArrayOutputStream outputStream) throws IOException {
		File docbasedir = new File(this.getLocalPath());
		ZipOutputStream out = new ZipOutputStream(outputStream);
		Vector<Object[]> source = new Vector<Object[]>();
		Vector<String> yaEntry = new Vector<String>();
		String baseDirStr = docbasedir.getAbsolutePath();
		for (int i = 0; i < getFiles().length; i++) {
			String tmpFile = getFiles()[i];
			
			if (tmpFile == null || "".equals(tmpFile.trim()) || "index.html".equals(tmpFile) )
				continue;
			if(yaEntry.contains(tmpFile))
				continue;
			
			String entryN = baseDirStr + "/" + tmpFile;
			entryN = entryN.replaceAll("\\\\", "/");
			yaEntry.add(tmpFile);
			Object[] a = {tmpFile, new File(entryN)};
			source.add(a);
		}
		String code = this.getHtmlCode();
		source.add(new Object[]{"index.html", code});
		
		out.setLevel(6);
		CRC32 crc = new CRC32();
		byte[] buf = new byte[1024];
		// Compress the getFiles()
		for (int i = 0; i < source.size(); i++) {
			Object[] sourceArray = source.get(i);
			String entryName = (String)sourceArray[0];
			Object entrySource = sourceArray[1];
			try {
				byte[] leido = null;
				ZipEntry entry = new ZipEntry(entryName);
				if(entrySource instanceof File){
					// Transfer bytes from the file to the ZIP file
					InputStream in = new FileInputStream((File)entrySource);

					ByteArrayOutputStream stream = new ByteArrayOutputStream();
					int len;
					while ((len = in.read(buf)) > 0) {
						// byteBuffer.put(buf,0,len);
						stream.write(buf, 0, len);
					}
					leido = stream.toByteArray();
					in.close();
				} else {
					leido = entrySource.toString().getBytes();
				}
				
				crc.reset();
				crc.update(leido);
				entry.setCrc(crc.getValue());
				entry.setSize(leido.length);

				// Add ZIP entry to output stream.
				out.putNextEntry(entry);
				out.write(leido);
				out.closeEntry();
			} catch (FileNotFoundException e) {
				System.out.println("No se encuentra " + entrySource);
			}
		}
		out.close();
		// Complete the ZIP file
		return out;
	}

}