/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package org.cnice.lms.client;
/*
 * @(#)Client.java    2.0  3 Sep 2002
 *
 * This software is property of CNICE
 *
 */
 
import java.io.*;
import java.net.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

import org.cnice.lms.common.*;

/**
 *
 * @version	2.0  3 Sep 2002
 * @author Jose-Luis Abreu
 */
public class Client {
	
	public static final int admin=0,tutor=1,learner=2;
	public static final String[][] userTypeNames={
		{"Administrator","Tutor","Learner"},
		{"Administrador","Tutor","Alumno"}
	};

	private	String host = null;
	protected String user_login = null;
	protected String user_password = null;
	private int TRACE=0;
	private boolean dbon=false;
	private int initial_port,final_port,PORT;
	private String userID,userName,userType,userEMail;
	
	public Client(String host) {
		this(host,8082);
	}
	
	public Client(String host,int port) {
		this(host,port,port);
	}
	
	public Client(String host,int initial_port,int final_port) {
		this.host=host;
		this.initial_port=initial_port;
		this.final_port=final_port;
	}
	
	/** Opens a connection to the server if it is running and available.
	  * Returns: true if the connection is possible and false otherwise.
	  */
	public boolean connect() {
		TRACE=0;
		try{
			executeHello();
			dbon=true;
		} catch(Exception ex){
			dbon=false;
		}
		TRACE=1;
		return dbon;
	}
	
	public boolean isConnected() {
		return dbon;
	}

	public void setUser(String user_login,String user_password) {
		this.user_login=user_login;
		this.user_password=user_password;
	}
	
	String getUserLogin() {
		return user_login;
	}

	public String getUserID() {
		return userID;
	}

	public String getUserName() {
		return userName;
	}

	public String getUserType() {
		return userType;
	}

	public static String getUserTypeName(String userType) {
		for (int i=0;i<userTypeNames[0].length;i++) {
			if (userTypeNames[0][i].equals(userType)) {
				return userTypeNames[util.Lang][i];
			}
		}
		return "";
	}

	public static String getUserType(String userTypeName) {
		for (int i=0;i<userTypeNames[0].length;i++) {
			if (userTypeNames[util.Lang][i].equals(userTypeName)) {
				return userTypeNames[0][i];
			}
		}
		return "";
	}

	public String getUserEMail() {
		return userEMail;
	}

	protected void tracePrint(Object ob) {
		if (TRACE>0) {
			System.out.println(ob);
		}
	}
	
	public boolean userAllowed(String login,String password) throws Exception {
		String[] data=executeUserData(login,password);
		if (data.length>0) {
			userID=data[0];
			userName=data[1];
			userType=data[2];
			userEMail=data[3];
			return true;
		} else {
			userID="";
			userName="";
			userType="";
			userEMail="";
			return false;
		}
	}
	
    public String[] executeUserData(String login,String password) throws Exception {
		Message request = new UserDataRequest(login,password);
		Message response = sendRequest(request);
		if(response==null) throw new Exception("Unreachable destination");
	
		if(response.getType()==Message.USERDATA_RESPONSE){
		    return ((UserDataResponse) response).getUserData();
		}
		if(response.getType()==Message.EXCEPTION_RESPONSE){
			throw new Exception(((ExceptionResponse) response).getExceptionMessage());
		}
		tracePrint("Unexpected Response Type");
		return new String[0];
    }
    
    protected Message sendRequest(Message msg) throws Exception {
		InetAddress addr=null;
		Socket socket=null;
		try{
		    if (host==null || host.equals("")) {
				addr = InetAddress.getByName(null);
		    } else {
				addr = InetAddress.getByName(host);
			}
		} catch(Exception ue){
		    System.err.println("Unable to get localhost direction "+host);
		    return null;
		}
		try {
			socket = new Socket(addr,PORT);
		} catch(Exception e) {
	    	tracePrint("Socket failed");
		}
		if (socket==null) {
			throw new Exception("Establishment of connection is impossible");
		}
	    ObjectInputStream in=null;
    	ObjectOutputStream out=null;
		try {
		    out = new ObjectOutputStream(socket.getOutputStream());
		    out.flush();
		    //Hay que poner el out primero porque si no podemos tener un deadlock
		    //Si cliente y servidor esperan por el in primero.
		    in = new ObjectInputStream(socket.getInputStream());
		    out.writeObject(msg);
		    out.flush();		    
		    Message msg2 = null;
		    try{
				msg2=(Message)in.readObject();
		    } catch(ClassNotFoundException ce){
				System.err.println("Unknown Object received");
				ce.printStackTrace();
				socket.close();
				return null;
		    }
		    socket.close();
		    return msg2;
	
		} catch(Exception e) {
		    // The socket should be closed on any
		    // failures other than the socket
		    // constructor:
		    try {
				socket.close();
		    } catch(Exception e2) {
				System.err.println("Socket not closed");
		    }
		}
		return null;
    }

    private void executeHello() throws Exception {
		InetAddress addr=null;
		Socket socket=null;
	    ObjectInputStream in=null;
    	ObjectOutputStream out=null;
		Message request = new HelloRequest();
		try{
		    if (host==null || host.equals("")) {
				addr = InetAddress.getByName(null);
		    } else {
				addr = InetAddress.getByName(host);
			}
			tracePrint("InetAddress="+addr);
		} catch(Exception ue){
		    tracePrint("Unable to get localhost direction "+host);
		    throw ue;
		}
		PORT=initial_port;
		boolean portok=false;
		while (!portok) {
			while (true) {
				try {
//System.out.println("check connect "+PORT);
//SecurityManager sm = System.getSecurityManager();
//if (sm != null) { sm.checkConnect(host,PORT); }
//System.out.println("socket construction BEGIN");
   					socket = new Socket(addr,PORT);
					break;
				} catch (Exception e) {
//System.out.println("socket construction exception");
//System.out.println(e);
					if (PORT==final_port) {
						tracePrint("No port available");
						throw e;
					}
					PORT++;
				}
//System.out.println("socket construction END");
			}
			if (socket==null) {
				throw new Exception("Establishment of connection is impossible");
			}
			try {
			    out = new ObjectOutputStream(socket.getOutputStream());
			    out.flush();
			    in = new ObjectInputStream(socket.getInputStream());
			    out.writeObject(request);
			    out.flush();		    
			    Message response = null;
			    try{
					response=(Message)in.readObject();
			    } catch(ClassNotFoundException ce){
					tracePrint("Unknown Object received");
					socket.close();
					throw ce;
			    }
			    socket.close();
				if(response.getType()==Message.HELLO_RESPONSE){
					portok=true;
				} else {
				}
			} catch(Exception e) {
			    try {
					socket.close();
			    } catch(Exception e2) {
					tracePrint("Socket not closed");
					throw e2;
			    }
			}
			if (!portok) {
				if (PORT==final_port) {
					tracePrint("No port available");
					throw new Exception("No port available");
				}
				PORT++;
			}
		}
    }

}