/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package org.cnice.lms.client.admin;
/*
 * @(#)Client.java    2.0  3 Sep 2002
 *
 * This software is property of CNICE
 *
 */
 
import java.io.*;
import java.net.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

import org.cnice.lms.admin.*;
import org.cnice.lms.common.*;
import org.cnice.lms.client.*;

/**
 *
 * @version	2.0  3 Sep 2002
 * @author Jose-Luis Abreu
 */
public class AdminClient extends Client {
	
	String getUserTypeName() {
		for (int i=0;i<userTypeNames[0].length;i++) {
			if (getUserType().equals(userTypeNames[0][i])) {
				return userTypeNames[util.Lang][i];
			}
		}
		return getUserType();
	}
	
	public AdminClient(String host) {
		super(host);
	}
	
	public AdminClient(String host,int port) {
		super(host,port);
	}
	
	public AdminClient(String host,int initial_port,int final_port) {
		super(host,initial_port,final_port);
	}
	
    public SqlResult executeQuery(String query) throws Exception {
		Message request = new QueryRequest(user_login,user_password,query);
		Message response = sendRequest(request);
		if(response==null) throw new Exception("Unreachable destination");
	
		if(response.getType()==Message.QUERY_RESPONSE){
		    return ((QueryResponse) response).getValues();
		}
		if(response.getType()==Message.EXCEPTION_RESPONSE){
			throw new Exception(((ExceptionResponse) response).getExceptionMessage());
		}
		throw new Exception("Unexpected Response Type");
    }
    
    public int executeUpdate(String updateStr,SqlValue[] values) throws Exception {
    	if (updateStr==null||updateStr.length()==0) {
			throw new Exception("Incomplete data");
    	} else {
			Message request = new UpdateRequest(user_login,user_password,updateStr,values);
			Message response = sendRequest(request);
			if(response==null) throw new Exception("Unreachable destination");
		
			if(response.getType()==Message.UPDATE_RESPONSE){
			    return ((UpdateResponse) response).getRowsAffected();
			}
			if(response.getType()==Message.EXCEPTION_RESPONSE){
				throw new Exception(((ExceptionResponse) response).getExceptionMessage());
			}
			throw new Exception("Unexpected Response Type");
		}
    }
    
	public void getCourses(AdminChoice ch) {
		ch.removeAll();
		ch.addIdAndItem("","");
		String query="select id,name from courses order by name";
		try {
			SqlResult values = executeQuery(query);													
			if (values!=null) {
				for (int n=0;n<values.size();n++) {
					SqlValueArray va=values.valueAt(n);
					ch.addIdAndItem(va.getString("id"),va.getString("name"));
				}
			}
		} catch (Exception e) {
			System.out.println(e.getMessage());
		}
	}

	public void getUsers(int type,AdminChoice ch) {
		ch.removeAll();
		String query="select distinct id,concat(lastname,' ',firstname) as name from users where type='"+userTypeNames[0][type]+"'";
		if (userTypeNames[0][type].equals(AdminClient.userTypeNames[0][AdminClient.tutor]) && 
			getUserType().equals(AdminClient.userTypeNames[0][AdminClient.tutor])) {
			query+=" and id="+getUserID();
		} else if (userTypeNames[0][type].equals(AdminClient.userTypeNames[0][AdminClient.learner]) && 
			getUserType().equals(AdminClient.userTypeNames[0][AdminClient.learner])) {
			query+=" and id="+getUserID();
		} else {
			ch.addIdAndItem("","");
		}
		query+=" order by name";
		try {
			SqlResult values = executeQuery(query);
			if (values!=null) {
				for (int n=0;n<values.size();n++) {
					SqlValueArray va=values.valueAt(n);
					ch.addIdAndItem(va.getString("id"),va.getString("name"));
				}
			}
			if (getUserType().equals(userTypeNames[util.Lang][tutor])) {
				ch.select(getUserName());
			}
		} catch (Exception e) {
			System.out.println(e.getMessage());
		}
	}
	
	public void getTutors(String courseid,AdminChoice ch) {
		ch.removeAll();
		ch.addIdAndItem("","");
		String query="select distinct users.id,concat(users.lastname,' ',users.firstname) as name from tutors,users where users.id=tutors.userid";
		if (courseid.length()>0) {
			query+=" and tutors.courseid='"+courseid+"'";
		}
		query+=" order by name";
		try {
			SqlResult values = executeQuery(query);													
			if (values!=null) {
				for (int n=0;n<values.size();n++) {
					SqlValueArray va=values.valueAt(n);
					ch.addIdAndItem(va.getString("id"),va.getString("name"));
				}
			}
		} catch (Exception e) {
			System.out.println(e.getMessage());
		}
	}
	
}