/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package org.cnice.lms.server;

import java.io.*;
import java.util.Date;

public class SaveOutput extends PrintStream {

//    static OutputStream logfile;
    static PrintStream logfile;
    static PrintStream oldStdout;
    static PrintStream oldStderr;

    public SaveOutput(OutputStream ps) {
		super(ps);
    }

    // Starts copying stdout and stderr to the file f.
    public static void start(String f) throws IOException {
		// Save old settings.
		oldStdout = System.out;
		oldStderr = System.err;

		// Create/Open logfile.
		logfile = new PrintStream(
			new BufferedOutputStream(
			new FileOutputStream(f,true)));
		Date date=new Date();
		logfile.println("#\n#\n# "+date.toString()+"\n#\n");

		// Start redirecting the output.
		System.setOut(new SaveOutput(System.out));
		System.setErr(new SaveOutput(System.err));
    }

    // Restores the original settings.
    public static void stop() {
		System.setOut(oldStdout);
		System.setErr(oldStderr);
        try {
			logfile.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    // PrintStream override.
    public void write(int b) {
        try {
		    logfile.write(b);
        } catch (Exception e) {
            e.printStackTrace();
            setError();
        }
		super.write(b);
    }
    // PrintStream override.
    public void write(byte buf[], int off, int len) {
        try {
			logfile.write(buf, off, len);
        } catch (Exception e) {
            e.printStackTrace();
            setError();
        }
		super.write(buf, off, len);
    }
}

/*
The start() method first saves the current standard output and
standard error print streams.  These print streams will be restored
when stop() is called. Next, the log file is opened.  If the log 
file does not exist, it's created.  Otherwise, the log file is
emptied.  Finally, System.setOut() and System.setErr() are called 
to replace the standard output and standard error print streams with
SaveOutput print streams.

The stop () method restores the original standard output and 
standard error. It then closes the log file.

A SaveOutput object is a PrintStream object that acts like a tee. 
Any characters it receives are forwarded to two places: the log file 
and the underlying print streams. The underlying print streams in 
the example are the original standard output and standard error
print streams; these are supplied to the SaveOutput constructor.  
Although both standard output and standard error are written into the 
same logfile, there is no need to synchronize this operation.  The 
reason is that the logfile output stream is itself a print stream 
and write operations are synchronized.

To implement this tee behavior, the program needs to override the 
two forms of the write method. These overrides simply write the 
characters into the logfile and then to the underlying print stream 
(by calling super.write()). The write() methods do not throw 
exceptions. Instead, they set a flag in the print stream if some 
problem occurs. They set the flag by calling setError(). If the client 
of the print stream wants to check if an error occurred, it can call 
checkError().

.  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
*/