/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package org.cnice.lms.server;

import java.io.*;
import java.net.*;
import java.sql.*;
import org.cnice.lms.common.*;
import org.cnice.lms.admin.*;
import org.cnice.lms.learner.*;

class ServeOneRequest implements Runnable {
	
	DatabaseConnection dbc;
	
	private static final int admin=0,tutor=1,learner=2;
	private Server server;
    private Socket socket;
    private ObjectInputStream in;
    private ObjectOutputStream out;
    
    ServeOneRequest(Server server,DatabaseConnection dbc,Socket s) throws IOException {
    	this.server=server;
    	this.dbc=dbc;
		socket = s;
		out =
		    new ObjectOutputStream( socket.getOutputStream());
		in =
		    new ObjectInputStream( socket.getInputStream());
		// If any of the above calls throw an
		// exception, the caller is responsible for
		// closing the socket. Otherwise the thread
		// will close it.
    }
    
    public void run() {
    	Message msg=null, rmsg=null;
		try {
			msg = (Message) in.readObject();
			System.out.print(msg.getMessageName()+"...");
			rmsg = processRequest(msg);
			System.out.print(" Respond "+rmsg.getMessageName());
			out.writeObject(rmsg);
		} catch(ClassNotFoundException ee) {
		    ee.printStackTrace();
		} catch(OptionalDataException ode) {
		    ode.printStackTrace();
		} catch(EOFException eoe){
			eoe.printStackTrace();
		} catch(IOException ioe){
		    ioe.printStackTrace();
		}finally {
		    try {
				socket.close();
				System.out.println(".");
		    } catch(IOException e) {
				System.err.println("Socket not closed");
		    }
		}
		if (rmsg!=null&&rmsg.getType()==Message.CLOSESERVER_RESPONSE) {
			server.shutdown();
		}
    }
    
    private int getUserType(String login,String password) {
    	int type=-1;
System.out.println("Connected: "+dbc.isConnected());
    	if (dbc.isConnected()) {
			try{
				String query="SELECT type FROM users WHERE login='"+login+"' AND password='"+password+"'";
			    Statement stmt = dbc.getConnection().createStatement();
			    ResultSet rs = stmt.executeQuery(query);
				if (rs.next()) {
					String stype=rs.getString("type");
System.out.println("stype="+stype);
					if (stype!=null) {
						if (stype.equals("Learner")) {
							type=learner;
						} else if (stype.equals("Tutor")) {
							type=tutor;
						} else if (stype.equals("Administrator")) {
							type=admin;
						}
					} else {
						type=-1;
					}
				}
			    rs.close();
			    stmt.close();
			} catch(SQLException se) {
System.out.println(se);
				type=-1;
			}
    	}
    	return type;
    }
    
    private Message processRequest(Message msg) {
System.out.print("1");
		Message rmsg=null;
		String emsg="";
		try{
			if (dbc.isConnected()) {
System.out.print("2");
				if (msg.getType()==Message.HELLO_REQUEST){
System.out.print("3");
					rmsg = new HelloResponse();
				} else {
System.out.print("4");
		    		int usertype=getUserType(msg.login,msg.password);
		    		if (usertype>=0) {
						if (msg.getType()==Message.USERDATA_REQUEST){
System.out.print("5");
						    rmsg = new UserDataResponse(msg.login,msg.password,getPublicUserData(msg.login,msg.password));
						} else if (msg.getType()==Message.QUERY_REQUEST){
System.out.print("6");
							String query=((QueryRequest)msg).getQuery();
						    SqlResult values = executeQuery(query);
						    rmsg = new QueryResponse(msg.login,msg.password,values);
						} else if(msg.getType()==Message.UPDATE_REQUEST){
							if (usertype==admin) {
System.out.print("7");
								String updateStr=((UpdateRequest)msg).getUpdateString();
			    				SqlValue[] values=((UpdateRequest)msg).getValues();
					    		int n=executeUpdate(updateStr,values);
					    		rmsg = new UpdateResponse(msg.login,msg.password,n);
							} else {
System.out.print("8");
					    		emsg="User not allowed error.";
							}
						} else if (msg.getType()==Message.INSERTACTIVITY_REQUEST){
System.out.print("9");
					    	String inserStr=((InsertActivityRequest)msg).getInsertString();
	    					String user_login=((InsertActivityRequest)msg).getUserLogin();
	    					String course=((InsertActivityRequest)msg).getCourse();
				    		String[] info = executeInsertActivity(msg.login,msg.password,
					    			inserStr,user_login,course);
							if (info==null || info[0].equals("")) {
System.out.print("A");
					    		rmsg = new ExceptionResponse(msg.login,msg.password,
				    					"Error of the assignation of the identifier");
					    	} else {
System.out.print("B");
				    			rmsg = new InsertActivityResponse(msg.login,msg.password,info);
				    		}
						} else if (msg.getType()==Message.UPDATEACTIVITY_REQUEST){
System.out.print("C");
							String id=((UpdateActivityRequest)msg).getID();
	    					String appendStr=((UpdateActivityRequest)msg).getAppendStr();
				    		executeUpdateActivity(msg.login,msg.password,
				    			id,appendStr);
				    		rmsg = new UpdateActivityResponse(msg.login,msg.password);
						} else if(msg.getType()==Message.CLOSEACTIVITY_REQUEST){
System.out.print("D");
							String id=((CloseActivityRequest)msg).getID();
				    		executeCloseActivity(msg.login,msg.password,id);
				    		rmsg = new CloseActivityResponse(msg.login,msg.password);
						} else if (msg.getType()==Message.CLOSESERVER_REQUEST){
							if (usertype==admin) {
System.out.print("E");
							    rmsg = new CloseServerResponse(msg.login,msg.password);
							} else {
System.out.print("F");
					    		emsg="User not allowed error.";
							}
						} else {
System.out.print("G");
							emsg="Fatal error, Unknown Request type: " + msg.getType();
							System.err.println(emsg);
						}
					} else {
System.out.print("H");
						emsg="Authentication error.";
					}
				}
			} else {
System.out.print("I");
				emsg="Connection to database failed.";
			}
		} catch (Exception ex) {
System.out.print("J");
		    rmsg = null;
		    emsg=ex.getMessage();
		}
		if (rmsg==null) {
System.out.print("K");
		    rmsg = new ExceptionResponse(msg.login,msg.password,emsg);
			System.out.println(emsg);
		}
System.out.println("L");
		return rmsg;
    }
    
    private String getID(String table,String field,String value) {
    	String id=null;
		try{
			String query="SELECT id FROM "+table+" WHERE "+
				field+"='"+value+"'";
		    Statement stmt = dbc.getConnection().createStatement();
		    ResultSet rs = stmt.executeQuery(query);
			if (rs.next()) {
				id=rs.getString(1);
			}
		    rs.close();
		    stmt.close();
		} catch(SQLException se) {
			id=null;
		}
	    return id;
    }
    
    private String[] getPublicUserData(String login,String password) {
    	String[] data=new String[0];
		try{
			String query="SELECT * FROM users WHERE login='"+login+"' AND password='"+password+"'";
		    Statement stmt = dbc.getConnection().createStatement();
		    ResultSet rs = stmt.executeQuery(query);
			if (rs.next()) {
				data=new String[4];
				data[0]=rs.getString("id");
				data[1]=rs.getString("lastname")+" "+rs.getString("firstname");
				data[2]=rs.getString("type");
				data[3]=rs.getString("email");
			}
		    rs.close();
		    stmt.close();
		} catch(SQLException se) {
			data=new String[0];
		}
    	return data;
    }
    
    private SqlResult executeQuery(String query) throws Exception {
    	SqlResult values=new SqlResult();
		try{
		    Statement stmt = dbc.getConnection().createStatement();
		    ResultSet rs = stmt.executeQuery(query);
			ResultSetMetaData rsmd = rs.getMetaData ();
			int numCols = rsmd.getColumnCount ();
			SqlMetadata[] metadata= new SqlMetadata[numCols];
			for (int i=1; i<=numCols; i++) {
				int type = SqlMetadata.getSqlMetadataType(rsmd.getColumnType(i));
				metadata[i-1]=new SqlMetadata(type,rsmd.getColumnName(i));
			}
			while (rs.next ()) {
				SqlValue[] value=new SqlValue[numCols];
				for (int i=1; i<=numCols; i++) {
					switch (metadata[i-1].getType()) {
					case SqlMetadata.INTEGER : 
						value[i-1]=new SqlValue(metadata[i-1].getType(),
							rs.getInt(i)); 
							break;
					case SqlMetadata.VARCHAR : 
						value[i-1]=new SqlValue(metadata[i-1].getType(),
							rs.getString(i)); 
							break;
					case SqlMetadata.DATE : 
						value[i-1]=new SqlValue(metadata[i-1].getType(),
							rs.getString(i)); 
							break;
					case SqlMetadata.TIME : 
						value[i-1]=new SqlValue(metadata[i-1].getType(),
							rs.getString(i)); 
							break;
					case SqlMetadata.TIMESTAMP : 
						value[i-1]=new SqlValue(metadata[i-1].getType(),
							rs.getString(i)); 
							break;
					case SqlMetadata.LONGVARBINARY : 
						value[i-1]=new SqlValue(metadata[i-1].getType(),
							rs.getBytes(i)); 
							break;
					}
				}
				values.addValue(new SqlValueArray(metadata,value));
			}
		    rs.close();
		    stmt.close();
		} catch(SQLException se) {
		    System.err.println(se);
		    throw new Exception(se.getMessage());
		}
		    
		return values;
    }

    private int executeUpdate(String updateStr,SqlValue[] values) throws Exception {
    	int n=-1;
		try{
		    PreparedStatement pstmt = dbc.getConnection().prepareStatement(updateStr);
		    if (values!=null) {
		    	for (int i=0;i<values.length;i++) {
		    		switch (values[i].getType()) {
		    			case SqlMetadata.INTEGER:
		    					pstmt.setInt(i+1,values[i].getInt());
		    					break;
		    			case SqlMetadata.DATE: 
		    					pstmt.setString(i+1,values[i].getString());
		    					break;
		    			case SqlMetadata.TIME: 
		    					pstmt.setString(i+1,values[i].getString());
		    					break;
		    			case SqlMetadata.TIMESTAMP: 
		    					pstmt.setString(i+1,values[i].getString());
		    					break;
		    			case SqlMetadata.VARCHAR: 
		    					pstmt.setString(i+1,values[i].getString());
		    					break;
		    			case SqlMetadata.LONGVARBINARY: 
		    					pstmt.setBytes(i+1,values[i].getBytes());
		    					break;
		    		}
		    	}
		    }
		    n=pstmt.executeUpdate();
		    pstmt.close();
		    return n;
		} catch (SQLException se) {
		    se.printStackTrace();
		    System.err.println(se);
		    throw new Exception(se.getMessage());
		}
    }

    private String[] executeInsertActivity(String login,String password,
    	String insertStr,String user_login,String course) throws Exception {
    	String userid=getID("users","login",user_login);
    	String courseid=getID("courses","name",course);
		String[] info=null;
    	if (userid!=null&&courseid!=null) {
			try{
				String s="INSERT INTO activities "+insertStr;
			    PreparedStatement pstmt = dbc.getConnection().prepareStatement(s);
	    		pstmt.setString(1,userid);
	    		pstmt.setString(2,courseid);
			    pstmt.executeUpdate();
			    pstmt.close();
		
			    Statement stmt = dbc.getConnection().createStatement();
			    ResultSet rs = stmt.executeQuery("SELECT LAST_INSERT_ID()");
			    if(!rs.next()){
					throw new Exception("Unable to retrieve identifier");
			    }
			    String identifier=rs.getString(1);
			    rs.close();
			    stmt.close();
			    s="UPDATE learners SET last_activity='"+identifier+"' where userid='"+userid+"' and courseid='"+courseid+"'" ;
			    pstmt = dbc.getConnection().prepareStatement(s);
			    pstmt.executeUpdate();
			    pstmt.close();
			    String query="SELECT email FROM users,learners WHERE users.id=learners.tutor_userid and learners.userid='"+userid+"' and learners.courseid='"+courseid+"'" ;
			    stmt = dbc.getConnection().createStatement();
			    rs = stmt.executeQuery(query);
			    String email="";
			    if(rs.next()){
			    	email=rs.getString("email");
			    }
				rs.close();
				stmt.close();
				info=new String[2];
				info[0]=identifier;
				info[1]=email;
				System.out.println("Register "+identifier+" inserted");
			} catch (SQLException se) {
			    se.printStackTrace();
			    System.err.println(se);
			    throw new Exception(se.getMessage());
			}
		} else {
			String s="";
			if (userid==null) {
				s+=" useird is null ";
			}
			if (courseid==null) {
				s+=" courseid is null ";
			}
		    System.err.println(s);
		    throw new Exception(s);
		}
		return info;
    }

    private void executeUpdateActivity(String login,String password,
    	String id,String appendStr) throws Exception {
    	String userid=getID("users","login",login);
		if (userid==null) {
			throw new Exception("User no found");
		}
		try{
			String query="SELECT * FROM activities WHERE id='"+id+"'";
		    Statement stmt = dbc.getConnection().createStatement();
		    ResultSet rs = stmt.executeQuery(query);
		    if(!rs.next()){
				throw new Exception("Unable to retrieve object");
		    }
		    if (!rs.getString("userid").equals(userid)) {
			    rs.close();
			    stmt.close();
		    	throw new Exception("User not allowed");
		    }
		    if (rs.getInt("state")>0) {
			    rs.close();
			    stmt.close();
		    	throw new Exception("Error with state field");
		    }
		    byte[] ba=rs.getBytes("info");
		    rs.close();
		    stmt.close();

		    String auxs=new String(ba)+appendStr;

    		String s="UPDATE activities SET info=? WHERE id='"+id+"'";
		    PreparedStatement pstmt = dbc.getConnection().prepareStatement(s);
		    pstmt.setBytes(1,auxs.getBytes());
		    pstmt.executeUpdate();
		    pstmt.close();	
		} catch (SQLException se) {
		    se.printStackTrace();
		    System.err.println(se);
		    throw new Exception(se.getMessage());
		}
    }

    private void executeCloseActivity(String login,String password,String id) throws Exception {
    	String userid=getID("users","login",login);
		if (userid==null) {
			throw new Exception("User no found");
		}
		try{
			String query="SELECT * FROM activities WHERE id='"+id+"'";
		    Statement stmt = dbc.getConnection().createStatement();
		    ResultSet rs = stmt.executeQuery(query);
		    if(!rs.next()){
				throw new Exception("Unable to retrieve object");
		    }
		    if (!rs.getString("userid").equals(userid)) {
			    rs.close();
			    stmt.close();
		    	throw new Exception("User not allowed");
		    }
		    if (rs.getInt("state")>0) {
			    rs.close();
			    stmt.close();
		    	throw new Exception("Error with state field");
		    }
		    rs.close();
		    stmt.close();


    		String s="UPDATE activities SET state=1 WHERE id='"+id+"'";
		    PreparedStatement pstmt = dbc.getConnection().prepareStatement(s);
		    pstmt.executeUpdate();
		    pstmt.close();	
		} catch (SQLException se) {
		    se.printStackTrace();
		    System.err.println(se);
		    throw new Exception(se.getMessage());
		}
    }

}