/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package org.cnice.rad.client;
/*
 * @(#)Client.java
 */

/**
 * @version	1.0  Ene 2007
 * @administrator: Jose-Luis Abreu
 */

import java.io.*;
import java.net.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

import org.cnice.rad.common.*;
import org.cnice.rad.admin.*;
import org.cnice.rad.learner.*;

public class Client {

	public static final int admin=0,teacher=1,learner=2;
	public static final String[] userTypeNames={"Administrador","Profesor","Alumno"};
	
	public static final int usr_id=0, usr_tipo=1, usr_grupo=2,
		usr_codigo_a=3, usr_contrasenya=4,
		usr_apellido_p=5, usr_apellido_m=6, usr_nombres=7,
		usr_alias=8, usr_correo_e=9;

	private	String host = null;
	private String host_name="";
	protected String user_login = null;
	protected String user_password = null;
//	private String nombre_equipo;
	private static boolean TRACE=false;
	private boolean dbon=false;
	private int initial_port,final_port,PORT;
	private String[] userData;

	public Client(String nombre_equipo,String host,int initial_port,int final_port) {
//		this.nombre_equipo=nombre_equipo;
		this.host=host;
		this.initial_port=initial_port;
		this.final_port=final_port;
	}

	/** Opens a connection to the server if it is running and available.
	  * Returns: true if the connection is possible and false otherwise.
	  */
	public boolean connect() {
		TRACE=false;
		tracePrint("Hello...");
		try{
			executeHello();
			dbon=true;
			tracePrint("Hello accepted");
		} catch(Exception ex){
			tracePrint("Hello rejected");
			dbon=false;
		}
		return dbon;
	}

	public boolean isConnected() {
		return dbon;
	}

	public void setUser(String user_login,String user_password) {
		this.user_login=user_login;
		this.user_password=user_password;
	}

	String getUserLogin() {
		return user_login;
	}

	String getUserPassword() {
		return user_password;
	}
	
	public String getNombreEquipo() {
		return host_name;
	}

	public String getUserID() {
		return userData[usr_id];
	}

	public String getUserName() {
		return userData[usr_nombres]+" "+
		userData[usr_apellido_p]+" "+userData[usr_apellido_m];
	}

	public String getUserType() {
		return userData[usr_tipo];
	}

	public String getUserGroupID() {
		return userData[usr_grupo];
	}

	public static String getUserTypeName(String userType) {
		for (int i=0;i<userTypeNames.length;i++) {
			if (userTypeNames[i].equals(userType)) {
				return userTypeNames[i];
			}
		}
		return "";
	}

	public static String getUserType(String userTypeName) {
		for (int i=0;i<userTypeNames.length;i++) {
			if (userTypeNames[i].equals(userTypeName)) {
				return userTypeNames[i];
			}
		}
		return "";
	}

	public String getUserEMail() {
		return userData[usr_correo_e];
	}

	protected static void tracePrint(Object ob) {
		if (TRACE) {
			System.out.println(ob);
		}
	}

/*
 *
				data[0]=rs.getString("id");
				data[1]=rs.getString("tipo");
				data[2]=String.valueOf(rs.getInt("grupo"));
				data[3]=rs.getString("codigo_a");
				data[4]=rs.getString("contrasenya");
				data[5]=rs.getString("apellido_p");
				data[6]=rs.getString("apellido_m");
				data[7]=rs.getString("nombres");
				data[8]=rs.getString("alias");
				data[9]=rs.getString("correo_e");
 */
	public boolean userAllowed(String login,String password) throws Exception {
		userData=executeUserData(login,password);
		return (userData.length>0);
	}

	public boolean grupoCorrecto(String login,String password,String grupo) throws Exception {
		return executeGrupoCorrecto(login,password,grupo);
	}
	
    public String[] executeUserData(String login,String password) throws Exception {
		Message request = new UserDataRequest(login,password);
		Message response = sendRequest(request);
		if(response==null) throw new Exception("Unreachable destination");

		if(response.getType()==Message.USERDATA_RESPONSE){
		    return ((UserDataResponse) response).getUserData();
		}
		if(response.getType()==Message.EXCEPTION_RESPONSE){
			throw new Exception(((ExceptionResponse) response).getExceptionMessage());
		}
		tracePrint("Unexpected Response Type");
		return new String[0];
    }

    public boolean executeGrupoCorrecto(String login,String password,String grupo) throws Exception {
    	String query="SELECT * FROM usuario,grupo "+
    	 "WHERE usuario.codigo_a='"+login+"' AND usuario.contrasenya='"+password+"'"+
    	 "      AND grupo.nombre='"+grupo+"'"+
    	 "      AND usuario.grupo=grupo.id";
		Message request = new QueryRequest(login,password,query);
		Message response = sendRequest(request);
		if(response==null) throw new Exception("Unreachable destination");

		if(response.getType()==Message.QUERY_RESPONSE){
		    SqlResult values=((QueryResponse) response).getValues();
			return (values!=null && values.size()>0);
		}
		if(response.getType()==Message.EXCEPTION_RESPONSE){
			throw new Exception(((ExceptionResponse) response).getExceptionMessage());
		}
		tracePrint("Unexpected Response Type");
		return false;
    }
    
    Vector Queue=new Vector();
    
    protected void queueRequest(Serializable rqst) {
    	Queue.add(rqst);
    	
    }
    
    protected void sendQueue() {

		try{
	    	sendRequest(Queue);
	    	Queue.clear();
    	}catch(Exception e ){
    		e.printStackTrace();	
    	} 
 /*
		Message response = sendRequest(request);
		if(response==null) throw new Exception("Unreachable destination");

		if(response.getType()==Message.CLOSEACTIVITY_RESPONSE){
		    return;
		}
		if(response.getType()==Message.EXCEPTION_RESPONSE){
			throw new Exception(((ExceptionResponse) response).getExceptionMessage());
		}
		tracePrint("Unexpected Response Type");
*/
    }

    protected Message sendRequest(Serializable msg/*Message msg*/) throws Exception {
		InetAddress addr=null;
		Socket socket=null;
		try{
		    if (host==null || host.equals("")) {
				addr = InetAddress.getByName(null);
		    } else {
				addr = InetAddress.getByName(host);
			}
		} catch(Exception ue){
		    System.err.println("Imposible obtener direccin del servidor "+host);
		    return null;
		}
		try {
			socket = new Socket(addr,PORT);
		} catch(Exception e) {
	    	tracePrint("Fallo de socket");
		}
		if (socket==null) {
			throw new Exception("Imposible establecer la coneccin");
		}
	    ObjectInputStream in=null;
    	ObjectOutputStream out=null;
		try {
		    out = new ObjectOutputStream(socket.getOutputStream());
		    out.flush();
		    //Hay que poner el out primero porque si no podemos tener un deadlock
		    //Si cliente y servidor esperan por el in primero.
		    in = new ObjectInputStream(socket.getInputStream());
		    out.writeObject(msg);
		    out.flush();
		    Message msg2 = null;
		    try{
				msg2=(Message)in.readObject();
		    } catch(ClassNotFoundException ce){
				System.err.println("Se recibi un objeto desconocido");
				ce.printStackTrace();
				socket.close();
				return null;
		    }
		    socket.close();
		    return msg2;

		} catch(Exception e) {
		    // The socket should be closed on any
		    // failures other than the socket
		    // constructor:
		    try {
				socket.close();
		    } catch(Exception e2) {
				System.err.println("No se cerr el socket");
		    }
		}
		return null;
    }

    private void executeHello() throws Exception {
		InetAddress addr=null;
		Socket socket=null;
	    ObjectInputStream in=null;
    	ObjectOutputStream out=null;
		Message request = new HelloRequest();
		try{
		    if (host==null || host.equals("")) {
				addr = InetAddress.getByName(null);
		    } else {
				addr = InetAddress.getByName(host);
			}
			tracePrint("InetAddress="+addr);
		} catch(Exception ue){
		    tracePrint("Imposible obtener direccin del servidor "+host);
		    throw ue;
		}
		PORT=initial_port;
		boolean portok=false;
		while (!portok) {
			while (true) {
				try {
					tracePrint("new socket addr:"+addr+" port:"+PORT);
   					socket = new Socket(addr,PORT);
   					
   					InetAddress ia=socket.getLocalAddress();
   					host_name=ia.getHostName();
  					break;
				} catch (Exception e) {
					tracePrint("port "+PORT+" at host "+addr+" is not available");
					if (PORT==final_port) {
						throw e;
					}
					PORT++;
				}
			}
			if (socket==null) {
				throw new Exception("Imposible esablecer la conexin");
			}
			try {
			    out = new ObjectOutputStream(socket.getOutputStream());
			    out.flush();
			    in = new ObjectInputStream(socket.getInputStream());
			    out.writeObject(request);
			    out.flush();
			    Message response = null;
			    try{
					response=(Message)in.readObject();
			    } catch(ClassNotFoundException ce){
					tracePrint("Se recibi un objeto desconocido");
					socket.close();
					throw ce;
			    }
			    socket.close();
				if(response.getType()==Message.HELLO_RESPONSE){
					portok=true;
				} else {
				}
			} catch(Exception e) {
			    try {
					socket.close();
			    } catch(Exception e2) {
					tracePrint("El socket no se cerr");
					throw e2;
			    }
			}
			if (!portok) {
				if (PORT==final_port) {
					tracePrint("No port available");
					throw new Exception("El puerto no est disponible");
				}
				PORT++;
			}
		}
    }

    public SqlResult executeSesionGrupo(String login,String password,String id_grupo) throws Exception {
    	String query=
    		"SELECT grupo.nombre as nombre_grupo,grupo.id as id_grupo, "+
    		"       sesion_grupo.* FROM grupo,sesion_grupo "+
    		"WHERE grupo.id='"+id_grupo+"'"+
    		"      AND sesion_grupo.grupo=grupo.id";
//System.out.println(query);
		Message request = new QueryRequest(login,password,query);
		Message response = sendRequest(request);
		if(response==null) throw new Exception("Destino inalcanzable");

		if(response.getType()==Message.QUERY_RESPONSE){
			return ((QueryResponse) response).getValues();
		}
		if(response.getType()==Message.EXCEPTION_RESPONSE){
			throw new Exception(((ExceptionResponse) response).getExceptionMessage());
		}
		tracePrint("Respuesta inesperada");
		return null;
    }

    public String[] executeCrearSesionEquipo(String login,String password,
    	String nom_equipo,String id_grupo,String[] id_alumno) throws Exception {
//    	String id_grupo,String[] id_alumno) throws Exception {
		Message request = new CreateSesionEquipoRequest(login,password,
			nom_equipo,id_grupo,id_alumno);
//			id_grupo,id_alumno);
		Message response = sendRequest(request);
		if(response==null) throw new Exception("Destino inalcanzable");

		if(response.getType()==Message.CREATESESIONEQUIPO_RESPONSE){			 
			return ((CreateSesionEquipoResponse)response).getInfo();
		}
		if(response.getType()==Message.EXCEPTION_RESPONSE){
			throw new Exception(((ExceptionResponse) response).getExceptionMessage());
		}
		tracePrint("Respuesta inesperada");
		return null;
    }

}