/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package org.cnice.rad.common;

/*
 * @(#)SqlQueryTableModel.java
 */

/**
 * @version	1.0  Ene 2007
 * @administrator: Jose-Luis Abreu
 */

import javax.swing.JTable;
import javax.swing.JComponent;
import javax.swing.table.*;

public class SqlQueryTableModel extends AbstractTableModel {
	
	JTable jtbl;
	JComponent parent;
	SqlResult values;
	String[] columns, columnNames;
	int[] columnWidths;
		
	public SqlQueryTableModel() {
		this(null,null,null,null);
	}
	
	public SqlQueryTableModel(JComponent parent,
		String[] cols,String[] names,
		int[] widths) {
		this.parent=parent;
		this.columns=cols;
		this.columnNames=names;
		this.columnWidths=widths;
		values = new SqlResult();	
	}
	
	public void setTable(JTable jtbl) {
		this.jtbl=jtbl;
	}
	
	public JTable getTable() {
		return jtbl;
	}
	
	public String getColumnName(int i) {
		if (columnNames==null) {
			if (values!=null&&values.size()>0) {
				SqlValueArray va=values.valueAt(0);
				SqlMetadata metadata=va.getMetadata(i);
				if (metadata!=null) {
					return metadata.getName();
				}
			}
		} else {
			if (i<columnNames.length) {
				return columnNames[i];
			}
		}
		return "";
	}
	
	public int getColumnCount() { 
		if (columns==null) {
			if (values!=null&&values.size()>0) {
				SqlValueArray va=values.valueAt(0);
				return va.getMetadata().length;
			}
		} else {
			return columns.length;
		}
		return 0;
	}
	
	public int getRowCount() { 
		if (values!=null) {
			return values.size();
		}
		return 0;
	}
	
	public Object getValueAt(int row, int col) {
		if (values!=null&&row<values.size()) {
			SqlValueArray va=values.valueAt(row);
			int ix=col;
			if (columns!=null) {
				if (col>=columns.length) {
					return null;
				} else {
					for (ix=0;ix<va.getMetadata().length;ix++) {
						SqlMetadata metadata=va.getMetadata(ix);
						if (metadata!=null&&
							columns[col].equals(metadata.getName())) {
								break;
						}
					}
				}
			}
			if (ix<va.getMetadata().length) {
				SqlValue v=va.getValue(ix);
				switch (v.getType()) {
				case SqlMetadata.INTEGER: return new Integer(v.getInt());
				case SqlMetadata.DATE   :
				case SqlMetadata.TIME   :
				case SqlMetadata.TIMESTAMP   :
				case SqlMetadata.VARCHAR: return v.getString();
				case SqlMetadata.LONGVARBINARY   : return v.getBytes();
				default                 : return v.getString();
				}
			}
		}
		return "";
	}
	
	public boolean isCellEditable(int row, int col) {
		return false;
	}
	
	public void fire() {
		fireTableChanged(null);
		if (jtbl!=null&&parent!=null&&columnWidths!=null) {
			int wp=parent.getSize().width;
			int wc=0;
			int nc=getColumnCount();
			for (int i=0;i<nc;i++) {
				TableColumn tcol = jtbl.getColumnModel().getColumn(i);
				int w=(int)(((double)wp*(double)columnWidths[i])/100.0);
				if (i+1==nc) {
					w=wp-wc;
				}
				if (w<0) {
					w=0;
				}
				tcol.setPreferredWidth(w);
				wc+=w;
			}
		}			
	}
	
	public void updateValues(SqlResult values) {
		this.values=values;
	}
}