/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package org.cnice.rad.server;

/*
 * @(#)Sesion.java
 */

/**
 * @version	1.0  Ene 2007
 * @administrator: Jose-Luis Abreu
 */

import java.io.*;
import java.net.*;
import java.sql.*;

import org.cnice.rad.client.learner.LearnerClient;
import org.cnice.rad.client.Client;
import org.cnice.rad.client.ClientDialog;
import org.cnice.rad.common.SqlMetadata;
import org.cnice.rad.common.Message;
import org.cnice.rad.common.ExceptionResponse;

public class CrearSesionGrupo {

	public static void main(String[] args) {
		crear(true);
	}
	
	public static String crear(boolean doExit) {
		LearnerClient cl=null;
		ClientDialog cd=null;
		String db_host="";
		int db_initial_port=8082, db_final_port=8082;
		try {
			cl=new LearnerClient("",db_host,db_initial_port,db_final_port);
			cl.connect();
			if (cl.isConnected()) {
				if (cd==null) {
					cd=new ClientDialog(cl);
				}
				String[] login=cd.getUserLogin(Client.teacher);
				if (login!=null && login.length>=2) {
					cl.setLog(System.out);
					String grupo=cd.getGrupo(login[0],login[1]);
					if (grupo!=null) {
						boolean ok=false;
						try {
							String date=SqlMetadata.getDateTimeString();
							try{
								String insertStr="(grupo,inicio,estado) "+
									"VALUES ("
									+"?,"
									+"'"+date+"'"+","
									+"0"+")";
								ok = executeCreateSesionGrupo(login[0],login[1],insertStr,grupo);
							} catch (Exception ex) {
								ex.printStackTrace();
							}
						} catch (Exception e) {
							System.out.println(e);
						}
						if (ok) {
							System.out.println("Sesin creada para el grupo "+grupo);
							System.out.println("Para cerrar la sesin basta pulsar <intro>");
							while (System.in.read()!='\n') {
							};
							executeCerrarSesionGrupo(login[0],login[1],grupo);
						} else {
							System.out.println("Problemas para crear sesin");
						}
					} else {
						System.out.println("Problemas para crear sesin");
					}
				} else {
					cl=null;
				}
			} else {
				System.out.println("RAD not connected");
				cl=null;
			}
		} catch (Exception e) {
			e.printStackTrace();
			cl=null;
		}
		if (doExit) {
			System.exit(0);
		}
		return null;
	}

    private static boolean executeCreateSesionGrupo(String login,String password,String insertStr,String grupo) throws Exception {
		Message request = new CreateSesionGrupoRequest(login,password,insertStr,grupo);
		Message response = sendRequest(request);
		if(response==null) throw new Exception("Unreachable destination");

		if(response.getType()==Message.CREATESESIONGRUPO_RESPONSE){
		    return true;
		}
		if(response.getType()==Message.EXCEPTION_RESPONSE){
			throw new Exception(((ExceptionResponse) response).getExceptionMessage());
		}
		System.out.println("Unexpected Response Type");
		return false;
    }

    private static boolean executeCerrarSesionGrupo(String login,String password,String grupo) throws Exception {
		Message request = new CerrarSesionGrupoRequest(login,password,grupo);
		Message response = sendRequest(request);
		if(response==null) throw new Exception("Unreachable destination");

		if(response.getType()==Message.CLOSESESIONGRUPO_RESPONSE){
		    return true;
		}
		if(response.getType()==Message.EXCEPTION_RESPONSE){
			throw new Exception(((ExceptionResponse) response).getExceptionMessage());
		}
		System.out.println("Unexpected Response Type "+response.getType());
		return false;
    }

    private static Message sendRequest(Message msg) throws Exception {
		InetAddress addr=null;
		Socket socket=null;
		String host="";
		int PORT=8082;
		try{
		    if (host==null || host.equals("")) {
				addr = InetAddress.getByName(null);
		    } else {
				addr = InetAddress.getByName(host);
			}
		} catch(UnknownHostException ue){
		    System.err.println("Unable to get localhost direction "+host);
		    return null;
		}
		try {
			socket = new Socket(addr,PORT);
		} catch(IOException e) {
	    	System.out.println("Socket failed");
		}
		if (socket==null) {
			throw new Exception("Establishment of connection is impossible");
		}
	    ObjectInputStream in=null;
    	ObjectOutputStream out=null;
		try {
		    out = new ObjectOutputStream(socket.getOutputStream());
		    out.flush();
		    //Hay que poner el out primero porque si no podemos tener un deadlock
		    //Si cliente y servidor esperan por el in primero.
		    in = new ObjectInputStream(socket.getInputStream());
		    out.writeObject(msg);
		    out.flush();
		    Message msg2 = null;
		    try{
				msg2=(Message)in.readObject();
		    } catch(ClassNotFoundException ce){
				System.err.println("Unknown Object received");
				ce.printStackTrace();
				socket.close();
				return null;
		    }
		    socket.close();
		    return msg2;

		} catch(IOException e) {
		    // The socket should be closed on any
		    // failures other than the socket
		    // constructor:
		    try {
				socket.close();
		    } catch(IOException e2) {
				System.err.println("Socket not closed");
		    }
		}
		return null;
    }

}