/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package org.cnice.rad.server;

/*
 * @(#)DatabaseConnection.java
 */

/**
 * @version	1.0  Ene 2007
 * @administrator: Jose-Luis Abreu
 */

import java.sql.*;
/**
 * This class makes the Data Base connection.
 * All classes needing to access the SQL inherit this class
 * which provides an object of type <code>Connection</code> upon which
 * the SQL requests are performed.
 */
class DatabaseConnection {

	String db_driver, db_host,
		db_user, db_password, db_initial_port, db_final_port;

    private Connection conn=null;

    private static boolean driver_loaded=false;

    DatabaseConnection(String db_driver,String db_host,
    	String db_user, String db_password, String db_initial_port, String db_final_port){
    	this.db_driver=db_driver;
    	this.db_host=db_host;
    	this.db_user=db_user;
    	this.db_password=db_password;
    	this.db_initial_port=db_initial_port;
    	this.db_final_port=db_final_port;
		loadDriver();
		if (driver_loaded) {
			connect();
		}
    }

    Connection getConnection() throws InternalError{
		if(driver_loaded==false) throw new InternalError("The driver has not been loaded");
		if(conn==null) throw new InternalError("The connection is not established");
		return conn;
    }

    boolean isConnected(){
		if(conn!=null) return true;
		return false;
    }

    boolean driverIsLoaded(){
		return driver_loaded;
    }

    protected void finalize(){
		try{
		    conn.close();
		} catch(SQLException se){
		    System.err.println("An exception has been throwed in Connection.close(): "+se);
		}
    }

    private void loadDriver(){
		if(driver_loaded==false){
		    try{
		    	System.out.println("Loading driver "+db_driver);
				Class.forName(db_driver).newInstance();
			    driver_loaded=true;
		    } catch(ClassNotFoundException ce){
				driver_loaded=false;
				System.err.println("The Driver can't be instantiated. An exception has been throwed: "+ce);
		    } catch(InstantiationException ie){
				driver_loaded=false;
				System.err.println("The Driver can't be instantiated. An exception has been throwed: "+ie);
		    } catch(IllegalAccessException iae){
				driver_loaded=false;
				System.err.println("The Driver can't be instantiated. An exception has been throwed: "+iae);
		    }
			if (driverIsLoaded()) {
				System.out.println("Database = "+db_host);
//		    	System.out.println("User = "+db_user);
			}
		}
    }

    private void connect() {
    	if (conn!=null) {
    		finalize();
    	}
		try{
//System.out.println("db_host="+db_host+" db_user="+db_user+" db_password="+db_password);
		    conn = DriverManager.getConnection(db_host,db_user,db_password);
		} catch(SQLException se){
		    conn = null;
		    System.out.println("Connection to database "+db_host+" failed.");
		    se.printStackTrace();
		}
    }

}