/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package org.unam.matem;
/*------------------------------------------------------------*/
/*                          Attribute                         */
/*                          (c)  MJA                          */
/*                       Febrero de 2001                      */
/*               ltima actualizacin Julio de 2002           */
/*------------------------------------------------------------*/
import java.util.*;
import java.awt.*;

public class Attribute {

	public String name, value;

	public Attribute() {
	}

	public Attribute(String n,String v) {
		name=n; value=v;
		if (name==null) {
			name="";
		}
		if (value==null) {
			value="";
		}
	}

	public static Attribute[] getAttrWithPrefix(Attribute[] AA,String prefix) {
		Vector v=new Vector();
		for (int i=0;i<AA.length;i++) {
			if (AA[i].name.startsWith(prefix)) {
				v.addElement(AA[i]);
			}
		}
		Attribute[] aa=new Attribute[v.size()];
		for (int i=0;i<aa.length;i++) {
			aa[i]=(Attribute) v.elementAt(i);
		}
		return aa;
	}

	public static String getValue(String s,String attrName) {
		Attribute[] aa=parse(s);
		return getValue(aa,attrName);
	}

	public static String getValue(String s,String attrName,String nullval) {
		Attribute[] aa=parse(s);
		return getValue(aa,attrName,nullval);
	}

	public static String getValueIgnoreCase(String s,String attrName,boolean trimvalue) {
		Attribute[] aa=parse(s,trimvalue);
		return getValueIgnoreCase(aa,attrName);
	}

	public static String getValueIgnoreCase(Attribute[] aa,String attrName) {
		if (aa!=null) {
			for (int i=0;i<aa.length;i++) {
				if (aa[i].name.equalsIgnoreCase(attrName)) {
					return aa[i].value;
				}
			}
		}
		return "";
	}
	
	public static Attribute[] copy (Attribute[] aa) {
		if (aa==null) {
			return null;
		}
		Attribute[] naa=new Attribute[aa.length];
		for (int i=0;i<aa.length;i++) {
			naa[i]=new Attribute(new String(aa[i].name),new String(aa[i].value));
		}
		return naa; 
	}

	public static String getValue(Attribute[] aa,String attrName) {
		return getValue(aa,attrName,"");
	}

	public static void setValue(Attribute[] aa,String attrName,String value) {
		if (aa!=null) {
			for (int i=0;i<aa.length;i++) {
				if (aa[i].name.equals(attrName)) {
					aa[i].value=new String(value);
					break;
				}
			}
		}
	}

	public static String getValue(Attribute[] aa,String attrName,String nullval) {
		if (aa!=null) {
			for (int i=0;i<aa.length;i++) {
				if (aa[i].name.equals(attrName)) {
					return aa[i].value;
				}
			}
		}
		return nullval;
	}

	private static final char[] Quotes = {
		'&',
		'>',
		'<',
		'"',
		'\''
	};

	private static final String[] codedQuotes = {
		"&amp;",
		"&gt;",
		"&lt;",
		"&quot;",
		"&squot;"
	};

	/** Get the HTML encoding of ",',<,>,&.
     *  @param value The string to convert.
     *  @return The string converted.
     */
    public static String encodeQuotes(String value){
		StringBuffer sb=new StringBuffer();
		if (value!=null) {
			for (int i=0; i<value.length(); i++) {
				char ch=value.charAt(i);
				boolean isQuote=false;
				for (int n=0;n<Quotes.length;n++) {
					if (ch==Quotes[n]) {
						isQuote=true;
						sb.append(codedQuotes[n]);
						break;
					}
				}
				if (!isQuote) {
					sb.append(ch);
				}
			}
		}
        return new String(sb);
    }

	/** Get the decodification of ",',<,>,&.
     *  @param value The string to convert.
     *  @return The string converted.
     */
    public static String decodeQuotes(String value){
         if (value==null || value.length()==0) {
        	return value;
        }
		StringBuffer sb=new StringBuffer();
		int i=0;
		while (i<value.length()) {
			char ch = value.charAt(i++);
			if (ch=='&') {
				int j=value.indexOf(";",i);
				if (j>=0) {
					String test=value.substring(i-1,j+1);
					for (int n=0;n<Quotes.length;n++) {
						if (test.equals(codedQuotes[n])) {
							ch=Quotes[n];
							i=j+1;
							break;
						}
					}
				}
			}
			sb.append(ch);
		}
		return sb.toString();
    }

	public static Attribute[] parse(String s) {
		return parse(s,false);
	}

	private static Attribute[] parse(String s,boolean trimvalue) {
		if (s==null) { return new Attribute[0]; }
		Vector aV = new Vector();
		int pos0=0;
		do {
			int poseq=s.indexOf("=",pos0);
			if (poseq<0) {
				break;
			}
			String name=s.substring(pos0,poseq).trim();
			int pos=poseq+1;
			int posesp=-1;
			boolean escape=false;
			boolean encomilla=false;
			char cha=(char)0;
			while (pos<s.length()) {
				char ch=s.charAt(pos);
				if (ch=='\''&&!escape) {
					if (encomilla) {
						pos++;
						break;
					} else {
						encomilla=true;
					}
				} else if (!encomilla) {
					if (ch==' '||ch=='\n') {
						posesp=pos;
					} else if (ch=='=') {
						if (pos+1<s.length()&&s.charAt(pos+1)=='=') { // ==
							pos++;
						} else if (cha=='<' || cha=='>' || cha=='!') { // <= || >= || !=
						} else {
							pos=posesp+1;
							break;
						}
					}
				}
				if (ch=='\\'&&!escape) {
					escape=true;
				} else {
					escape=false;
				}
				pos++;
				cha=ch;
			}
			String auxs=s.substring(poseq+1,pos).trim();
			if (trimvalue) {
				auxs=trim(auxs,' ');
			}
			auxs=trim(auxs,'\'');
			auxs=decodeQuotes(auxs);
			aV.addElement(new Attribute(name,auxs));
			pos0=pos;
		} while (pos0<s.length());
		Attribute[] aa = new Attribute[aV.size()];
		Enumeration e=aV.elements();
		for (int i=0; i<aa.length;i++) {
			aa[i] = (Attribute) e.nextElement();
		}
		return aa;
	}

	public static String[] get100Values(String prefix, Attribute[] aa) {
		Vector v=new Vector();
		for (int i=0;i<aa.length;i++) {
			if (aa[i].name.startsWith(prefix)) {
				v.addElement(aa[i].value);
			}
		}
		String[] sa=new String[v.size()];
		Enumeration e=v.elements();
		for (int i=0;i<sa.length;i++) {
			sa[i]=(String) e.nextElement();
		}
		return sa;
	}

	private static String trim(String s,char ch) {
		if (s==null||s.length()==0) {
			return s;
		}
		int i1=0;
		while (s.charAt(i1)==ch) {
			i1++;
			if (i1>=s.length()) {
				return "";
			}
		}
		int i2=s.length()-1;
		while (s.charAt(i2)==ch) {
			i2--;
		}
		return s.substring(i1,i2+1);
	}


}