/*
_____________________________________________

Descartes. Interactive Mathematics for Teaching and Learning
This program is free software; you can redistribute it and/or 
modify it under the terms of the European Public License 
(EUPL) v 1.1 as published by the IDABC (see http://ec.europa.eu/idabc/). 

Developing institutions
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Software development: Jos Luis Abreu Len, Marta Oliver Serrat
Functional Design: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Copyright by Ministerio de Educacin de Espaa.

This programme is distributed in good faith, but WITHOUT ANY WARRANTY, not even
with the implicit warranties of USABILITY or FITNESS FOR A CONCRETE PURPOSE.
For more information consult the EUPL License European Public License. 

You should have received a copy of the EUPL European Public License along with this 
program. If not, see http://ec.europa.eu/idabc/servlets/Docbb6d.pdf?id=31979
_____________________________________________

Descartes. Matemticas interactivas para la enseanza y el aprendizaje
Este programa es software libre; usted tiene derecho a redistribuirlo y/o
a modificarlo bajo los trminos de la Licencia pblica europea 
(EUPL) v 1.1 publicados por la IDABC (ver  http://ec.europa.eu/idabc/).

Instituciones desarrolladoras:
- Instituto de Tecnologas Educativas del Ministerio de Educacin de Espaa
- Instituto de Matemticas de la Universidad Nacional Autnoma de Mxico

Desarrollo de software: Jos Luis Abreu Len, Marta Oliver Serrat
Diseo funcional: Jos Luis Abreu Len, Juan Madrigal Muga, Jos R. Galo Snchez

Derechos de explotacin propiedad del Ministerio de Educacin de Espaa.

Este programa se distribuye de buena fe, pero SIN NINGUNA GARANTA, incluso sin 
las presuntas garantas implcitas de USABILIDAD o ADECUACIN A PROPSITO CONCRETO. 
Para mas informacin consulte la Licencia EUPL European Public License.

Usted debe haber recibido una copia de la Licencia EUPLjunto con este 
programa, si por algn motivo no le es posible visualizarla, puede consultarla en
la siguiente URL: http://ec.europa.eu/idabc/servlets/Docb4f4.pdf?id=31980
_____________________________________________

*/
package org.unam.matem;

import java.util.*;

class Node {
	
	String token;
	int symb;
	int type;
	double r;
	Node left,right;
	
	Node(String s) {
		token=s;
		symb=Symbol(token);
		type=Type(symb);
	}
	
	static final int // SYMBOL
		LP    = 0,
		RP    = 1,
		MAS   = 2,
		MENOS = 3, 
		POR   = 4,
		DIV   = 5,
		VAR   = 6,
		NOOP  = 7;
	
	static int Symbol(String token) {
		if (token.equals("("))  { return LP; }
		if (token.equals(")"))  { return RP; }
		if (token.equals("+"))  { return MAS; }
		if (token.equals("-"))  { return MENOS; }
		if (token.equals("*"))  { return POR; }
		if (token.equals("/"))  { return DIV; }
		return VAR;
	}
	
	static final int // TYPE
		No=0,
		Lp=1,
		Rp=2,
		B1=3,
		B2=4,
		Vi=5,
		In=6;

	static int Type(int symb) {
		switch (symb) {
		case LP     : return Lp;
		case RP     : return Rp;
		case MAS    :
		case MENOS  : return B1;
		case POR	:
		case DIV    : return B2;
		case VAR    : return Vi;
		default     : return No;
		}
	}
	
	String toExpr() {
		if (left!=null && right!=null) {
			return "("+left.toExpr()+token+right.toExpr()+")";
		} else {
			return token;
		}
	}
	
	double Evaluate() {
		if (left!=null && right!=null) {
			switch (symb) {
				case MAS   : return left.Evaluate() + right.Evaluate();
				case MENOS : return left.Evaluate() - right.Evaluate();
				case POR   : return left.Evaluate() * right.Evaluate();
				case DIV   : return left.Evaluate() / right.Evaluate();
				default    : return 0; 
			}
		} else {
			return r;
		}
	}

}


public class Parser {
	
	static boolean isOperator(char ch) {
		switch (ch) {
			case '+' : 
			case '-' : 
			case '*' : 
			case '/' : return true;
			default  : return false;
		}
	}

	static boolean isDecimal(char ch)  {
		return isDigit(ch)||ch=='.';
	}

	static boolean isDigit(char ch) {
		return ch>='0'&&ch<='9';
	}
	
	static boolean isSpace(char ch) {
		return ch==' ' || ch=='\10' || ch=='\13';
	}

	static String[] getTokens(String expr) {
		int pos=0;
		Vector v=new Vector();
		while (pos<expr.length()) {
			char ch=expr.charAt(pos++);
			if (!isSpace(ch)) { 
				StringBuffer sb=new StringBuffer();
				sb.append(ch);
				if (isDecimal(ch)) {
					while ( (pos<expr.length()) && isDecimal(expr.charAt(pos)) ) { 
						sb.append(expr.charAt(pos++));
					}
				}
				String auxs=new String(sb);
				v.addElement(auxs);
			}
		} 
		String[] sa=new String[v.size()];
		Enumeration e=v.elements();
		for (int i=0;i<sa.length;i++) {
			sa[i]=(String) e.nextElement();
		}
		return sa;
	}
	
	static Node Parse(String expr) throws Exception {
		String[] tok=getTokens(expr);
		Stack NS=new Stack();
		Stack TNS=new Stack();
		boolean op_exp=false;
		int countLp=0;
		for (int i=0;i<tok.length;i++) {
			String token=tok[i];
			System.out.println("NEW TOKEN = "+token);
			Node node=new Node (token);
			if (node.type==Node.Rp) {
				TNS.push(node);
				printStacks(TNS,NS,op_exp);
				TNS.pop();
				if (countLp>0) {
					countLp--;
					op_exp=true;
					while (true) {
						Node N=(Node)TNS.pop();
						if (N.type==Node.Lp) {
							printStacks(TNS,NS,op_exp);
							break;
						} else {
							addNode(NS,N);
						}
					}
				} else {
					throw new Exception("too many ) in "+expr);
				}
			} else if (!op_exp) {
				if (node.type==Node.Lp) {
					countLp++;
					TNS.push(node);
					printStacks(TNS,NS,op_exp);
				} else if (node.type==Node.Vi) {
					addNode(NS,node); 
					op_exp=true;
					printStacks(TNS,NS,op_exp);
				} else {
					throw new Exception("expression expected in "+expr+"\nat token "+i+" \""+node.token+"\"");
				}
			} else if (node.type==Node.B1 || node.type==Node.B2) {
				while (!TNS.isEmpty()) {
					Node N=(Node)TNS.peek();
					if (N.type<node.type) {
						break;
					}
					addNode(NS,(Node)TNS.pop());
					printStacks(TNS,NS,op_exp);
				}
				TNS.push(node); 
				op_exp=false;
				printStacks(TNS,NS,op_exp);
			} else {
				throw new Exception("operator expected in "+expr+"\nat token "+i+" \""+node.token+"\"");
			}
		}
		if (!op_exp) {
			throw new Exception("incomplete expression "+expr);
		} else if (countLp>0) {
			throw new Exception(" ) expected inn "+expr);
		}
		while (!TNS.isEmpty()) {
			addNode(NS,(Node)TNS.pop());
			printStacks(TNS,NS,op_exp);
		}
		return (Node) NS.peek();
	}

	private static void addNode(Stack NS,Node node) throws Exception {
		switch (node.type) {
		case Node.B1 :
		case Node.B2 :
			node.right=new Node("");
			if (!NS.isEmpty()) {
				node.right=(Node) NS.pop();
			}
			if (!NS.isEmpty()) {
				node.left=(Node) NS.pop();
			}
			break;
		case Node.Vi :
			try {
				node.r=Double.valueOf(node.token).doubleValue();
			} catch(NumberFormatException nfe) {
				throw nfe;
			}
			break;
		default : node.type=Node.In; break;
		}
		NS.push(node); 
	}
	
	static void printStacks(Stack S1,Stack S2,boolean op_exp) {
		printVector("PILA_O",S1);
		printVector("PILA_N",S2);
		System.out.println("op_exp = "+op_exp+"\n");
	}

	static void printVector(String title,Vector V) {
		System.out.print(title+": ");
		for (int i=0;i<V.size();i++) {
			System.out.print(((Node) V.elementAt(i)).toExpr()+"   ");
		}
		System.out.println("");
	}

	public static void main(String[] args) {
		try {
			Node N=Parse(args[0]);
			System.out.println(N.toExpr()+" = "+N.Evaluate());
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
}